@@ -126,7 +126,7 @@ static std::vector<int> ParseCPUList(const std::string &filename) {
 	std::string data;
 	std::vector<int> results;
 
-	if (File::ReadFileToString(true, Path(filename), data)) {
+	if (File::ReadSysTextFileToString(Path(filename), &data)) {
 		std::vector<std::string> ranges;
 		SplitString(data, ',', ranges);
 		for (auto range : ranges) {
@@ -426,7 +426,7 @@ void CPUInfo::Detect() {
 		if (GetLogicalProcessorInformation(&processors[0], &len)) {
 			num_cores = 0;
 			logical_cpu_count = 0;
-			for (auto processor : processors) {
+			for (const auto &processor : processors) {
 				if (processor.Relationship == RelationProcessorCore) {
 					num_cores++;
 					for (int i = 0; i < sizeof(processor.ProcessorMask) * 8; ++i) {
@@ -542,3 +542,29 @@ std::string CPUInfo::Summarize() {
 }
 
 #endif // PPSSPP_ARCH(X86) || PPSSPP_ARCH(AMD64)
+
+const char *GetCompilerABI() {
+#if PPSSPP_ARCH(ARMV7)
+	return "armeabi-v7a";
+#elif PPSSPP_ARCH(ARM64)
+	return "arm64";
+#elif PPSSPP_ARCH(X86)
+	return "x86";
+#elif PPSSPP_ARCH(AMD64)
+	return "x86-64";
+#elif PPSSPP_ARCH(RISCV64)
+    //https://github.com/riscv/riscv-toolchain-conventions#cc-preprocessor-definitions
+    //https://github.com/riscv/riscv-c-api-doc/blob/master/riscv-c-api.md#abi-related-preprocessor-definitions
+    #if defined(__riscv_float_abi_single)
+        return "lp64f";
+    #elif defined(__riscv_float_abi_double)
+        return "lp64d";
+    #elif defined(__riscv_float_abi_quad)
+        return "lp64q";
+    #elif defined(__riscv_float_abi_soft)
+        return "lp64";
+    #endif
+#else
+	return "other";
+#endif
+}
