@@ -44,6 +44,11 @@ VK_DEFINE_HANDLE(VmaAllocation);
 
 std::string VulkanVendorString(uint32_t vendorId);
 
+template<class R, class T> inline void ChainStruct(R &root, T *newStruct) {
+	newStruct->pNext = root.pNext;
+	root.pNext = newStruct;
+}
+
 // Not all will be usable on all platforms, of course...
 enum WindowSystem {
 #ifdef _WIN32
@@ -180,11 +185,18 @@ public:
 	void DestroyInstance();
 
 	int GetBestPhysicalDevice();
-	int GetPhysicalDeviceByName(std::string name);
-	void ChooseDevice(int physical_device);
-	bool EnableInstanceExtension(const char *extension);
-	bool EnableDeviceExtension(const char *extension);
-	VkResult CreateDevice();
+	int GetPhysicalDeviceByName(const std::string &name);
+
+	// Convenience method to avoid code duplication.
+	// If it returns false, delete the context.
+	bool CreateInstanceAndDevice(const CreateInfo &info);
+
+	// The coreVersion is to avoid enabling extensions that are merged into core Vulkan from a certain version.
+	bool EnableInstanceExtension(const char *extension, uint32_t coreVersion);
+	bool EnableDeviceExtension(const char *extension, uint32_t coreVersion);
+
+	// Was previously two functions, ChooseDevice and CreateDevice.
+	VkResult CreateDevice(int physical_device);
 
 	const std::string &InitError() const { return init_error_; }
 
@@ -272,6 +284,7 @@ public:
 		VkPhysicalDeviceMultiviewFeatures multiview;
 		VkPhysicalDevicePresentWaitFeaturesKHR presentWait;
 		VkPhysicalDevicePresentIdFeaturesKHR presentId;
+		VkPhysicalDeviceProvokingVertexFeaturesEXT provokingVertex;
 	};
 
 	const PhysicalDeviceProps &GetPhysicalDeviceProperties(int i = -1) const {
@@ -347,6 +360,7 @@ public:
 		// out of MAX_INFLIGHT_FRAMES.
 		return inflightFrames_;
 	}
+
 	// Don't call while a frame is in progress.
 	void UpdateInflightFrames(int n);
 
@@ -401,6 +415,14 @@ public:
 		return frame_[curFrame_].deleteList.GetLastDeleteCount();
 	}
 
+	u32 InstanceApiVersion() const {
+		return vulkanInstanceApiVersion_;
+	}
+
+	u32 DeviceApiVersion() const {
+		return vulkanDeviceApiVersion_;
+	}
+
 private:
 	bool ChooseQueue();
 
@@ -416,7 +438,7 @@ private:
 
 	bool CheckLayers(const std::vector<LayerProperties> &layer_props, const std::vector<const char *> &layer_names) const;
 
-	WindowSystem winsys_;
+	WindowSystem winsys_{};
 
 	// Don't use the real types here to avoid having to include platform-specific stuff
 	// that we really don't want in everything that uses VulkanContext.
@@ -428,6 +450,8 @@ private:
 	VkDevice device_ = VK_NULL_HANDLE;
 	VkQueue gfx_queue_ = VK_NULL_HANDLE;
 	VkSurfaceKHR surface_ = VK_NULL_HANDLE;
+	u32 vulkanInstanceApiVersion_ = 0;
+	u32 vulkanDeviceApiVersion_ = 0;
 
 	std::string init_error_;
 	std::vector<const char *> instance_layer_names_;
@@ -479,7 +503,7 @@ private:
 	std::vector<VkDebugUtilsMessengerEXT> utils_callbacks;
 
 	VkSwapchainKHR swapchain_ = VK_NULL_HANDLE;
-	VkFormat swapchainFormat_;
+	VkFormat swapchainFormat_ = VK_FORMAT_UNDEFINED;
 
 	uint32_t queue_count = 0;
 
@@ -488,7 +512,7 @@ private:
 	VkSurfaceCapabilitiesKHR surfCapabilities_{};
 	std::vector<VkSurfaceFormatKHR> surfFormats_{};
 
-	VkPresentModeKHR presentMode_;
+	VkPresentModeKHR presentMode_ = VK_PRESENT_MODE_FIFO_KHR;
 	std::vector<VkPresentModeKHR> availablePresentModes_;
 
 	std::vector<VkCommandBuffer> cmdQueue_;
@@ -496,12 +520,6 @@ private:
 	VmaAllocator allocator_ = VK_NULL_HANDLE;
 };
 
-// Detailed control.
-void TransitionImageLayout2(VkCommandBuffer cmd, VkImage image, int baseMip, int mipLevels, int numLayers, VkImageAspectFlags aspectMask,
-	VkImageLayout oldImageLayout, VkImageLayout newImageLayout,
-	VkPipelineStageFlags srcStageMask, VkPipelineStageFlags dstStageMask,
-	VkAccessFlags srcAccessMask, VkAccessFlags dstAccessMask);
-
 // GLSL compiler
 void init_glslang();
 void finalize_glslang();
@@ -517,8 +535,10 @@ bool GLSLtoSPV(const VkShaderStageFlagBits shader_type, const char *sourceCode,
 const char *VulkanColorSpaceToString(VkColorSpaceKHR colorSpace);
 const char *VulkanFormatToString(VkFormat format);
 const char *VulkanPresentModeToString(VkPresentModeKHR presentMode);
+const char *VulkanImageLayoutToString(VkImageLayout imageLayout);
 
 std::string FormatDriverVersion(const VkPhysicalDeviceProperties &props);
+std::string FormatAPIVersion(u32 version);
 
 // Simple heuristic.
 bool IsHashMaliDriverVersion(const VkPhysicalDeviceProperties &props);
