@@ -1,7 +1,8 @@
 #pragma once
-
 #include <cstdint>
 
+void TimeInit();
+
 // Seconds.
 double time_now_d();
 
@@ -13,24 +14,32 @@ uint64_t time_now_raw();
 double from_time_raw(uint64_t raw_time);
 double from_time_raw_relative(uint64_t raw_time);
 
+// Seconds, Unix UTC time
+double time_now_unix_utc();
+
 // Sleep. Does not necessarily have millisecond granularity, especially on Windows.
 void sleep_ms(int ms);
 
+// Precise sleep. Can consume a little bit of CPU on Windows at least.
+void sleep_precise(double seconds);
+
 // Yield. Signals that this thread is busy-waiting but wants to allow other hyperthreads to run.
 void yield();
 
-void GetTimeFormatted(char formattedTime[13]);
+void GetCurrentTimeFormatted(char formattedTime[13]);
 
-// Rust-style Instant for clear and easy timing.
+// Most accurate timer possible - no extra double conversions. Only for spans.
 class Instant {
 public:
 	static Instant Now() {
-		return Instant(time_now_d());
-	}
-	double Elapsed() const {
-		return time_now_d() - instantTime_;
+		return Instant();
 	}
+	double ElapsedSeconds() const;
+	int64_t ElapsedNanos() const;
 private:
-	explicit Instant(double initTime) : instantTime_(initTime) {}
-	double instantTime_;
+	Instant();
+	uint64_t nativeStart_;
+#ifndef _WIN32
+	int64_t nsecs_;
+#endif
 };
