@@ -90,32 +90,8 @@ enum class RestoreSettingsBits : int {
 };
 ENUM_CLASS_BITOPS(RestoreSettingsBits);
 
-inline std::string GPUBackendToString(GPUBackend backend) {
-	switch (backend) {
-	case GPUBackend::OPENGL:
-		return "OPENGL";
-	case GPUBackend::DIRECT3D9:
-		return "DIRECT3D9";
-	case GPUBackend::DIRECT3D11:
-		return "DIRECT3D11";
-	case GPUBackend::VULKAN:
-		return "VULKAN";
-	}
-	// Intentionally not a default so we get a warning.
-	return "INVALID";
-}
-
-inline GPUBackend GPUBackendFromString(const std::string &backend) {
-	if (!strcasecmp(backend.c_str(), "OPENGL") || backend == "0")
-		return GPUBackend::OPENGL;
-	if (!strcasecmp(backend.c_str(), "DIRECT3D9") || backend == "1")
-		return GPUBackend::DIRECT3D9;
-	if (!strcasecmp(backend.c_str(), "DIRECT3D11") || backend == "2")
-		return GPUBackend::DIRECT3D11;
-	if (!strcasecmp(backend.c_str(), "VULKAN") || backend == "3")
-		return GPUBackend::VULKAN;
-	return GPUBackend::OPENGL;
-}
+std::string GPUBackendToString(GPUBackend backend);
+GPUBackend GPUBackendFromString(std::string_view backend);
 
 enum AudioBackendType {
 	AUDIO_BACKEND_AUTO,
@@ -128,6 +104,7 @@ enum IOTimingMethods {
 	IOTIMING_FAST = 0,
 	IOTIMING_HOST = 1,
 	IOTIMING_REALISTIC = 2,
+	IOTIMING_UMDSLOWREALISTIC = 3,
 };
 
 enum class AutoLoadSaveState {
@@ -149,6 +126,12 @@ enum class BackgroundAnimation {
 	MOVING_BACKGROUND = 4,
 };
 
+// iOS only
+enum class AppSwitchMode {
+	SINGLE_SWIPE_NO_INDICATOR = 0,
+	DOUBLE_SWIPE_INDICATOR = 1,
+};
+
 // for Config.iShowStatusFlags
 enum class ShowStatusFlags {
 	FPS_COUNTER = 1 << 1,
@@ -174,6 +157,7 @@ enum class ScreenEdgePosition {
 	TOP_RIGHT = 5,
 	CENTER_LEFT = 6,
 	CENTER_RIGHT = 7,
+	CENTER = 8,  // Used for REALLY important messages! Not RetroAchievements notifications.
 	VALUE_COUNT,
 };
 
@@ -186,7 +170,27 @@ enum class DebugOverlay : int {
 	FRAME_PROFILE,
 #endif
 	CONTROL,
-	AUDIO,
+	Audio,
 	GPU_PROFILE,
 	GPU_ALLOCATOR,
+	FRAMEBUFFER_LIST,
+};
+
+// Android-only for now
+enum class DisplayFramerateMode : int {
+	DEFAULT,
+	REQUEST_60HZ,
+	FORCE_60HZ_METHOD1,
+	FORCE_60HZ_METHOD2,
+};
+
+enum class SkipGPUReadbackMode : int {
+	NO_SKIP,
+	SKIP,
+	COPY_TO_TEXTURE,
+};
+
+enum class RemoteISOShareType : int {
+	RECENT,
+	LOCAL_FOLDER,
 };
