@@ -76,7 +76,7 @@ protected:
 
 private:
 	void FormatErrorCode(uint32_t code);
-	void DisplayMessage(std::string text, bool hasYesNo = false, bool hasOK = false);
+	void DisplayMessage(const std::string &text, bool hasYesNo = false, bool hasOK = false);
 
 	enum Flags
 	{
