@@ -94,8 +94,8 @@ private:
 	void DisplaySaveIcon(bool checkExists);
 	void DisplaySaveDataInfo1();
 	void DisplaySaveDataInfo2(bool showNewData = false);
-	void DisplayMessage(std::string text, bool hasYesNo = false);
-	const std::string GetSelectedSaveDirName() const;
+	void DisplayMessage(std::string_view text, bool hasYesNo = false);
+	std::string GetSelectedSaveDirName() const;
 
 	void JoinIOThread();
 	void StartIOThread();
