@@ -63,7 +63,7 @@ namespace
 		truncate_cpy(str, strLength, value.c_str());
 	}
 
-	bool ReadPSPFile(std::string filename, u8 **data, s64 dataSize, s64 *readSize)
+	bool ReadPSPFile(const std::string &filename, u8 **data, s64 dataSize, s64 *readSize)
 	{
 		int handle = pspFileSystem.OpenFile(filename, FILEACCESS_READ);
 		if (handle < 0)
@@ -86,7 +86,7 @@ namespace
 		return result != 0;
 	}
 
-	bool WritePSPFile(std::string filename, const u8 *data, SceSize dataSize)
+	bool WritePSPFile(const std::string &filename, const u8 *data, SceSize dataSize)
 	{
 		int handle = pspFileSystem.OpenFile(filename, (FileAccess)(FILEACCESS_WRITE | FILEACCESS_CREATE | FILEACCESS_TRUNCATE));
 		if (handle < 0)
@@ -110,32 +110,31 @@ namespace
 		return info;
 	}
 
-	bool PSPMatch(std::string text, std::string regexp)
-	{
-		if(text.empty() && regexp.empty())
+	bool PSPMatch(std::string_view text, std::string_view regexp) {
+		if (text.empty() && regexp.empty())
 			return true;
-		else if(regexp == "*")
+		else if (regexp == "*")
 			return true;
-		else if(text.empty())
+		else if (text.empty())
 			return false;
-		else if(regexp.empty())
+		else if (regexp.empty())
 			return false;
-		else if(regexp == "?" && text.length() == 1)
+		else if (regexp == "?" && text.length() == 1)
 			return true;
-		else if(text == regexp)
+		else if (text == regexp)
 			return true;
-		else if(regexp.data()[0] == '*')
+		else if (regexp.data()[0] == '*')
 		{
 			bool res = PSPMatch(text.substr(1),regexp.substr(1));
 			if(!res)
 				res = PSPMatch(text.substr(1),regexp);
 			return res;
 		}
-		else if(regexp.data()[0] == '?')
+		else if (regexp.data()[0] == '?')
 		{
 			return PSPMatch(text.substr(1),regexp.substr(1));
 		}
-		else if(regexp.data()[0] == text.data()[0])
+		else if (regexp.data()[0] == text.data()[0])
 		{
 			return PSPMatch(text.substr(1),regexp.substr(1));
 		}
@@ -145,7 +144,7 @@ namespace
 
 	int align16(int address)
 	{
-		return ((address + 0xF) >> 4) << 4;
+		return (address + 15) & ~15;
 	}
 
 	int GetSDKMainVersion(int sdkVersion)
@@ -210,19 +209,16 @@ void SaveFileInfo::DoState(PointerWrap &p)
 
 SavedataParam::SavedataParam() { }
 
-void SavedataParam::Init()
-{
-	if (!pspFileSystem.GetFileInfo(savePath).exists)
-	{
-		pspFileSystem.MkDir(savePath);
-	}
+void SavedataParam::Init() {
+	// If the folder already exists, this is a no-op.
+	pspFileSystem.MkDir(savePath);
 	// Create a nomedia file to hide save icons form Android image viewer
 #if PPSSPP_PLATFORM(ANDROID)
 	int handle = pspFileSystem.OpenFile(savePath + ".nomedia", (FileAccess)(FILEACCESS_CREATE | FILEACCESS_WRITE), 0);
 	if (handle >= 0) {
 		pspFileSystem.CloseFile(handle);
 	} else {
-		INFO_LOG(IO, "Failed to create .nomedia file (might be ok if it already exists)");
+		INFO_LOG(Log::IO, "Failed to create .nomedia file (might be ok if it already exists)");
 	}
 #endif
 }
@@ -332,13 +328,13 @@ bool SavedataParam::Delete(SceUtilitySavedataParam* param, int saveId) {
 
 	// Sanity check, preventing full delete of savedata/ in MGS PW demo (!)
 	if (!strlen(param->gameName) && param->mode != SCE_UTILITY_SAVEDATA_TYPE_LISTALLDELETE) {
-		ERROR_LOG(SCEUTILITY, "Bad param with gameName empty - cannot delete save directory");
+		ERROR_LOG(Log::sceUtility, "Bad param with gameName empty - cannot delete save directory");
 		return false;
 	}
 
 	std::string dirPath = GetSaveFilePath(param, GetSaveDir(saveId));
 	if (dirPath.size() == 0) {
-		ERROR_LOG(SCEUTILITY, "GetSaveFilePath returned empty - cannot delete save directory");
+		ERROR_LOG(Log::sceUtility, "GetSaveFilePath (%.*s) returned empty - cannot delete save directory. Might already be deleted?", (int)sizeof(param->gameName), param->gameName);
 		return false;
 	}
 
@@ -346,7 +342,7 @@ bool SavedataParam::Delete(SceUtilitySavedataParam* param, int saveId) {
 		return false;
 	}
 
-	ClearCaches();
+	ClearSFOCache();
 	pspFileSystem.RmDir(dirPath);
 	return true;
 }
@@ -378,11 +374,11 @@ int SavedataParam::DeleteData(SceUtilitySavedataParam* param) {
 	}
 
 	if (!subFolder.size()) {
-		ERROR_LOG(SCEUTILITY, "Bad subfolder, ignoring delete of %s", filePath.c_str());
+		ERROR_LOG(Log::sceUtility, "Bad subfolder, ignoring delete of %s", filePath.c_str());
 		return 0;
 	}
 
-	ClearCaches();
+	ClearSFOCache();
 	pspFileSystem.RemoveFile(filePath);
 
 	// Update PARAM.SFO to remove the file, if it was in the list.
@@ -404,7 +400,7 @@ int SavedataParam::DeleteData(SceUtilitySavedataParam* param) {
 		}
 
 		if (changed) {
-			std::unique_ptr<u8[]> updatedList(new u8[fileListSize]);
+			auto updatedList = std::make_unique<u8[]> (fileListSize);
 			memcpy(updatedList.get(), fileList, fileListSize);
 			sfoFile->SetValue("SAVEDATA_FILE_LIST", updatedList.get(), fileListSize, (int)FILE_LIST_TOTAL_SIZE);
 
@@ -412,7 +408,7 @@ int SavedataParam::DeleteData(SceUtilitySavedataParam* param) {
 			size_t sfoSize;
 			sfoFile->WriteSFO(&sfoData, &sfoSize);
 
-			ClearCaches();
+			ClearSFOCache();
 			WritePSPFile(sfoPath, sfoData, (SceSize)sfoSize);
 			delete[] sfoData;
 		}
@@ -426,12 +422,12 @@ int SavedataParam::Save(SceUtilitySavedataParam* param, const std::string &saveD
 		return SCE_UTILITY_SAVEDATA_ERROR_SAVE_MS_NOSPACE;
 	}
 	if (param->dataSize > param->dataBufSize) {
-		ERROR_LOG_REPORT(SCEUTILITY, "Savedata buffer overflow: %d / %d", param->dataSize, param->dataBufSize);
+		ERROR_LOG_REPORT(Log::sceUtility, "Savedata buffer overflow: %d / %d", param->dataSize, param->dataBufSize);
 		return SCE_UTILITY_SAVEDATA_ERROR_RW_BAD_PARAMS;
 	}
 	auto validateSize = [](const PspUtilitySavedataFileData &data) {
 		if (data.buf.IsValid() && data.bufSize < data.size) {
-			ERROR_LOG_REPORT(SCEUTILITY, "Savedata subdata buffer overflow: %d / %d", data.size, data.bufSize);
+			ERROR_LOG_REPORT(Log::sceUtility, "Savedata subdata buffer overflow: %d / %d", data.size, data.bufSize);
 			return false;
 		}
 		return true;
@@ -441,14 +437,14 @@ int SavedataParam::Save(SceUtilitySavedataParam* param, const std::string &saveD
 	}
 
 	if (param->secureVersion > 3) {
-		ERROR_LOG_REPORT(SCEUTILITY, "Savedata version requested on save: %d", param->secureVersion);
+		ERROR_LOG_REPORT(Log::sceUtility, "Savedata version requested on save: %d", param->secureVersion);
 		return SCE_UTILITY_SAVEDATA_ERROR_SAVE_PARAM;
 	} else if (param->secureVersion != 0) {
 		if (param->secureVersion != 1 && !HasKey(param) && secureMode) {
-			ERROR_LOG_REPORT(SCEUTILITY, "Savedata version with missing key on save: %d", param->secureVersion);
+			ERROR_LOG_REPORT(Log::sceUtility, "Savedata version with missing key on save: %d", param->secureVersion);
 			return SCE_UTILITY_SAVEDATA_ERROR_SAVE_PARAM;
 		}
-		WARN_LOG(SCEUTILITY, "Savedata version requested on save: %d", param->secureVersion);
+		WARN_LOG(Log::sceUtility, "Savedata version requested on save: %d", param->secureVersion);
 	}
 
 	std::string dirPath = GetSaveFilePath(param, GetSaveDir(param, saveDirName));
@@ -460,7 +456,7 @@ int SavedataParam::Save(SceUtilitySavedataParam* param, const std::string &saveD
 		}
 	}
 
-	u8* cryptedData = 0;
+	u8* cryptedData = nullptr;
 	int cryptedSize = 0;
 	u8 cryptedHash[0x10]{};
 	// Encrypt save.
@@ -468,13 +464,21 @@ int SavedataParam::Save(SceUtilitySavedataParam* param, const std::string &saveD
 	if (param->dataBuf.IsValid() && g_Config.bEncryptSave && secureMode)
 	{
 		cryptedSize = param->dataSize;
-		if(cryptedSize == 0 || (SceSize)cryptedSize > param->dataBufSize)
+		if (cryptedSize == 0 || (SceSize)cryptedSize > param->dataBufSize) {
+			ERROR_LOG(Log::sceUtility, "Bad cryptedSize %d", cryptedSize);
 			cryptedSize = param->dataBufSize; // fallback, should never use this
+		}
 		u8 *data_ = param->dataBuf;
 
 		int aligned_len = align16(cryptedSize);
-		cryptedData = new u8[aligned_len + 0x10];
+		if (aligned_len != cryptedSize) {
+			WARN_LOG(Log::sceUtility, "cryptedSize unaligned: %d (%d)", cryptedSize, cryptedSize & 15);
+		}
+
+		cryptedData = new u8[aligned_len + 0x10]();
 		memcpy(cryptedData, data_, cryptedSize);
+		// EncryptData will do a memmove to make room for the key in front.
+		// Technically we could just copy it into place here to avoid that.
 
 		int decryptMode = DetermineCryptMode(param);
 		bool hasKey = decryptMode > 1;
@@ -486,7 +490,7 @@ int SavedataParam::Save(SceUtilitySavedataParam* param, const std::string &saveD
 		if (EncryptData(decryptMode, cryptedData, &cryptedSize, &aligned_len, cryptedHash, (hasKey ? param->key : 0)) != 0) {
 			auto err = GetI18NCategory(I18NCat::ERRORS);
 			g_OSD.Show(OSDType::MESSAGE_WARNING, err->T("Save encryption failed. This save won't work on real PSP"), 6.0f);
-			ERROR_LOG(SCEUTILITY,"Save encryption failed. This save won't work on real PSP");
+			ERROR_LOG(Log::sceUtility,"Save encryption failed. This save won't work on real PSP");
 			delete[] cryptedData;
 			cryptedData = 0;
 		}
@@ -496,13 +500,20 @@ int SavedataParam::Save(SceUtilitySavedataParam* param, const std::string &saveD
 	std::string sfopath = dirPath + "/" + SFO_FILENAME;
 	std::shared_ptr<ParamSFOData> sfoFile = LoadCachedSFO(sfopath, true);
 
-	// Update values
-	sfoFile->SetValue("TITLE", param->sfoParam.title, 128);
-	sfoFile->SetValue("SAVEDATA_TITLE", param->sfoParam.savedataTitle, 128);
-	sfoFile->SetValue("SAVEDATA_DETAIL", param->sfoParam.detail, 1024);
-	sfoFile->SetValue("PARENTAL_LEVEL", param->sfoParam.parentalLevel, 4);
-	sfoFile->SetValue("CATEGORY", "MS", 4);
-	sfoFile->SetValue("SAVEDATA_DIRECTORY", GetSaveDir(param, saveDirName), 64);
+	// This was added in #18430, see below.
+	bool subWrite = param->mode == SCE_UTILITY_SAVEDATA_TYPE_WRITEDATASECURE || param->mode == SCE_UTILITY_SAVEDATA_TYPE_WRITEDATA;
+	bool wasCrypted = GetSaveCryptMode(param, saveDirName) != 0;
+
+	// Update values. NOTE! #18430 made this conditional on !subWrite, but this is not correct, as it causes #18687.
+	// So now we do a hacky trick and just check for a valid title before we proceed with updating the sfoFile.
+	if (strnlen(param->sfoParam.title, sizeof(param->sfoParam.title)) > 0) {
+		sfoFile->SetValue("TITLE", param->sfoParam.title, 128);
+		sfoFile->SetValue("SAVEDATA_TITLE", param->sfoParam.savedataTitle, 128);
+		sfoFile->SetValue("SAVEDATA_DETAIL", param->sfoParam.detail, 1024);
+		sfoFile->SetValue("PARENTAL_LEVEL", param->sfoParam.parentalLevel, 4);
+		sfoFile->SetValue("CATEGORY", "MS", 4);
+		sfoFile->SetValue("SAVEDATA_DIRECTORY", GetSaveDir(param, saveDirName), 64);
+	}
 
 	// Always write and update the file list.
 	// For each file, 13 bytes for filename, 16 bytes for file hash (0 in PPSSPP), 3 byte for padding
@@ -532,23 +543,21 @@ int SavedataParam::Save(SceUtilitySavedataParam* param, const std::string &saveD
 	delete[] updatedList;
 
 	// Init param with 0. This will be used to detect crypted save or not on loading
-	u8 *tmpData = new u8[128];
-	memset(tmpData, 0, 128);
-	sfoFile->SetValue("SAVEDATA_PARAMS", tmpData, 128, 128);
-	delete[] tmpData;
+	u8 zeroes[128]{};
+	sfoFile->SetValue("SAVEDATA_PARAMS", zeroes, 128, 128);
 
 	u8 *sfoData;
 	size_t sfoSize;
 	sfoFile->WriteSFO(&sfoData, &sfoSize);
 
 	// Calc SFO hash for PSP.
-	if (cryptedData != 0) {
+	if (cryptedData != 0 || (subWrite && wasCrypted)) {
 		int offset = sfoFile->GetDataOffset(sfoData, "SAVEDATA_PARAMS");
-		if(offset >= 0)
+		if (offset >= 0)
 			UpdateHash(sfoData, (int)sfoSize, offset, DetermineCryptMode(param));
 	}
 
-	ClearCaches();
+	ClearSFOCache();
 	WritePSPFile(sfopath, sfoData, (SceSize)sfoSize);
 	delete[] sfoData;
 	sfoData = nullptr;
@@ -573,24 +582,21 @@ int SavedataParam::Save(SceUtilitySavedataParam* param, const std::string &saveD
 			saveSize = cryptedSize;
 		}
 
-		INFO_LOG(SCEUTILITY,"Saving file with size %u in %s",saveSize,filePath.c_str());
+		INFO_LOG(Log::sceUtility,"Saving file with size %u in %s",saveSize,filePath.c_str());
 
 		// copy back save name in request
 		strncpy(param->saveName, saveDirName.c_str(), 20);
 
-		if (fileName.empty()) {
-			delete[] cryptedData;
-		} else {
+		if (!fileName.empty()) {
 			if (!WritePSPFile(filePath, data_, saveSize)) {
-				ERROR_LOG(SCEUTILITY, "Error writing file %s", filePath.c_str());
+				ERROR_LOG(Log::sceUtility, "Error writing file %s", filePath.c_str());
 				delete[] cryptedData;
 				return SCE_UTILITY_SAVEDATA_ERROR_SAVE_MS_NOSPACE;
 			}
-			delete[] cryptedData;
 		}	
+		delete[] cryptedData;
 	}
 
-
 	// SAVE ICON0
 	if (param->icon0FileData.buf.IsValid())
 	{
@@ -646,6 +652,7 @@ int SavedataParam::Load(SceUtilitySavedataParam *param, const std::string &saveD
 
 	// Load sfo
 	if (!LoadSFO(param, dirPath)) {
+		WARN_LOG(Log::sceUtility, "Load: Failed to load SFO from %s", dirPath.c_str());
 		return isRWMode ? SCE_UTILITY_SAVEDATA_ERROR_RW_DATA_BROKEN : SCE_UTILITY_SAVEDATA_ERROR_LOAD_DATA_BROKEN;
 	}
 
@@ -668,14 +675,14 @@ int SavedataParam::Load(SceUtilitySavedataParam *param, const std::string &saveD
 
 int SavedataParam::LoadSaveData(SceUtilitySavedataParam *param, const std::string &saveDirName, const std::string &dirPath, bool secureMode) {
 	if (param->secureVersion > 3) {
-		ERROR_LOG_REPORT(SCEUTILITY, "Savedata version requested: %d", param->secureVersion);
+		ERROR_LOG_REPORT(Log::sceUtility, "Savedata version requested: %d", param->secureVersion);
 		return SCE_UTILITY_SAVEDATA_ERROR_LOAD_PARAM;
 	} else if (param->secureVersion != 0) {
 		if (param->secureVersion != 1 && !HasKey(param) && secureMode) {
-			ERROR_LOG_REPORT(SCEUTILITY, "Savedata version with missing key: %d", param->secureVersion);
+			ERROR_LOG_REPORT(Log::sceUtility, "Savedata version with missing key: %d", param->secureVersion);
 			return SCE_UTILITY_SAVEDATA_ERROR_LOAD_PARAM;
 		}
-		WARN_LOG_REPORT(SCEUTILITY, "Savedata version requested: %d", param->secureVersion);
+		WARN_LOG_REPORT(Log::sceUtility, "Savedata version requested: %d", param->secureVersion);
 	}
 
 	std::string filename = GetFileName(param);
@@ -685,11 +692,11 @@ int SavedataParam::LoadSaveData(SceUtilitySavedataParam *param, const std::strin
 		return 0;
 
 	s64 readSize;
-	INFO_LOG(SCEUTILITY, "Loading file with size %u in %s", param->dataBufSize, filePath.c_str());
+	INFO_LOG(Log::sceUtility, "Loading file with size %u in %s", param->dataBufSize, filePath.c_str());
 	u8 *saveData = nullptr;
 	int saveSize = -1;
 	if (!ReadPSPFile(filePath, &saveData, saveSize, &readSize)) {
-		ERROR_LOG(SCEUTILITY,"Error reading file %s",filePath.c_str());
+		ERROR_LOG(Log::sceUtility,"Error reading file %s",filePath.c_str());
 		return SCE_UTILITY_SAVEDATA_ERROR_LOAD_NO_DATA;
 	}
 	saveSize = (int)readSize;
@@ -702,9 +709,9 @@ int SavedataParam::LoadSaveData(SceUtilitySavedataParam *param, const std::strin
 	bool saveDone = false;
 	u32 loadedSize = 0;
 	if (isCrypted) {
-		if (DetermineCryptMode(param) > 1 && !HasKey(param))
+		if (DetermineCryptMode(param) > 1 && !HasKey(param)) {
 			return SCE_UTILITY_SAVEDATA_ERROR_LOAD_PARAM;
-
+		}
 		u8 hash[16];
 		bool hasExpectedHash = GetExpectedHash(dirPath, filename, hash);
 		loadedSize = LoadCryptedSave(param, param->dataBuf, saveData, saveSize, prevCryptMode, hasExpectedHash ? hash : nullptr, saveDone);
@@ -769,7 +776,7 @@ u32 SavedataParam::LoadCryptedSave(SceUtilitySavedataParam *param, u8 *data, con
 	if (decryptMode != prevCryptMode) {
 		if (prevCryptMode == 1 && param->key[0] == 0) {
 			// Backwards compat for a bug we used to have.
-			WARN_LOG(SCEUTILITY, "Savedata loading with hashmode %d instead of detected %d", prevCryptMode, decryptMode);
+			WARN_LOG(Log::sceUtility, "Savedata loading with hashmode %d instead of detected %d", prevCryptMode, decryptMode);
 			decryptMode = prevCryptMode;
 
 			// Don't notify the user if we're not going to upgrade the save.
@@ -780,9 +787,9 @@ u32 SavedataParam::LoadCryptedSave(SceUtilitySavedataParam *param, u8 *data, con
 			}
 		} else {
 			if (decryptMode == 5 && prevCryptMode == 3) {
-				WARN_LOG(SCEUTILITY, "Savedata loading with detected hashmode %d instead of file's %d", decryptMode, prevCryptMode);
+				WARN_LOG(Log::sceUtility, "Savedata loading with detected hashmode %d instead of file's %d", decryptMode, prevCryptMode);
 			} else {
-				WARN_LOG_REPORT(SCEUTILITY, "Savedata loading with detected hashmode %d instead of file's %d", decryptMode, prevCryptMode);
+				WARN_LOG_REPORT(Log::sceUtility, "Savedata loading with detected hashmode %d instead of file's %d", decryptMode, prevCryptMode);
 			}
 			if (g_Config.bSavedataUpgrade) {
 				decryptMode = prevCryptMode;
@@ -794,17 +801,17 @@ u32 SavedataParam::LoadCryptedSave(SceUtilitySavedataParam *param, u8 *data, con
 		hasKey = decryptMode > 1;
 	}
 
-	int err = DecryptSave(decryptMode, data_base, &saveSize, &align_len, hasKey ? cryptKey : nullptr, expectedHash);
+	int err = DecryptData(decryptMode, data_base, &saveSize, &align_len, hasKey ? cryptKey : nullptr, expectedHash);
 	// Perhaps the file had the wrong mode....
 	if (err != 0 && detectedMode != decryptMode) {
 		resetData(detectedMode);
-		err = DecryptSave(detectedMode, data_base, &saveSize, &align_len, hasKey ? cryptKey : nullptr, expectedHash);
+		err = DecryptData(detectedMode, data_base, &saveSize, &align_len, hasKey ? cryptKey : nullptr, expectedHash);
 	}
 	// TODO: Should return an error, but let's just try with a bad hash.
 	if (err != 0 && expectedHash != nullptr) {
-		WARN_LOG(SCEUTILITY, "Incorrect hash on save data, likely corrupt");
+		WARN_LOG(Log::sceUtility, "Incorrect hash on save data, likely corrupt");
 		resetData(decryptMode);
-		err = DecryptSave(decryptMode, data_base, &saveSize, &align_len, hasKey ? cryptKey : nullptr, nullptr);
+		err = DecryptData(decryptMode, data_base, &saveSize, &align_len, hasKey ? cryptKey : nullptr, nullptr);
 	}
 
 	u32 sz = 0;
@@ -875,8 +882,8 @@ std::set<std::string> SavedataParam::GetSecureFileNames(const std::string &dirPa
 	auto entries = GetSFOEntries(dirPath);
 
 	std::set<std::string> secureFileNames;
-	for (auto entry : entries) {
-		char temp[14];
+	for (const auto &entry : entries) {
+		char temp[14]{};
 		truncate_cpy(temp, entry.filename);
 		secureFileNames.insert(temp);
 	}
@@ -907,9 +914,13 @@ void SavedataParam::LoadFile(const std::string& dirPath, const std::string& file
 		fileData->size = readSize;
 		const std::string tag = "SavedataLoad/" + filePath;
 		NotifyMemInfo(MemBlockFlags::WRITE, fileData->buf.ptr, fileData->size, tag.c_str(), tag.size());
+		INFO_LOG(Log::sceUtility, "Loaded subfile %s (size: %d bytes) into %08x", filePath.c_str(), fileData->size, fileData->buf.ptr);
+	} else {
+		WARN_LOG(Log::sceUtility, "Failed to load subfile %s into %08x", filePath.c_str(), fileData->buf.ptr);
 	}
 }
 
+// Note: The work is done in-place, hence the memmove etc.
 int SavedataParam::EncryptData(unsigned int mode,
 		 unsigned char *data,
 		 int *dataLen,
@@ -917,16 +928,18 @@ int SavedataParam::EncryptData(unsigned int mode,
 		 unsigned char *hash,
 		 unsigned char *cryptkey)
 {
-	pspChnnlsvContext1 ctx1;
-	pspChnnlsvContext2 ctx2;
+	pspChnnlsvContext1 ctx1{};
+	pspChnnlsvContext2 ctx2{};
+
+	INFO_LOG(Log::sceUtility, "EncryptData(mode=%d, *dataLen=%d, *alignedLen=%d)", mode, *dataLen, *alignedLen);
 
 	/* Make room for the IV in front of the data. */
 	memmove(data + 0x10, data, *alignedLen);
 
 	/* Set up buffers */
-	memset(&ctx1, 0, sizeof(pspChnnlsvContext1));
-	memset(&ctx2, 0, sizeof(pspChnnlsvContext2));
 	memset(hash, 0, 0x10);
+
+	// Zero out the IV before we begin.
 	memset(data, 0, 0x10);
 
 	/* Build the 0x10-byte IV and setup encryption */
@@ -962,9 +975,10 @@ int SavedataParam::EncryptData(unsigned int mode,
 	return 0;
 }
 
-int SavedataParam::DecryptSave(unsigned int mode, unsigned char *data, int *dataLen, int *alignedLen, unsigned char *cryptkey, const u8 *expectedHash) {
-	pspChnnlsvContext1 ctx1;
-	pspChnnlsvContext2 ctx2;
+// Note: The work is done in-place, hence the memmove etc.
+int SavedataParam::DecryptData(unsigned int mode, unsigned char *data, int *dataLen, int *alignedLen, unsigned char *cryptkey, const u8 *expectedHash) {
+	pspChnnlsvContext1 ctx1{};
+	pspChnnlsvContext2 ctx2{};
 
 	/* Need a 16-byte IV plus some data */
 	if (*alignedLen <= 0x10)
@@ -972,10 +986,6 @@ int SavedataParam::DecryptSave(unsigned int mode, unsigned char *data, int *data
 	*dataLen -= 0x10;
 	*alignedLen -= 0x10;
 
-	/* Set up buffers */
-	memset(&ctx1, 0, sizeof(pspChnnlsvContext1));
-	memset(&ctx2, 0, sizeof(pspChnnlsvContext2));
-
 	/* Perform the magic */
 	if (sceSdSetIndex_(ctx1, mode) < 0)
 		return -2;
@@ -1005,10 +1015,11 @@ int SavedataParam::DecryptSave(unsigned int mode, unsigned char *data, int *data
 	return 0;
 }
 
+// Requires sfoData to be padded with zeroes to the next 16-byte boundary (due to BuildHash)
 int SavedataParam::UpdateHash(u8* sfoData, int sfoSize, int sfoDataParamsOffset, int encryptmode)
 {
 	int alignedLen = align16(sfoSize);
-	memset(sfoData+sfoDataParamsOffset, 0, 128);
+	memset(sfoData + sfoDataParamsOffset, 0, 128);
 	u8 filehash[16];
 	int ret = 0;
 
@@ -1027,8 +1038,8 @@ int SavedataParam::UpdateHash(u8* sfoData, int sfoSize, int sfoDataParamsOffset,
 	}
 
 	// Copy 11D0 hash to param.sfo and set flag indicating it's there
-	memcpy(sfoData+sfoDataParamsOffset + 0x20, filehash, 0x10);
-	*(sfoData+sfoDataParamsOffset) |= 0x01;
+	memcpy(sfoData + sfoDataParamsOffset + 0x20, filehash, 0x10);
+	*(sfoData + sfoDataParamsOffset) |= 0x01;
 
 	// If new encryption mode, compute and insert the 1220 hash.
 	if (encryptmode & 6)
@@ -1054,19 +1065,18 @@ int SavedataParam::UpdateHash(u8* sfoData, int sfoSize, int sfoDataParamsOffset,
 	return 0;
 }
 
-int SavedataParam::BuildHash(unsigned char *output,
-		unsigned char *data,
+// Requires sfoData to be padded with zeroes to the next 16-byte boundary.
+int SavedataParam::BuildHash(uint8_t *output,
+		const uint8_t *data,
 		unsigned int len,
 		unsigned int alignedLen,
 		int mode,
-		unsigned char *cryptkey)
-{
+		const uint8_t *cryptkey) {
 	pspChnnlsvContext1 ctx1;
 
 	/* Set up buffers */
 	memset(&ctx1, 0, sizeof(pspChnnlsvContext1));
 	memset(output, 0, 0x10);
-	memset(data + len, 0, alignedLen - len);
 
 	/* Perform the magic */
 	if (sceSdSetIndex_(ctx1, mode & 0xFF) < 0)
@@ -1087,7 +1097,7 @@ int SavedataParam::BuildHash(unsigned char *output,
 std::string SavedataParam::GetSpaceText(u64 size, bool roundUp)
 {
 	char text[50];
-	static const char *suffixes[] = {"B", "KB", "MB", "GB"};
+	static const char * const suffixes[] = {"B", "KB", "MB", "GB"};
 	for (size_t i = 0; i < ARRAY_SIZE(suffixes); ++i)
 	{
 		if (size < 1024)
@@ -1105,6 +1115,10 @@ std::string SavedataParam::GetSpaceText(u64 size, bool roundUp)
 	return std::string(text);
 }
 
+inline std::string FmtPspTime(const ScePspDateTime &dt) {
+	return StringFromFormat("%04d-%02d-%02d %02d:%02d:%02d.%06d", dt.year, dt.month, dt.day, dt.hour, dt.minute, dt.second, dt.microsecond);
+}
+
 int SavedataParam::GetSizes(SceUtilitySavedataParam *param)
 {
 	if (!param) {
@@ -1192,9 +1206,13 @@ int SavedataParam::GetSizes(SceUtilitySavedataParam *param)
 
 		// TODO: Maybe these are rounded to the nearest 32KB?  Or something?
 		param->utilityData->usedSpace32KB = total_size / 0x400;
-		spaceTxt = SavedataParam::GetSpaceText(total_size, true);
+		std::string spaceTxt32 = SavedataParam::GetSpaceText(total_size, true);
 		memset(param->utilityData->usedSpace32Str, 0, sizeof(param->utilityData->usedSpace32Str));
-		strncpy(param->utilityData->usedSpace32Str, spaceTxt.c_str(), sizeof(param->utilityData->usedSpace32Str));
+		strncpy(param->utilityData->usedSpace32Str, spaceTxt32.c_str(), sizeof(param->utilityData->usedSpace32Str));
+
+		INFO_LOG(Log::sceUtility, "GetSize: usedSpaceKB: %d (str: %s) (clusters: %d)", param->utilityData->usedSpaceKB, spaceTxt.c_str(), param->utilityData->usedClusters);
+		INFO_LOG(Log::sceUtility, "GetSize: usedSpace32KB: %d (str32: %s)", param->utilityData->usedSpace32KB, spaceTxt32.c_str());
+
 		NotifyMemInfo(MemBlockFlags::WRITE, param->utilityData.ptr, sizeof(SceUtilitySavedataUsedDataInfo), "SavedataGetSizes");
 	}
 	return ret;
@@ -1208,18 +1226,16 @@ bool SavedataParam::GetList(SceUtilitySavedataParam *param)
 
 	if (param->idList.IsValid())
 	{
-		u32 maxFile = param->idList->maxCount;
+		u32 maxFileCount = param->idList->maxCount;
 
 		std::vector<PSPFileInfo> validDir;
 		std::vector<PSPFileInfo> sfoFiles;
 		std::vector<PSPFileInfo> allDir = pspFileSystem.GetDirListing(savePath);
 
 		std::string searchString = GetGameName(param) + GetSaveName(param);
-		for (size_t i = 0; i < allDir.size() && validDir.size() < maxFile; i++)
-		{
+		for (size_t i = 0; i < allDir.size() && validDir.size() < maxFileCount; i++) {
 			std::string dirName = allDir[i].name;
-			if (PSPMatch(dirName, searchString))
-			{
+			if (PSPMatch(dirName, searchString)) {
 				validDir.push_back(allDir[i]);
 			}
 		}
@@ -1252,7 +1268,14 @@ bool SavedataParam::GetList(SceUtilitySavedataParam *param)
 		}
 		// Save num of folder found
 		param->idList->resultCount = (u32)validDir.size();
-
+		// Log out the listing.
+		if (GenericLogEnabled(LogLevel::LINFO, Log::sceUtility)) {
+			INFO_LOG(Log::sceUtility, "LIST (searchstring=%s): %d files (max: %d)", searchString.c_str(), param->idList->resultCount, maxFileCount);
+			for (int i = 0; i < validDir.size(); i++) {
+				INFO_LOG(Log::sceUtility, "%s: mode %08x, ctime: %s, atime: %s, mtime: %s",
+					entries[i].name, entries[i].st_mode, FmtPspTime(entries[i].st_ctime).c_str(), FmtPspTime(entries[i].st_atime).c_str(), FmtPspTime(entries[i].st_mtime).c_str());
+			}
+		}
 		NotifyMemInfo(MemBlockFlags::WRITE, param->idList.ptr, sizeof(SceUtilitySavedataIdListInfo), "SavedataGetList");
 		NotifyMemInfo(MemBlockFlags::WRITE, param->idList->entries.ptr, (uint32_t)validDir.size() * sizeof(SceUtilitySavedataIdListEntry), "SavedataGetList");
 	}
@@ -1265,23 +1288,23 @@ int SavedataParam::GetFilesList(SceUtilitySavedataParam *param, u32 requestAddr)
 	}
 
 	if (!param->fileList.IsValid()) {
-		ERROR_LOG_REPORT(SCEUTILITY, "SavedataParam::GetFilesList(): bad fileList address %08x", param->fileList.ptr);
+		ERROR_LOG_REPORT(Log::sceUtility, "SavedataParam::GetFilesList(): bad fileList address %08x", param->fileList.ptr);
 		// Should crash.
 		return -1;
 	}
 
 	auto &fileList = param->fileList;
 	if (fileList->secureEntries.IsValid() && fileList->maxSecureEntries > 99) {
-		ERROR_LOG_REPORT(SCEUTILITY, "SavedataParam::GetFilesList(): too many secure entries, %d", fileList->maxSecureEntries);
+		ERROR_LOG_REPORT(Log::sceUtility, "SavedataParam::GetFilesList(): too many secure entries, %d", fileList->maxSecureEntries);
 		return SCE_UTILITY_SAVEDATA_ERROR_RW_BAD_PARAMS;
 	}
 	if (fileList->normalEntries.IsValid() && fileList->maxNormalEntries > 8192) {
-		ERROR_LOG_REPORT(SCEUTILITY, "SavedataParam::GetFilesList(): too many normal entries, %d", fileList->maxNormalEntries);
+		ERROR_LOG_REPORT(Log::sceUtility, "SavedataParam::GetFilesList(): too many normal entries, %d", fileList->maxNormalEntries);
 		return SCE_UTILITY_SAVEDATA_ERROR_RW_BAD_PARAMS;
 	}
 	if (sceKernelGetCompiledSdkVersion() >= 0x02060000) {
 		if (fileList->systemEntries.IsValid() && fileList->maxSystemEntries > 5) {
-			ERROR_LOG_REPORT(SCEUTILITY, "SavedataParam::GetFilesList(): too many system entries, %d", fileList->maxSystemEntries);
+			ERROR_LOG_REPORT(Log::sceUtility, "SavedataParam::GetFilesList(): too many system entries, %d", fileList->maxSystemEntries);
 			return SCE_UTILITY_SAVEDATA_ERROR_RW_BAD_PARAMS;
 		}
 	}
@@ -1290,7 +1313,7 @@ int SavedataParam::GetFilesList(SceUtilitySavedataParam *param, u32 requestAddr)
 	bool dirPathExists = false;
 	auto files = pspFileSystem.GetDirListing(dirPath, &dirPathExists);
 	if (!dirPathExists) {
-		DEBUG_LOG(SCEUTILITY, "SavedataParam::GetFilesList(): directory %s does not exist", dirPath.c_str());
+		DEBUG_LOG(Log::sceUtility, "SavedataParam::GetFilesList(): directory %s does not exist", dirPath.c_str());
 		return SCE_UTILITY_SAVEDATA_ERROR_RW_NO_DATA;
 	}
 
@@ -1324,7 +1347,7 @@ int SavedataParam::GetFilesList(SceUtilitySavedataParam *param, u32 requestAddr)
 		}
 		// TODO: What are the exact rules?  It definitely skips lowercase, and allows FILE or FILE.EXT.
 		if (file->name.find_first_of("abcdefghijklmnopqrstuvwxyz") != file->name.npos) {
-			DEBUG_LOG(SCEUTILITY, "SavedataParam::GetFilesList(): skipping file %s with lowercase", file->name.c_str());
+			DEBUG_LOG(Log::sceUtility, "SavedataParam::GetFilesList(): skipping file %s with lowercase", file->name.c_str());
 			continue;
 		}
 
@@ -1366,6 +1389,15 @@ int SavedataParam::GetFilesList(SceUtilitySavedataParam *param, u32 requestAddr)
 		entry->name[15] = '\0';
 	}
 
+	if (GenericLogEnabled(LogLevel::LINFO, Log::sceUtility)) {
+		INFO_LOG(Log::sceUtility, "FILES: %d files listed", fileList->resultNumNormalEntries);
+		for (int i = 0; i < (int)fileList->resultNumNormalEntries; i++) {
+			const SceUtilitySavedataFileListEntry &info = fileList->systemEntries[i];
+			INFO_LOG(Log::sceUtility, "%s: mode %08x, ctime: %s, atime: %s, mtime: %s",
+				info.name, info.st_mode, FmtPspTime(info.st_ctime).c_str(), FmtPspTime(info.st_atime).c_str(), FmtPspTime(info.st_mtime).c_str());
+		}
+	}
+
 	NotifyMemInfo(MemBlockFlags::WRITE, fileList.ptr, sizeof(SceUtilitySavedataFileListInfo), "SavedataGetFilesList");
 	if (fileList->resultNumSystemEntries != 0)
 		NotifyMemInfo(MemBlockFlags::WRITE, fileList->systemEntries.ptr, fileList->resultNumSystemEntries * sizeof(SceUtilitySavedataFileListEntry), "SavedataGetFilesList");
@@ -1377,19 +1409,16 @@ int SavedataParam::GetFilesList(SceUtilitySavedataParam *param, u32 requestAddr)
 	return 0;
 }
 
-bool SavedataParam::GetSize(SceUtilitySavedataParam *param)
-{
-	if (!param)
-	{
+bool SavedataParam::GetSize(SceUtilitySavedataParam *param) {
+	if (!param) {
 		return false;
 	}
 
 	const std::string saveDir = savePath + GetGameName(param) + GetSaveName(param);
 	bool exists = false;
-	auto listing = pspFileSystem.GetDirListing(saveDir, &exists);
 
-	if (param->sizeInfo.IsValid())
-	{
+	if (param->sizeInfo.IsValid()) {
+		auto listing = pspFileSystem.GetDirListing(saveDir, &exists);
 		const u64 freeBytes = MemoryStick_FreeSpace();
 
 		s64 overwriteBytes = 0;
@@ -1420,27 +1449,30 @@ bool SavedataParam::GetSize(SceUtilitySavedataParam *param)
 			param->sizeInfo->overwriteKB = 0;
 
 			spaceTxt = GetSpaceText(0, true);
-			truncate_cpy(param->sizeInfo->neededString, spaceTxt.c_str());
-			truncate_cpy(param->sizeInfo->overwriteString, spaceTxt.c_str());
+			truncate_cpy(param->sizeInfo->neededString, spaceTxt);
+			truncate_cpy(param->sizeInfo->overwriteString, spaceTxt);
 		} else {
 			// Bytes needed to save additional data.
 			s64 neededBytes = writeBytes - freeBytes;
 			param->sizeInfo->neededKB = (neededBytes + 1023) / 1024;
 			spaceTxt = GetSpaceText(neededBytes, true);
-			truncate_cpy(param->sizeInfo->neededString, spaceTxt.c_str());
+			truncate_cpy(param->sizeInfo->neededString, spaceTxt);
 
 			if (writeBytes - overwriteBytes < (s64)freeBytes) {
 				param->sizeInfo->overwriteKB = 0;
 				spaceTxt = GetSpaceText(0, true);
-				truncate_cpy(param->sizeInfo->overwriteString, spaceTxt.c_str());
+				truncate_cpy(param->sizeInfo->overwriteString, spaceTxt);
 			} else {
 				s64 neededOverwriteBytes = writeBytes - freeBytes - overwriteBytes;
 				param->sizeInfo->overwriteKB = (neededOverwriteBytes + 1023) / 1024;
 				spaceTxt = GetSpaceText(neededOverwriteBytes, true);
-				truncate_cpy(param->sizeInfo->overwriteString, spaceTxt.c_str());
+				truncate_cpy(param->sizeInfo->overwriteString, spaceTxt);
 			}
 		}
 
+		INFO_LOG(Log::sceUtility, "SectorSize: %d FreeSectors: %d FreeKB: %d neededKb: %d overwriteKb: %d",
+			param->sizeInfo->sectorSize, param->sizeInfo->freeSectors, param->sizeInfo->freeKB, param->sizeInfo->neededKB, param->sizeInfo->overwriteKB);
+
 		NotifyMemInfo(MemBlockFlags::WRITE, param->sizeInfo.ptr, sizeof(PspUtilitySavedataSizeInfo), "SavedataGetSize");
 	}
 
@@ -1459,16 +1491,15 @@ void SavedataParam::Clear()
 		}
 
 		delete [] saveDataList;
-		saveDataList = 0;
+		saveDataList = NULL;
 		saveDataListCount = 0;
 	}
 	if (noSaveIcon)
 	{
-		if (noSaveIcon->texture != NULL)
-			delete noSaveIcon->texture;
+		delete noSaveIcon->texture;
 		noSaveIcon->texture = NULL;
 		delete noSaveIcon;
-		noSaveIcon = 0;
+		noSaveIcon = NULL;
 	}
 }
 
@@ -1538,7 +1569,7 @@ int SavedataParam::SetPspParam(SceUtilitySavedataParam *param)
 							std::string fileDataPath = savePath + it->name;
 							if (it->exists) {
 								SetFileInfo(realCount, *it, saveName);
-								DEBUG_LOG(SCEUTILITY, "%s Exist", fileDataPath.c_str());
+								DEBUG_LOG(Log::sceUtility, "%s Exist", fileDataPath.c_str());
 								++realCount;
 							} else {
 								if (listEmptyFile) {
@@ -1558,15 +1589,15 @@ int SavedataParam::SetPspParam(SceUtilitySavedataParam *param)
 				PSPFileInfo info = GetSaveInfo(fileDataDir);
 				if (info.exists) {
 					SetFileInfo(realCount, info, thisSaveName);
-					DEBUG_LOG(SCEUTILITY, "Save data exists: %s = %s", thisSaveName.c_str(), fileDataDir.c_str());
+					INFO_LOG(Log::sceUtility, "Save data exists: %s = %s", thisSaveName.c_str(), fileDataDir.c_str());
 					realCount++;
 				} else {
 					if (listEmptyFile) {
 						ClearFileInfo(saveDataList[realCount], thisSaveName);
-						DEBUG_LOG(SCEUTILITY, "Listing missing save data: %s = %s", thisSaveName.c_str(), fileDataDir.c_str());
+						INFO_LOG(Log::sceUtility, "Listing missing save data: %s = %s", thisSaveName.c_str(), fileDataDir.c_str());
 						realCount++;
 					} else {
-						DEBUG_LOG(SCEUTILITY, "Save data not found: %s = %s", thisSaveName.c_str(), fileDataDir.c_str());
+						INFO_LOG(Log::sceUtility, "Save data not found: %s = %s", thisSaveName.c_str(), fileDataDir.c_str());
 					}
 				}
 			}
@@ -1586,14 +1617,14 @@ int SavedataParam::SetPspParam(SceUtilitySavedataParam *param)
 		PSPFileInfo info = GetSaveInfo(fileDataDir);
 		if (info.exists) {
 			SetFileInfo(0, info, GetSaveName(param));
-			DEBUG_LOG(SCEUTILITY, "Save data exists: %s = %s", GetSaveName(param).c_str(), fileDataDir.c_str());
+			INFO_LOG(Log::sceUtility, "Save data exists: %s = %s", GetSaveName(param).c_str(), fileDataDir.c_str());
 			saveNameListDataCount = 1;
 		} else {
 			if (listEmptyFile) {
 				ClearFileInfo(saveDataList[0], GetSaveName(param));
-				DEBUG_LOG(SCEUTILITY, "Listing missing save data: %s = %s", GetSaveName(param).c_str(), fileDataDir.c_str());
+				INFO_LOG(Log::sceUtility, "Listing missing save data: %s = %s", GetSaveName(param).c_str(), fileDataDir.c_str());
 			} else {
-				DEBUG_LOG(SCEUTILITY, "Save data not found: %s = %s", GetSaveName(param).c_str(), fileDataDir.c_str());
+				INFO_LOG(Log::sceUtility, "Save data not found: %s = %s", GetSaveName(param).c_str(), fileDataDir.c_str());
 			}
 			saveNameListDataCount = 0;
 			return 0;
@@ -1602,7 +1633,7 @@ int SavedataParam::SetPspParam(SceUtilitySavedataParam *param)
 	return 0;
 }
 
-void SavedataParam::SetFileInfo(SaveFileInfo &saveInfo, PSPFileInfo &info, std::string saveName, std::string savrDir)
+void SavedataParam::SetFileInfo(SaveFileInfo &saveInfo, PSPFileInfo &info, const std::string &saveName, const std::string &savrDir)
 {
 	saveInfo.size = info.size;
 	saveInfo.saveName = saveName;
@@ -1638,11 +1669,11 @@ void SavedataParam::SetFileInfo(SaveFileInfo &saveInfo, PSPFileInfo &info, std::
 		SetStringFromSFO(*sfoFile, "SAVEDATA_DETAIL", saveInfo.saveDetail, sizeof(saveInfo.saveDetail));
 	} else {
 		saveInfo.broken = true;
-		truncate_cpy(saveInfo.title, saveDir.c_str());
+		truncate_cpy(saveInfo.title, saveDir);
 	}
 }
 
-void SavedataParam::SetFileInfo(int idx, PSPFileInfo &info, std::string saveName, std::string saveDir)
+void SavedataParam::SetFileInfo(int idx, PSPFileInfo &info, const std::string &saveName, const std::string &saveDir)
 {
 	SetFileInfo(saveDataList[idx], info, saveName, saveDir);
 	saveDataList[idx].idx = idx;
@@ -1674,7 +1705,7 @@ void SavedataParam::ClearFileInfo(SaveFileInfo &saveInfo, const std::string &sav
 	}
 }
 
-PSPFileInfo SavedataParam::GetSaveInfo(std::string saveDir) {
+PSPFileInfo SavedataParam::GetSaveInfo(const std::string &saveDir) {
 	PSPFileInfo info = pspFileSystem.GetFileInfo(saveDir);
 	if (info.exists) {
 		info.access = 0777;
@@ -1889,8 +1920,7 @@ void SavedataParam::DoState(PointerWrap &p) {
 	Do(p, saveDataListCount);
 	Do(p, saveNameListDataCount);
 	if (p.mode == p.MODE_READ) {
-		if (saveDataList)
-			delete [] saveDataList;
+		delete [] saveDataList;
 		if (saveDataListCount != 0) {
 			saveDataList = new SaveFileInfo[saveDataListCount];
 			DoArray(p, saveDataList, saveDataListCount);
@@ -1908,7 +1938,7 @@ void SavedataParam::DoState(PointerWrap &p) {
 	}
 }
 
-void SavedataParam::ClearCaches() {
+void SavedataParam::ClearSFOCache() {
 	std::lock_guard<std::mutex> guard(cacheLock_);
 	sfoCache_.clear();
 }
@@ -1917,7 +1947,7 @@ std::shared_ptr<ParamSFOData> SavedataParam::LoadCachedSFO(const std::string &pa
 	std::lock_guard<std::mutex> guard(cacheLock_);
 	if (sfoCache_.find(path) == sfoCache_.end()) {
 		std::vector<u8> data;
-		if (pspFileSystem.ReadEntireFile(path, data) < 0) {
+		if (pspFileSystem.ReadEntireFile(path, data, true) < 0) {
 			// Mark as not existing for later.
 			sfoCache_[path].reset();
 		} else {
@@ -1958,14 +1988,14 @@ int SavedataParam::GetSaveCryptMode(const SceUtilitySavedataParam *param, const
 			return 5;
 		default:
 			// Well, it's not zero, so yes.
-			ERROR_LOG_REPORT(SCEUTILITY, "Unexpected SAVEDATA_PARAMS hash flag: %02x", tmpDataOrig[0]);
+			ERROR_LOG_REPORT(Log::sceUtility, "Unexpected SAVEDATA_PARAMS hash flag: %02x", tmpDataOrig[0]);
 			return 1;
 		}
 	}
 	return 0;
 }
 
-bool SavedataParam::IsInSaveDataList(std::string saveName, int count) {
+bool SavedataParam::IsInSaveDataList(const std::string &saveName, int count) {
 	for(int i = 0; i < count; ++i) {
 		if(strcmp(saveDataList[i].saveName.c_str(),saveName.c_str()) == 0)
 			return true;
