@@ -45,10 +45,12 @@ public:
 		return true;
 	}
 	int GetBlockSize() const { return 2048;}  // forced, it cannot be changed by subclasses
-	virtual u32 GetNumBlocks() = 0;
-	virtual bool IsDisc() = 0;
+	virtual u32 GetNumBlocks() const = 0;
+	virtual u64 GetUncompressedSize() const {
+		return (u64)GetNumBlocks() * (u64)GetBlockSize();
+	}
+	virtual bool IsDisc() const = 0;
 
-	u32 CalculateCRC(volatile bool *cancel = nullptr);
 	void NotifyReadError();
 
 protected:
@@ -62,20 +64,20 @@ public:
 	~CISOFileBlockDevice();
 	bool ReadBlock(int blockNumber, u8 *outPtr, bool uncached = false) override;
 	bool ReadBlocks(u32 minBlock, int count, u8 *outPtr) override;
-	u32 GetNumBlocks() override { return numBlocks; }
-	bool IsDisc() override { return true; }
+	u32 GetNumBlocks() const override { return numBlocks; }
+	bool IsDisc() const override { return true; }
 
 private:
-	u32 *index;
-	u8 *readBuffer;
-	u8 *zlibBuffer;
-	u32 zlibBufferFrame;
-	u8 indexShift;
-	u8 blockShift;
-	u32 frameSize;
-	u32 numBlocks;
-	u32 numFrames;
-	int ver_;
+	u32 *index = nullptr;
+	u8 *readBuffer = nullptr;
+	u8 *zlibBuffer = nullptr;
+	u32 zlibBufferFrame = 0;
+	u8 indexShift = 0;
+	u8 blockShift = 0;
+	u32 frameSize = 0;
+	u32 numBlocks = 0;
+	u32 numFrames = 0;
+	int ver_ = 0;
 };
 
 
@@ -85,9 +87,11 @@ public:
 	~FileBlockDevice();
 	bool ReadBlock(int blockNumber, u8 *outPtr, bool uncached = false) override;
 	bool ReadBlocks(u32 minBlock, int count, u8 *outPtr) override;
-	u32 GetNumBlocks() override {return (u32)(filesize_ / GetBlockSize());}
-	bool IsDisc() override { return true; }
-
+	u32 GetNumBlocks() const override {return (u32)(filesize_ / GetBlockSize());}
+	bool IsDisc() const override { return true; }
+	u64 GetUncompressedSize() const override {
+		return filesize_;
+	}
 private:
 	u64 filesize_;
 };
@@ -109,26 +113,48 @@ public:
 	~NPDRMDemoBlockDevice();
 
 	bool ReadBlock(int blockNumber, u8 *outPtr, bool uncached = false) override;
-	u32 GetNumBlocks() override {return (u32)lbaSize;}
-	bool IsDisc() override { return false; }
+	u32 GetNumBlocks() const override {return (u32)lbaSize_;}
+	bool IsDisc() const override { return false; }
 
 private:
-	static std::mutex mutex_;
-	u32 lbaSize;
+	// TODO: Doubt this mutex is actually needed.
+	std::mutex mutex_;
 
-	u32 psarOffset;
-	int blockSize;
-	int blockLBAs;
-	u32 numBlocks;
+	u32 lbaSize_ = 0;
 
-	u8 vkey[16];
-	u8 hkey[16];
-	struct table_info *table;
+	u32 psarOffset = 0;
+	int blockSize_ = 0;
+	int blockLBAs_ = 0;
+	u32 numBlocks_ = 0;
 
-	int currentBlock;
-	u8 *blockBuf;
-	u8 *tempBuf;
+	u8 vkey[16]{};
+	u8 hkey[16]{};
+	struct table_info *table_ = nullptr;
+
+	int currentBlock_ = 0;
+	u8 *blockBuf_ = nullptr;
+	u8 *tempBuf_ = nullptr;
 };
 
+struct CHDImpl;
+
+struct ExtendedCoreFile;
+
+class CHDFileBlockDevice : public BlockDevice {
+public:
+	CHDFileBlockDevice(FileLoader *fileLoader);
+	~CHDFileBlockDevice();
+	bool ReadBlock(int blockNumber, u8 *outPtr, bool uncached = false) override;
+	bool ReadBlocks(u32 minBlock, int count, u8 *outPtr) override;
+	u32 GetNumBlocks() const override { return numBlocks; }
+	bool IsDisc() const override { return true; }
+private:
+	struct ExtendedCoreFile *core_file_ = nullptr;
+	std::unique_ptr<CHDImpl> impl_;
+	u8 *readBuffer = nullptr;
+	u32 currentHunk = 0;
+	u32 blocksPerHunk = 0;
+	u32 numBlocks = 0;
+};
 
 BlockDevice *constructBlockDevice(FileLoader *fileLoader);
