@@ -39,17 +39,13 @@
 #endif
 
 #include <atomic>
-#include <climits>
 #include <mutex>
 #include <thread>
+#include <climits>
 
 #include "Common/Net/Resolve.h"
 #include "Common/Serialize/Serializer.h"
-
-#include "Core/CoreTiming.h"
 #include "Core/MemMap.h"
-#include "Core/HLE/HLE.h"
-#include "Core/HLE/HLEHelperThread.h"
 #include "Core/HLE/sceKernelThread.h"
 #include "Core/HLE/sceKernel.h"
 #include "Core/HLE/sceKernelMutex.h"
@@ -111,6 +107,10 @@ inline bool isDisconnected(int errcode) { return (errcode == EPIPE || errcode ==
 #define POLLPRI POLL_PRI
 #endif
 
+#ifndef SD_RECEIVE
+#define SD_RECEIVE SHUT_RD //0x00
+#endif
+
 #ifndef SD_BOTH
 #define SD_BOTH SHUT_RDWR //0x02
 #endif
@@ -1038,7 +1038,7 @@ void addFriend(SceNetAdhocctlConnectPacketS2C * packet);
 * Send chat or get that
 * @param std::string ChatString 
 */
-void sendChat(std::string chatString);
+void sendChat(const std::string &chatString);
 std::vector<std::string> getChatLog();
 int GetChatChangeID();
 int GetChatMessageCount();
