@@ -35,6 +35,7 @@ const int AUDIO_ROUTING_SPEAKER_ON = 1;
 int defaultRoutingMode = AUDIO_ROUTING_SPEAKER_ON;
 int defaultRoutingVolMode = AUDIO_ROUTING_SPEAKER_ON;
 
+// TODO: These are way oversized and together consume 4MB of memory.
 extern FixedSizeQueue<s16, 32768 * 8> chanSampleQueues[PSP_AUDIO_CHANNEL_MAX + 1];
 
 // The extra channel is for SRC/Output2/Vaudio.
@@ -85,16 +86,16 @@ void AudioChannel::clear()
 // max or 50%?
 static u32 sceAudioOutputBlocking(u32 chan, int vol, u32 samplePtr) {
 	if (vol > 0xFFFF) {
-		ERROR_LOG(SCEAUDIO, "sceAudioOutputBlocking() - invalid volume");
+		ERROR_LOG(Log::sceAudio, "sceAudioOutputBlocking() - invalid volume");
 		return SCE_ERROR_AUDIO_INVALID_VOLUME;
 	} else if (chan >= PSP_AUDIO_CHANNEL_MAX) {
-		ERROR_LOG(SCEAUDIO, "sceAudioOutputBlocking() - bad channel");
+		ERROR_LOG(Log::sceAudio, "sceAudioOutputBlocking() - bad channel");
 		return SCE_ERROR_AUDIO_INVALID_CHANNEL;
 	} else if (!chans[chan].reserved) {
-		ERROR_LOG(SCEAUDIO, "sceAudioOutputBlocking() - channel not reserved");
+		ERROR_LOG(Log::sceAudio, "sceAudioOutputBlocking() - channel not reserved");
 		return SCE_ERROR_AUDIO_CHANNEL_NOT_INIT;
 	} else {
-		DEBUG_LOG(SCEAUDIO, "sceAudioOutputBlocking(%08x, %08x, %08x)", chan, vol, samplePtr);
+		DEBUG_LOG(Log::sceAudio, "sceAudioOutputBlocking(%08x, %08x, %08x)", chan, vol, samplePtr);
 		if (vol >= 0) {
 			chans[chan].leftVolume = vol;
 			chans[chan].rightVolume = vol;
@@ -108,13 +109,13 @@ static u32 sceAudioOutputPannedBlocking(u32 chan, int leftvol, int rightvol, u32
 	int result = 0;
 	// For some reason, this is the only one that checks for negative.
 	if (leftvol > 0xFFFF || rightvol > 0xFFFF || leftvol < 0 || rightvol < 0) {
-		ERROR_LOG(SCEAUDIO, "sceAudioOutputPannedBlocking() - invalid volume");
+		ERROR_LOG(Log::sceAudio, "sceAudioOutputPannedBlocking() - invalid volume");
 		result = SCE_ERROR_AUDIO_INVALID_VOLUME;
 	} else if (chan >= PSP_AUDIO_CHANNEL_MAX) {
-		ERROR_LOG(SCEAUDIO, "sceAudioOutputPannedBlocking() - bad channel");
+		ERROR_LOG(Log::sceAudio, "sceAudioOutputPannedBlocking() - bad channel");
 		result = SCE_ERROR_AUDIO_INVALID_CHANNEL;
 	} else if (!chans[chan].reserved) {
-		ERROR_LOG(SCEAUDIO, "sceAudioOutputPannedBlocking() - channel not reserved");
+		ERROR_LOG(Log::sceAudio, "sceAudioOutputPannedBlocking() - channel not reserved");
 		result = SCE_ERROR_AUDIO_CHANNEL_NOT_INIT;
 	} else {
 		if (leftvol >= 0) {
@@ -127,23 +128,23 @@ static u32 sceAudioOutputPannedBlocking(u32 chan, int leftvol, int rightvol, u32
 		result = __AudioEnqueue(chans[chan], chan, true);
 	}
 
-	DEBUG_LOG(SCEAUDIO, "%08x = sceAudioOutputPannedBlocking(%08x, %08x, %08x, %08x)", result, chan, leftvol, rightvol, samplePtr);
+	DEBUG_LOG(Log::sceAudio, "%08x = sceAudioOutputPannedBlocking(%08x, %08x, %08x, %08x)", result, chan, leftvol, rightvol, samplePtr);
 	return result;
 
 }
 
 static u32 sceAudioOutput(u32 chan, int vol, u32 samplePtr) {
 	if (vol > 0xFFFF) {
-		ERROR_LOG(SCEAUDIO, "sceAudioOutput() - invalid volume");
+		ERROR_LOG(Log::sceAudio, "sceAudioOutput() - invalid volume");
 		return SCE_ERROR_AUDIO_INVALID_VOLUME;
 	} else if (chan >= PSP_AUDIO_CHANNEL_MAX) {
-		ERROR_LOG(SCEAUDIO, "sceAudioOutput() - bad channel");
+		ERROR_LOG(Log::sceAudio, "sceAudioOutput() - bad channel");
 		return SCE_ERROR_AUDIO_INVALID_CHANNEL;
 	} else if (!chans[chan].reserved)	{
-		ERROR_LOG(SCEAUDIO, "sceAudioOutput(%08x, %08x, %08x) - channel not reserved", chan, vol, samplePtr);
+		ERROR_LOG(Log::sceAudio, "sceAudioOutput(%08x, %08x, %08x) - channel not reserved", chan, vol, samplePtr);
 		return SCE_ERROR_AUDIO_CHANNEL_NOT_INIT;
 	} else {
-		DEBUG_LOG(SCEAUDIO, "sceAudioOutputPanned(%08x, %08x, %08x)", chan, vol, samplePtr);
+		DEBUG_LOG(Log::sceAudio, "sceAudioOutput(%08x, %08x, %08x)", chan, vol, samplePtr);
 		if (vol >= 0) {
 			chans[chan].leftVolume = vol;
 			chans[chan].rightVolume = vol;
@@ -155,16 +156,16 @@ static u32 sceAudioOutput(u32 chan, int vol, u32 samplePtr) {
 
 static u32 sceAudioOutputPanned(u32 chan, int leftvol, int rightvol, u32 samplePtr) {
 	if (leftvol > 0xFFFF || rightvol > 0xFFFF) {
-		ERROR_LOG(SCEAUDIO, "sceAudioOutputPanned() - invalid volume");
+		ERROR_LOG(Log::sceAudio, "sceAudioOutputPanned() - invalid volume");
 		return SCE_ERROR_AUDIO_INVALID_VOLUME;
 	} else if (chan >= PSP_AUDIO_CHANNEL_MAX) {
-		ERROR_LOG(SCEAUDIO, "sceAudioOutputPanned() - bad channel");
+		ERROR_LOG(Log::sceAudio, "sceAudioOutputPanned() - bad channel");
 		return SCE_ERROR_AUDIO_INVALID_CHANNEL;
 	} else if (!chans[chan].reserved) {
-		ERROR_LOG(SCEAUDIO, "sceAudioOutputPanned(%08x, %08x, %08x, %08x) - channel not reserved", chan, leftvol, rightvol, samplePtr);
+		ERROR_LOG(Log::sceAudio, "sceAudioOutputPanned(%08x, %08x, %08x, %08x) - channel not reserved", chan, leftvol, rightvol, samplePtr);
 		return SCE_ERROR_AUDIO_CHANNEL_NOT_INIT;
 	} else {
-		DEBUG_LOG(SCEAUDIO, "sceAudioOutputPanned(%08x, %08x, %08x, %08x)", chan, leftvol, rightvol, samplePtr);
+		DEBUG_LOG(Log::sceAudio, "sceAudioOutputPanned(%08x, %08x, %08x, %08x)", chan, leftvol, rightvol, samplePtr);
 		if (leftvol >= 0) {
 			chans[chan].leftVolume = leftvol;
 		}
@@ -178,21 +179,21 @@ static u32 sceAudioOutputPanned(u32 chan, int leftvol, int rightvol, u32 sampleP
 
 static int sceAudioGetChannelRestLen(u32 chan) {
 	if (chan >= PSP_AUDIO_CHANNEL_MAX) {
-		ERROR_LOG(SCEAUDIO, "sceAudioGetChannelRestLen(%08x) - bad channel", chan);
+		ERROR_LOG(Log::sceAudio, "sceAudioGetChannelRestLen(%08x) - bad channel", chan);
 		return SCE_ERROR_AUDIO_INVALID_CHANNEL;
 	}
 	int remainingSamples = (int)chanSampleQueues[chan].size() / 2;
-	VERBOSE_LOG(SCEAUDIO, "%d=sceAudioGetChannelRestLen(%08x)", remainingSamples, chan);
+	VERBOSE_LOG(Log::sceAudio, "%d=sceAudioGetChannelRestLen(%08x)", remainingSamples, chan);
 	return remainingSamples;
 }
 
 static int sceAudioGetChannelRestLength(u32 chan) {
 	if (chan >= PSP_AUDIO_CHANNEL_MAX) {
-		ERROR_LOG(SCEAUDIO, "sceAudioGetChannelRestLength(%08x) - bad channel", chan);
+		ERROR_LOG(Log::sceAudio, "sceAudioGetChannelRestLength(%08x) - bad channel", chan);
 		return SCE_ERROR_AUDIO_INVALID_CHANNEL;
 	}
 	int remainingSamples = (int)chanSampleQueues[chan].size() / 2;
-	VERBOSE_LOG(SCEAUDIO, "%d=sceAudioGetChannelRestLength(%08x)", remainingSamples, chan);
+	VERBOSE_LOG(Log::sceAudio, "%d=sceAudioGetChannelRestLength(%08x)", remainingSamples, chan);
 	return remainingSamples;
 }
 
@@ -208,28 +209,28 @@ static u32 sceAudioChReserve(int chan, u32 sampleCount, u32 format) {
 	if (chan < 0) {
 		chan = GetFreeChannel();
 		if (chan < 0) {
-			ERROR_LOG(SCEAUDIO, "sceAudioChReserve - no channels remaining");
+			ERROR_LOG(Log::sceAudio, "sceAudioChReserve - no channels remaining");
 			return SCE_ERROR_AUDIO_NO_CHANNELS_AVAILABLE;
 		}
 	}
 	if ((u32)chan >= PSP_AUDIO_CHANNEL_MAX)	{
-		ERROR_LOG(SCEAUDIO, "sceAudioChReserve(%08x, %08x, %08x) - bad channel", chan, sampleCount, format);
+		ERROR_LOG(Log::sceAudio, "sceAudioChReserve(%08x, %08x, %08x) - bad channel", chan, sampleCount, format);
 		return SCE_ERROR_AUDIO_INVALID_CHANNEL;
 	}
 	if ((sampleCount & 63) != 0 || sampleCount == 0 || sampleCount > PSP_AUDIO_SAMPLE_MAX) {
-		ERROR_LOG(SCEAUDIO, "sceAudioChReserve(%08x, %08x, %08x) - invalid sample count", chan, sampleCount, format);
+		ERROR_LOG(Log::sceAudio, "sceAudioChReserve(%08x, %08x, %08x) - invalid sample count", chan, sampleCount, format);
 		return SCE_ERROR_AUDIO_OUTPUT_SAMPLE_DATA_SIZE_NOT_ALIGNED;
 	}
 	if (format != PSP_AUDIO_FORMAT_MONO && format != PSP_AUDIO_FORMAT_STEREO) {
-		ERROR_LOG(SCEAUDIO, "sceAudioChReserve(%08x, %08x, %08x) - invalid format", chan, sampleCount, format);
+		ERROR_LOG(Log::sceAudio, "sceAudioChReserve(%08x, %08x, %08x) - invalid format", chan, sampleCount, format);
 		return SCE_ERROR_AUDIO_INVALID_FORMAT;
 	}
 	if (chans[chan].reserved) {
-		ERROR_LOG(SCEAUDIO, "sceAudioChReserve - reserve channel failed");
+		ERROR_LOG(Log::sceAudio, "sceAudioChReserve - reserve channel failed");
 		return SCE_ERROR_AUDIO_INVALID_CHANNEL;
 	}
 
-	DEBUG_LOG(SCEAUDIO, "sceAudioChReserve(%08x, %08x, %08x)", chan, sampleCount, format);
+	DEBUG_LOG(Log::sceAudio, "sceAudioChReserve(%08x, %08x, %08x)", chan, sampleCount, format);
 	chans[chan].sampleCount = sampleCount;
 	chans[chan].format = format;
 	chans[chan].reserved = true;
@@ -240,47 +241,47 @@ static u32 sceAudioChReserve(int chan, u32 sampleCount, u32 format) {
 
 static u32 sceAudioChRelease(u32 chan) {
 	if (chan >= PSP_AUDIO_CHANNEL_MAX) {
-		ERROR_LOG(SCEAUDIO, "sceAudioChRelease(%i) - bad channel", chan);
+		ERROR_LOG(Log::sceAudio, "sceAudioChRelease(%i) - bad channel", chan);
 		return SCE_ERROR_AUDIO_INVALID_CHANNEL;
 	}
 
 	if (!chans[chan].reserved) {
-		ERROR_LOG(SCEAUDIO, "sceAudioChRelease(%i) - channel not reserved", chan);
+		ERROR_LOG(Log::sceAudio, "sceAudioChRelease(%i) - channel not reserved", chan);
 		return SCE_ERROR_AUDIO_CHANNEL_NOT_RESERVED;
 	}
 	// TODO: Does this error if busy?
 	chans[chan].reset();
 	chans[chan].reserved = false;
-	return hleLogSuccessI(SCEAUDIO, 0);
+	return hleLogSuccessI(Log::sceAudio, 0);
 }
 
 static u32 sceAudioSetChannelDataLen(u32 chan, u32 len) {
 	int result = 0;
 	if (chan >= PSP_AUDIO_CHANNEL_MAX) {
-		ERROR_LOG(SCEAUDIO, "sceAudioSetChannelDataLen(%08x, %08x) - bad channel", chan, len);
+		ERROR_LOG(Log::sceAudio, "sceAudioSetChannelDataLen(%08x, %08x) - bad channel", chan, len);
 		result = SCE_ERROR_AUDIO_INVALID_CHANNEL;
 	} else if (!chans[chan].reserved)	{
-		ERROR_LOG(SCEAUDIO, "sceAudioSetChannelDataLen(%08x, %08x) - channel not reserved", chan, len);
+		ERROR_LOG(Log::sceAudio, "sceAudioSetChannelDataLen(%08x, %08x) - channel not reserved", chan, len);
 		result = SCE_ERROR_AUDIO_CHANNEL_NOT_INIT;
 	} else if ((len & 63) != 0 || len == 0 || len > PSP_AUDIO_SAMPLE_MAX) {
-		ERROR_LOG(SCEAUDIO, "sceAudioSetChannelDataLen(%08x, %08x) - invalid sample count", chan, len);
+		ERROR_LOG(Log::sceAudio, "sceAudioSetChannelDataLen(%08x, %08x) - invalid sample count", chan, len);
 		result = SCE_ERROR_AUDIO_OUTPUT_SAMPLE_DATA_SIZE_NOT_ALIGNED;
 	} else {
 		chans[chan].sampleCount = len;
 	}
-	DEBUG_LOG(SCEAUDIO, "%08x = sceAudioSetChannelDataLen(%08x, %08x)", result , chan, len);
+	DEBUG_LOG(Log::sceAudio, "%08x = sceAudioSetChannelDataLen(%08x, %08x)", result , chan, len);
 	return result;
 }
 
 static u32 sceAudioChangeChannelConfig(u32 chan, u32 format) {
 	if (chan >= PSP_AUDIO_CHANNEL_MAX) {
-		ERROR_LOG(SCEAUDIO, "sceAudioChangeChannelConfig(%08x, %08x) - invalid channel number", chan, format);
+		ERROR_LOG(Log::sceAudio, "sceAudioChangeChannelConfig(%08x, %08x) - invalid channel number", chan, format);
 		return SCE_ERROR_AUDIO_INVALID_CHANNEL;
 	} else if (!chans[chan].reserved) {
-		ERROR_LOG(SCEAUDIO, "sceAudioChangeChannelConfig(%08x, %08x) - channel not reserved", chan, format);
+		ERROR_LOG(Log::sceAudio, "sceAudioChangeChannelConfig(%08x, %08x) - channel not reserved", chan, format);
 		return SCE_ERROR_AUDIO_CHANNEL_NOT_RESERVED;
 	} else {
-		DEBUG_LOG(SCEAUDIO, "sceAudioChangeChannelConfig(%08x, %08x)", chan, format);
+		DEBUG_LOG(Log::sceAudio, "sceAudioChangeChannelConfig(%08x, %08x)", chan, format);
 		chans[chan].format = format;
 		return 0;
 	}
@@ -288,16 +289,16 @@ static u32 sceAudioChangeChannelConfig(u32 chan, u32 format) {
 
 static u32 sceAudioChangeChannelVolume(u32 chan, u32 leftvol, u32 rightvol) {
 	if (leftvol > 0xFFFF || rightvol > 0xFFFF) {
-		ERROR_LOG(SCEAUDIO, "sceAudioChangeChannelVolume(%08x, %08x, %08x) - invalid volume", chan, leftvol, rightvol);
+		ERROR_LOG(Log::sceAudio, "sceAudioChangeChannelVolume(%08x, %08x, %08x) - invalid volume", chan, leftvol, rightvol);
 		return SCE_ERROR_AUDIO_INVALID_VOLUME;
 	} else if (chan >= PSP_AUDIO_CHANNEL_MAX) {
-		ERROR_LOG(SCEAUDIO, "sceAudioChangeChannelVolume(%08x, %08x, %08x) - invalid channel number", chan, leftvol, rightvol);
+		ERROR_LOG(Log::sceAudio, "sceAudioChangeChannelVolume(%08x, %08x, %08x) - invalid channel number", chan, leftvol, rightvol);
 		return SCE_ERROR_AUDIO_INVALID_CHANNEL;
 	} else if (!chans[chan].reserved)	{
-		ERROR_LOG(SCEAUDIO, "sceAudioChangeChannelVolume(%08x, %08x, %08x) - channel not reserved", chan, leftvol, rightvol);
+		ERROR_LOG(Log::sceAudio, "sceAudioChangeChannelVolume(%08x, %08x, %08x) - channel not reserved", chan, leftvol, rightvol);
 		return SCE_ERROR_AUDIO_CHANNEL_NOT_RESERVED;
 	} else {
-		DEBUG_LOG(SCEAUDIO, "sceAudioChangeChannelVolume(%08x, %08x, %08x)", chan, leftvol, rightvol);
+		DEBUG_LOG(Log::sceAudio, "sceAudioChangeChannelVolume(%08x, %08x, %08x)", chan, leftvol, rightvol);
 		chans[chan].leftVolume = leftvol;
 		chans[chan].rightVolume = rightvol;
 		return 0;
@@ -305,13 +306,13 @@ static u32 sceAudioChangeChannelVolume(u32 chan, u32 leftvol, u32 rightvol) {
 }
 
 static u32 sceAudioInit() {
-	DEBUG_LOG(SCEAUDIO, "sceAudioInit()");
+	DEBUG_LOG(Log::sceAudio, "sceAudioInit()");
 	// Don't need to do anything
 	return 0;
 }
 
 static u32 sceAudioEnd() {
-	DEBUG_LOG(SCEAUDIO, "sceAudioEnd()");
+	DEBUG_LOG(Log::sceAudio, "sceAudioEnd()");
 	// Don't need to do anything
 	return 0;
 }
@@ -321,27 +322,27 @@ static u32 sceAudioOutput2Reserve(u32 sampleCount) {
 	// This seems to ignore the MSB, for some reason.
 	sampleCount &= 0x7FFFFFFF;
 	if (sampleCount < 17 || sampleCount > 4111) {
-		return hleLogError(SCEAUDIO, SCE_KERNEL_ERROR_INVALID_SIZE, "invalid sample count");
+		return hleLogError(Log::sceAudio, SCE_KERNEL_ERROR_INVALID_SIZE, "invalid sample count");
 	} else if (chan.reserved) {
-		return hleLogError(SCEAUDIO, SCE_ERROR_AUDIO_CHANNEL_ALREADY_RESERVED, "channel already reserved");
+		return hleLogError(Log::sceAudio, SCE_ERROR_AUDIO_CHANNEL_ALREADY_RESERVED, "channel already reserved");
 	}
 
 	chan.sampleCount = sampleCount;
 	chan.format = PSP_AUDIO_FORMAT_STEREO;
 	chan.reserved = true;
 	__AudioSetSRCFrequency(0);
-	return hleLogSuccessI(SCEAUDIO, 0);
+	return hleLogSuccessI(Log::sceAudio, 0);
 }
 
 static u32 sceAudioOutput2OutputBlocking(u32 vol, u32 dataPtr) {
 	// Note: 0xFFFFF, not 0xFFFF!
 	if (vol > 0xFFFFF) {
-		return hleLogError(SCEAUDIO, SCE_ERROR_AUDIO_INVALID_VOLUME, "invalid volume");
+		return hleLogError(Log::sceAudio, SCE_ERROR_AUDIO_INVALID_VOLUME, "invalid volume");
 	}
 
 	auto &chan = chans[PSP_AUDIO_CHANNEL_OUTPUT2];
 	if (!chan.reserved) {
-		return hleLogError(SCEAUDIO, SCE_ERROR_AUDIO_CHANNEL_NOT_RESERVED, "channel not reserved");
+		return hleLogError(Log::sceAudio, SCE_ERROR_AUDIO_CHANNEL_NOT_RESERVED, "channel not reserved");
 	}
 
 	chan.leftVolume = vol;
@@ -351,58 +352,58 @@ static u32 sceAudioOutput2OutputBlocking(u32 vol, u32 dataPtr) {
 	hleEatCycles(10000);
 	int result = __AudioEnqueue(chan, PSP_AUDIO_CHANNEL_OUTPUT2, true);
 	if (result < 0)
-		return hleLogError(SCEAUDIO, result);
-	return hleLogSuccessI(SCEAUDIO, result);
+		return hleLogError(Log::sceAudio, result);
+	return hleLogSuccessI(Log::sceAudio, result);
 }
 
 static u32 sceAudioOutput2ChangeLength(u32 sampleCount) {
 	auto &chan = chans[PSP_AUDIO_CHANNEL_OUTPUT2];
 	if (!chan.reserved) {
-		return hleLogError(SCEAUDIO, SCE_ERROR_AUDIO_CHANNEL_NOT_RESERVED, "channel not reserved");
+		return hleLogError(Log::sceAudio, SCE_ERROR_AUDIO_CHANNEL_NOT_RESERVED, "channel not reserved");
 	}
 	chan.sampleCount = sampleCount;
-	return hleLogSuccessI(SCEAUDIO, 0);
+	return hleLogSuccessI(Log::sceAudio, 0);
 }
 
 static u32 sceAudioOutput2GetRestSample() {
 	auto &chan = chans[PSP_AUDIO_CHANNEL_OUTPUT2];
 	if (!chan.reserved) {
-		return hleLogError(SCEAUDIO, SCE_ERROR_AUDIO_CHANNEL_NOT_RESERVED, "channel not reserved");
+		return hleLogError(Log::sceAudio, SCE_ERROR_AUDIO_CHANNEL_NOT_RESERVED, "channel not reserved");
 	}
 	u32 size = (u32)chanSampleQueues[PSP_AUDIO_CHANNEL_OUTPUT2].size() / 2;
 	if (size > chan.sampleCount) {
 		// If ChangeLength reduces the size, it still gets output but this return is clamped.
 		size = chan.sampleCount;
 	}
-	return hleLogSuccessI(SCEAUDIO, size);
+	return hleLogSuccessI(Log::sceAudio, size);
 }
 
 static u32 sceAudioOutput2Release() {
 	auto &chan = chans[PSP_AUDIO_CHANNEL_OUTPUT2];
 	if (!chan.reserved)
-		return hleLogError(SCEAUDIO, SCE_ERROR_AUDIO_CHANNEL_NOT_RESERVED, "channel not reserved");
+		return hleLogError(Log::sceAudio, SCE_ERROR_AUDIO_CHANNEL_NOT_RESERVED, "channel not reserved");
 	if (!chanSampleQueues[PSP_AUDIO_CHANNEL_OUTPUT2].empty())
-		return hleLogError(SCEAUDIO, SCE_ERROR_AUDIO_CHANNEL_ALREADY_RESERVED, "output busy");
+		return hleLogError(Log::sceAudio, SCE_ERROR_AUDIO_CHANNEL_ALREADY_RESERVED, "output busy");
 
 	chan.reset();
 	chan.reserved = false;
-	return hleLogSuccessI(SCEAUDIO, 0);
+	return hleLogSuccessI(Log::sceAudio, 0);
 }
 
 static u32 sceAudioSetFrequency(u32 freq) {
 	// TODO: Not available from user code.
 	if (freq == 44100 || freq == 48000) {
-		INFO_LOG(SCEAUDIO, "sceAudioSetFrequency(%08x)", freq);
+		INFO_LOG(Log::sceAudio, "sceAudioSetFrequency(%08x)", freq);
 		__AudioSetOutputFrequency(freq);
 		return 0;
 	} else {
-		ERROR_LOG(SCEAUDIO, "sceAudioSetFrequency(%08x) - invalid frequency (must be 44.1 or 48 khz)", freq);
+		ERROR_LOG(Log::sceAudio, "sceAudioSetFrequency(%08x) - invalid frequency (must be 44.1 or 48 khz)", freq);
 		return SCE_ERROR_AUDIO_INVALID_FREQUENCY;
 	}
 }
 
 static u32 sceAudioSetVolumeOffset() {
-	ERROR_LOG(SCEAUDIO, "UNIMPL sceAudioSetVolumeOffset()");
+	ERROR_LOG(Log::sceAudio, "UNIMPL sceAudioSetVolumeOffset()");
 	return 0;
 }
 
@@ -419,15 +420,15 @@ static u32 sceAudioSRCChReserve(u32 sampleCount, u32 freq, u32 format) {
 	// This seems to ignore the MSB, for some reason.
 	sampleCount &= 0x7FFFFFFF;
 	if (format == 4) {
-		return hleReportError(SCEAUDIO, PSP_AUDIO_ERROR_SRC_FORMAT_4, "unexpected format");
+		return hleReportError(Log::sceAudio, PSP_AUDIO_ERROR_SRC_FORMAT_4, "unexpected format");
 	} else if (format != 2) {
-		return hleLogError(SCEAUDIO, SCE_KERNEL_ERROR_INVALID_SIZE, "unexpected format");
+		return hleLogError(Log::sceAudio, SCE_KERNEL_ERROR_INVALID_SIZE, "unexpected format");
 	} else if (sampleCount < 17 || sampleCount > 4111) {
-		return hleLogError(SCEAUDIO, SCE_KERNEL_ERROR_INVALID_SIZE, "invalid sample count");
+		return hleLogError(Log::sceAudio, SCE_KERNEL_ERROR_INVALID_SIZE, "invalid sample count");
 	} else if (freq != 0 && !SRCFrequencyAllowed(freq)) {
-		return hleLogError(SCEAUDIO, SCE_ERROR_AUDIO_INVALID_FREQUENCY, "invalid frequency");
+		return hleLogError(Log::sceAudio, SCE_ERROR_AUDIO_INVALID_FREQUENCY, "invalid frequency");
 	} else if (chan.reserved) {
-		return hleLogError(SCEAUDIO, SCE_ERROR_AUDIO_CHANNEL_ALREADY_RESERVED, "channel already reserved");
+		return hleLogError(Log::sceAudio, SCE_ERROR_AUDIO_CHANNEL_ALREADY_RESERVED, "channel already reserved");
 	}
 
 	chan.reserved = true;
@@ -435,29 +436,29 @@ static u32 sceAudioSRCChReserve(u32 sampleCount, u32 freq, u32 format) {
 	chan.format = format == 2 ? PSP_AUDIO_FORMAT_STEREO : PSP_AUDIO_FORMAT_MONO;
 	// Zero means default to 44.1kHz.
 	__AudioSetSRCFrequency(freq);
-	return hleLogSuccessI(SCEAUDIO, 0);
+	return hleLogSuccessI(Log::sceAudio, 0);
 }
 
 static u32 sceAudioSRCChRelease() {
 	auto &chan = chans[PSP_AUDIO_CHANNEL_SRC];
 	if (!chan.reserved)
-		return hleLogError(SCEAUDIO, SCE_ERROR_AUDIO_CHANNEL_NOT_RESERVED, "channel not reserved");
+		return hleLogError(Log::sceAudio, SCE_ERROR_AUDIO_CHANNEL_NOT_RESERVED, "channel not reserved");
 	if (!chanSampleQueues[PSP_AUDIO_CHANNEL_SRC].empty())
-		return hleLogError(SCEAUDIO, SCE_ERROR_AUDIO_CHANNEL_ALREADY_RESERVED, "output busy");
+		return hleLogError(Log::sceAudio, SCE_ERROR_AUDIO_CHANNEL_ALREADY_RESERVED, "output busy");
 
 	chan.reset();
 	chan.reserved = false;
-	return hleLogSuccessI(SCEAUDIO, 0);
+	return hleLogSuccessI(Log::sceAudio, 0);
 }
 
 static u32 sceAudioSRCOutputBlocking(u32 vol, u32 buf) {
 	if (vol > 0xFFFFF) {
-		return hleLogError(SCEAUDIO, SCE_ERROR_AUDIO_INVALID_VOLUME, "invalid volume");
+		return hleLogError(Log::sceAudio, SCE_ERROR_AUDIO_INVALID_VOLUME, "invalid volume");
 	}
 
 	auto &chan = chans[PSP_AUDIO_CHANNEL_SRC];
 	if (!chan.reserved) {
-		return hleLogError(SCEAUDIO, SCE_ERROR_AUDIO_CHANNEL_NOT_RESERVED, "channel not reserved");
+		return hleLogError(Log::sceAudio, SCE_ERROR_AUDIO_CHANNEL_NOT_RESERVED, "channel not reserved");
 	}
 
 	chan.leftVolume = vol;
@@ -467,57 +468,77 @@ static u32 sceAudioSRCOutputBlocking(u32 vol, u32 buf) {
 	hleEatCycles(10000);
 	int result = __AudioEnqueue(chan, PSP_AUDIO_CHANNEL_SRC, true);
 	if (result < 0)
-		return hleLogError(SCEAUDIO, result);
-	return hleLogSuccessI(SCEAUDIO, result);
+		return hleLogError(Log::sceAudio, result);
+	return hleLogSuccessI(Log::sceAudio, result);
 }
 
 static int sceAudioInputBlocking(u32 maxSamples, u32 sampleRate, u32 bufAddr) {
 	if (!Memory::IsValidAddress(bufAddr)) {
-		ERROR_LOG(HLE, "sceAudioInputBlocking(%d, %d, %08x): invalid addresses", maxSamples, sampleRate, bufAddr);
+		ERROR_LOG(Log::HLE, "sceAudioInputBlocking(%d, %d, %08x): invalid addresses", maxSamples, sampleRate, bufAddr);
 		return -1;
 	}
 
-	INFO_LOG(HLE, "sceAudioInputBlocking: maxSamples: %d, samplerate: %d, bufAddr: %08x", maxSamples, sampleRate, bufAddr);
+	INFO_LOG(Log::HLE, "sceAudioInputBlocking: maxSamples: %d, samplerate: %d, bufAddr: %08x", maxSamples, sampleRate, bufAddr);
 	return __MicInput(maxSamples, sampleRate, bufAddr, AUDIOINPUT);
 }
 
 static int sceAudioInput(u32 maxSamples, u32 sampleRate, u32 bufAddr) {
 	if (!Memory::IsValidAddress(bufAddr)) {
-		ERROR_LOG(HLE, "sceAudioInput(%d, %d, %08x): invalid addresses", maxSamples, sampleRate, bufAddr);
+		ERROR_LOG(Log::HLE, "sceAudioInput(%d, %d, %08x): invalid addresses", maxSamples, sampleRate, bufAddr);
 		return -1;
 	}
 
-	ERROR_LOG(HLE, "UNTEST sceAudioInput: maxSamples: %d, samplerate: %d, bufAddr: %08x", maxSamples, sampleRate, bufAddr);
+	ERROR_LOG(Log::HLE, "UNTEST sceAudioInput: maxSamples: %d, samplerate: %d, bufAddr: %08x", maxSamples, sampleRate, bufAddr);
 	return __MicInput(maxSamples, sampleRate, bufAddr, AUDIOINPUT, false);
 }
 
+static int sceAudioInputInit(int unknown1, int gain, int unknown2) {
+	ERROR_LOG(Log::HLE, "UNIMPL sceAudioInputInit: unknown1: %d, gain: %d, unknown2: %d", unknown1, gain, unknown2);
+	return 0;
+}
+
+static int sceAudioInputInitEx(u32 paramAddr) {
+	ERROR_LOG(Log::HLE, "UNIMPL sceAudioInputInitEx: paramAddr: %08x", paramAddr);
+	return 0;
+}
+
+static int sceAudioPollInputEnd() {
+	ERROR_LOG(Log::HLE, "UNIMPL sceAudioPollInputEnd");
+	return 0;
+}
+
+static int sceAudioWaitInputEnd() {
+	ERROR_LOG(Log::HLE, "UNIMPL sceAudioWaitInputEnd");
+	return 0;
+}
+
 static int sceAudioGetInputLength() {
 	int ret = Microphone::getReadMicDataLength() / 2;
-	ERROR_LOG(HLE, "UNTEST sceAudioGetInputLength(ret: %d)", ret);
+	ERROR_LOG(Log::HLE, "UNTEST sceAudioGetInputLength(ret: %d)", ret);
 	return ret;
 }
 
 static u32 sceAudioRoutingSetMode(u32 mode) {
-	ERROR_LOG_REPORT(SCEAUDIO, "sceAudioRoutingSetMode(%08x)", mode);
+	ERROR_LOG_REPORT(Log::sceAudio, "sceAudioRoutingSetMode(%08x)", mode);
 	int previousMode = defaultRoutingMode;
 	defaultRoutingMode = mode;
 	return previousMode;
 }
 
 static u32 sceAudioRoutingGetMode() {
-	ERROR_LOG_REPORT(SCEAUDIO, "sceAudioRoutingGetMode()");
+	ERROR_LOG_REPORT(Log::sceAudio, "sceAudioRoutingGetMode()");
 	return defaultRoutingMode;
 }
 
 static u32 sceAudioRoutingSetVolumeMode(u32 mode) {
-	ERROR_LOG_REPORT(SCEAUDIO, "sceAudioRoutingSetVolumeMode(%08x)", mode);
+	ERROR_LOG_REPORT(Log::sceAudio, "sceAudioRoutingSetVolumeMode(%08x)", mode);
 	int previousMode = defaultRoutingVolMode;
 	defaultRoutingVolMode = mode;
 	return previousMode;
 }
 
 static u32 sceAudioRoutingGetVolumeMode() {
-	ERROR_LOG_REPORT(SCEAUDIO, "sceAudioRoutingGetVolumeMode()");
+	ERROR_LOG_REPORT(Log::sceAudio, "sceAudioRoutingGetVolumeMode()");
 	return defaultRoutingVolMode;
 }
 
@@ -558,13 +579,13 @@ const HLEFunction sceAudio[] =
 	{0XB61595C0, nullptr,                                   "sceAudioLoopbackTest",          '?', ""    },
 
 	// Microphone interface
-	{0X7DE61688, nullptr,                                   "sceAudioInputInit",             '?', ""    },
-	{0XE926D3FB, nullptr,                                   "sceAudioInputInitEx",           '?', ""    },
+	{0X7DE61688, &WrapI_III<sceAudioInputInit>,             "sceAudioInputInit",             'i', "iii" },
+	{0XE926D3FB, &WrapI_U<sceAudioInputInitEx>,             "sceAudioInputInitEx",           'i', "x"   },
 	{0X6D4BEC68, &WrapI_UUU<sceAudioInput>,                 "sceAudioInput",                 'i', "xxx" },
 	{0X086E5895, &WrapI_UUU<sceAudioInputBlocking>,         "sceAudioInputBlocking",         'i', "xxx" },
 	{0XA708C6A6, &WrapI_V<sceAudioGetInputLength>,          "sceAudioGetInputLength",        'i', ""    },
-	{0XA633048E, nullptr,                                   "sceAudioPollInputEnd",          '?', ""    },
-	{0X87B2E651, nullptr,                                   "sceAudioWaitInputEnd",          '?', ""    },
+	{0XA633048E, &WrapI_V<sceAudioPollInputEnd>,            "sceAudioPollInputEnd",          'i', ""    },
+	{0X87B2E651, &WrapI_V<sceAudioWaitInputEnd>,            "sceAudioWaitInputEnd",          'i', ""    },
 
 	{0X36FD8AA9, &WrapU_U<sceAudioRoutingSetMode>,          "sceAudioRoutingSetMode",        'x', "x"   },
 	{0X39240E7D, &WrapU_V<sceAudioRoutingGetMode>,          "sceAudioRoutingGetMode",        'x', ""    },
