@@ -19,8 +19,11 @@
 #include <vector>
 #include <mutex>
 
-#include "Common/ChunkFile.h"
-#include "Common/ThreadSafeList.h"
+#include "Common/Serialize/Serializer.h"
+#include "Common/Serialize/SerializeFuncs.h"
+#include "Common/Serialize/SerializeList.h"
+#include "Common/Serialize/SerializeMap.h"
+#include "Common/Data/Collections/ThreadSafeList.h"
 #include "Core/HLE/HLE.h"
 #include "Core/HLE/FunctionWrappers.h"
 #include "Core/MIPS/MIPS.h"
@@ -63,7 +66,7 @@ public:
 
 	bool run(PendingInterrupt& pend) override {
 		if (ge_pending_cb.empty()) {
-			ERROR_LOG_REPORT(SCEGE, "Unable to run GE interrupt: no pending interrupt");
+			ERROR_LOG_REPORT(Log::sceGe, "Unable to run GE interrupt: no pending interrupt");
 			return false;
 		}
 
@@ -71,12 +74,12 @@ public:
 		DisplayList* dl = gpu->getList(intrdata.listid);
 
 		if (dl == NULL) {
-			WARN_LOG(SCEGE, "Unable to run GE interrupt: list doesn't exist: %d", intrdata.listid);
+			WARN_LOG(Log::sceGe, "Unable to run GE interrupt: list doesn't exist: %d", intrdata.listid);
 			return false;
 		}
 
 		if (!dl->interruptsEnabled) {
-			ERROR_LOG_REPORT(SCEGE, "Unable to run GE interrupt: list has interrupts disabled, should not happen");
+			ERROR_LOG_REPORT(Log::sceGe, "Unable to run GE interrupt: list has interrupts disabled, should not happen");
 			return false;
 		}
 
@@ -115,7 +118,7 @@ public:
 
 		SubIntrHandler* handler = get(subintr);
 		if (handler != NULL) {
-			DEBUG_LOG(CPU, "Entering GE interrupt handler %08x", handler->handlerAddress);
+			DEBUG_LOG(Log::CPU, "Entering GE interrupt handler %08x", handler->handlerAddress);
 			currentMIPS->pc = handler->handlerAddress;
 			u32 data = dl->subIntrToken;
 			currentMIPS->r[MIPS_REG_A0] = data & 0xFFFF;
@@ -139,7 +142,7 @@ public:
 
 		// Seen in GoW.
 		if (subintr >= 0)
-			DEBUG_LOG(SCEGE, "Ignoring interrupt for display list %d, already been released.", intrdata.listid);
+			DEBUG_LOG(Log::sceGe, "Ignoring interrupt for display list %d, already been released.", intrdata.listid);
 		return false;
 	}
 
@@ -149,7 +152,7 @@ public:
 
 		DisplayList* dl = gpu->getList(intrdata.listid);
 		if (!dl->interruptsEnabled) {
-			ERROR_LOG_REPORT(SCEGE, "Unable to finish GE interrupt: list has interrupts disabled, should not happen");
+			ERROR_LOG_REPORT(Log::sceGe, "Unable to finish GE interrupt: list has interrupts disabled, should not happen");
 			return;
 		}
 
@@ -160,9 +163,9 @@ public:
 				DisplayListState newState = static_cast<DisplayListState>(Memory::ReadUnchecked_U32(intrdata.pc - 4) & 0xFF);
 				//dl->status = static_cast<DisplayListStatus>(Memory::ReadUnchecked_U32(intrdata.pc) & 0xFF);
 				//if(dl->status < 0 || dl->status > PSP_GE_LIST_PAUSED)
-				//	ERROR_LOG(SCEGE, "Weird DL status after signal suspend %x", dl->status);
+				//	ERROR_LOG(Log::sceGe, "Weird DL status after signal suspend %x", dl->status);
 				if (newState != PSP_GE_DL_STATE_RUNNING) {
-					DEBUG_LOG_REPORT(SCEGE, "GE Interrupt: newState might be %d", newState);
+					DEBUG_LOG_REPORT(Log::sceGe, "GE Interrupt: newState might be %d", newState);
 				}
 
 				if (dl->state != PSP_GE_DL_STATE_NONE && dl->state != PSP_GE_DL_STATE_COMPLETED) {
@@ -219,31 +222,31 @@ void __GeDoState(PointerWrap &p) {
 	if (!s)
 		return;
 
-	p.DoArray(ge_callback_data, ARRAY_SIZE(ge_callback_data));
-	p.DoArray(ge_used_callbacks, ARRAY_SIZE(ge_used_callbacks));
+	DoArray(p, ge_callback_data, ARRAY_SIZE(ge_callback_data));
+	DoArray(p, ge_used_callbacks, ARRAY_SIZE(ge_used_callbacks));
 
 	if (s >= 2) {
-		p.Do(ge_pending_cb);
+		Do(p, ge_pending_cb);
 	} else {
 		std::list<GeInterruptData_v1> old;
-		p.Do(old);
+		Do(p, old);
 		ge_pending_cb.clear();
-		for (auto it = old.begin(), end = old.end(); it != end; ++it) {
-			GeInterruptData intrdata = {it->listid, it->pc};
-			intrdata.cmd = Memory::ReadUnchecked_U32(it->pc - 4) >> 24;
+		for (const auto &ge : old) {
+			GeInterruptData intrdata = {ge.listid, ge.pc};
+			intrdata.cmd = Memory::ReadUnchecked_U32(ge.pc - 4) >> 24;
 			ge_pending_cb.push_back(intrdata);
 		}
 	}
 
-	p.Do(geSyncEvent);
+	Do(p, geSyncEvent);
 	CoreTiming::RestoreRegisterEvent(geSyncEvent, "GeSyncEvent", &__GeExecuteSync);
-	p.Do(geInterruptEvent);
+	Do(p, geInterruptEvent);
 	CoreTiming::RestoreRegisterEvent(geInterruptEvent, "GeInterruptEvent", &__GeExecuteInterrupt);
-	p.Do(geCycleEvent);
+	Do(p, geCycleEvent);
 	CoreTiming::RestoreRegisterEvent(geCycleEvent, "GeCycleEvent", &__GeCheckCycles);
 
-	p.Do(listWaitingThreads);
-	p.Do(drawWaitingThreads);
+	Do(p, listWaitingThreads);
+	Do(p, drawWaitingThreads);
 
 	// Everything else is done in sceDisplay.
 }
@@ -285,7 +288,7 @@ void __GeWaitCurrentThread(GPUSyncType type, SceUID waitId, const char *reason)
 		listWaitingThreads[waitId].push_back(__KernelGetCurThread());
 		waitType = WAITTYPE_GELISTSYNC;
 	} else {
-		ERROR_LOG_REPORT(SCEGE, "__GeWaitCurrentThread: bad wait type");
+		ERROR_LOG_REPORT(Log::sceGe, "__GeWaitCurrentThread: bad wait type");
 		return;
 	}
 
@@ -295,8 +298,8 @@ void __GeWaitCurrentThread(GPUSyncType type, SceUID waitId, const char *reason)
 static bool __GeTriggerWait(WaitType waitType, SceUID waitId, WaitingThreadList &waitingThreads) {
 	// TODO: Do they ever get a result other than 0?
 	bool wokeThreads = false;
-	for (auto it = waitingThreads.begin(), end = waitingThreads.end(); it != end; ++it)
-		wokeThreads |= HLEKernel::ResumeFromWait(*it, waitType, waitId, 0);
+	for (int threadID : waitingThreads)
+		wokeThreads |= HLEKernel::ResumeFromWait(threadID, waitType, waitId, 0);
 	waitingThreads.clear();
 	return wokeThreads;
 }
@@ -308,13 +311,13 @@ bool __GeTriggerWait(GPUSyncType type, SceUID waitId) {
 	else if (type == GPU_SYNC_LIST || (WaitType)type == WAITTYPE_GELISTSYNC)
 		return __GeTriggerWait(WAITTYPE_GELISTSYNC, waitId, listWaitingThreads[waitId]);
 	else
-		ERROR_LOG_REPORT(SCEGE, "__GeTriggerWait: bad wait type");
+		ERROR_LOG_REPORT(Log::sceGe, "__GeTriggerWait: bad wait type");
 	return false;
 }
 
 static u32 sceGeEdramGetAddr() {
 	u32 retVal = 0x04000000;
-	DEBUG_LOG(SCEGE, "%08x = sceGeEdramGetAddr", retVal);
+	DEBUG_LOG(Log::sceGe, "%08x = sceGeEdramGetAddr", retVal);
 	hleEatCycles(150);
 	return retVal;
 }
@@ -322,7 +325,7 @@ static u32 sceGeEdramGetAddr() {
 // TODO: Return a different value for the PS3 enhanced-emulator games?
 static u32 sceGeEdramGetSize() {
 	const u32 retVal = 0x00200000;
-	DEBUG_LOG(SCEGE, "%08x = sceGeEdramGetSize()", retVal);
+	DEBUG_LOG(Log::sceGe, "%08x = sceGeEdramGetSize()", retVal);
 	return retVal;
 }
 
@@ -331,7 +334,7 @@ static int __GeSubIntrBase(int callbackId) {
 }
 
 u32 sceGeListEnQueue(u32 listAddress, u32 stallAddress, int callbackId, u32 optParamAddr) {
-	DEBUG_LOG(SCEGE,
+	DEBUG_LOG(Log::sceGe,
 			"sceGeListEnQueue(addr=%08x, stall=%08x, cbid=%08x, param=%08x)",
 			listAddress, stallAddress, callbackId, optParamAddr);
 	auto optParam = PSPPointer<PspGeListArgs>::Create(optParamAddr);
@@ -340,30 +343,28 @@ u32 sceGeListEnQueue(u32 listAddress, u32 stallAddress, int callbackId, u32 optP
 	if ((int)listID >= 0)
 		listID = LIST_ID_MAGIC ^ listID;
 
-	DEBUG_LOG(SCEGE, "List %i enqueued.", listID);
 	hleEatCycles(490);
 	CoreTiming::ForceCheck();
-	return listID;
+	return hleLogSuccessX(Log::sceGe, listID);
 }
 
 u32 sceGeListEnQueueHead(u32 listAddress, u32 stallAddress, int callbackId, u32 optParamAddr) {
-	DEBUG_LOG(SCEGE,
+	DEBUG_LOG(Log::sceGe,
 			"sceGeListEnQueueHead(addr=%08x, stall=%08x, cbid=%08x, param=%08x)",
 			listAddress, stallAddress, callbackId, optParamAddr);
 	auto optParam = PSPPointer<PspGeListArgs>::Create(optParamAddr);
 
 	u32 listID = gpu->EnqueueList(listAddress, stallAddress, __GeSubIntrBase(callbackId), optParam, true);
-	if ((int)listID >= 0) {
+	if ((int)listID >= 0)
 		listID = LIST_ID_MAGIC ^ listID;
-		DEBUG_LOG(SCEGE, "List %i enqueued at head.", listID);
-	}
+
 	hleEatCycles(480);
 	CoreTiming::ForceCheck();
-	return listID;
+	return hleLogSuccessX(Log::sceGe, listID);
 }
 
 static int sceGeListDeQueue(u32 listID) {
-	WARN_LOG(SCEGE, "sceGeListDeQueue(%08x)", listID);
+	WARN_LOG(Log::sceGe, "sceGeListDeQueue(%08x)", listID);
 	int result = gpu->DequeueList(LIST_ID_MAGIC ^ listID);
 	hleReSchedule("dlist dequeued");
 	return result;
@@ -375,13 +376,14 @@ static int sceGeListUpdateStallAddr(u32 displayListID, u32 stallAddress) {
 	hleEatCycles(190);
 	CoreTiming::ForceCheck();
 
-	DEBUG_LOG(SCEGE, "sceGeListUpdateStallAddr(dlid=%i, stalladdr=%08x)", displayListID, stallAddress);
+	DEBUG_LOG(Log::sceGe, "sceGeListUpdateStallAddr(dlid=%i, stalladdr=%08x)", displayListID, stallAddress);
 	return gpu->UpdateStall(LIST_ID_MAGIC ^ displayListID, stallAddress);
 }
 
 // 0 : wait for completion. 1:check and return
 int sceGeListSync(u32 displayListID, u32 mode) {
-	DEBUG_LOG(SCEGE, "sceGeListSync(dlid=%08x, mode=%08x)", displayListID, mode);
+	DEBUG_LOG(Log::sceGe, "sceGeListSync(dlid=%08x, mode=%08x)", displayListID, mode);
+	hleEatCycles(220);  // Fudged without measuring, copying sceGeContinue.
 	return gpu->ListSync(LIST_ID_MAGIC ^ displayListID, mode);
 }
 
@@ -389,12 +391,14 @@ static u32 sceGeDrawSync(u32 mode) {
 	//wait/check entire drawing state
 	if (PSP_CoreParameter().compat.flags().DrawSyncEatCycles)
 		hleEatCycles(500000); //HACK(?) : Potential fix for Crash Tag Team Racing and a few Gundam games
-	DEBUG_LOG(SCEGE, "sceGeDrawSync(mode=%d)  (0=wait for completion, 1=peek)", mode);
+	else if (!PSP_CoreParameter().compat.flags().DrawSyncInstant)
+		hleEatCycles(1240);
+	DEBUG_LOG(Log::sceGe, "sceGeDrawSync(mode=%d)  (0=wait for completion, 1=peek)", mode);
 	return gpu->DrawSync(mode);
 }
 
 int sceGeContinue() {
-	DEBUG_LOG(SCEGE, "sceGeContinue");
+	DEBUG_LOG(Log::sceGe, "sceGeContinue");
 	int ret = gpu->Continue();
 	hleEatCycles(220);
 	hleReSchedule("ge continue");
@@ -403,19 +407,19 @@ int sceGeContinue() {
 
 static int sceGeBreak(u32 mode, u32 unknownPtr) {
 	if (mode > 1) {
-		WARN_LOG(SCEGE, "sceGeBreak(mode=%d, unknown=%08x): invalid mode", mode, unknownPtr);
+		WARN_LOG(Log::sceGe, "sceGeBreak(mode=%d, unknown=%08x): invalid mode", mode, unknownPtr);
 		return SCE_KERNEL_ERROR_INVALID_MODE;
 	}
 	// Not sure what this is supposed to be for...
-	if ((int)unknownPtr < 0 || (int)unknownPtr + 16 < 0) {
-		WARN_LOG_REPORT(SCEGE, "sceGeBreak(mode=%d, unknown=%08x): invalid ptr", mode, unknownPtr);
+	if ((int)unknownPtr < 0 || (int)(unknownPtr + 16) < 0) {
+		WARN_LOG_REPORT(Log::sceGe, "sceGeBreak(mode=%d, unknown=%08x): invalid ptr", mode, unknownPtr);
 		return SCE_KERNEL_ERROR_PRIV_REQUIRED;
 	} else if (unknownPtr != 0) {
-		WARN_LOG_REPORT(SCEGE, "sceGeBreak(mode=%d, unknown=%08x): unknown ptr (%s)", mode, unknownPtr, Memory::IsValidAddress(unknownPtr) ? "valid" : "invalid");
+		WARN_LOG_REPORT(Log::sceGe, "sceGeBreak(mode=%d, unknown=%08x): unknown ptr (%s)", mode, unknownPtr, Memory::IsValidAddress(unknownPtr) ? "valid" : "invalid");
 	}
 
 	//mode => 0 : current dlist 1: all drawing
-	DEBUG_LOG(SCEGE, "sceGeBreak(mode=%d, unknown=%08x)", mode, unknownPtr);
+	DEBUG_LOG(Log::sceGe, "sceGeBreak(mode=%d, unknown=%08x)", mode, unknownPtr);
 	int result = gpu->Break(mode);
 	if (result >= 0 && mode == 0) {
 		return LIST_ID_MAGIC ^ result;
@@ -424,8 +428,6 @@ static int sceGeBreak(u32 mode, u32 unknownPtr) {
 }
 
 static u32 sceGeSetCallback(u32 structAddr) {
-	DEBUG_LOG(SCEGE, "sceGeSetCallback(struct=%08x)", structAddr);
-
 	int cbID = -1;
 	for (size_t i = 0; i < ARRAY_SIZE(ge_used_callbacks); ++i) {
 		if (!ge_used_callbacks[i]) {
@@ -435,12 +437,13 @@ static u32 sceGeSetCallback(u32 structAddr) {
 	}
 
 	if (cbID == -1) {
-		WARN_LOG(SCEGE, "sceGeSetCallback(): out of callback ids");
-		return SCE_KERNEL_ERROR_OUT_OF_MEMORY;
+		return hleLogWarning(Log::sceGe, SCE_KERNEL_ERROR_OUT_OF_MEMORY, "out of callback ids");
 	}
 
 	ge_used_callbacks[cbID] = true;
-	Memory::ReadStruct(structAddr, &ge_callback_data[cbID]);
+	auto callbackData = PSPPointer<PspGeCallbackData>::Create(structAddr);
+	ge_callback_data[cbID] = *callbackData;
+	callbackData.NotifyRead("GeSetCallback");
 
 	int subIntrBase = __GeSubIntrBase(cbID);
 
@@ -455,14 +458,14 @@ static u32 sceGeSetCallback(u32 structAddr) {
 		sceKernelEnableSubIntr(PSP_GE_INTR, subIntrBase | PSP_GE_SUBINTR_SIGNAL);
 	}
 
-	return cbID;
+	return hleLogSuccessI(Log::sceGe, cbID);
 }
 
 static int sceGeUnsetCallback(u32 cbID) {
-	DEBUG_LOG(SCEGE, "sceGeUnsetCallback(cbid=%08x)", cbID);
+	DEBUG_LOG(Log::sceGe, "sceGeUnsetCallback(cbid=%08x)", cbID);
 
 	if (cbID >= ARRAY_SIZE(ge_used_callbacks)) {
-		WARN_LOG(SCEGE, "sceGeUnsetCallback(cbid=%08x): invalid callback id", cbID);
+		WARN_LOG(Log::sceGe, "sceGeUnsetCallback(cbid=%08x): invalid callback id", cbID);
 		return SCE_KERNEL_ERROR_INVALID_ID;
 	}
 
@@ -472,7 +475,7 @@ static int sceGeUnsetCallback(u32 cbID) {
 		sceKernelReleaseSubIntrHandler(PSP_GE_INTR, subIntrBase | PSP_GE_SUBINTR_FINISH);
 		sceKernelReleaseSubIntrHandler(PSP_GE_INTR, subIntrBase | PSP_GE_SUBINTR_SIGNAL);
 	} else {
-		WARN_LOG(SCEGE, "sceGeUnsetCallback(cbid=%08x): ignoring unregistered callback id", cbID);
+		WARN_LOG(Log::sceGe, "sceGeUnsetCallback(cbid=%08x): ignoring unregistered callback id", cbID);
 	}
 
 	ge_used_callbacks[cbID] = false;
@@ -482,10 +485,10 @@ static int sceGeUnsetCallback(u32 cbID) {
 // Points to 512 32-bit words, where we can probably layout the context however we want
 // unless some insane game pokes it and relies on it...
 u32 sceGeSaveContext(u32 ctxAddr) {
-	DEBUG_LOG(SCEGE, "sceGeSaveContext(%08x)", ctxAddr);
+	DEBUG_LOG(Log::sceGe, "sceGeSaveContext(%08x)", ctxAddr);
 
 	if (gpu->BusyDrawing()) {
-		WARN_LOG(SCEGE, "sceGeSaveContext(%08x): lists in process, aborting", ctxAddr);
+		WARN_LOG(Log::sceGe, "sceGeSaveContext(%08x): lists in process, aborting", ctxAddr);
 		// Real error code.
 		return -1;
 	}
@@ -501,10 +504,10 @@ u32 sceGeSaveContext(u32 ctxAddr) {
 }
 
 u32 sceGeRestoreContext(u32 ctxAddr) {
-	DEBUG_LOG(SCEGE, "sceGeRestoreContext(%08x)", ctxAddr);
+	DEBUG_LOG(Log::sceGe, "sceGeRestoreContext(%08x)", ctxAddr);
 
 	if (gpu->BusyDrawing()) {
-		WARN_LOG(SCEGE, "sceGeRestoreContext(%08x): lists in process, aborting", ctxAddr);
+		WARN_LOG(Log::sceGe, "sceGeRestoreContext(%08x): lists in process, aborting", ctxAddr);
 		return SCE_KERNEL_ERROR_BUSY;
 	}
 
@@ -516,97 +519,86 @@ u32 sceGeRestoreContext(u32 ctxAddr) {
 	return 0;
 }
 
-static void __GeCopyMatrix(u32 matrixPtr, float *mtx, u32 size) {
-	for (u32 i = 0; i < size / sizeof(float); ++i) {
-		Memory::Write_U32(toFloat24(mtx[i]), matrixPtr + i * sizeof(float));
-	}
-}
-
 static int sceGeGetMtx(int type, u32 matrixPtr) {
-	if (!Memory::IsValidAddress(matrixPtr)) {
-		ERROR_LOG(SCEGE, "sceGeGetMtx(%d, %08x) - bad matrix ptr", type, matrixPtr);
-		return -1;
+	int size = type == GE_MTX_PROJECTION ? 16 : 12;
+	if (!Memory::IsValidRange(matrixPtr, size * sizeof(float))) {
+		return hleLogError(Log::sceGe, -1, "bad matrix ptr");
 	}
 
-	INFO_LOG(SCEGE, "sceGeGetMtx(%d, %08x)", type, matrixPtr);
-	switch (type) {
-	case GE_MTX_BONE0:
-	case GE_MTX_BONE1:
-	case GE_MTX_BONE2:
-	case GE_MTX_BONE3:
-	case GE_MTX_BONE4:
-	case GE_MTX_BONE5:
-	case GE_MTX_BONE6:
-	case GE_MTX_BONE7:
-		{
-			int n = type - GE_MTX_BONE0;
-			__GeCopyMatrix(matrixPtr, gstate.boneMatrix + n * 12, 12 * sizeof(float));
-		}
-		break;
-	case GE_MTX_TEXGEN:
-		__GeCopyMatrix(matrixPtr, gstate.tgenMatrix, 12 * sizeof(float));
-		break;
-	case GE_MTX_WORLD:
-		__GeCopyMatrix(matrixPtr, gstate.worldMatrix, 12 * sizeof(float));
-		break;
-	case GE_MTX_VIEW:
-		__GeCopyMatrix(matrixPtr, gstate.viewMatrix, 12 * sizeof(float));
-		break;
-	case GE_MTX_PROJECTION:
-		__GeCopyMatrix(matrixPtr, gstate.projMatrix, 16 * sizeof(float));
-		break;
-	default:
-		return SCE_KERNEL_ERROR_INVALID_INDEX;
-	}
-	return 0;
+	u32_le *dest = (u32_le *)Memory::GetPointerWriteUnchecked(matrixPtr);
+	// Note: this reads the CPU-visible matrix values, which may differ from the actual used values.
+	// They only differ when more DATA commands are sent than are valid for a matrix.
+	if (!gpu || !gpu->GetMatrix24(GEMatrixType(type), dest, 0))
+		return hleLogError(Log::sceGe, SCE_KERNEL_ERROR_INVALID_INDEX, "invalid matrix");
+
+	return hleLogSuccessInfoI(Log::sceGe, 0);
 }
 
 static u32 sceGeGetCmd(int cmd) {
-	INFO_LOG(SCEGE, "sceGeGetCmd(%i)", cmd);
 	if (cmd >= 0 && cmd < (int)ARRAY_SIZE(gstate.cmdmem)) {
-		return gstate.cmdmem[cmd];  // Does not mask away the high bits.
-	} else {
-		return SCE_KERNEL_ERROR_INVALID_INDEX;
+		// Does not mask away the high bits.  But matrix regs don't read back.
+		u32 val = gstate.cmdmem[cmd];
+		switch (cmd) {
+		case GE_CMD_BONEMATRIXDATA:
+		case GE_CMD_WORLDMATRIXDATA:
+		case GE_CMD_VIEWMATRIXDATA:
+		case GE_CMD_PROJMATRIXDATA:
+		case GE_CMD_TGENMATRIXDATA:
+			val &= 0xFF000000;
+			break;
+
+		case GE_CMD_BONEMATRIXNUMBER:
+			val &= 0xFF00007F;
+			break;
+
+		case GE_CMD_WORLDMATRIXNUMBER:
+		case GE_CMD_VIEWMATRIXNUMBER:
+		case GE_CMD_PROJMATRIXNUMBER:
+		case GE_CMD_TGENMATRIXNUMBER:
+			val &= 0xFF00000F;
+			break;
+
+		default:
+			break;
+		}
+		return hleLogSuccessInfoX(Log::sceGe, val);
 	}
+	return hleLogError(Log::sceGe, SCE_KERNEL_ERROR_INVALID_INDEX);
 }
 
 static int sceGeGetStack(int index, u32 stackPtr) {
-	WARN_LOG_REPORT(SCEGE, "sceGeGetStack(%i, %08x)", index, stackPtr);
+	WARN_LOG_REPORT(Log::sceGe, "sceGeGetStack(%i, %08x)", index, stackPtr);
 	return gpu->GetStack(index, stackPtr);
 }
 
-static u32 sceGeEdramSetAddrTranslation(int new_size) {
+static u32 sceGeEdramSetAddrTranslation(u32 new_size) {
 	bool outsideRange = new_size != 0 && (new_size < 0x200 || new_size > 0x1000);
 	bool notPowerOfTwo = (new_size & (new_size - 1)) != 0;
 	if (outsideRange || notPowerOfTwo) {
-		WARN_LOG(SCEGE, "sceGeEdramSetAddrTranslation(%i): invalid value", new_size);
-		return SCE_KERNEL_ERROR_INVALID_VALUE;
+		return hleLogWarning(Log::sceGe, SCE_KERNEL_ERROR_INVALID_VALUE, "invalid value");
+	}
+	if (!gpu) {
+		return hleLogError(Log::sceGe, -1, "GPUInterface not available");
 	}
 
-	DEBUG_LOG(SCEGE, "sceGeEdramSetAddrTranslation(%i)", new_size);
-
-	// TODO: This isn't safe. EDRamWidth should be global and saved.
-	static int EDRamWidth = 0x400;
-	int last = EDRamWidth;
-	EDRamWidth = new_size;
-	return last;
+	return hleReportDebug(Log::sceGe, gpu->SetAddrTranslation(new_size));
 }
 
 const HLEFunction sceGe_user[] = {
 	{0XE47E40E4, &WrapU_V<sceGeEdramGetAddr>,            "sceGeEdramGetAddr",            'x', ""    },
-	{0XAB49E76A, &WrapU_UUIU<sceGeListEnQueue>,          "sceGeListEnQueue",             'x', "xxix"},
-	{0X1C0D95A6, &WrapU_UUIU<sceGeListEnQueueHead>,      "sceGeListEnQueueHead",         'x', "xxix"},
+	{0XAB49E76A, &WrapU_UUIU<sceGeListEnQueue>,          "sceGeListEnQueue",             'x', "xxip"},
+	{0X1C0D95A6, &WrapU_UUIU<sceGeListEnQueueHead>,      "sceGeListEnQueueHead",         'x', "xxip"},
 	{0XE0D68148, &WrapI_UU<sceGeListUpdateStallAddr>,    "sceGeListUpdateStallAddr",     'i', "xx"  },
 	{0X03444EB4, &WrapI_UU<sceGeListSync>,               "sceGeListSync",                'i', "xx"  },
 	{0XB287BD61, &WrapU_U<sceGeDrawSync>,                "sceGeDrawSync",                'x', "x"   },
 	{0XB448EC0D, &WrapI_UU<sceGeBreak>,                  "sceGeBreak",                   'i', "xx"  },
 	{0X4C06E472, &WrapI_V<sceGeContinue>,                "sceGeContinue",                'i', ""    },
-	{0XA4FC06A4, &WrapU_U<sceGeSetCallback>,             "sceGeSetCallback",             'x', "x"   },
+	{0XA4FC06A4, &WrapU_U<sceGeSetCallback>,             "sceGeSetCallback",             'i', "p"   },
 	{0X05DB22CE, &WrapI_U<sceGeUnsetCallback>,           "sceGeUnsetCallback",           'i', "x"   },
 	{0X1F6752AD, &WrapU_V<sceGeEdramGetSize>,            "sceGeEdramGetSize",            'x', ""    },
-	{0XB77905EA, &WrapU_I<sceGeEdramSetAddrTranslation>, "sceGeEdramSetAddrTranslation", 'x', "i"   },
+	{0XB77905EA, &WrapU_U<sceGeEdramSetAddrTranslation>, "sceGeEdramSetAddrTranslation", 'x', "x"   },
 	{0XDC93CFEF, &WrapU_I<sceGeGetCmd>,                  "sceGeGetCmd",                  'x', "i"   },
-	{0X57C8945B, &WrapI_IU<sceGeGetMtx>,                 "sceGeGetMtx",                  'i', "ix"  },
+	{0X57C8945B, &WrapI_IU<sceGeGetMtx>,                 "sceGeGetMtx",                  'i', "ip"  },
 	{0X438A385A, &WrapU_U<sceGeSaveContext>,             "sceGeSaveContext",             'x', "x"   },
 	{0X0BF608FB, &WrapU_U<sceGeRestoreContext>,          "sceGeRestoreContext",          'x', "x"   },
 	{0X5FB86AB0, &WrapI_U<sceGeListDeQueue>,             "sceGeListDeQueue",             'i', "x"   },
