@@ -104,7 +104,7 @@ static u32 convertYCbCrToABGR(int y, int cb, int cr) {
 
 static int JpegCsc(u32 imageAddr, u32 yCbCrAddr, int widthHeight, int bufferWidth, uint32_t chroma, int &usec) {
 	if ((chroma & 0x000FFFFF) != 0x00020202 && (chroma & 0x000FFFFF) != 0x00020201 && (chroma & 0x000FFFFF) != 0x00020101)
-		return hleLogError(ME, ERROR_JPEG_INVALID_COLORSPACE, "invalid colorspace");
+		return hleLogError(Log::ME, ERROR_JPEG_INVALID_COLORSPACE, "invalid colorspace");
 	if (bufferWidth < 0)
 		bufferWidth = 0;
 
@@ -120,9 +120,9 @@ static int JpegCsc(u32 imageAddr, u32 yCbCrAddr, int widthHeight, int bufferWidt
 
 	uint64_t destSize = ((uint64_t)bufferWidth * (height - 1) + width) * 4;
 	if (destSize > 0x3FFFFFFF || !Memory::IsValidRange(imageAddr, (uint32_t)destSize))
-		return hleLogError(ME, ERROR_JPEG_INVALID_VALUE, "invalid dest address or size");
+		return hleLogError(Log::ME, ERROR_JPEG_INVALID_VALUE, "invalid dest address or size");
 	if (sizeY > 0x3FFFFFFF || !Memory::IsValidRange(yCbCrAddr, sizeY + sizeCb + sizeCb))
-		return hleLogError(ME, ERROR_JPEG_INVALID_VALUE, "invalid src address or size");
+		return hleLogError(Log::ME, ERROR_JPEG_INVALID_VALUE, "invalid src address or size");
 
 	u32_le *imageBuffer = (u32_le *)Memory::GetPointerWriteUnchecked(imageAddr);
 	const u8 *Y = (const u8 *)Memory::GetPointerUnchecked(yCbCrAddr);
@@ -164,8 +164,8 @@ static int JpegCsc(u32 imageAddr, u32 yCbCrAddr, int widthHeight, int bufferWidt
 	NotifyMemInfo(MemBlockFlags::WRITE, imageAddr, (uint32_t)destSize, "JpegCsc");
 
 	if ((widthHeight & 0xFFFF) == 0)
-		return hleLogSuccessI(ME, -1);
-	return hleLogSuccessI(ME, 0);
+		return hleLogSuccessI(Log::ME, -1);
+	return hleLogSuccessI(Log::ME, 0);
 }
 
 static int JpegMJpegCsc(u32 imageAddr, u32 yCbCrAddr, int widthHeight, int bufferWidth, int &usec) {
@@ -180,12 +180,12 @@ static int JpegMJpegCsc(u32 imageAddr, u32 yCbCrAddr, int widthHeight, int buffe
 	int sizeCb = sizeY >> 2;
 
 	if (width > 720 || height > 480)
-		return hleLogError(ME, ERROR_JPEG_INVALID_SIZE, "invalid size, max 720x480");
+		return hleLogError(Log::ME, ERROR_JPEG_INVALID_SIZE, "invalid size, max 720x480");
 	if (bufferWidth > 1024)
-		return hleLogError(ME, ERROR_JPEG_INVALID_SIZE, "invalid stride, max 1024");
+		return hleLogError(Log::ME, ERROR_JPEG_INVALID_SIZE, "invalid stride, max 1024");
 	uint32_t destSize = (bufferWidth * (height - 1) + width) * 4;
 	if (!Memory::IsValidRange(imageAddr, destSize))
-		return hleLogError(ME, SCE_KERNEL_ERROR_INVALID_POINTER, "invalid dest address or size");
+		return hleLogError(Log::ME, SCE_KERNEL_ERROR_INVALID_POINTER, "invalid dest address or size");
 
 	u32_le *imageBuffer = (u32_le *)Memory::GetPointerWriteUnchecked(imageAddr);
 	const u8 *Y = (const u8 *)Memory::GetPointerUnchecked(yCbCrAddr);
@@ -237,12 +237,12 @@ static int JpegMJpegCsc(u32 imageAddr, u32 yCbCrAddr, int widthHeight, int buffe
 
 	NotifyMemInfo(MemBlockFlags::WRITE, imageAddr, destSize, "JpegMJpegCsc");
 
-	return hleLogSuccessI(ME, 0);
+	return hleLogSuccessI(Log::ME, 0);
 }
 
 static int sceJpegMJpegCsc(u32 imageAddr, u32 yCbCrAddr, int widthHeight, int bufferWidth) {
 	if (mjpegInited == 0)
-		return hleLogError(ME, 0x80000001, "not yet inited");
+		return hleLogError(Log::ME, 0x80000001, "not yet inited");
 
 	int usec = 0;
 	int result = JpegMJpegCsc(imageAddr, yCbCrAddr, widthHeight, bufferWidth, usec);
@@ -263,15 +263,15 @@ static u32 convertARGBtoABGR(u32 argb) {
 
 static int DecodeJpeg(u32 jpegAddr, int jpegSize, u32 imageAddr, int &usec) {
 	if (!Memory::IsValidRange(jpegAddr, jpegSize))
-		return hleLogError(ME, ERROR_JPEG_NO_SOI, "invalid jpeg address");
+		return hleLogError(Log::ME, ERROR_JPEG_NO_SOI, "invalid jpeg address");
 	if (jpegSize == 0)
-		return hleLogError(ME, ERROR_JPEG_INVALID_DATA, "invalid jpeg data");
+		return hleLogError(Log::ME, ERROR_JPEG_INVALID_DATA, "invalid jpeg data");
 
 	NotifyMemInfo(MemBlockFlags::READ, jpegAddr, jpegSize, "JpegDecodeMJpeg");
 
 	const u8 *buf = Memory::GetPointerUnchecked(jpegAddr);
 	if (jpegSize < 2 || buf[0] != 0xFF || buf[1] != 0xD8)
-		return hleLogError(ME, ERROR_JPEG_NO_SOI, "no SOI found, invalid data");
+		return hleLogError(Log::ME, ERROR_JPEG_NO_SOI, "no SOI found, invalid data");
 
 	int width, height, actual_components;
 	unsigned char *jpegBuf = jpgd::decompress_jpeg_image_from_memory(buf, jpegSize, &width, &height, &actual_components, 3);
@@ -284,24 +284,24 @@ static int DecodeJpeg(u32 jpegAddr, int jpegSize, u32 imageAddr, int &usec) {
 	}
 
 	if (jpegBuf == nullptr) {
-		return hleLogError(ME, ERROR_JPEG_INVALID_DATA, "unable to decompress jpeg");
+		return hleLogError(Log::ME, ERROR_JPEG_INVALID_DATA, "unable to decompress jpeg");
 	}
 
 	usec += (width * height) / 14;
 
 	if (!Memory::IsValidRange(imageAddr, mjpegWidth * mjpegHeight * 4)) {
 		free(jpegBuf);
-		return hleLogError(ME, SCE_KERNEL_ERROR_INVALID_POINTER, "invalid output address");
+		return hleLogError(Log::ME, SCE_KERNEL_ERROR_INVALID_POINTER, "invalid output address");
 	}
 	// Note: even if you Delete, the size is still allowed.
 	if (width > mjpegWidth || height > mjpegHeight) {
 		free(jpegBuf);
-		return hleLogError(ME, ERROR_JPEG_INVALID_SIZE, "invalid output address");
+		return hleLogError(Log::ME, ERROR_JPEG_INVALID_SIZE, "invalid output address");
 	}
 	if (mjpegInited == 0) {
 		// If you finish after setting the size, then call this - you get an interesting error.
 		free(jpegBuf);
-		return hleLogError(ME, 0x80000001, "mjpeg not inited");
+		return hleLogError(Log::ME, 0x80000001, "mjpeg not inited");
 	}
 
 	usec += (width * height) / 110;
@@ -320,14 +320,14 @@ static int DecodeJpeg(u32 jpegAddr, int jpegSize, u32 imageAddr, int &usec) {
 	}
 
 	free(jpegBuf);
-	return hleLogSuccessX(ME, getWidthHeight(width, height));
+	return hleLogSuccessX(Log::ME, getWidthHeight(width, height));
 }
 
 static int sceJpegDecodeMJpeg(u32 jpegAddr, int jpegSize, u32 imageAddr, int dhtMode) {
 	if ((jpegAddr | jpegSize | (jpegAddr + jpegSize)) & 0x80000000)
-		return hleLogError(ME, SCE_KERNEL_ERROR_PRIV_REQUIRED, "invalid jpeg address");
+		return hleLogError(Log::ME, SCE_KERNEL_ERROR_PRIV_REQUIRED, "invalid jpeg address");
 	if (imageAddr & 0x80000000)
-		return hleLogError(ME, SCE_KERNEL_ERROR_PRIV_REQUIRED, "invalid output address");
+		return hleLogError(Log::ME, SCE_KERNEL_ERROR_PRIV_REQUIRED, "invalid output address");
 
 	int usec = 300;
 	int result = DecodeJpeg(jpegAddr, jpegSize, imageAddr, usec);
@@ -336,9 +336,9 @@ static int sceJpegDecodeMJpeg(u32 jpegAddr, int jpegSize, u32 imageAddr, int dht
 
 static int sceJpegDecodeMJpegSuccessively(u32 jpegAddr, int jpegSize, u32 imageAddr, int dhtMode) {
 	if ((jpegAddr | jpegSize | (jpegAddr + jpegSize)) & 0x80000000)
-		return hleLogError(ME, SCE_KERNEL_ERROR_PRIV_REQUIRED, "invalid jpeg address");
+		return hleLogError(Log::ME, SCE_KERNEL_ERROR_PRIV_REQUIRED, "invalid jpeg address");
 	if (imageAddr & 0x80000000)
-		return hleLogError(ME, SCE_KERNEL_ERROR_PRIV_REQUIRED, "invalid output address");
+		return hleLogError(Log::ME, SCE_KERNEL_ERROR_PRIV_REQUIRED, "invalid output address");
 
 	int usec = 300;
 	int result = DecodeJpeg(jpegAddr, jpegSize, imageAddr, usec);
@@ -360,15 +360,15 @@ static int getYCbCrBufferSize(int w, int h) {
 
 static int JpegGetOutputInfo(u32 jpegAddr, int jpegSize, u32 colourInfoAddr) {
 	if (!Memory::IsValidRange(jpegAddr, jpegSize))
-		return hleLogError(ME, ERROR_JPEG_NO_SOI, "invalid jpeg address");
+		return hleLogError(Log::ME, ERROR_JPEG_NO_SOI, "invalid jpeg address");
 	if (jpegSize == 0)
-		return hleLogError(ME, ERROR_JPEG_INVALID_DATA, "invalid jpeg data");
+		return hleLogError(Log::ME, ERROR_JPEG_INVALID_DATA, "invalid jpeg data");
 
 	NotifyMemInfo(MemBlockFlags::READ, jpegAddr, jpegSize, "JpegGetOutputInfo");
 
 	const u8 *buf = Memory::GetPointerUnchecked(jpegAddr);
 	if (jpegSize < 2 || buf[0] != 0xFF || buf[1] != 0xD8)
-		return hleLogError(ME, ERROR_JPEG_NO_SOI, "no SOI found, invalid data");
+		return hleLogError(Log::ME, ERROR_JPEG_NO_SOI, "no SOI found, invalid data");
 
 	int width, height, actual_components;
 	unsigned char *jpegBuf = jpgd::decompress_jpeg_image_from_memory(buf, jpegSize, &width, &height, &actual_components, 3);
@@ -381,7 +381,7 @@ static int JpegGetOutputInfo(u32 jpegAddr, int jpegSize, u32 colourInfoAddr) {
 	}
 
 	if (jpegBuf == nullptr) {
-		return hleLogError(ME, ERROR_JPEG_INVALID_DATA, "unable to decompress jpeg");
+		return hleLogError(Log::ME, ERROR_JPEG_INVALID_DATA, "unable to decompress jpeg");
 	}
 
 	free(jpegBuf);
@@ -411,12 +411,12 @@ static int JpegGetOutputInfo(u32 jpegAddr, int jpegSize, u32 colourInfoAddr) {
 		fclose(wfp);
 #endif //JPEG_DEBUG
 
-	return hleLogSuccessX(ME, getYCbCrBufferSize(width, height));
+	return hleLogSuccessX(Log::ME, getYCbCrBufferSize(width, height));
 }
 
 static int sceJpegGetOutputInfo(u32 jpegAddr, int jpegSize, u32 colourInfoAddr, int dhtMode) {
 	if ((jpegAddr | jpegSize | (jpegAddr + jpegSize)) & 0x80000000)
-		return hleLogError(ME, SCE_KERNEL_ERROR_PRIV_REQUIRED, "invalid jpeg address");
+		return hleLogError(Log::ME, SCE_KERNEL_ERROR_PRIV_REQUIRED, "invalid jpeg address");
 
 	int result = JpegGetOutputInfo(jpegAddr, jpegSize, colourInfoAddr);
 	// Time taken varies a bit, this is the low end.  Depends on data.
@@ -489,15 +489,15 @@ static int JpegConvertRGBToYCbCr(const void *data, u8 *output, int width, int he
 
 static int JpegDecodeMJpegYCbCr(u32 jpegAddr, int jpegSize, u32 yCbCrAddr, int yCbCrSize, int &usec) {
 	if (!Memory::IsValidRange(jpegAddr, jpegSize))
-		return hleLogError(ME, ERROR_JPEG_NO_SOI, "invalid jpeg address");
+		return hleLogError(Log::ME, ERROR_JPEG_NO_SOI, "invalid jpeg address");
 	if (jpegSize == 0)
-		return hleLogError(ME, ERROR_JPEG_INVALID_DATA, "invalid jpeg data");
+		return hleLogError(Log::ME, ERROR_JPEG_INVALID_DATA, "invalid jpeg data");
 
 	NotifyMemInfo(MemBlockFlags::READ, jpegAddr, jpegSize, "JpegDecodeMJpegYCbCr");
 
 	const u8 *buf = Memory::GetPointerUnchecked(jpegAddr);
 	if (jpegSize < 2 || buf[0] != 0xFF || buf[1] != 0xD8)
-		return hleLogError(ME, ERROR_JPEG_NO_SOI, "no SOI found, invalid data");
+		return hleLogError(Log::ME, ERROR_JPEG_NO_SOI, "no SOI found, invalid data");
 
 	int width, height, actual_components;
 	unsigned char *jpegBuf = jpgd::decompress_jpeg_image_from_memory(buf, jpegSize, &width, &height, &actual_components, 3);
@@ -510,12 +510,12 @@ static int JpegDecodeMJpegYCbCr(u32 jpegAddr, int jpegSize, u32 yCbCrAddr, int y
 	}
 
 	if (jpegBuf == nullptr) {
-		return hleLogError(ME, ERROR_JPEG_INVALID_DATA, "unable to decompress jpeg");
+		return hleLogError(Log::ME, ERROR_JPEG_INVALID_DATA, "unable to decompress jpeg");
 	}
 
 	if (yCbCrSize < getYCbCrBufferSize(width, height)) {
 		free(jpegBuf);
-		return hleLogError(ME, ERROR_JPEG_OUT_OF_MEMORY, "buffer not large enough");
+		return hleLogError(Log::ME, ERROR_JPEG_OUT_OF_MEMORY, "buffer not large enough");
 	}
 
 	// Technically, it seems like the PSP doesn't support grayscale, but we might as well.
@@ -525,7 +525,7 @@ static int JpegDecodeMJpegYCbCr(u32 jpegAddr, int jpegSize, u32 yCbCrAddr, int y
 			NotifyMemInfo(MemBlockFlags::WRITE, yCbCrAddr, getYCbCrBufferSize(width, height), "JpegDecodeMJpegYCbCr");
 		} else {
 			// There's some weird behavior on the PSP where it writes data around the last passed address.
-			WARN_LOG_REPORT(ME, "JpegDecodeMJpegYCbCr: Invalid output address (%08x / %08x) for %dx%d", yCbCrAddr, yCbCrSize, width, height);
+			WARN_LOG_REPORT(Log::ME, "JpegDecodeMJpegYCbCr: Invalid output address (%08x / %08x) for %dx%d", yCbCrAddr, yCbCrSize, width, height);
 		}
 	}
 
@@ -533,16 +533,16 @@ static int JpegDecodeMJpegYCbCr(u32 jpegAddr, int jpegSize, u32 yCbCrAddr, int y
 
 	// Rough estimate based on observed timing.
 	usec += (width * height) / 14;
-	return hleLogSuccessX(ME, getWidthHeight(width, height));
+	return hleLogSuccessX(Log::ME, getWidthHeight(width, height));
 }
 
 static int sceJpegDecodeMJpegYCbCr(u32 jpegAddr, int jpegSize, u32 yCbCrAddr, int yCbCrSize, int dhtMode) {
 	if ((jpegAddr | jpegSize | (jpegAddr + jpegSize)) & 0x80000000)
-		return hleLogError(ME, SCE_KERNEL_ERROR_PRIV_REQUIRED, "invalid jpeg address");
+		return hleLogError(Log::ME, SCE_KERNEL_ERROR_PRIV_REQUIRED, "invalid jpeg address");
 	if ((yCbCrAddr | yCbCrSize | (yCbCrAddr + yCbCrSize)) & 0x80000000)
-		return hleLogError(ME, SCE_KERNEL_ERROR_PRIV_REQUIRED, "invalid output address");
+		return hleLogError(Log::ME, SCE_KERNEL_ERROR_PRIV_REQUIRED, "invalid output address");
 	if (!Memory::IsValidRange(jpegAddr, jpegSize))
-		return hleLogError(ME, ERROR_JPEG_INVALID_VALUE, "invalid jpeg address");
+		return hleLogError(Log::ME, ERROR_JPEG_INVALID_VALUE, "invalid jpeg address");
 	
 	int usec = 300;
 	int result = JpegDecodeMJpegYCbCr(jpegAddr, jpegSize, yCbCrAddr, yCbCrSize, usec);
@@ -551,9 +551,9 @@ static int sceJpegDecodeMJpegYCbCr(u32 jpegAddr, int jpegSize, u32 yCbCrAddr, in
 
 static int sceJpegDecodeMJpegYCbCrSuccessively(u32 jpegAddr, int jpegSize, u32 yCbCrAddr, int yCbCrSize, int dhtMode) {
 	if ((jpegAddr | jpegSize | (jpegAddr + jpegSize)) & 0x80000000)
-		return hleLogError(ME, SCE_KERNEL_ERROR_PRIV_REQUIRED, "invalid jpeg address");
+		return hleLogError(Log::ME, SCE_KERNEL_ERROR_PRIV_REQUIRED, "invalid jpeg address");
 	if ((yCbCrAddr | yCbCrSize | (yCbCrAddr + yCbCrSize)) & 0x80000000)
-		return hleLogError(ME, SCE_KERNEL_ERROR_PRIV_REQUIRED, "invalid output address");
+		return hleLogError(Log::ME, SCE_KERNEL_ERROR_PRIV_REQUIRED, "invalid output address");
 	
 	// Do as same way as sceJpegDecodeMJpegYCbCr() but with smaller block size
 	int usec = 300;
@@ -562,63 +562,63 @@ static int sceJpegDecodeMJpegYCbCrSuccessively(u32 jpegAddr, int jpegSize, u32 y
 }
 
 static int sceJpeg_9B36444C() {
-	ERROR_LOG_REPORT(ME, "UNIMPL sceJpeg_9B36444C()");
+	ERROR_LOG_REPORT(Log::ME, "UNIMPL sceJpeg_9B36444C()");
 	return 0;
 }
 
 static int sceJpegCreateMJpeg(int width, int height) {
 	if (mjpegInited == 0)
-		return hleLogError(ME, ERROR_JPEG_INVALID_STATE, "not yet inited");
+		return hleLogError(Log::ME, ERROR_JPEG_INVALID_STATE, "not yet inited");
 	if (mjpegInited == 2)
-		return hleLogError(ME, ERROR_JPEG_INVALID_STATE, "already created");
+		return hleLogError(Log::ME, ERROR_JPEG_INVALID_STATE, "already created");
 	if (width > 1024)
-		return hleLogError(ME, ERROR_JPEG_INVALID_SIZE, "width outside bounds");
+		return hleLogError(Log::ME, ERROR_JPEG_INVALID_SIZE, "width outside bounds");
 
 	mjpegInited = 2;
 	mjpegWidth = width;
 	mjpegHeight = height;
 
-	return hleLogSuccessInfoI(ME, 0);
+	return hleLogSuccessInfoI(Log::ME, 0);
 }
 
 static int sceJpegDeleteMJpeg() {
 	if (mjpegInited == 0)
-		return hleLogError(ME, ERROR_JPEG_INVALID_STATE, "not yet inited");
+		return hleLogError(Log::ME, ERROR_JPEG_INVALID_STATE, "not yet inited");
 	if (mjpegInited == 1)
-		return hleLogError(ME, ERROR_JPEG_INVALID_STATE, "not yet created");
+		return hleLogError(Log::ME, ERROR_JPEG_INVALID_STATE, "not yet created");
 
 	mjpegInited = 1;
-	return hleLogSuccessInfoI(ME, 0);
+	return hleLogSuccessInfoI(Log::ME, 0);
 }
 
 static int sceJpegInitMJpeg() {
 	if (mjpegInited == 1 || mjpegInited == 2)
-		return hleLogError(ME, ERROR_JPEG_ALREADY_INIT, "already inited");
+		return hleLogError(Log::ME, ERROR_JPEG_ALREADY_INIT, "already inited");
 
 	// If it was -1, it's from an old save state, avoid double init error but assume inited.
 	if (mjpegInited == 0)
 		mjpegInited = 1;
-	return hleLogSuccessI(ME, hleDelayResult(0, "mjpeg init", 130));
+	return hleLogSuccessI(Log::ME, hleDelayResult(0, "mjpeg init", 130));
 }
 
 static int sceJpegFinishMJpeg() {
 	if (mjpegInited == 0)
-		return hleLogError(ME, ERROR_JPEG_INVALID_STATE, "already inited");
+		return hleLogError(Log::ME, ERROR_JPEG_INVALID_STATE, "already inited");
 	if (mjpegInited == 2)
-		return hleLogError(ME, ERROR_JPEG_INVALID_STATE, "mjpeg not deleted");
+		return hleLogError(Log::ME, ERROR_JPEG_INVALID_STATE, "mjpeg not deleted");
 
 	// Even from an old save state, if we see this we leave compat mode.
 	mjpegInited = 0;
-	return hleLogSuccessI(ME, hleDelayResult(0, "mjpeg finish", 120));
+	return hleLogSuccessI(Log::ME, hleDelayResult(0, "mjpeg finish", 120));
 }
 
 static int sceJpegMJpegCscWithColorOption() {
-	ERROR_LOG_REPORT(ME, "UNIMPL sceJpegMJpegCscWithColorOption()");
+	ERROR_LOG_REPORT(Log::ME, "UNIMPL sceJpegMJpegCscWithColorOption()");
 	return 0;
 }
 
 static int sceJpegDecompressAllImage() {
-	ERROR_LOG_REPORT(ME, "UNIMPL sceJpegDecompressAllImage()");
+	ERROR_LOG_REPORT(Log::ME, "UNIMPL sceJpegDecompressAllImage()");
 	return 0;
 }
 
