@@ -42,15 +42,15 @@ void __VaudioDoState(PointerWrap &p) {
 
 static u32 sceVaudioChReserve(int sampleCount, int freq, int format) {
 	if (vaudioReserved) {
-		ERROR_LOG(SCEAUDIO, "sceVaudioChReserve(%i, %i, %i) - already reserved", sampleCount, freq, format);
+		ERROR_LOG(Log::sceAudio, "sceVaudioChReserve(%i, %i, %i) - already reserved", sampleCount, freq, format);
 		return SCE_KERNEL_ERROR_BUSY;
 	}
 	// We still have to check the channel also, which gives a different error.
 	if (chans[PSP_AUDIO_CHANNEL_VAUDIO].reserved) {
-		ERROR_LOG(SCEAUDIO, "sceVaudioChReserve(%i, %i, %i) - channel already reserved", sampleCount, freq, format);
+		ERROR_LOG(Log::sceAudio, "sceVaudioChReserve(%i, %i, %i) - channel already reserved", sampleCount, freq, format);
 		return SCE_ERROR_AUDIO_CHANNEL_ALREADY_RESERVED;
 	}
-	DEBUG_LOG(SCEAUDIO, "sceVaudioChReserve(%i, %i, %i)", sampleCount, freq, format);
+	DEBUG_LOG(Log::sceAudio, "sceVaudioChReserve(%i, %i, %i)", sampleCount, freq, format);
 	chans[PSP_AUDIO_CHANNEL_VAUDIO].reserved = true;
 	chans[PSP_AUDIO_CHANNEL_VAUDIO].sampleCount = sampleCount;
 	chans[PSP_AUDIO_CHANNEL_VAUDIO].format = format == 2 ? PSP_AUDIO_FORMAT_STEREO : PSP_AUDIO_FORMAT_MONO;
@@ -62,7 +62,7 @@ static u32 sceVaudioChReserve(int sampleCount, int freq, int format) {
 }
 
 static u32 sceVaudioChRelease() {
-	DEBUG_LOG(SCEAUDIO, "sceVaudioChRelease(...)");
+	DEBUG_LOG(Log::sceAudio, "sceVaudioChRelease(...)");
 	if (!chans[PSP_AUDIO_CHANNEL_VAUDIO].reserved) {
 		return SCE_ERROR_AUDIO_CHANNEL_NOT_RESERVED;
 	} else {
@@ -74,7 +74,7 @@ static u32 sceVaudioChRelease() {
 }
 
 static u32 sceVaudioOutputBlocking(int vol, u32 buffer) {
-	DEBUG_LOG(SCEAUDIO, "sceVaudioOutputBlocking(%i, %08x)", vol, buffer);
+	DEBUG_LOG(Log::sceAudio, "sceVaudioOutputBlocking(%i, %08x)", vol, buffer);
 	chans[PSP_AUDIO_CHANNEL_VAUDIO].leftVolume = vol;
 	chans[PSP_AUDIO_CHANNEL_VAUDIO].rightVolume = vol;
 	// TODO: This may be wrong, not sure if's in a different format?
@@ -83,12 +83,12 @@ static u32 sceVaudioOutputBlocking(int vol, u32 buffer) {
 }
 
 static u32 sceVaudioSetEffectType(int effectType, int vol) {
-	ERROR_LOG_REPORT(SCEAUDIO, "UNIMPL sceVaudioSetEffectType(%i, %i)", effectType, vol);
+	ERROR_LOG_REPORT(Log::sceAudio, "UNIMPL sceVaudioSetEffectType(%i, %i)", effectType, vol);
 	return 0;
 }
 
 static u32 sceVaudioSetAlcMode(int alcMode) {
-	ERROR_LOG_REPORT(SCEAUDIO, "UNIMPL sceVaudioSetAlcMode(%i)", alcMode);
+	ERROR_LOG_REPORT(Log::sceAudio, "UNIMPL sceVaudioSetAlcMode(%i)", alcMode);
 	return 0;
 }
 
