@@ -16,7 +16,9 @@
 // https://github.com/hrydgard/ppsspp and http://www.ppsspp.org/.
 
 #include "Common/Serialize/SerializeFuncs.h"
+#include "Common/Math/CrossSIMD.h"
 #include "Core/Config.h"
+#include "Core/Core.h"
 #include "Core/Debugger/MemBlockInfo.h"
 #include "Core/HW/MediaEngine.h"
 #include "Core/MemMap.h"
@@ -29,6 +31,18 @@
 
 #include <algorithm>
 
+#ifdef _M_SSE
+#include <emmintrin.h>
+#endif
+
+#if PPSSPP_ARCH(ARM_NEON)
+#if defined(_MSC_VER) && PPSSPP_ARCH(ARM64)
+#include <arm64_neon.h>
+#else
+#include <arm_neon.h>
+#endif
+#endif
+
 #ifdef USE_FFMPEG
 
 extern "C" {
@@ -42,6 +56,9 @@ extern "C" {
 #endif // USE_FFMPEG
 
 #ifdef USE_FFMPEG
+
+#include "Core/FFMPEGCompat.h"
+
 static AVPixelFormat getSwsFormat(int pspFormat)
 {
 	switch (pspFormat)
@@ -55,7 +72,7 @@ static AVPixelFormat getSwsFormat(int pspFormat)
 	case GE_CMODE_32BIT_ABGR8888:
 		return AV_PIX_FMT_RGBA;
 	default:
-		ERROR_LOG(ME, "Unknown pixel format");
+		ERROR_LOG(Log::ME, "Unknown pixel format");
 		return (AVPixelFormat)0;
 	}
 }
@@ -80,11 +97,11 @@ void ffmpeg_logger(void *, int level, const char *format, va_list va_args) {
 
 	// Let's color the log line appropriately.
 	if (level <= AV_LOG_PANIC) {
-		ERROR_LOG(ME, "FF: %s", tmp);
+		ERROR_LOG(Log::ME, "FF: %s", tmp);
 	} else if (level >= AV_LOG_VERBOSE) {
-		DEBUG_LOG(ME, "FF: %s", tmp);
+		DEBUG_LOG(Log::ME, "FF: %s", tmp);
 	} else {
-		INFO_LOG(ME, "FF: %s", tmp);
+		INFO_LOG(Log::ME, "FF: %s", tmp);
 	}
 }
 
@@ -112,7 +129,7 @@ static int getPixelFormatBytes(int pspFormat)
 		return 4;
 
 	default:
-		ERROR_LOG(ME, "Unknown pixel format");
+		ERROR_LOG(Log::ME, "Unknown pixel format");
 		return 4;
 	}
 }
@@ -130,10 +147,8 @@ MediaEngine::~MediaEngine() {
 
 void MediaEngine::closeMedia() {
 	closeContext();
-	if (m_pdata)
-		delete m_pdata;
-	if (m_demux)
-		delete m_demux;
+	delete m_pdata;
+	delete m_demux;
 	m_pdata = nullptr;
 	m_demux = nullptr;
 	AudioClose(&m_audioContext);
@@ -229,13 +244,13 @@ bool MediaEngine::SetupStreams() {
 #ifdef USE_FFMPEG
 	const u32 magic = *(u32_le *)&m_mpegheader[0];
 	if (magic != PSMF_MAGIC) {
-		WARN_LOG_REPORT(ME, "Could not setup streams, bad magic: %08x", magic);
+		WARN_LOG_REPORT(Log::ME, "Could not setup streams, bad magic: %08x", magic);
 		return false;
 	}
 	int numStreams = *(u16_be *)&m_mpegheader[0x80];
 	if (numStreams <= 0 || numStreams > 8) {
 		// Looks crazy.  Let's bail out and let FFmpeg handle it.
-		WARN_LOG_REPORT(ME, "Could not setup streams, unexpected stream count: %d", numStreams);
+		WARN_LOG_REPORT(Log::ME, "Could not setup streams, unexpected stream count: %d", numStreams);
 		return false;
 	}
 
@@ -287,18 +302,20 @@ bool MediaEngine::openContext(bool keepReadPos) {
 	}
 	av_dict_free(&open_opt);
 
-	if (!SetupStreams()) {
+	bool usedFFMPEGFindStreamInfo = false;
+	if (!SetupStreams() || PSP_CoreParameter().compat.flags().UseFFMPEGFindStreamInfo) {
 		// Fallback to old behavior.  Reads too much and corrupts when game doesn't read fast enough.
 		// SetupStreams sometimes work for newer FFmpeg 3.1+ now, but sometimes framerate is missing.
-		WARN_LOG_REPORT_ONCE(setupStreams, ME, "Failed to read valid video stream data from header");
+		WARN_LOG_REPORT_ONCE(setupStreams, Log::ME, "Failed to read valid video stream data from header");
 		if (avformat_find_stream_info(m_pFormatCtx, nullptr) < 0) {
 			closeContext();
 			return false;
 		}
+		usedFFMPEGFindStreamInfo = true;
 	}
 
 	if (m_videoStream >= (int)m_pFormatCtx->nb_streams) {
-		WARN_LOG_REPORT(ME, "Bad video stream %d", m_videoStream);
+		WARN_LOG_REPORT(Log::ME, "Bad video stream %d", m_videoStream);
 		m_videoStream = -1;
 	}
 
@@ -324,8 +341,13 @@ bool MediaEngine::openContext(bool keepReadPos) {
 		return false;
 
 	setVideoDim();
-	m_audioContext = new SimpleAudio(m_audioType, 44100, 2);
+	m_audioContext = CreateAudioDecoder((PSPAudioType)m_audioType);
 	m_isVideoEnd = false;
+
+	if (PSP_CoreParameter().compat.flags().UseFFMPEGFindStreamInfo && usedFFMPEGFindStreamInfo) {
+		m_mpegheaderReadPos++;
+		av_seek_frame(m_pFormatCtx, m_videoStream, 0, 0);
+	}
 #endif // USE_FFMPEG
 	return true;
 }
@@ -394,7 +416,7 @@ bool MediaEngine::addVideoStream(int streamNum, int streamId) {
 		// no need to add an existing stream.
 		if ((u32)streamNum < m_pFormatCtx->nb_streams)
 			return true;
-		const AVCodec *h264_codec = avcodec_find_decoder(AV_CODEC_ID_H264);
+		AVCodec *h264_codec = avcodec_find_decoder(AV_CODEC_ID_H264);
 		if (!h264_codec)
 			return false;
 		AVStream *stream = avformat_new_stream(m_pFormatCtx, h264_codec);
@@ -409,14 +431,20 @@ bool MediaEngine::addVideoStream(int streamNum, int streamId) {
 			stream->codecpar->codec_id = AV_CODEC_ID_H264;
 #else
 			stream->request_probe = 0;
-#endif
 			stream->need_parsing = AVSTREAM_PARSE_FULL;
+#endif
 			// We could set the width here, but we don't need to.
 			if (streamNum >= m_expectedVideoStreams) {
 				++m_expectedVideoStreams;
 			}
 
-			m_codecsToClose.push_back(stream->codec);
+#if LIBAVFORMAT_VERSION_INT >= AV_VERSION_INT(59, 16, 100)
+			AVCodec *codec = avcodec_find_decoder(stream->codecpar->codec_id);
+			AVCodecContext *codecCtx = avcodec_alloc_context3(codec);
+#else
+			AVCodecContext *codecCtx = stream->codec;
+#endif
+			m_codecsToClose.push_back(codecCtx);
 			return true;
 		}
 	}
@@ -501,13 +529,13 @@ bool MediaEngine::setVideoStream(int streamNum, bool force) {
 #if LIBAVFORMAT_VERSION_INT >= AV_VERSION_INT(57, 33, 100)
 		AVCodec *pCodec = avcodec_find_decoder(stream->codecpar->codec_id);
 		if (!pCodec) {
-			WARN_LOG_REPORT(ME, "Could not find decoder for %d", (int)stream->codecpar->codec_id);
+			WARN_LOG_REPORT(Log::ME, "Could not find decoder for %d", (int)stream->codecpar->codec_id);
 			return false;
 		}
 		AVCodecContext *m_pCodecCtx = avcodec_alloc_context3(pCodec);
 		int paramResult = avcodec_parameters_to_context(m_pCodecCtx, stream->codecpar);
 		if (paramResult < 0) {
-			WARN_LOG_REPORT(ME, "Failed to prepare context parameters: %08x", paramResult);
+			WARN_LOG_REPORT(Log::ME, "Failed to prepare context parameters: %08x", paramResult);
 			return false;
 		}
 #else
@@ -744,9 +772,35 @@ inline void writeVideoLineRGBA(void *destp, const void *srcp, int width) {
 	u32_le *dest = (u32_le *)destp;
 	const u32_le *src = (u32_le *)srcp;
 
-	const u32 mask = 0x00FFFFFF;
-	for (int i = 0; i < width; ++i) {
-		dest[i] = src[i] & mask;
+	int count = width;
+
+#if PPSSPP_ARCH(SSE2)
+	__m128i mask = _mm_set1_epi32(0x00FFFFFF);
+	while (count >= 8) {
+		__m128i pixels1 = _mm_and_si128(_mm_loadu_si128((const __m128i *)src), mask);
+		__m128i pixels2 = _mm_and_si128(_mm_loadu_si128((const __m128i *)src + 1), mask);
+		_mm_storeu_si128((__m128i *)dest, pixels1);
+		_mm_storeu_si128((__m128i *)dest + 1, pixels2);
+		src += 8;
+		dest += 8;
+		count -= 8;
+	}
+#elif PPSSPP_ARCH(ARM_NEON)
+	uint32x4_t mask = vdupq_n_u32(0x00FFFFFF);
+	while (count >= 8) {
+		uint32x4_t pixels1 = vandq_u32(vld1q_u32(src), mask);
+		uint32x4_t pixels2 = vandq_u32(vld1q_u32(src + 4), mask);
+		vst1q_u32(dest, pixels1);
+		vst1q_u32(dest + 4, pixels2);
+		src += 8;
+		dest += 8;
+		count -= 8;
+	}
+#endif
+	const u32 mask32 = 0x00FFFFFF;
+	DO_NOT_VECTORIZE_LOOP
+	while (count--) {
+		*dest++ = *src++ & mask32;
 	}
 }
 
@@ -793,7 +847,7 @@ int MediaEngine::writeVideoImage(u32 bufferPtr, int frameWidth, int videoPixelMo
 
 	if (!Memory::IsValidRange(bufferPtr, videoImageSize) || frameWidth > 2048) {
 		// Clearly invalid values.  Let's just not.
-		ERROR_LOG_REPORT(ME, "Ignoring invalid video decode address %08x/%x", bufferPtr, frameWidth);
+		ERROR_LOG_REPORT(Log::ME, "Ignoring invalid video decode address %08x/%x", bufferPtr, frameWidth);
 		return 0;
 	}
 
@@ -844,7 +898,7 @@ int MediaEngine::writeVideoImage(u32 bufferPtr, int frameWidth, int videoPixelMo
 		break;
 
 	default:
-		ERROR_LOG_REPORT(ME, "Unsupported video pixel format %d", videoPixelMode);
+		ERROR_LOG_REPORT(Log::ME, "Unsupported video pixel format %d", videoPixelMode);
 		break;
 	}
 
@@ -882,7 +936,7 @@ int MediaEngine::writeVideoImageWithRange(u32 bufferPtr, int frameWidth, int vid
 
 	if (!Memory::IsValidRange(bufferPtr, videoImageSize) || frameWidth > 2048) {
 		// Clearly invalid values.  Let's just not.
-		ERROR_LOG_REPORT(ME, "Ignoring invalid video decode address %08x/%x", bufferPtr, frameWidth);
+		ERROR_LOG_REPORT(Log::ME, "Ignoring invalid video decode address %08x/%x", bufferPtr, frameWidth);
 		return 0;
 	}
 
@@ -944,12 +998,12 @@ int MediaEngine::writeVideoImageWithRange(u32 bufferPtr, int frameWidth, int vid
 		break;
 
 	default:
-		ERROR_LOG_REPORT(ME, "Unsupported video pixel format %d", videoPixelMode);
+		ERROR_LOG_REPORT(Log::ME, "Unsupported video pixel format %d", videoPixelMode);
 		break;
 	}
 
 	if (swizzle) {
-		WARN_LOG_REPORT_ONCE(vidswizzle, ME, "Swizzling Video with range");
+		WARN_LOG_REPORT_ONCE(vidswizzle, Log::ME, "Swizzling Video with range");
 
 		const int bxc = videoLineSize / 16;
 		int byc = (height + 7) / 8;
@@ -1006,9 +1060,9 @@ int MediaEngine::getNextAudioFrame(u8 **buf, int *headerCode1, int *headerCode2)
 }
 
 int MediaEngine::getAudioSamples(u32 bufferPtr) {
-	u8 *buffer = Memory::GetPointerWriteRange(bufferPtr, 8192);
+	int16_t *buffer = (int16_t *)Memory::GetPointerWriteRange(bufferPtr, 8192);
 	if (buffer == nullptr) {
-		ERROR_LOG_REPORT(ME, "Ignoring bad audio decode address %08x during video playback", bufferPtr);
+		ERROR_LOG_REPORT(Log::ME, "Ignoring bad audio decode address %08x during video playback", bufferPtr);
 	}
 	if (!m_demux) {
 		return 0;
@@ -1020,7 +1074,7 @@ int MediaEngine::getAudioSamples(u32 bufferPtr) {
 	if (frameSize == 0) {
 		return 0;
 	}
-	int outbytes = 0;
+	int outSamples = 0;
 
 	if (m_audioContext != nullptr) {
 		if (headerCode1 == 0x24) {
@@ -1029,11 +1083,13 @@ int MediaEngine::getAudioSamples(u32 bufferPtr) {
 			m_audioContext->SetChannels(1);
 		}
 
-		if (!m_audioContext->Decode(audioFrame, frameSize, buffer, &outbytes)) {
-			ERROR_LOG(ME, "Audio (%s) decode failed during video playback", GetCodecName(m_audioType));
+		int inbytesConsumed = 0;
+		if (!m_audioContext->Decode(audioFrame, frameSize, &inbytesConsumed, 2, buffer, &outSamples)) {
+			ERROR_LOG(Log::ME, "Audio (%s) decode failed during video playback", GetCodecName(m_audioType));
 		}
+		int outBytes = outSamples * sizeof(int16_t) * 2;
 
-		NotifyMemInfo(MemBlockFlags::WRITE, bufferPtr, outbytes, "VideoDecodeAudio");
+		NotifyMemInfo(MemBlockFlags::WRITE, bufferPtr, outBytes, "VideoDecodeAudio");
 	}
 
 	return 0x2000;
