@@ -56,7 +56,7 @@ static const bool disasm = false;
 extern volatile CoreState coreState;
 
 void ShowPC(u32 sp) {
-	ERROR_LOG(JIT, "ShowPC : %08x  ArmSP : %08x", currentMIPS->pc, sp);
+	ERROR_LOG(Log::JIT, "ShowPC : %08x  ArmSP : %08x", currentMIPS->pc, sp);
 	// Sleep(1);
 }
 
@@ -127,7 +127,7 @@ void ArmJit::GenerateFixedCode() {
 
 	enterDispatcher = AlignCode16();
 
-	DEBUG_LOG(JIT, "Base: %08x", (u32)Memory::base);
+	DEBUG_LOG(Log::JIT, "Base: %08x", (u32)Memory::base);
 
 	SetCC(CC_AL);
 
@@ -256,9 +256,9 @@ void ArmJit::GenerateFixedCode() {
 
 	// Uncomment if you want to see the output...
 	if (disasm) {
-		INFO_LOG(JIT, "THE DISASM ========================");
+		INFO_LOG(Log::JIT, "THE DISASM ========================");
 		DisassembleArm(start, GetCodePtr() - start);
-		INFO_LOG(JIT, "END OF THE DISASM ========================");
+		INFO_LOG(Log::JIT, "END OF THE DISASM ========================");
 	}
 
 	// Don't forget to zap the instruction cache!
