@@ -2,6 +2,7 @@
 #include <cmath>
 
 #include "ppsspp_config.h"
+#include "Common/BitSet.h"
 #include "Common/BitScan.h"
 #include "Common/Common.h"
 #include "Common/Data/Convert/SmallDataConvert.h"
@@ -31,6 +32,7 @@
 #include "Core/MIPS/IR/IRInst.h"
 #include "Core/MIPS/IR/IRInterpreter.h"
 #include "Core/System.h"
+#include "Core/MIPS/MIPSTracer.h"
 
 #ifdef mips
 // Why do MIPS compilers define something so generic?  Try to keep defined, at least...
@@ -88,33 +90,68 @@ u32 IRRunMemCheck(u32 pc, u32 addr) {
 	return coreState != CORE_RUNNING ? 1 : 0;
 }
 
-template <uint32_t alignment>
-u32 RunValidateAddress(u32 pc, u32 addr, u32 isWrite) {
-	const auto toss = [&](MemoryExceptionType t) {
-		Core_MemoryException(addr, alignment, pc, t);
-		return coreState != CORE_RUNNING ? 1 : 0;
-	};
+void IRApplyRounding(MIPSState *mips) {
+	u32 fcr1Bits = mips->fcr31 & 0x01000003;
+	// If these are 0, we just leave things as they are.
+	if (fcr1Bits) {
+		int rmode = fcr1Bits & 3;
+		bool ftz = (fcr1Bits & 0x01000000) != 0;
+#if PPSSPP_ARCH(SSE2)
+		u32 csr = _mm_getcsr() & ~0x6000;
+		// Translate the rounding mode bits to X86, the same way as in Asm.cpp.
+		if (rmode & 1) {
+			rmode ^= 2;
+		}
+		csr |= rmode << 13;
 
-	if (!Memory::IsValidRange(addr, alignment)) {
-		MemoryExceptionType t = isWrite == 1 ? MemoryExceptionType::WRITE_WORD : MemoryExceptionType::READ_WORD;
-		if constexpr (alignment > 4)
-			t = isWrite ? MemoryExceptionType::WRITE_BLOCK : MemoryExceptionType::READ_BLOCK;
-		return toss(t);
+		if (ftz) {
+			// Flush to zero
+			csr |= 0x8000;
+		}
+		_mm_setcsr(csr);
+#elif PPSSPP_ARCH(ARM64) && !PPSSPP_PLATFORM(WINDOWS)
+		// On ARM64 we need to use inline assembly for a portable solution.
+		// Unfortunately we don't have this possibility on Windows with MSVC, so ifdeffed out above.
+		// Note that in the JIT, for fcvts, we use specific conversions. We could use the FCVTS variants
+		// directly through inline assembly.
+		u64 fpcr;  // not really 64-bit, just to match the register size.
+		asm volatile ("mrs %0, fpcr" : "=r" (fpcr));
+
+		// Translate MIPS to ARM rounding mode
+		static const u8 lookup[4] = {0, 3, 1, 2};
+
+		fpcr &= ~(3 << 22);    // Clear bits [23:22]
+		fpcr |= (lookup[rmode] << 22);
+
+		if (ftz) {
+			fpcr |= 1 << 24;
+		}
+		// Write back the modified FPCR
+		asm volatile ("msr fpcr, %0" : : "r" (fpcr));
+#endif
 	}
-	if constexpr (alignment > 1)
-		if ((addr & (alignment - 1)) != 0)
-			return toss(MemoryExceptionType::ALIGNMENT);
-	return 0;
+}
+
+void IRRestoreRounding() {
+#if PPSSPP_ARCH(SSE2)
+	// TODO: We should avoid this if we didn't apply rounding in the first place.
+	// In the meantime, clear out FTZ and rounding mode bits.
+	u32 csr = _mm_getcsr();
+	csr &= ~(7 << 13);
+	_mm_setcsr(csr);
+#elif PPSSPP_ARCH(ARM64) && !PPSSPP_PLATFORM(WINDOWS)
+	u64 fpcr;  // not really 64-bit, just to match the regsiter size.
+	asm volatile ("mrs %0, fpcr" : "=r" (fpcr));
+	fpcr &= ~(7 << 22);    // Clear bits [23:22] for rounding, 24 for FTZ
+	// Write back the modified FPCR
+	asm volatile ("msr fpcr, %0" : : "r" (fpcr));
+#endif
 }
 
 // We cannot use NEON on ARM32 here until we make it a hard dependency. We can, however, on ARM64.
-u32 IRInterpret(MIPSState *mips, const IRInst *inst, int count) {
-	const IRInst *end = inst + count;
-	while (inst != end) {
+u32 IRInterpret(MIPSState *mips, const IRInst *inst) {
+	while (true) {
 		switch (inst->op) {
-		case IROp::Nop:
-			_assert_(false);
-			break;
 		case IROp::SetConst:
 			mips->r[inst->dest] = inst->constant;
 			break;
@@ -142,20 +179,29 @@ u32 IRInterpret(MIPSState *mips, const IRInst *inst, int count) {
 		case IROp::AddConst:
 			mips->r[inst->dest] = mips->r[inst->src1] + inst->constant;
 			break;
+		case IROp::OptAddConst:  // For this one, it's worth having a "unary" variant of the above that only needs to read one register param.
+			mips->r[inst->dest] += inst->constant;
+			break;
 		case IROp::SubConst:
 			mips->r[inst->dest] = mips->r[inst->src1] - inst->constant;
 			break;
 		case IROp::AndConst:
 			mips->r[inst->dest] = mips->r[inst->src1] & inst->constant;
 			break;
+		case IROp::OptAndConst:  // For this one, it's worth having a "unary" variant of the above that only needs to read one register param.
+			mips->r[inst->dest] &= inst->constant;
+			break;
 		case IROp::OrConst:
 			mips->r[inst->dest] = mips->r[inst->src1] | inst->constant;
 			break;
+		case IROp::OptOrConst:
+			mips->r[inst->dest] |= inst->constant;
+			break;
 		case IROp::XorConst:
 			mips->r[inst->dest] = mips->r[inst->src1] ^ inst->constant;
 			break;
 		case IROp::Neg:
-			mips->r[inst->dest] = -(s32)mips->r[inst->src1];
+			mips->r[inst->dest] = (u32)(-(s32)mips->r[inst->src1]);
 			break;
 		case IROp::Not:
 			mips->r[inst->dest] = ~mips->r[inst->src1];
@@ -170,31 +216,6 @@ u32 IRInterpret(MIPSState *mips, const IRInst *inst, int count) {
 			mips->r[inst->dest] = ReverseBits32(mips->r[inst->src1]);
 			break;
 
-		case IROp::ValidateAddress8:
-			if (RunValidateAddress<1>(mips->pc, mips->r[inst->src1] + inst->constant, inst->src2)) {
-				CoreTiming::ForceCheck();
-				return mips->pc;
-			}
-		break;
-		case IROp::ValidateAddress16:
-			if (RunValidateAddress<2>(mips->pc, mips->r[inst->src1] + inst->constant, inst->src2)) {
-				CoreTiming::ForceCheck();
-				return mips->pc;
-			}
-			break;
-		case IROp::ValidateAddress32:
-			if (RunValidateAddress<4>(mips->pc, mips->r[inst->src1] + inst->constant, inst->src2)) {
-				CoreTiming::ForceCheck();
-				return mips->pc;
-			}
-			break;
-		case IROp::ValidateAddress128:
-			if (RunValidateAddress<16>(mips->pc, mips->r[inst->src1] + inst->constant, inst->src2)) {
-				CoreTiming::ForceCheck();
-				return mips->pc;
-			}
-			break;
-
 		case IROp::Load8:
 			mips->r[inst->dest] = Memory::ReadUnchecked_U8(mips->r[inst->src1] + inst->constant);
 			break;
@@ -283,33 +304,20 @@ u32 IRInterpret(MIPSState *mips, const IRInst *inst, int count) {
 		case IROp::LoadVec4:
 		{
 			u32 base = mips->r[inst->src1] + inst->constant;
-#if defined(_M_SSE)
-			_mm_store_ps(&mips->f[inst->dest], _mm_load_ps((const float *)Memory::GetPointerUnchecked(base)));
-#else
-			for (int i = 0; i < 4; i++)
-				mips->f[inst->dest + i] = Memory::ReadUnchecked_Float(base + 4 * i);
-#endif
+			// This compiles to a nice SSE load/store on x86, and hopefully similar on ARM.
+			memcpy(&mips->f[inst->dest], Memory::GetPointerUnchecked(base), 4 * 4);
 			break;
 		}
 		case IROp::StoreVec4:
 		{
 			u32 base = mips->r[inst->src1] + inst->constant;
-#if defined(_M_SSE)
-			_mm_store_ps((float *)Memory::GetPointerUnchecked(base), _mm_load_ps(&mips->f[inst->dest]));
-#else
-			for (int i = 0; i < 4; i++)
-				Memory::WriteUnchecked_Float(mips->f[inst->dest + i], base + 4 * i);
-#endif
+			memcpy((float *)Memory::GetPointerUnchecked(base), &mips->f[inst->dest], 4 * 4);
 			break;
 		}
 
 		case IROp::Vec4Init:
 		{
-#if defined(_M_SSE)
-			_mm_store_ps(&mips->f[inst->dest], _mm_load_ps(vec4InitValues[inst->src1]));
-#else
 			memcpy(&mips->f[inst->dest], vec4InitValues[inst->src1], 4 * sizeof(float));
-#endif
 			break;
 		}
 
@@ -320,22 +328,30 @@ u32 IRInterpret(MIPSState *mips, const IRInst *inst, int count) {
 			float temp[4];
 			for (int i = 0; i < 4; i++)
 				temp[i] = mips->f[inst->src1 + ((inst->src2 >> (i * 2)) & 3)];
+			const int dest = inst->dest;
 			for (int i = 0; i < 4; i++)
-				mips->f[inst->dest + i] = temp[i];
+				mips->f[dest + i] = temp[i];
 			break;
 		}
 
 		case IROp::Vec4Blend:
+		{
+			const int dest = inst->dest;
+			const int src1 = inst->src1;
+			const int src2 = inst->src2;
+			const int constant = inst->constant;
+			// 90% of calls to this is inst->constant == 7 or inst->constant == 8. Some are 1 and 4, others very rare.
 			// Could use _mm_blendv_ps (SSE4+BMI), vbslq_f32 (ARM), __riscv_vmerge_vvm (RISC-V)
 			for (int i = 0; i < 4; i++)
-				mips->f[inst->dest + i] = ((inst->constant >> i) & 1) ? mips->f[inst->src2 + i] : mips->f[inst->src1 + i];
+				mips->f[dest + i] = ((constant >> i) & 1) ? mips->f[src2 + i] : mips->f[src1 + i];
 			break;
+		}
 
 		case IROp::Vec4Mov:
 		{
 #if defined(_M_SSE)
 			_mm_store_ps(&mips->f[inst->dest], _mm_load_ps(&mips->f[inst->src1]));
-#elif PPSSPP_ARCH(ARM64_NEON)
+#elif PPSSPP_ARCH(ARM_NEON)
 			vst1q_f32(&mips->f[inst->dest], vld1q_f32(&mips->f[inst->src1]));
 #else
 			memcpy(&mips->f[inst->dest], &mips->f[inst->src1], 4 * sizeof(float));
@@ -347,7 +363,7 @@ u32 IRInterpret(MIPSState *mips, const IRInst *inst, int count) {
 		{
 #if defined(_M_SSE)
 			_mm_store_ps(&mips->f[inst->dest], _mm_add_ps(_mm_load_ps(&mips->f[inst->src1]), _mm_load_ps(&mips->f[inst->src2])));
-#elif PPSSPP_ARCH(ARM64_NEON)
+#elif PPSSPP_ARCH(ARM_NEON)
 			vst1q_f32(&mips->f[inst->dest], vaddq_f32(vld1q_f32(&mips->f[inst->src1]), vld1q_f32(&mips->f[inst->src2])));
 #else
 			for (int i = 0; i < 4; i++)
@@ -360,7 +376,7 @@ u32 IRInterpret(MIPSState *mips, const IRInst *inst, int count) {
 		{
 #if defined(_M_SSE)
 			_mm_store_ps(&mips->f[inst->dest], _mm_sub_ps(_mm_load_ps(&mips->f[inst->src1]), _mm_load_ps(&mips->f[inst->src2])));
-#elif PPSSPP_ARCH(ARM64_NEON)
+#elif PPSSPP_ARCH(ARM_NEON)
 			vst1q_f32(&mips->f[inst->dest], vsubq_f32(vld1q_f32(&mips->f[inst->src1]), vld1q_f32(&mips->f[inst->src2])));
 #else
 			for (int i = 0; i < 4; i++)
@@ -373,7 +389,7 @@ u32 IRInterpret(MIPSState *mips, const IRInst *inst, int count) {
 		{
 #if defined(_M_SSE)
 			_mm_store_ps(&mips->f[inst->dest], _mm_mul_ps(_mm_load_ps(&mips->f[inst->src1]), _mm_load_ps(&mips->f[inst->src2])));
-#elif PPSSPP_ARCH(ARM64_NEON)
+#elif PPSSPP_ARCH(ARM_NEON)
 			vst1q_f32(&mips->f[inst->dest], vmulq_f32(vld1q_f32(&mips->f[inst->src1]), vld1q_f32(&mips->f[inst->src2])));
 #else
 			for (int i = 0; i < 4; i++)
@@ -386,6 +402,8 @@ u32 IRInterpret(MIPSState *mips, const IRInst *inst, int count) {
 		{
 #if defined(_M_SSE)
 			_mm_store_ps(&mips->f[inst->dest], _mm_div_ps(_mm_load_ps(&mips->f[inst->src1]), _mm_load_ps(&mips->f[inst->src2])));
+#elif PPSSPP_ARCH(ARM64_NEON)
+			vst1q_f32(&mips->f[inst->dest], vdivq_f32(vld1q_f32(&mips->f[inst->src1]), vld1q_f32(&mips->f[inst->src2])));
 #else
 			for (int i = 0; i < 4; i++)
 				mips->f[inst->dest + i] = mips->f[inst->src1 + i] / mips->f[inst->src2 + i];
@@ -397,9 +415,12 @@ u32 IRInterpret(MIPSState *mips, const IRInst *inst, int count) {
 		{
 #if defined(_M_SSE)
 			_mm_store_ps(&mips->f[inst->dest], _mm_mul_ps(_mm_load_ps(&mips->f[inst->src1]), _mm_set1_ps(mips->f[inst->src2])));
+#elif PPSSPP_ARCH(ARM_NEON)
+			vst1q_f32(&mips->f[inst->dest], vmulq_lane_f32(vld1q_f32(&mips->f[inst->src1]), vdup_n_f32(mips->f[inst->src2]), 0));
 #else
+			const float factor = mips->f[inst->src2];
 			for (int i = 0; i < 4; i++)
-				mips->f[inst->dest + i] = mips->f[inst->src1 + i] * mips->f[inst->src2];
+				mips->f[inst->dest + i] = mips->f[inst->src1 + i] * factor;
 #endif
 			break;
 		}
@@ -408,7 +429,7 @@ u32 IRInterpret(MIPSState *mips, const IRInst *inst, int count) {
 		{
 #if defined(_M_SSE)
 			_mm_store_ps(&mips->f[inst->dest], _mm_xor_ps(_mm_load_ps(&mips->f[inst->src1]), _mm_load_ps((const float *)signBits)));
-#elif PPSSPP_ARCH(ARM64_NEON)
+#elif PPSSPP_ARCH(ARM_NEON)
 			vst1q_f32(&mips->f[inst->dest], vnegq_f32(vld1q_f32(&mips->f[inst->src1])));
 #else
 			for (int i = 0; i < 4; i++)
@@ -421,7 +442,7 @@ u32 IRInterpret(MIPSState *mips, const IRInst *inst, int count) {
 		{
 #if defined(_M_SSE)
 			_mm_store_ps(&mips->f[inst->dest], _mm_and_ps(_mm_load_ps(&mips->f[inst->src1]), _mm_load_ps((const float *)noSignMask)));
-#elif PPSSPP_ARCH(ARM64_NEON)
+#elif PPSSPP_ARCH(ARM_NEON)
 			vst1q_f32(&mips->f[inst->dest], vabsq_f32(vld1q_f32(&mips->f[inst->src1])));
 #else
 			for (int i = 0; i < 4; i++)
@@ -432,15 +453,19 @@ u32 IRInterpret(MIPSState *mips, const IRInst *inst, int count) {
 
 		case IROp::Vec2Unpack16To31:
 		{
-			mips->fi[inst->dest] = (mips->fi[inst->src1] << 16) >> 1;
-			mips->fi[inst->dest + 1] = (mips->fi[inst->src1] & 0xFFFF0000) >> 1;
+			const int dest = inst->dest;
+			const int src1 = inst->src1;
+			mips->fi[dest] = (mips->fi[src1] << 16) >> 1;
+			mips->fi[dest + 1] = (mips->fi[src1] & 0xFFFF0000) >> 1;
 			break;
 		}
 
 		case IROp::Vec2Unpack16To32:
 		{
-			mips->fi[inst->dest] = (mips->fi[inst->src1] << 16);
-			mips->fi[inst->dest + 1] = (mips->fi[inst->src1] & 0xFFFF0000);
+			const int dest = inst->dest;
+			const int src1 = inst->src1;
+			mips->fi[dest] = (mips->fi[src1] << 16);
+			mips->fi[dest + 1] = (mips->fi[src1] & 0xFFFF0000);
 			break;
 		}
 
@@ -451,6 +476,11 @@ u32 IRInterpret(MIPSState *mips, const IRInst *inst, int count) {
 			src = _mm_unpacklo_epi8(src, _mm_setzero_si128());
 			src = _mm_unpacklo_epi16(src, _mm_setzero_si128());
 			_mm_store_si128((__m128i *)&mips->fi[inst->dest], _mm_slli_epi32(src, 24));
+#elif PPSSPP_ARCH(ARM_NEON) && 0 // Untested
+			const uint8x8_t value = (uint8x8_t)vdup_n_u32(mips->fi[inst->src1]);
+			const uint16x8_t value16 = vmovl_u8(value);
+			const uint32x4_t value32 = vshll_n_u16(vget_low_u16(value16), 24);
+			vst1q_u32(&mips->fi[inst->dest], value32);
 #else
 			mips->fi[inst->dest] = (mips->fi[inst->src1] << 24);
 			mips->fi[inst->dest + 1] = (mips->fi[inst->src1] << 16) & 0xFF000000;
@@ -469,6 +499,8 @@ u32 IRInterpret(MIPSState *mips, const IRInst *inst, int count) {
 
 		case IROp::Vec2Pack31To16:
 		{
+			// Used in Tekken 6
+
 			u32 val = (mips->fi[inst->src1] >> 15) & 0xFFFF;
 			val |= (mips->fi[inst->src1 + 1] << 1) & 0xFFFF0000;
 			mips->fi[inst->dest] = val;
@@ -489,13 +521,24 @@ u32 IRInterpret(MIPSState *mips, const IRInst *inst, int count) {
 
 		case IROp::Vec4Pack31To8:
 		{
+			// Used in Tekken 6
+
 			// Removed previous SSE code due to the need for unsigned 16-bit pack, which I'm too lazy to work around the lack of in SSE2.
 			// pshufb or SSE4 instructions can be used instead.
+#if PPSSPP_ARCH(ARM_NEON) && 0
+			// Untested
+			uint32x4_t value = vld1q_u32(&mips->fi[inst->src1]);
+			value = vshlq_n_u32(value, 1);
+			uint32x2_t halved = vshrn_n_u32(value, 8);
+			uint32x2_t halvedAgain = vshrn_n_u32(vcombine_u32(halved, vdup_n_u32(0)), 8);
+			mips->fi[inst->dest] = vget_lane_u32(halvedAgain, 0);
+#else
 			u32 val = (mips->fi[inst->src1] >> 23) & 0xFF;
 			val |= (mips->fi[inst->src1 + 1] >> 15) & 0xFF00;
 			val |= (mips->fi[inst->src1 + 2] >> 7) & 0xFF0000;
 			val |= (mips->fi[inst->src1 + 3] << 1) & 0xFF000000;
 			mips->fi[inst->dest] = val;
+#endif
 			break;
 		}
 
@@ -517,9 +560,11 @@ u32 IRInterpret(MIPSState *mips, const IRInst *inst, int count) {
 			val = _mm_andnot_si128(mask, val);
 			_mm_store_si128((__m128i *)&mips->fi[inst->dest], val);
 #else
+			const int src1 = inst->src1;
+			const int dest = inst->dest;
 			for (int i = 0; i < 4; i++) {
-				u32 val = mips->fi[inst->src1 + i];
-				mips->fi[inst->dest + i] = (int)val >= 0 ? val : 0;
+				u32 val = mips->fi[src1 + i];
+				mips->fi[dest + i] = (int)val >= 0 ? val : 0;
 			}
 #endif
 			break;
@@ -527,20 +572,22 @@ u32 IRInterpret(MIPSState *mips, const IRInst *inst, int count) {
 
 		case IROp::Vec4DuplicateUpperBitsAndShift1:  // For vuc2i, the weird one.
 		{
+			const int src1 = inst->src1;
+			const int dest = inst->dest;
 			for (int i = 0; i < 4; i++) {
-				u32 val = mips->fi[inst->src1 + i];
+				u32 val = mips->fi[src1 + i];
 				val = val | (val >> 8);
 				val = val | (val >> 16);
 				val >>= 1;
-				mips->fi[inst->dest + i] = val;
+				mips->fi[dest + i] = val;
 			}
 			break;
 		}
 
 		case IROp::FCmpVfpuBit:
 		{
-			int op = inst->dest & 0xF;
-			int bit = inst->dest >> 4;
+			const int op = inst->dest & 0xF;
+			const int bit = inst->dest >> 4;
 			int result = 0;
 			switch (op) {
 			case VC_EQ: result = mips->f[inst->src1] == mips->f[inst->src2]; break;
@@ -572,8 +619,8 @@ u32 IRInterpret(MIPSState *mips, const IRInst *inst, int count) {
 
 		case IROp::FCmpVfpuAggregate:
 		{
-			u32 mask = inst->dest;
-			u32 cc = mips->vfpuCtrl[VFPU_CTRL_CC];
+			const u32 mask = inst->dest;
+			const u32 cc = mips->vfpuCtrl[VFPU_CTRL_CC];
 			int anyBit = (cc & mask) ? 0x10 : 0x00;
 			int allBit = (cc & mask) == mask ? 0x20 : 0x00;
 			mips->vfpuCtrl[VFPU_CTRL_CC] = (cc & ~0x30) | anyBit | allBit;
@@ -586,9 +633,11 @@ u32 IRInterpret(MIPSState *mips, const IRInst *inst, int count) {
 			}
 			break;
 
-		// Not quickly implementable on all platforms, unfortunately.
 		case IROp::Vec4Dot:
 		{
+			// Not quickly implementable on all platforms, unfortunately.
+			// Though, this is still pretty fast compared to one split into multiple IR instructions.
+			// This might be good though: https://gist.github.com/rikusalminen/3040241
 			float dot = mips->f[inst->src1] * mips->f[inst->src2];
 			for (int i = 1; i < 4; i++)
 				dot += mips->f[inst->src1 + i] * mips->f[inst->src2 + i];
@@ -775,13 +824,13 @@ u32 IRInterpret(MIPSState *mips, const IRInst *inst, int count) {
 		case IROp::BSwap16:
 		{
 			u32 x = mips->r[inst->src1];
+			// Don't think we can beat this with intrinsics.
 			mips->r[inst->dest] = ((x & 0xFF00FF00) >> 8) | ((x & 0x00FF00FF) << 8);
 			break;
 		}
 		case IROp::BSwap32:
 		{
-			u32 x = mips->r[inst->src1];
-			mips->r[inst->dest] = ((x & 0xFF000000) >> 24) | ((x & 0x00FF0000) >> 8) | ((x & 0x0000FF00) << 8) | ((x & 0x000000FF) << 24);
+			mips->r[inst->dest] = swap32(mips->r[inst->src1]);
 			break;
 		}
 
@@ -792,12 +841,31 @@ u32 IRInterpret(MIPSState *mips, const IRInst *inst, int count) {
 			mips->f[inst->dest] = mips->f[inst->src1] - mips->f[inst->src2];
 			break;
 		case IROp::FMul:
-			if ((my_isinf(mips->f[inst->src1]) && mips->f[inst->src2] == 0.0f) || (my_isinf(mips->f[inst->src2]) && mips->f[inst->src1] == 0.0f)) {
+#if 1
+		{
+			float a = mips->f[inst->src1];
+			float b = mips->f[inst->src2];
+			if ((b == 0.0f && my_isinf(a)) || (a == 0.0f && my_isinf(b))) {
 				mips->fi[inst->dest] = 0x7fc00000;
 			} else {
-				mips->f[inst->dest] = mips->f[inst->src1] * mips->f[inst->src2];
+				mips->f[inst->dest] = a * b;
 			}
+		}
 			break;
+#else
+			// Not sure if faster since it needs to load the operands twice? But the code is simpler.
+			{
+				// Takes care of negative zero by masking away the top bit, which also makes the inf check shorter.
+				u32 a = mips->fi[inst->src1] & 0x7FFFFFFF;
+				u32 b = mips->fi[inst->src2] & 0x7FFFFFFF;
+				if ((a == 0 && b == 0x7F800000) || (b == 0 && a == 0x7F800000)) {
+					mips->fi[inst->dest] = 0x7fc00000;
+				} else {
+					mips->f[inst->dest] = mips->f[inst->src1] * mips->f[inst->src2];
+				}
+				break;
+			}
+#endif
 		case IROp::FDiv:
 			mips->f[inst->dest] = mips->f[inst->src1] / mips->f[inst->src2];
 			break;
@@ -846,9 +914,9 @@ u32 IRInterpret(MIPSState *mips, const IRInst *inst, int count) {
 			mips->f[inst->dest] = vfpu_clamp(mips->f[inst->src1], -1.0f, 1.0f);
 			break;
 
-		// Bitwise trickery
 		case IROp::FSign:
 		{
+			// Bitwise trickery
 			u32 val;
 			memcpy(&val, &mips->f[inst->src1], sizeof(u32));
 			if (val == 0 || val == 0x80000000)
@@ -976,6 +1044,7 @@ u32 IRInterpret(MIPSState *mips, const IRInst *inst, int count) {
 				mips->fs[inst->dest] = my_isinf(src) && src < 0.0f ? -2147483648LL : 2147483647LL;
 				break;
 			}
+			// TODO: Inline assembly to use here would be better.
 			switch (IRRoundMode(mips->fcr31 & 3)) {
 			case IRRoundMode::RINT_0: mips->fs[inst->dest] = (int)round_ieee_754(src); break;
 			case IRRoundMode::CAST_1: mips->fs[inst->dest] = (int)src; break;
@@ -1017,9 +1086,19 @@ u32 IRInterpret(MIPSState *mips, const IRInst *inst, int count) {
 		case IROp::FMovFromGPR:
 			memcpy(&mips->f[inst->dest], &mips->r[inst->src1], 4);
 			break;
+		case IROp::OptFCvtSWFromGPR:
+			mips->f[inst->dest] = (float)(int)mips->r[inst->src1];
+			break;
 		case IROp::FMovToGPR:
 			memcpy(&mips->r[inst->dest], &mips->f[inst->src1], 4);
 			break;
+		case IROp::OptFMovToGPRShr8:
+		{
+			u32 temp;
+			memcpy(&temp, &mips->f[inst->src1], 4);
+			mips->r[inst->dest] = temp >> 8;
+			break;
+		}
 
 		case IROp::ExitToConst:
 			return inst->constant;
@@ -1089,14 +1168,11 @@ u32 IRInterpret(MIPSState *mips, const IRInst *inst, int count) {
 			int funcIndex = inst->constant;
 			const ReplacementTableEntry *f = GetReplacementFunc(funcIndex);
 			int cycles = f->replaceFunc();
-			mips->downcount -= cycles;
+			mips->r[inst->dest] = cycles < 0 ? -1 : 0;
+			mips->downcount -= cycles < 0 ? -cycles : cycles;
 			break;
 		}
 
-		case IROp::Break:
-			Core_Break(mips->pc);
-			return mips->pc + 4;
-
 		case IROp::SetCtrlVFPU:
 			mips->vfpuCtrl[inst->dest] = inst->constant;
 			break;
@@ -1109,6 +1185,20 @@ u32 IRInterpret(MIPSState *mips, const IRInst *inst, int count) {
 			memcpy(&mips->vfpuCtrl[inst->dest], &mips->f[inst->src1], 4);
 			break;
 
+		case IROp::ApplyRoundingMode:
+			IRApplyRounding(mips);
+			break;
+		case IROp::RestoreRoundingMode:
+			IRRestoreRounding();
+			break;
+		case IROp::UpdateRoundingMode:
+			// TODO: Implement
+			break;
+
+		case IROp::Break:
+			Core_Break(mips->pc);
+			return mips->pc + 4;
+
 		case IROp::Breakpoint:
 			if (IRRunBreakpoint(inst->constant)) {
 				CoreTiming::ForceCheck();
@@ -1123,20 +1213,44 @@ u32 IRInterpret(MIPSState *mips, const IRInst *inst, int count) {
 			}
 			break;
 
-		case IROp::ApplyRoundingMode:
-			// TODO: Implement
+		case IROp::ValidateAddress8:
+			if (RunValidateAddress<1>(mips->pc, mips->r[inst->src1] + inst->constant, inst->src2)) {
+				CoreTiming::ForceCheck();
+				return mips->pc;
+			}
 			break;
-		case IROp::RestoreRoundingMode:
-			// TODO: Implement
+		case IROp::ValidateAddress16:
+			if (RunValidateAddress<2>(mips->pc, mips->r[inst->src1] + inst->constant, inst->src2)) {
+				CoreTiming::ForceCheck();
+				return mips->pc;
+			}
 			break;
-		case IROp::UpdateRoundingMode:
-			// TODO: Implement
+		case IROp::ValidateAddress32:
+			if (RunValidateAddress<4>(mips->pc, mips->r[inst->src1] + inst->constant, inst->src2)) {
+				CoreTiming::ForceCheck();
+				return mips->pc;
+			}
+			break;
+		case IROp::ValidateAddress128:
+			if (RunValidateAddress<16>(mips->pc, mips->r[inst->src1] + inst->constant, inst->src2)) {
+				CoreTiming::ForceCheck();
+				return mips->pc;
+			}
+			break;
+		case IROp::LogIRBlock:
+			if (mipsTracer.tracing_enabled) {
+				mipsTracer.executed_blocks.push_back(inst->constant);
+			}
 			break;
 
+		case IROp::Nop: // TODO: This shouldn't crash, but for now we should not emit nops, so...
+		case IROp::Bad:
 		default:
-			// Unimplemented IR op. Bad.
 			Crash();
+			break;
+			// Unimplemented IR op. Bad.
 		}
+
 #ifdef _DEBUG
 		if (mips->r[0] != 0)
 			Crash();
@@ -1144,6 +1258,6 @@ u32 IRInterpret(MIPSState *mips, const IRInst *inst, int count) {
 		inst++;
 	}
 
-	// We hit count.  If this is a full block, it was badly constructed.
+	// We should not reach here anymore.
 	return 0;
 }
