@@ -17,6 +17,7 @@
 
 #include "ppsspp_config.h"
 #include <set>
+#include <algorithm>
 
 #include "ext/xxhash.h"
 #include "Common/Profiler/Profiler.h"
@@ -40,14 +41,21 @@
 #include "Core/MIPS/IR/IRNativeCommon.h"
 #include "Core/MIPS/JitCommon/JitCommon.h"
 #include "Core/Reporting.h"
+#include "Common/TimeUtil.h"
+#include "Core/MIPS/MIPSTracer.h"
+
 
 namespace MIPSComp {
 
-IRJit::IRJit(MIPSState *mipsState) : frontend_(mipsState->HasDefaultPrefix()), mips_(mipsState) {
+IRJit::IRJit(MIPSState *mipsState, bool actualJit) : frontend_(mipsState->HasDefaultPrefix()), mips_(mipsState), blocks_(actualJit) {
 	// u32 size = 128 * 1024;
-	// blTrampolines_ = kernelMemory.Alloc(size, true, "trampoline");
 	InitIR();
 
+	compileToNative_ = actualJit;
+
+	// If this IRJit instance will be used to drive a "JIT using IR", don't optimize for interpretation.
+	jo.optimizeForInterpreter = !actualJit;
+
 	IROptions opts{};
 	opts.disableFlags = g_Config.uJitDisableFlags;
 #if PPSSPP_ARCH(RISCV64)
@@ -55,16 +63,17 @@ IRJit::IRJit(MIPSState *mipsState) : frontend_(mipsState->HasDefaultPrefix()), m
 	opts.unalignedLoadStore = false;
 	opts.unalignedLoadStoreVec4 = true;
 	opts.preferVec4 = cpu_info.RiscV_V;
-#elif PPSSPP_ARCH(ARM)
+#elif PPSSPP_ARCH(ARM) || PPSSPP_ARCH(ARM64)
 	opts.unalignedLoadStore = (opts.disableFlags & (uint32_t)JitDisable::LSU_UNALIGNED) == 0;
 	opts.unalignedLoadStoreVec4 = true;
-	opts.preferVec4 = cpu_info.bASIMD || cpu_info.bNEON;
+	opts.preferVec4 = true;
 #else
 	opts.unalignedLoadStore = (opts.disableFlags & (uint32_t)JitDisable::LSU_UNALIGNED) == 0;
 	// TODO: Could allow on x86 pretty easily...
 	opts.unalignedLoadStoreVec4 = false;
 	opts.preferVec4 = true;
 #endif
+	opts.optimizeForInterpreter = jo.optimizeForInterpreter;
 	frontend_.SetOptions(opts);
 }
 
@@ -79,33 +88,45 @@ void IRJit::UpdateFCR31() {
 }
 
 void IRJit::ClearCache() {
-	INFO_LOG(JIT, "IRJit: Clearing the cache!");
+	INFO_LOG(Log::JIT, "IRJit: Clearing the block cache!");
 	blocks_.Clear();
 }
 
 void IRJit::InvalidateCacheAt(u32 em_address, int length) {
 	std::vector<int> numbers = blocks_.FindInvalidatedBlockNumbers(em_address, length);
+	if (numbers.empty()) {
+		return;
+	}
+
+	DEBUG_LOG(Log::JIT, "Invalidating IR block cache at %08x (%d bytes): %d blocks", em_address, length, (int)numbers.size());
+
 	for (int block_num : numbers) {
 		auto block = blocks_.GetBlock(block_num);
-		int cookie = block->GetTargetOffset() < 0 ? block_num : block->GetTargetOffset();
+		// TODO: We are invalidating a lot of blocks that are already invalid (yu gi oh).
+		// INFO_LOG(Log::JIT, "Block at %08x invalidated: valid: %d", block->GetOriginalStart(), block->IsValid());
+		// If we're a native JIT (IR->JIT, not just IR interpreter), we write native offsets into the blocks.
+		int cookie = compileToNative_ ? block->GetNativeOffset() : block->GetIRArenaOffset();
+		blocks_.RemoveBlockFromPageLookup(block_num);
 		block->Destroy(cookie);
 	}
 }
 
 void IRJit::Compile(u32 em_address) {
+	_dbg_assert_(compilerEnabled_);
+
 	PROFILE_THIS_SCOPE("jitc");
 
 	if (g_Config.bPreloadFunctions) {
 		// Look to see if we've preloaded this block.
 		int block_num = blocks_.FindPreloadBlock(em_address);
 		if (block_num != -1) {
-			IRBlock *b = blocks_.GetBlock(block_num);
+			IRBlock *block = blocks_.GetBlock(block_num);
 			// Okay, let's link and finalize the block now.
-			int cookie = b->GetTargetOffset() < 0 ? block_num : b->GetTargetOffset();
-			b->Finalize(cookie);
-			if (b->IsValid()) {
+			int cookie = compileToNative_ ? block->GetNativeOffset() : block->GetIRArenaOffset();
+			block->Finalize(cookie);
+			if (block->IsValid()) {
 				// Success, we're done.
-				FinalizeTargetBlock(b, block_num);
+				FinalizeNativeBlock(&blocks_, block_num);
 				return;
 			}
 		}
@@ -115,7 +136,7 @@ void IRJit::Compile(u32 em_address) {
 	u32 mipsBytes;
 	if (!CompileBlock(em_address, instructions, mipsBytes, false)) {
 		// Ran out of block numbers - need to reset.
-		ERROR_LOG(JIT, "Ran out of block numbers, clearing cache");
+		ERROR_LOG(Log::JIT, "Ran out of block numbers, clearing cache");
 		ClearCache();
 		CompileBlock(em_address, instructions, mipsBytes, false);
 	}
@@ -127,7 +148,10 @@ void IRJit::Compile(u32 em_address) {
 	}
 }
 
+// WARNING! This can be called from IRInterpret / the JIT, through the function preload stuff!
 bool IRJit::CompileBlock(u32 em_address, std::vector<IRInst> &instructions, u32 &mipsBytes, bool preload) {
+	_dbg_assert_(compilerEnabled_);
+
 	frontend_.DoJit(em_address, instructions, mipsBytes, preload);
 	if (instructions.empty()) {
 		_dbg_assert_(preload);
@@ -135,31 +159,37 @@ bool IRJit::CompileBlock(u32 em_address, std::vector<IRInst> &instructions, u32
 		return preload;
 	}
 
-	int block_num = blocks_.AllocateBlock(em_address);
+	int block_num = blocks_.AllocateBlock(em_address, mipsBytes, instructions);
 	if ((block_num & ~MIPS_EMUHACK_VALUE_MASK) != 0) {
+		WARN_LOG(Log::JIT, "Failed to allocate block for %08x (%d instructions)", em_address, (int)instructions.size());
 		// Out of block numbers.  Caller will handle.
 		return false;
 	}
 
 	IRBlock *b = blocks_.GetBlock(block_num);
-	b->SetInstructions(instructions);
-	b->SetOriginalSize(mipsBytes);
-	if (preload) {
+	if (preload || mipsTracer.tracing_enabled) {
 		// Hash, then only update page stats, don't link yet.
 		// TODO: Should we always hash?  Then we can reuse blocks.
 		b->UpdateHash();
 	}
-	if (!CompileTargetBlock(b, block_num, preload))
+
+	if (!CompileNativeBlock(&blocks_, block_num, preload))
 		return false;
-	// Overwrites the first instruction, and also updates stats.
+
+	if (mipsTracer.tracing_enabled) {
+		mipsTracer.prepare_block(b, blocks_);
+	}
+
+	// Updates stats, also patches the first MIPS instruction into an emuhack if 'preload == false'
 	blocks_.FinalizeBlock(block_num, preload);
 	if (!preload)
-		FinalizeTargetBlock(b, block_num);
-
+		FinalizeNativeBlock(&blocks_, block_num);
 	return true;
 }
 
 void IRJit::CompileFunction(u32 start_address, u32 length) {
+	_dbg_assert_(compilerEnabled_);
+
 	PROFILE_THIS_SCOPE("jitc");
 
 	// Note: we don't actually write emuhacks yet, so we can validate hashes.
@@ -186,7 +216,7 @@ void IRJit::CompileFunction(u32 start_address, u32 length) {
 		if (!CompileBlock(em_address, instructions, mipsBytes, true)) {
 			// Ran out of block numbers - let's hope there's no more code it needs to run.
 			// Will flush when actually compiling.
-			ERROR_LOG(JIT, "Ran out of block numbers while compiling function");
+			ERROR_LOG(Log::JIT, "Ran out of block numbers while compiling function");
 			return;
 		}
 
@@ -246,32 +276,62 @@ void IRJit::RunLoopUntil(u64 globalticks) {
 		if (coreState != 0) {
 			break;
 		}
-		while (mips_->downcount >= 0) {
-			u32 inst = Memory::ReadUnchecked_U32(mips_->pc);
+
+		MIPSState *mips = mips_;
+#ifdef _DEBUG
+		compilerEnabled_ = false;
+#endif
+		while (mips->downcount >= 0) {
+			u32 inst = Memory::ReadUnchecked_U32(mips->pc);
 			u32 opcode = inst & 0xFF000000;
 			if (opcode == MIPS_EMUHACK_OPCODE) {
-				u32 data = inst & 0xFFFFFF;
-				IRBlock *block = blocks_.GetBlock(data);
-				u32 startPC = mips_->pc;
-				mips_->pc = IRInterpret(mips_, block->GetInstructions(), block->GetNumInstructions());
+				u32 offset = inst & 0x00FFFFFF; // Alternatively, inst - opcode
+				const IRInst *instPtr = blocks_.GetArenaPtr() + offset;
+				// First op is always, except when using breakpoints, downcount, to save one dispatch inside IRInterpret.
+				// This branch is very cpu-branch-predictor-friendly so this still beats the dispatch.
+				if (instPtr->op == IROp::Downcount) {
+					mips->downcount -= instPtr->constant;
+					instPtr++;
+				}
+#ifdef IR_PROFILING
+				IRBlock *block = blocks_.GetBlock(blocks_.GetBlockNumFromOffset(offset));
+				Instant start = Instant::Now();
+				mips->pc = IRInterpret(mips, instPtr);
+				int64_t elapsedNanos = start.ElapsedNanos();
+				block->profileStats_.executions += 1;
+				block->profileStats_.totalNanos += elapsedNanos;
+#else
+				mips->pc = IRInterpret(mips, instPtr);
+#endif
 				// Note: this will "jump to zero" on a badly constructed block missing exits.
-				if (!Memory::IsValidAddress(mips_->pc) || (mips_->pc & 3) != 0) {
-					Core_ExecException(mips_->pc, startPC, ExecExceptionType::JUMP);
+				if (!Memory::IsValid4AlignedAddress(mips->pc)) {
+					int blockNum = blocks_.GetBlockNumFromIRArenaOffset(offset);
+					IRBlock *block = blocks_.GetBlockUnchecked(blockNum);
+					Core_ExecException(mips->pc, block->GetOriginalStart(), ExecExceptionType::JUMP);
 					break;
 				}
 			} else {
 				// RestoreRoundingMode(true);
-				Compile(mips_->pc);
+#ifdef _DEBUG
+				compilerEnabled_ = true;
+#endif
+				Compile(mips->pc);
+#ifdef _DEBUG
+				compilerEnabled_ = false;
+#endif
 				// ApplyRoundingMode(true);
 			}
 		}
+#ifdef _DEBUG
+		compilerEnabled_ = true;
+#endif
 	}
 
 	// RestoreRoundingMode(true);
 }
 
 bool IRJit::DescribeCodePtr(const u8 *ptr, std::string &name) {
-	// Used in target disassembly viewer.
+	// Used in native disassembly viewer.
 	return false;
 }
 
@@ -285,16 +345,72 @@ void IRJit::UnlinkBlock(u8 *checkedEntry, u32 originalAddress) {
 
 void IRBlockCache::Clear() {
 	for (int i = 0; i < (int)blocks_.size(); ++i) {
-		int cookie = blocks_[i].GetTargetOffset() < 0 ? i : blocks_[i].GetTargetOffset();
+		int cookie = compileToNative_ ? blocks_[i].GetNativeOffset() : blocks_[i].GetIRArenaOffset();
 		blocks_[i].Destroy(cookie);
 	}
 	blocks_.clear();
 	byPage_.clear();
+	arena_.clear();
+	arena_.shrink_to_fit();
 }
 
-std::vector<int> IRBlockCache::FindInvalidatedBlockNumbers(u32 address, u32 length) {
+IRBlockCache::IRBlockCache(bool compileToNative) : compileToNative_(compileToNative) {}
+
+int IRBlockCache::AllocateBlock(int emAddr, u32 origSize, const std::vector<IRInst> &insts) {
+	// We have 24 bits to represent offsets with.
+	const u32 MAX_ARENA_SIZE = 0x1000000 - 1;
+	int offset = (int)arena_.size();
+	if (offset >= MAX_ARENA_SIZE) {
+		WARN_LOG(Log::JIT, "Filled JIT arena, restarting");
+		return -1;
+	}
+	// TODO: Use memcpy.
+	for (int i = 0; i < insts.size(); i++) {
+		arena_.push_back(insts[i]);
+	}
+	int newBlockIndex = (int)blocks_.size();
+	blocks_.push_back(IRBlock(emAddr, origSize, offset, (u32)insts.size()));
+	return newBlockIndex;
+}
+
+int IRBlockCache::GetBlockNumFromIRArenaOffset(int offset) const {
+	// Block offsets are always in rising order (we don't go back and replace them when invalidated). So we can binary search.
+	int low = 0;
+	int high = (int)blocks_.size() - 1;
+	int found = -1;
+	while (low <= high) {
+		int mid = low + (high - low) / 2;
+		const int blockOffset = blocks_[mid].GetIRArenaOffset();
+		if (blockOffset == offset) {
+			found = mid;
+			break;
+		}
+		if (blockOffset < offset) {
+			low = mid + 1;
+		} else {
+			high = mid - 1;
+		}
+	}
+
+#ifndef _DEBUG
+	// Then, in debug builds, cross check the result.
+	return found;
+#else
+	// TODO: Optimize if we need to call this often.
+	for (int i = 0; i < (int)blocks_.size(); i++) {
+		if (blocks_[i].GetIRArenaOffset() == offset) {
+			_dbg_assert_(i == found);
+			return i;
+		}
+	}
+#endif
+	_dbg_assert_(found == -1);
+	return -1;
+}
+
+std::vector<int> IRBlockCache::FindInvalidatedBlockNumbers(u32 address, u32 lengthInBytes) {
 	u32 startPage = AddressToPage(address);
-	u32 endPage = AddressToPage(address + length);
+	u32 endPage = AddressToPage(address + lengthInBytes);
 
 	std::vector<int> found;
 	for (u32 page = startPage; page <= endPage; ++page) {
@@ -304,8 +420,8 @@ std::vector<int> IRBlockCache::FindInvalidatedBlockNumbers(u32 address, u32 leng
 
 		const std::vector<int> &blocksInPage = iter->second;
 		for (int i : blocksInPage) {
-			if (blocks_[i].OverlapsRange(address, length)) {
-				// Not removing from the page, hopefully doesn't build up with small recompiles.
+			if (blocks_[i].OverlapsRange(address, lengthInBytes)) {
+				// We now try to remove these during invalidation.
 				found.push_back(i);
 			}
 		}
@@ -314,23 +430,56 @@ std::vector<int> IRBlockCache::FindInvalidatedBlockNumbers(u32 address, u32 leng
 	return found;
 }
 
-void IRBlockCache::FinalizeBlock(int i, bool preload) {
+void IRBlockCache::FinalizeBlock(int blockIndex, bool preload) {
+	// TODO: What's different about preload blocks?
+	IRBlock &block = blocks_[blockIndex];
 	if (!preload) {
-		int cookie = blocks_[i].GetTargetOffset() < 0 ? i : blocks_[i].GetTargetOffset();
-		blocks_[i].Finalize(cookie);
+		int cookie = compileToNative_ ? block.GetNativeOffset() : block.GetIRArenaOffset();
+		block.Finalize(cookie);
 	}
 
 	u32 startAddr, size;
-	blocks_[i].GetRange(startAddr, size);
+	block.GetRange(&startAddr, &size);
 
 	u32 startPage = AddressToPage(startAddr);
 	u32 endPage = AddressToPage(startAddr + size);
 
 	for (u32 page = startPage; page <= endPage; ++page) {
-		byPage_[page].push_back(i);
+		byPage_[page].push_back(blockIndex);
 	}
 }
 
+// Call after Destroy-ing it.
+void IRBlockCache::RemoveBlockFromPageLookup(int blockIndex) {
+	// We need to remove the block from the byPage lookup.
+	IRBlock &block = blocks_[blockIndex];
+
+	u32 startAddr, size;
+	block.GetRange(&startAddr, &size);
+
+	u32 startPage = AddressToPage(startAddr);
+	u32 endPage = AddressToPage(startAddr + size);
+
+	for (u32 page = startPage; page <= endPage; ++page) {
+		auto iter = std::find(byPage_[page].begin(), byPage_[page].end(), blockIndex);
+		if (iter != byPage_[page].end()) {
+			byPage_[page].erase(iter);
+		} else if (block.IsValid()) {
+			// If it was previously invalidated, we don't care, hence the above check.
+			WARN_LOG(Log::JIT, "RemoveBlock: Block at %08x was not found where expected in byPage table.", startAddr);
+		}
+	}
+
+	// Additionally, we'd like to zap the block in the IR arena.
+	// However, this breaks if calling sceKernelIcacheClearAll(), since as soon as we return, we'll be executing garbage.
+	/*
+	IRInst bad{ IROp::Bad };
+	for (int off = block.GetIRArenaOffset(); off < (int)(block.GetIRArenaOffset() + block.GetNumIRInstructions()); off++) {
+		arena_[off] = bad;
+	}
+	*/
+}
+
 u32 IRBlockCache::AddressToPage(u32 addr) const {
 	// Use relatively small pages since basic blocks are typically small.
 	return (addr & 0x3FFFFFFF) >> 10;
@@ -357,16 +506,18 @@ int IRBlockCache::FindPreloadBlock(u32 em_address) {
 int IRBlockCache::FindByCookie(int cookie) {
 	if (blocks_.empty())
 		return -1;
-	// TODO: Maybe a flag to determine target offset mode?
-	if (blocks_[0].GetTargetOffset() < 0)
-		return cookie;
 
+	// TODO: Maybe a flag to determine native offset mode?
+	if (!compileToNative_) {
+		return GetBlockNumFromIRArenaOffset(cookie);
+	}
+
+	// TODO: This could also use a binary search.
 	for (int i = 0; i < GetNumBlocks(); ++i) {
-		int offset = blocks_[i].GetTargetOffset();
+		int offset = blocks_[i].GetNativeOffset();
 		if (offset == cookie)
 			return i;
 	}
-
 	return -1;
 }
 
@@ -376,7 +527,7 @@ std::vector<u32> IRBlockCache::SaveAndClearEmuHackOps() {
 
 	for (int number = 0; number < (int)blocks_.size(); ++number) {
 		IRBlock &b = blocks_[number];
-		int cookie = b.GetTargetOffset() < 0 ? number : b.GetTargetOffset();
+		int cookie = compileToNative_ ? b.GetNativeOffset() : b.GetIRArenaOffset();
 		if (b.IsValid() && b.RestoreOriginalFirstOp(cookie)) {
 			result[number] = number;
 		} else {
@@ -387,9 +538,9 @@ std::vector<u32> IRBlockCache::SaveAndClearEmuHackOps() {
 	return result;
 }
 
-void IRBlockCache::RestoreSavedEmuHackOps(std::vector<u32> saved) {
+void IRBlockCache::RestoreSavedEmuHackOps(const std::vector<u32> &saved) {
 	if ((int)blocks_.size() != (int)saved.size()) {
-		ERROR_LOG(JIT, "RestoreSavedEmuHackOps: Wrong saved block size.");
+		ERROR_LOG(Log::JIT, "RestoreSavedEmuHackOps: Wrong saved block size.");
 		return;
 	}
 
@@ -397,7 +548,7 @@ void IRBlockCache::RestoreSavedEmuHackOps(std::vector<u32> saved) {
 		IRBlock &b = blocks_[number];
 		// Only if we restored it, write it back.
 		if (b.IsValid() && saved[number] != 0 && b.HasOriginalFirstOp()) {
-			int cookie = b.GetTargetOffset() < 0 ? number : b.GetTargetOffset();
+			int cookie = compileToNative_ ? b.GetNativeOffset() : b.GetIRArenaOffset();
 			b.Finalize(cookie);
 		}
 	}
@@ -407,9 +558,10 @@ JitBlockDebugInfo IRBlockCache::GetBlockDebugInfo(int blockNum) const {
 	const IRBlock &ir = blocks_[blockNum];
 	JitBlockDebugInfo debugInfo{};
 	uint32_t start, size;
-	ir.GetRange(start, size);
+	ir.GetRange(&start, &size);
 	debugInfo.originalAddress = start;  // TODO
 
+	debugInfo.origDisasm.reserve(((start + size) - start) / 4);
 	for (u32 addr = start; addr < start + size; addr += 4) {
 		char temp[256];
 		MIPSDisAsm(Memory::Read_Instruction(addr), addr, temp, sizeof(temp), true);
@@ -417,8 +569,10 @@ JitBlockDebugInfo IRBlockCache::GetBlockDebugInfo(int blockNum) const {
 		debugInfo.origDisasm.push_back(mipsDis);
 	}
 
-	for (int i = 0; i < ir.GetNumInstructions(); i++) {
-		IRInst inst = ir.GetInstructions()[i];
+	debugInfo.irDisasm.reserve(ir.GetNumIRInstructions());
+	const IRInst *instructions = GetBlockInstructionPtr(ir);
+	for (int i = 0; i < ir.GetNumIRInstructions(); i++) {
+		IRInst inst = instructions[i];
 		char buffer[256];
 		DisassembleIR(buffer, sizeof(buffer), inst);
 		debugInfo.irDisasm.push_back(buffer);
@@ -431,12 +585,11 @@ void IRBlockCache::ComputeStats(BlockCacheStats &bcStats) const {
 	double maxBloat = 0.0;
 	double minBloat = 1000000000.0;
 	for (const auto &b : blocks_) {
-		double codeSize = (double)b.GetNumInstructions() * sizeof(IRInst);
+		double codeSize = (double)b.GetNumIRInstructions() * 4;  // We count bloat in instructions, not bytes. sizeof(IRInst);
 		if (codeSize == 0)
 			continue;
-
 		u32 origAddr, mipsBytes;
-		b.GetRange(origAddr, mipsBytes);
+		b.GetRange(&origAddr, &mipsBytes);
 		double origSize = (double)mipsBytes;
 		double bloat = codeSize / origSize;
 		if (bloat < minBloat) {
@@ -448,7 +601,6 @@ void IRBlockCache::ComputeStats(BlockCacheStats &bcStats) const {
 			bcStats.maxBloatBlock = origAddr;
 		}
 		totalBloat += bloat;
-		bcStats.bloatMap[bloat] = origAddr;
 	}
 	bcStats.numBlocks = (int)blocks_.size();
 	bcStats.minBloat = minBloat;
@@ -496,15 +648,22 @@ void IRBlock::Finalize(int cookie) {
 		origFirstOpcode_ = Memory::Read_Opcode_JIT(origAddr_);
 		MIPSOpcode opcode = MIPSOpcode(MIPS_EMUHACK_OPCODE | cookie);
 		Memory::Write_Opcode_JIT(origAddr_, opcode);
+	} else {
+		WARN_LOG(Log::JIT, "Finalizing invalid block (cookie: %d)", cookie);
 	}
 }
 
 void IRBlock::Destroy(int cookie) {
 	if (origAddr_) {
 		MIPSOpcode opcode = MIPSOpcode(MIPS_EMUHACK_OPCODE | cookie);
-		if (Memory::ReadUnchecked_U32(origAddr_) == opcode.encoding)
+		u32 memOp = Memory::ReadUnchecked_U32(origAddr_);
+		if (memOp == opcode.encoding) {
 			Memory::Write_Opcode_JIT(origAddr_, origFirstOpcode_);
-
+		} else {
+			// NOTE: This is not an error. Just interesting to log.
+			DEBUG_LOG(Log::JIT, "IRBlock::Destroy: Note: Block at %08x was overwritten - checked for %08x, got %08x when restoring the MIPS op to %08x", origAddr_, opcode.encoding, memOp, origFirstOpcode_.encoding);
+		}
+		// TODO: Also wipe the block in the IR opcode arena.
 		// Let's mark this invalid so we don't try to clear it again.
 		origAddr_ = 0;
 	}
@@ -512,7 +671,8 @@ void IRBlock::Destroy(int cookie) {
 
 u64 IRBlock::CalculateHash() const {
 	if (origAddr_) {
-		// This is unfortunate.  In case of emuhacks, we have to make a copy.
+		// This is unfortunate. In case there are emuhacks, we have to make a copy.
+		// If we could hash while reading we could avoid this.
 		std::vector<u32> buffer;
 		buffer.resize(origSize_ / 4);
 		size_t pos = 0;
@@ -521,10 +681,8 @@ u64 IRBlock::CalculateHash() const {
 			MIPSOpcode instr = Memory::ReadUnchecked_Instruction(origAddr_ + off, false);
 			buffer[pos++] = instr.encoding;
 		}
-
 		return XXH3_64bits(&buffer[0], origSize_);
 	}
-
 	return 0;
 }
 
