@@ -42,6 +42,7 @@
 #define FsI(i) (currentMIPS->fs[i])
 #define PC (currentMIPS->pc)
 
+#define _SIMM16_SHL2 ((u32)(s32)(s16)(op & 0xFFFF) << 2)
 #define _RS   ((op>>21) & 0x1F)
 #define _RT   ((op>>16) & 0x1F)
 #define _RD   ((op>>11) & 0x1F)
@@ -94,13 +95,13 @@ namespace MIPSInt
 {
 	void Int_Cache(MIPSOpcode op)
 	{
-		int imm = SignExtend16ToS32(op & 0xFFFF);
+		int imm = SignExtend16ToS32(op);
 		int rs = _RS;
 		uint32_t addr = R(rs) + imm;
 		int func = (op >> 16) & 0x1F;
 
 		// Let's only report this once per run to be safe from impacting perf.
-		static bool reportedAlignment = false;
+		static bool loggedAlignment = false;
 
 		// It appears that a cache line is 0x40 (64) bytes, loops in games
 		// issue the cache instruction at that interval.
@@ -119,13 +120,14 @@ namespace MIPSInt
 				int size = 0x40 + (addr & 0x3F);
 				MIPSComp::jit->InvalidateCacheAt(alignedAddr, size);
 				// Using a bool to avoid locking/etc. in case it's slow.
-				if (!reportedAlignment && (addr & 0x3F) != 0) {
-					WARN_LOG_REPORT(JIT, "Unaligned icache invalidation of %08x (%08x + %d) at PC=%08x", addr, R(rs), imm, PC);
-					reportedAlignment = true;
+				if (!loggedAlignment && (addr & 0x3F) != 0) {
+					// These are seen exclusively in Lego games, and are really no big deal. Reporting removed.
+					WARN_LOG(Log::JIT, "Unaligned icache invalidation of %08x (%08x + %d) at PC=%08x", addr, R(rs), imm, PC);
+					loggedAlignment = true;
 				}
 				if (alignedAddr <= PC + 4 && alignedAddr + size >= PC - 4) {
 					// This is probably rare so we don't use a static bool.
-					WARN_LOG_REPORT_ONCE(icacheInvalidatePC, JIT, "Invalidating address near PC: %08x (%08x + %d) at PC=%08x", addr, R(rs), imm, PC);
+					WARN_LOG_REPORT_ONCE(icacheInvalidatePC, Log::JIT, "Invalidating address near PC: %08x (%08x + %d) at PC=%08x", addr, R(rs), imm, PC);
 				}
 			}
 			break;
@@ -144,7 +146,7 @@ namespace MIPSInt
 			break;
 
 		default:
-			DEBUG_LOG(CPU, "cache instruction affecting %08x : function %i", addr, func);
+			DEBUG_LOG(Log::CPU, "cache instruction affecting %08x : function %i", addr, func);
 		}
 
 		PC += 4;
@@ -153,7 +155,7 @@ namespace MIPSInt
 	void Int_Syscall(MIPSOpcode op)
 	{
 		// Need to pre-move PC, as CallSyscall may result in a rescheduling!
-		// To do this neater, we'll need a little generated kernel loop that syscall can jump to and then RFI from 
+		// To do this neater, we'll need a little generated kernel loop that syscall can jump to and then RFI from
 		// but I don't see a need to bother.
 		if (mipsr4k.inDelaySlot)
 		{
@@ -169,7 +171,7 @@ namespace MIPSInt
 
 	void Int_Sync(MIPSOpcode op)
 	{
-		//DEBUG_LOG(CPU, "sync");
+		//DEBUG_LOG(Log::CPU, "sync");
 		PC += 4;
 	}
 
@@ -182,12 +184,12 @@ namespace MIPSInt
 
 	void Int_RelBranch(MIPSOpcode op)
 	{
-		int imm = (signed short)(op&0xFFFF)<<2;
+		int imm = _SIMM16_SHL2;
 		int rs = _RS;
 		int rt = _RT;
 		u32 addr = PC + imm + 4;
 
-		switch (op >> 26) 
+		switch (op >> 26)
 		{
 		case 4:  if (R(rt) == R(rs))  DelayBranchTo(addr); else PC += 4; break; //beq
 		case 5:  if (R(rt) != R(rs))  DelayBranchTo(addr); else PC += 4; break; //bne
@@ -207,7 +209,7 @@ namespace MIPSInt
 
 	void Int_RelBranchRI(MIPSOpcode op)
 	{
-		int imm = (signed short)(op&0xFFFF)<<2;
+		int imm = _SIMM16_SHL2;
 		int rs = _RS;
 		u32 addr = PC + imm + 4;
 
@@ -230,7 +232,7 @@ namespace MIPSInt
 
 	void Int_VBranch(MIPSOpcode op)
 	{
-		int imm = (signed short)(op&0xFFFF)<<2;
+		int imm = _SIMM16_SHL2;
 		u32 addr = PC + imm + 4;
 
 		// x, y, z, w, any, all, (invalid), (invalid)
@@ -248,7 +250,7 @@ namespace MIPSInt
 
 	void Int_FPUBranch(MIPSOpcode op)
 	{
-		int imm = (signed short)(op&0xFFFF)<<2;
+		int imm = _SIMM16_SHL2;
 		u32 addr = PC + imm + 4;
 		switch((op>>16)&0x1f)
 		{
@@ -261,16 +263,16 @@ namespace MIPSInt
 			break;
 		}
 	}
-	
+
 	void Int_JumpType(MIPSOpcode op)
 	{
 		if (mipsr4k.inDelaySlot)
-			ERROR_LOG(CPU, "Jump in delay slot :(");
+			ERROR_LOG(Log::CPU, "Jump in delay slot :(");
 
 		u32 off = ((op & 0x03FFFFFF) << 2);
 		u32 addr = (currentMIPS->pc & 0xF0000000) | off;
 
-		switch (op>>26) 
+		switch (op>>26)
 		{
 		case 2: //j
 			if (!mipsr4k.inDelaySlot)
@@ -292,13 +294,13 @@ namespace MIPSInt
 		if (mipsr4k.inDelaySlot)
 		{
 			// There's one of these in Star Soldier at 0881808c, which seems benign.
-			ERROR_LOG(CPU, "Jump in delay slot :(");
+			ERROR_LOG(Log::CPU, "Jump in delay slot :(");
 		}
 
 		int rs = _RS;
 		int rd = _RD;
 		u32 addr = R(rs);
-		switch (op & 0x3f) 
+		switch (op & 0x3f)
 		{
 		case 8: //jr
 			if (!mipsr4k.inDelaySlot)
@@ -328,7 +330,7 @@ namespace MIPSInt
 			return; //nop
 		}
 
-		switch (op>>26) 
+		switch (op>>26)
 		{
 		case 8:	R(rt) = R(rs) + simm; break; //addi
 		case 9:	R(rt) = R(rs) + simm; break;	//addiu
@@ -391,7 +393,7 @@ namespace MIPSInt
 			return;
 		}
 
-		switch (op & 63) 
+		switch (op & 63)
 		{
 		case 10: if (R(rt) == 0) R(rd) = R(rs); break; //movz
 		case 11: if (R(rt) != 0) R(rd) = R(rs); break; //movn
@@ -428,7 +430,7 @@ namespace MIPSInt
 			return;
 		}
 
-		switch (op >> 26) 
+		switch (op >> 26)
 		{
 		case 32: R(rt) = SignExtend8ToU32(Memory::Read_U8(addr)); break; //lb
 		case 33: R(rt) = SignExtend16ToU32(Memory::Read_U16(addr)); break; //lh
@@ -522,7 +524,7 @@ namespace MIPSInt
 				} else if (fs == 0) {
 					R(rt) = MIPSState::FCR0_VALUE;
 				} else {
-					WARN_LOG_REPORT(CPU, "ReadFCR: Unexpected reg %d", fs);
+					WARN_LOG_REPORT(Log::CPU, "ReadFCR: Unexpected reg %d", fs);
 					R(rt) = 0;
 				}
 				break;
@@ -544,12 +546,12 @@ namespace MIPSInt
 						MIPSComp::jit->UpdateFCR31();
 					}
 				} else {
-					WARN_LOG_REPORT(CPU, "WriteFCR: Unexpected reg %d (value %08x)", fs, value);
+					WARN_LOG_REPORT(Log::CPU, "WriteFCR: Unexpected reg %d (value %08x)", fs, value);
 				}
-				DEBUG_LOG(CPU, "FCR%i written to, value %08x", fs, value);
+				DEBUG_LOG(Log::CPU, "FCR%i written to, value %08x", fs, value);
 				break;
 			}
-		
+
 		default:
 			_dbg_assert_msg_(false,"Trying to interpret instruction that can't be interpreted");
 			break;
@@ -590,7 +592,7 @@ namespace MIPSInt
 		int rs = _RS;
 		int rd = _RD;
 
-		switch (op & 63) 
+		switch (op & 63)
 		{
 		case 24: //mult
 			{
@@ -702,16 +704,16 @@ namespace MIPSInt
 			PC += 4;
 			return;
 		}
-		
+
 		switch (op & 0x3f)
 		{
 		case 0: R(rd) = R(rt) << sa;					 break; //sll
-		case 2: 
+		case 2:
 			if (_RS == 0) //srl
 			{
 				R(rd) = R(rt) >> sa;
-				break; 
-			} 
+				break;
+			}
 			else if (_RS == 1) //rotr
 			{
 				R(rd) = __rotr(R(rt), sa);
@@ -726,7 +728,7 @@ namespace MIPSInt
 			if (_FD == 0) //srlv
 			{
 				R(rd) = R(rt) >> (R(rs)&0x1F);
-				break; 
+				break;
 			}
 			else if (_FD == 1) // rotrv
 			{
@@ -821,14 +823,14 @@ namespace MIPSInt
 		case 36:  // mfic
 			if (!reported) {
 				Reporting::ReportMessage("MFIC instruction hit (%08x) at %08x", op.encoding, currentMIPS->pc);
-				WARN_LOG(CPU,"MFIC Disable/Enable Interrupt CPU instruction");
+				WARN_LOG(Log::CPU,"MFIC Disable/Enable Interrupt CPU instruction");
 				reported = 1;
 			}
 			break;
 		case 38:  // mtic
 			if (!reported) {
 				Reporting::ReportMessage("MTIC instruction hit (%08x) at %08x", op.encoding, currentMIPS->pc);
-				WARN_LOG(CPU,"MTIC Disable/Enable Interrupt CPU instruction");
+				WARN_LOG(Log::CPU,"MTIC Disable/Enable Interrupt CPU instruction");
 				reported = 1;
 			}
 			break;
@@ -1021,7 +1023,7 @@ namespace MIPSInt
 		case 0:
 			if (!reported) {
 				Reporting::ReportMessage("INTERRUPT instruction hit (%08x) at %08x", op.encoding, currentMIPS->pc);
-				WARN_LOG(CPU,"Disable/Enable Interrupt CPU instruction");
+				WARN_LOG(Log::CPU,"Disable/Enable Interrupt CPU instruction");
 				reported = 1;
 			}
 			break;
@@ -1038,17 +1040,21 @@ namespace MIPSInt
 		int index = op.encoding & 0xFFFFFF;
 		const ReplacementTableEntry *entry = GetReplacementFunc(index);
 		if (entry && entry->replaceFunc && (entry->flags & REPFLAG_DISABLED) == 0) {
-			entry->replaceFunc();
+			int cycles = entry->replaceFunc();
 
 			if (entry->flags & (REPFLAG_HOOKENTER | REPFLAG_HOOKEXIT)) {
 				// Interpret the original instruction under the hook.
 				MIPSInterpret(Memory::Read_Instruction(PC, true));
+			} else if (cycles < 0) {
+				// Leave PC unchanged, call the replacement again (assumes args are modified.)
+				currentMIPS->downcount += cycles;
 			} else {
 				PC = currentMIPS->r[MIPS_REG_RA];
+				currentMIPS->downcount -= cycles;
 			}
 		} else {
 			if (!entry || !entry->replaceFunc) {
-				ERROR_LOG(CPU, "Bad replacement function index %i", index);
+				ERROR_LOG(Log::CPU, "Bad replacement function index %i", index);
 			}
 			// Interpret the original instruction under it.
 			MIPSInterpret(Memory::Read_Instruction(PC, true));
