@@ -18,8 +18,10 @@
 #include <cstring>
 
 #include "ppsspp_config.h"
-#include "CPUDetect.h"
-#include "Common.h"
+
+#include "Common/CPUDetect.h"
+#include "Common/Common.h"
+#include "Common/Log.h"
 
 #ifdef _M_SSE
 #include <emmintrin.h>
@@ -32,7 +34,8 @@
 #include <arm_neon.h>
 #endif
 #endif
-#include "IndexGenerator.h"
+
+#include "GPU/Common/IndexGenerator.h"
 
 // Points don't need indexing...
 const u8 IndexGenerator::indexedPrimitiveType[7] = {
@@ -45,67 +48,46 @@ const u8 IndexGenerator::indexedPrimitiveType[7] = {
 	GE_PRIM_RECTANGLES,
 };
 
-void IndexGenerator::Setup(u16 *inds) {
-	this->indsBase_ = inds;
-	Reset();
-}
-
-void IndexGenerator::AddPrim(int prim, int vertexCount, bool clockwise) {
+void IndexGenerator::AddPrim(int prim, int vertexCount, int indexOffset, bool clockwise) {
 	switch (prim) {
-	case GE_PRIM_POINTS: AddPoints(vertexCount); break;
-	case GE_PRIM_LINES: AddLineList(vertexCount); break;
-	case GE_PRIM_LINE_STRIP: AddLineStrip(vertexCount); break;
-	case GE_PRIM_TRIANGLES: AddList(vertexCount, clockwise); break;
-	case GE_PRIM_TRIANGLE_STRIP: AddStrip(vertexCount, clockwise); break;
-	case GE_PRIM_TRIANGLE_FAN: AddFan(vertexCount, clockwise); break;
-	case GE_PRIM_RECTANGLES: AddRectangles(vertexCount); break;  // Same
+	case GE_PRIM_POINTS: AddPoints(vertexCount, indexOffset); break;
+	case GE_PRIM_LINES: AddLineList(vertexCount, indexOffset); break;
+	case GE_PRIM_LINE_STRIP: AddLineStrip(vertexCount, indexOffset); break;
+	case GE_PRIM_TRIANGLES: AddList(vertexCount, indexOffset, clockwise); break;
+	case GE_PRIM_TRIANGLE_STRIP: AddStrip(vertexCount, indexOffset, clockwise); break;
+	case GE_PRIM_TRIANGLE_FAN: AddFan(vertexCount, indexOffset, clockwise); break;
+	case GE_PRIM_RECTANGLES: AddRectangles(vertexCount, indexOffset); break;  // Same
 	}
 }
 
-void IndexGenerator::AddPoints(int numVerts) {
+void IndexGenerator::AddPoints(int numVerts, int indexOffset) {
 	u16 *outInds = inds_;
-	const int startIndex = index_;
 	for (int i = 0; i < numVerts; i++)
-		*outInds++ = startIndex + i;
+		*outInds++ = indexOffset + i;
 	inds_ = outInds;
-	// ignore overflow verts
-	index_ += numVerts;
-	count_ += numVerts;
-	prim_ = GE_PRIM_POINTS;
-	seenPrims_ |= 1 << GE_PRIM_POINTS;
 }
 
-void IndexGenerator::AddList(int numVerts, bool clockwise) {
+void IndexGenerator::AddList(int numVerts, int indexOffset, bool clockwise) {
 	u16 *outInds = inds_;
-	const int startIndex = index_;
 	const int v1 = clockwise ? 1 : 2;
 	const int v2 = clockwise ? 2 : 1;
 	for (int i = 0; i < numVerts; i += 3) {
-		*outInds++ = startIndex + i;
-		*outInds++ = startIndex + i + v1;
-		*outInds++ = startIndex + i + v2;
+		*outInds++ = indexOffset + i;
+		*outInds++ = indexOffset + i + v1;
+		*outInds++ = indexOffset + i + v2;
 	}
 	inds_ = outInds;
-	// ignore overflow verts
-	index_ += numVerts;
-	count_ += numVerts;
-	prim_ = GE_PRIM_TRIANGLES;
-	seenPrims_ |= 1 << GE_PRIM_TRIANGLES;
-	if (!clockwise) {
-		// Make sure we don't treat this as pure.
-		seenPrims_ |= 1 << GE_PRIM_TRIANGLE_STRIP;
-	}
 }
 
 alignas(16) static const u16 offsets_clockwise[24] = {
 	0, (u16)(0 + 1), (u16)(0 + 2),
-	1, (u16)(1 + 2), (u16)(1 + 1),
+	(u16)(1 + 1), 1, (u16)(1 + 2),
 	2, (u16)(2 + 1), (u16)(2 + 2),
-	3, (u16)(3 + 2), (u16)(3 + 1),
+	(u16)(3 + 1), 3, (u16)(3 + 2),
 	4, (u16)(4 + 1), (u16)(4 + 2),
-	5, (u16)(5 + 2), (u16)(5 + 1),
+	(u16)(5 + 1), 5, (u16)(5 + 2),
 	6, (u16)(6 + 1), (u16)(6 + 2),
-	7, (u16)(7 + 2), (u16)(7 + 1),
+	(u16)(7 + 1), 7, (u16)(7 + 2),
 };
 
 alignas(16) static const uint16_t offsets_counter_clockwise[24] = {
@@ -119,8 +101,11 @@ alignas(16) static const uint16_t offsets_counter_clockwise[24] = {
 	7, (u16)(7 + 1), (u16)(7 + 2),
 };
 
-void IndexGenerator::AddStrip(int numVerts, bool clockwise) {
+void IndexGenerator::AddStrip(int numVerts, int indexOffset, bool clockwise) {
 	int numTris = numVerts - 2;
+	if (numTris <= 0) {
+		return;
+	}
 #ifdef _M_SSE
 	// In an SSE2 register we can fit 8 16-bit integers.
 	// However, we need to output a multiple of 3 indices.
@@ -129,8 +114,7 @@ void IndexGenerator::AddStrip(int numVerts, bool clockwise) {
 
 	// We allow ourselves to write some extra indices to avoid the fallback loop.
 	// That's alright as we're appending to a buffer - they will get overwritten anyway.
-	int numChunks = (numTris + 7) >> 3;
-	__m128i ibase8 = _mm_set1_epi16(index_);
+	__m128i ibase8 = _mm_set1_epi16(indexOffset);
 	const __m128i *offsets = (const __m128i *)(clockwise ? offsets_clockwise : offsets_counter_clockwise);
 	__m128i *dst = (__m128i *)inds_;
 	__m128i offsets0 = _mm_add_epi16(ibase8, _mm_load_si128(offsets));
@@ -143,6 +127,7 @@ void IndexGenerator::AddStrip(int numVerts, bool clockwise) {
 			__m128i offsets2 = _mm_add_epi16(ibase8, _mm_load_si128(offsets + 2));
 			_mm_storeu_si128(dst + 2, offsets2);
 			__m128i increment = _mm_set1_epi16(8);
+			int numChunks = (numTris + 7) >> 3;
 			for (int i = 1; i < numChunks; i++) {
 				dst += 3;
 				offsets0 = _mm_add_epi16(offsets0, increment);
@@ -157,8 +142,7 @@ void IndexGenerator::AddStrip(int numVerts, bool clockwise) {
 	inds_ += numTris * 3;
 	// wind doesn't need to be updated, an even number of triangles have been drawn.
 #elif PPSSPP_ARCH(ARM_NEON)
-	int numChunks = (numTris + 7) >> 3;
-	uint16x8_t ibase8 = vdupq_n_u16(index_);
+	uint16x8_t ibase8 = vdupq_n_u16(indexOffset);
 	const u16 *offsets = clockwise ? offsets_clockwise : offsets_counter_clockwise;
 	u16 *dst = inds_;
 	uint16x8_t offsets0 = vaddq_u16(ibase8, vld1q_u16(offsets));
@@ -170,6 +154,7 @@ void IndexGenerator::AddStrip(int numVerts, bool clockwise) {
 			uint16x8_t offsets2 = vaddq_u16(ibase8, vld1q_u16(offsets + 16));
 			vst1q_u16(dst + 16, offsets2);
 			uint16x8_t increment = vdupq_n_u16(8);
+			int numChunks = (numTris + 7) >> 3;
 			for (int i = 1; i < numChunks; i++) {
 				dst += 3 * 8;
 				offsets0 = vaddq_u16(offsets0, increment);
@@ -185,7 +170,7 @@ void IndexGenerator::AddStrip(int numVerts, bool clockwise) {
 #else
 	// Slow fallback loop.
 	int wind = clockwise ? 1 : 2;
-	int ibase = index_;
+	int ibase = indexOffset;
 	size_t numPairs = numTris / 2;
 	u16 *outInds = inds_;
 	while (numPairs > 0) {
@@ -206,105 +191,63 @@ void IndexGenerator::AddStrip(int numVerts, bool clockwise) {
 	}
 	inds_ = outInds;
 #endif
-
-	index_ += numVerts;
-	if (numTris > 0)
-		count_ += numTris * 3;
-	// This is so we can detect one single strip by just looking at seenPrims_.
-	if (!seenPrims_ && clockwise) {
-		seenPrims_ = 1 << GE_PRIM_TRIANGLE_STRIP;
-		prim_ = GE_PRIM_TRIANGLE_STRIP;
-		pureCount_ = numVerts;
-	} else {
-		seenPrims_ |= (1 << GE_PRIM_TRIANGLE_STRIP) | (1 << GE_PRIM_TRIANGLES);
-		prim_ = GE_PRIM_TRIANGLES;
-		pureCount_ = 0;
-	}
 }
 
-void IndexGenerator::AddFan(int numVerts, bool clockwise) {
+void IndexGenerator::AddFan(int numVerts, int indexOffset, bool clockwise) {
 	const int numTris = numVerts - 2;
 	u16 *outInds = inds_;
-	const int startIndex = index_;
 	const int v1 = clockwise ? 1 : 2;
 	const int v2 = clockwise ? 2 : 1;
 	for (int i = 0; i < numTris; i++) {
-		*outInds++ = startIndex;
-		*outInds++ = startIndex + i + v1;
-		*outInds++ = startIndex + i + v2;
+		*outInds++ = indexOffset;
+		*outInds++ = indexOffset + i + v1;
+		*outInds++ = indexOffset + i + v2;
 	}
 	inds_ = outInds;
-	index_ += numVerts;
-	count_ += numTris * 3;
-	prim_ = GE_PRIM_TRIANGLES;
-	seenPrims_ |= 1 << GE_PRIM_TRIANGLE_FAN;
-	if (!clockwise) {
-		// Make sure we don't treat this as pure.
-		seenPrims_ |= 1 << GE_PRIM_TRIANGLE_STRIP;
-	}
 }
 
 //Lines
-void IndexGenerator::AddLineList(int numVerts) {
+void IndexGenerator::AddLineList(int numVerts, int indexOffset) {
 	u16 *outInds = inds_;
-	const int startIndex = index_;
+	numVerts &= ~1;
 	for (int i = 0; i < numVerts; i += 2) {
-		*outInds++ = startIndex + i;
-		*outInds++ = startIndex + i + 1;
+		*outInds++ = indexOffset + i;
+		*outInds++ = indexOffset + i + 1;
 	}
 	inds_ = outInds;
-	index_ += numVerts;
-	count_ += numVerts;
-	prim_ = GE_PRIM_LINES;
-	seenPrims_ |= 1 << prim_;
 }
 
-void IndexGenerator::AddLineStrip(int numVerts) {
+void IndexGenerator::AddLineStrip(int numVerts, int indexOffset) {
 	const int numLines = numVerts - 1;
 	u16 *outInds = inds_;
-	const int startIndex = index_;
 	for (int i = 0; i < numLines; i++) {
-		*outInds++ = startIndex + i;
-		*outInds++ = startIndex + i + 1;
+		*outInds++ = indexOffset + i;
+		*outInds++ = indexOffset + i + 1;
 	}
 	inds_ = outInds;
-	index_ += numVerts;
-	count_ += numLines * 2;
-	prim_ = GE_PRIM_LINES;
-	seenPrims_ |= 1 << GE_PRIM_LINE_STRIP;
 }
 
-void IndexGenerator::AddRectangles(int numVerts) {
+void IndexGenerator::AddRectangles(int numVerts, int indexOffset) {
 	u16 *outInds = inds_;
-	const int startIndex = index_;
 	//rectangles always need 2 vertices, disregard the last one if there's an odd number
 	numVerts = numVerts & ~1;
 	for (int i = 0; i < numVerts; i += 2) {
-		*outInds++ = startIndex + i;
-		*outInds++ = startIndex + i + 1;
+		*outInds++ = indexOffset + i;
+		*outInds++ = indexOffset + i + 1;
 	}
 	inds_ = outInds;
-	index_ += numVerts;
-	count_ += numVerts;
-	prim_ = GE_PRIM_RECTANGLES;
-	seenPrims_ |= 1 << GE_PRIM_RECTANGLES;
 }
 
-template <class ITypeLE, int flag>
+template <class ITypeLE>
 void IndexGenerator::TranslatePoints(int numInds, const ITypeLE *inds, int indexOffset) {
-	indexOffset = index_ - indexOffset;
 	u16 *outInds = inds_;
 	for (int i = 0; i < numInds; i++)
 		*outInds++ = indexOffset + inds[i];
 	inds_ = outInds;
-	count_ += numInds;
-	prim_ = GE_PRIM_POINTS;
-	seenPrims_ |= (1 << GE_PRIM_POINTS) | flag;
 }
 
-template <class ITypeLE, int flag>
+template <class ITypeLE>
 void IndexGenerator::TranslateLineList(int numInds, const ITypeLE *inds, int indexOffset) {
-	indexOffset = index_ - indexOffset;
 	u16 *outInds = inds_;
 	numInds = numInds & ~1;
 	for (int i = 0; i < numInds; i += 2) {
@@ -312,14 +255,10 @@ void IndexGenerator::TranslateLineList(int numInds, const ITypeLE *inds, int ind
 		*outInds++ = indexOffset + inds[i + 1];
 	}
 	inds_ = outInds;
-	count_ += numInds;
-	prim_ = GE_PRIM_LINES;
-	seenPrims_ |= (1 << GE_PRIM_LINES) | flag;
 }
 
-template <class ITypeLE, int flag>
+template <class ITypeLE>
 void IndexGenerator::TranslateLineStrip(int numInds, const ITypeLE *inds, int indexOffset) {
-	indexOffset = index_ - indexOffset;
 	int numLines = numInds - 1;
 	u16 *outInds = inds_;
 	for (int i = 0; i < numLines; i++) {
@@ -327,42 +266,34 @@ void IndexGenerator::TranslateLineStrip(int numInds, const ITypeLE *inds, int in
 		*outInds++ = indexOffset + inds[i + 1];
 	}
 	inds_ = outInds;
-	count_ += numLines * 2;
-	prim_ = GE_PRIM_LINES;
-	seenPrims_ |= (1 << GE_PRIM_LINE_STRIP) | flag;
 }
 
-template <class ITypeLE, int flag>
+template <class ITypeLE>
 void IndexGenerator::TranslateList(int numInds, const ITypeLE *inds, int indexOffset, bool clockwise) {
-	indexOffset = index_ - indexOffset;
 	// We only bother doing this minor optimization in triangle list, since it's by far the most
 	// common operation that can benefit.
 	if (sizeof(ITypeLE) == sizeof(inds_[0]) && indexOffset == 0 && clockwise) {
 		memcpy(inds_, inds, numInds * sizeof(ITypeLE));
 		inds_ += numInds;
-		count_ += numInds;
 	} else {
 		u16 *outInds = inds_;
 		int numTris = numInds / 3;  // Round to whole triangles
 		numInds = numTris * 3;
 		const int v1 = clockwise ? 1 : 2;
 		const int v2 = clockwise ? 2 : 1;
+		// TODO: This can actually be SIMD-d, although will need complex shuffles if clockwise.
 		for (int i = 0; i < numInds; i += 3) {
 			*outInds++ = indexOffset + inds[i];
 			*outInds++ = indexOffset + inds[i + v1];
 			*outInds++ = indexOffset + inds[i + v2];
 		}
 		inds_ = outInds;
-		count_ += numInds;
 	}
-	prim_ = GE_PRIM_TRIANGLES;
-	seenPrims_ |= (1 << GE_PRIM_TRIANGLES) | flag;
 }
 
-template <class ITypeLE, int flag>
+template <class ITypeLE>
 void IndexGenerator::TranslateStrip(int numInds, const ITypeLE *inds, int indexOffset, bool clockwise) {
 	int wind = clockwise ? 1 : 2;
-	indexOffset = index_ - indexOffset;
 	int numTris = numInds - 2;
 	u16 *outInds = inds_;
 	for (int i = 0; i < numTris; i++) {
@@ -372,15 +303,11 @@ void IndexGenerator::TranslateStrip(int numInds, const ITypeLE *inds, int indexO
 		*outInds++ = indexOffset + inds[i + wind];
 	}
 	inds_ = outInds;
-	count_ += numTris * 3;
-	prim_ = GE_PRIM_TRIANGLES;
-	seenPrims_ |= (1 << GE_PRIM_TRIANGLE_STRIP) | flag;
 }
 
-template <class ITypeLE, int flag>
+template <class ITypeLE>
 void IndexGenerator::TranslateFan(int numInds, const ITypeLE *inds, int indexOffset, bool clockwise) {
 	if (numInds <= 0) return;
-	indexOffset = index_ - indexOffset;
 	int numTris = numInds - 2;
 	u16 *outInds = inds_;
 	const int v1 = clockwise ? 1 : 2;
@@ -391,14 +318,10 @@ void IndexGenerator::TranslateFan(int numInds, const ITypeLE *inds, int indexOff
 		*outInds++ = indexOffset + inds[i + v2];
 	}
 	inds_ = outInds;
-	count_ += numTris * 3;
-	prim_ = GE_PRIM_TRIANGLES;
-	seenPrims_ |= (1 << GE_PRIM_TRIANGLE_FAN) | flag;
 }
 
-template <class ITypeLE, int flag>
+template <class ITypeLE>
 inline void IndexGenerator::TranslateRectangles(int numInds, const ITypeLE *inds, int indexOffset) {
-	indexOffset = index_ - indexOffset;
 	u16 *outInds = inds_;
 	//rectangles always need 2 vertices, disregard the last one if there's an odd number
 	numInds = numInds & ~1;
@@ -407,44 +330,41 @@ inline void IndexGenerator::TranslateRectangles(int numInds, const ITypeLE *inds
 		*outInds++ = indexOffset + inds[i+1];
 	}
 	inds_ = outInds;
-	count_ += numInds;
-	prim_ = GE_PRIM_RECTANGLES;
-	seenPrims_ |= (1 << GE_PRIM_RECTANGLES) | flag;
 }
 
 // Could template this too, but would have to define in header.
 void IndexGenerator::TranslatePrim(int prim, int numInds, const u8 *inds, int indexOffset, bool clockwise) {
 	switch (prim) {
-	case GE_PRIM_POINTS: TranslatePoints<u8, SEEN_INDEX8>(numInds, inds, indexOffset); break;
-	case GE_PRIM_LINES: TranslateLineList<u8, SEEN_INDEX8>(numInds, inds, indexOffset); break;
-	case GE_PRIM_LINE_STRIP: TranslateLineStrip<u8, SEEN_INDEX8>(numInds, inds, indexOffset); break;
-	case GE_PRIM_TRIANGLES: TranslateList<u8, SEEN_INDEX8>(numInds, inds, indexOffset, clockwise); break;
-	case GE_PRIM_TRIANGLE_STRIP: TranslateStrip<u8, SEEN_INDEX8>(numInds, inds, indexOffset, clockwise); break;
-	case GE_PRIM_TRIANGLE_FAN: TranslateFan<u8, SEEN_INDEX8>(numInds, inds, indexOffset, clockwise); break;
-	case GE_PRIM_RECTANGLES: TranslateRectangles<u8, SEEN_INDEX8>(numInds, inds, indexOffset); break;  // Same
+	case GE_PRIM_POINTS: TranslatePoints<u8>(numInds, inds, indexOffset); break;
+	case GE_PRIM_LINES: TranslateLineList<u8>(numInds, inds, indexOffset); break;
+	case GE_PRIM_LINE_STRIP: TranslateLineStrip<u8>(numInds, inds, indexOffset); break;
+	case GE_PRIM_TRIANGLES: TranslateList<u8>(numInds, inds, indexOffset, clockwise); break;
+	case GE_PRIM_TRIANGLE_STRIP: TranslateStrip<u8>(numInds, inds, indexOffset, clockwise); break;
+	case GE_PRIM_TRIANGLE_FAN: TranslateFan<u8>(numInds, inds, indexOffset, clockwise); break;
+	case GE_PRIM_RECTANGLES: TranslateRectangles<u8>(numInds, inds, indexOffset); break;  // Same
 	}
 }
 
 void IndexGenerator::TranslatePrim(int prim, int numInds, const u16_le *inds, int indexOffset, bool clockwise) {
 	switch (prim) {
-	case GE_PRIM_POINTS: TranslatePoints<u16_le, SEEN_INDEX16>(numInds, inds, indexOffset); break;
-	case GE_PRIM_LINES: TranslateLineList<u16_le, SEEN_INDEX16>(numInds, inds, indexOffset); break;
-	case GE_PRIM_LINE_STRIP: TranslateLineStrip<u16_le, SEEN_INDEX16>(numInds, inds, indexOffset); break;
-	case GE_PRIM_TRIANGLES: TranslateList<u16_le, SEEN_INDEX16>(numInds, inds, indexOffset, clockwise); break;
-	case GE_PRIM_TRIANGLE_STRIP: TranslateStrip<u16_le, SEEN_INDEX16>(numInds, inds, indexOffset, clockwise); break;
-	case GE_PRIM_TRIANGLE_FAN: TranslateFan<u16_le, SEEN_INDEX16>(numInds, inds, indexOffset, clockwise); break;
-	case GE_PRIM_RECTANGLES: TranslateRectangles<u16_le, SEEN_INDEX16>(numInds, inds, indexOffset); break;  // Same
+	case GE_PRIM_POINTS: TranslatePoints<u16_le>(numInds, inds, indexOffset); break;
+	case GE_PRIM_LINES: TranslateLineList<u16_le>(numInds, inds, indexOffset); break;
+	case GE_PRIM_LINE_STRIP: TranslateLineStrip<u16_le>(numInds, inds, indexOffset); break;
+	case GE_PRIM_TRIANGLES: TranslateList<u16_le>(numInds, inds, indexOffset, clockwise); break;
+	case GE_PRIM_TRIANGLE_STRIP: TranslateStrip<u16_le>(numInds, inds, indexOffset, clockwise); break;
+	case GE_PRIM_TRIANGLE_FAN: TranslateFan<u16_le>(numInds, inds, indexOffset, clockwise); break;
+	case GE_PRIM_RECTANGLES: TranslateRectangles<u16_le>(numInds, inds, indexOffset); break;  // Same
 	}
 }
 
 void IndexGenerator::TranslatePrim(int prim, int numInds, const u32_le *inds, int indexOffset, bool clockwise) {
 	switch (prim) {
-	case GE_PRIM_POINTS: TranslatePoints<u32_le, SEEN_INDEX32>(numInds, inds, indexOffset); break;
-	case GE_PRIM_LINES: TranslateLineList<u32_le, SEEN_INDEX32>(numInds, inds, indexOffset); break;
-	case GE_PRIM_LINE_STRIP: TranslateLineStrip<u32_le, SEEN_INDEX32>(numInds, inds, indexOffset); break;
-	case GE_PRIM_TRIANGLES: TranslateList<u32_le, SEEN_INDEX32>(numInds, inds, indexOffset, clockwise); break;
-	case GE_PRIM_TRIANGLE_STRIP: TranslateStrip<u32_le, SEEN_INDEX32>(numInds, inds, indexOffset, clockwise); break;
-	case GE_PRIM_TRIANGLE_FAN: TranslateFan<u32_le, SEEN_INDEX32>(numInds, inds, indexOffset, clockwise); break;
-	case GE_PRIM_RECTANGLES: TranslateRectangles<u32_le, SEEN_INDEX32>(numInds, inds, indexOffset); break;  // Same
+	case GE_PRIM_POINTS: TranslatePoints<u32_le>(numInds, inds, indexOffset); break;
+	case GE_PRIM_LINES: TranslateLineList<u32_le>(numInds, inds, indexOffset); break;
+	case GE_PRIM_LINE_STRIP: TranslateLineStrip<u32_le>(numInds, inds, indexOffset); break;
+	case GE_PRIM_TRIANGLES: TranslateList<u32_le>(numInds, inds, indexOffset, clockwise); break;
+	case GE_PRIM_TRIANGLE_STRIP: TranslateStrip<u32_le>(numInds, inds, indexOffset, clockwise); break;
+	case GE_PRIM_TRIANGLE_FAN: TranslateFan<u32_le>(numInds, inds, indexOffset, clockwise); break;
+	case GE_PRIM_RECTANGLES: TranslateRectangles<u32_le>(numInds, inds, indexOffset); break;  // Same
 	}
 }
