@@ -18,7 +18,9 @@
 #include <d3d11.h>
 #include <d3d11_1.h>
 
-#include "math/dataconv.h"
+#include <algorithm>
+
+#include "Common/Data/Convert/SmallDataConvert.h"
 
 #include "GPU/Math3D.h"
 #include "GPU/GPUState.h"
@@ -26,9 +28,8 @@
 #include "GPU/Common/GPUStateUtils.h"
 #include "Core/System.h"
 #include "Core/Config.h"
-#include "Core/Reporting.h"
 
-#include "GPU/Common/FramebufferCommon.h"
+#include "GPU/Common/FramebufferManagerCommon.h"
 #include "GPU/D3D11/DrawEngineD3D11.h"
 #include "GPU/D3D11/StateMappingD3D11.h"
 #include "GPU/D3D11/FramebufferManagerD3D11.h"
@@ -92,13 +93,13 @@ static const D3D11_STENCIL_OP stencilOps[] = {
 };
 
 static const D3D11_PRIMITIVE_TOPOLOGY primToD3D11[8] = {
-	D3D11_PRIMITIVE_TOPOLOGY_POINTLIST,
-	D3D11_PRIMITIVE_TOPOLOGY_LINELIST,
-	D3D11_PRIMITIVE_TOPOLOGY_LINESTRIP,
+	D3D11_PRIMITIVE_TOPOLOGY_TRIANGLELIST, // Points are expanded to triangles.
+	D3D11_PRIMITIVE_TOPOLOGY_TRIANGLELIST, // Lines are expanded to triangles too.
+	D3D11_PRIMITIVE_TOPOLOGY_TRIANGLELIST, // Lines are expanded to triangles too.
 	D3D11_PRIMITIVE_TOPOLOGY_TRIANGLELIST,
 	D3D11_PRIMITIVE_TOPOLOGY_TRIANGLESTRIP,
 	D3D11_PRIMITIVE_TOPOLOGY_UNDEFINED,  // D3D11 doesn't do triangle fans.
-	D3D11_PRIMITIVE_TOPOLOGY_TRIANGLELIST,
+	D3D11_PRIMITIVE_TOPOLOGY_TRIANGLELIST, // Rectangles are expanded to triangles.
 };
 
 static const D3D11_LOGIC_OP logicOps[] = {
@@ -120,14 +121,6 @@ static const D3D11_LOGIC_OP logicOps[] = {
 	D3D11_LOGIC_OP_SET,
 };
 
-void DrawEngineD3D11::ResetShaderBlending() {
-	if (fboTexBound_) {
-		ID3D11ShaderResourceView *srv = nullptr;
-		context_->PSSetShaderResources(0, 1, &srv);
-		fboTexBound_ = false;
-	}
-}
-
 class FramebufferManagerD3D11;
 class ShaderManagerD3D11;
 
@@ -139,10 +132,9 @@ void DrawEngineD3D11::ApplyDrawState(int prim) {
 		return;
 	}
 
-	bool useBufferedRendering = g_Config.iRenderingMode != FB_NON_BUFFERED_MODE;
+	bool useBufferedRendering = framebufferManager_->UseBufferedRendering();
 	// Blend
 	if (gstate_c.IsDirty(DIRTY_BLEND_STATE)) {
-		gstate_c.SetAllowShaderBlend(!g_Config.bDisableSlowFramebufEffects);
 		if (gstate.isModeClear()) {
 			keys_.blend.value = 0;  // full wipe
 			keys_.blend.blendEnable = false;
@@ -153,23 +145,41 @@ void DrawEngineD3D11::ApplyDrawState(int prim) {
 			keys_.blend.colorWriteMask = (colorMask ? (1 | 2 | 4) : 0) | (alphaMask ? 8 : 0);
 		} else {
 			keys_.blend.value = 0;
-			// Set blend - unless we need to do it in the shader.
-			GenericBlendState blendState;
-			ConvertBlendState(blendState, gstate_c.allowShaderBlend);
-			if (blendState.applyShaderBlending) {
-				if (ApplyShaderBlending()) {
-					// We may still want to do something about stencil -> alpha.
-					ApplyStencilReplaceAndLogicOp(blendState.replaceAlphaWithStencil, blendState);
-				} else {
-					// Until next time, force it off.
-					ResetShaderBlending();
-					gstate_c.SetAllowShaderBlend(false);
+
+			pipelineState_.Convert(draw_->GetShaderLanguageDesc().bitwiseOps);
+			GenericMaskState &maskState = pipelineState_.maskState;
+			GenericBlendState &blendState = pipelineState_.blendState;
+			// We ignore the logicState on D3D since there's no support, the emulation of it is blend-and-shader only.
+
+			if (pipelineState_.FramebufferRead()) {
+				FBOTexState fboTexBindState = FBO_TEX_NONE;
+				ApplyFramebufferRead(&fboTexBindState);
+				// The shader takes over the responsibility for blending, so recompute.
+				ApplyStencilReplaceAndLogicOpIgnoreBlend(blendState.replaceAlphaWithStencil, blendState);
+
+				if (fboTexBindState == FBO_TEX_COPY_BIND_TEX) {
+					framebufferManager_->BindFramebufferAsColorTexture(1, framebufferManager_->GetCurrentRenderVFB(), BINDFBCOLOR_MAY_COPY | BINDFBCOLOR_UNCACHED, 0);
+					// No sampler required, we do a plain Load in the pixel shader.
+					fboTexBound_ = true;
+					fboTexBindState = FBO_TEX_NONE;
+
+					framebufferManager_->RebindFramebuffer("RebindFramebuffer - ApplyDrawState");
+					// Must dirty blend state here so we re-copy next time.  Example: Lunar's spell effects.
+					dirtyRequiresRecheck_ |= DIRTY_BLEND_STATE;
+					gstate_c.Dirty(DIRTY_BLEND_STATE);
+				}
+
+				dirtyRequiresRecheck_ |= DIRTY_FRAGMENTSHADER_STATE;
+				gstate_c.Dirty(DIRTY_FRAGMENTSHADER_STATE);
+			} else {
+				if (fboTexBound_) {
+					fboTexBound_ = false;
+					dirtyRequiresRecheck_ |= DIRTY_FRAGMENTSHADER_STATE;
+					gstate_c.Dirty(DIRTY_FRAGMENTSHADER_STATE);
 				}
-			} else if (blendState.resetShaderBlending) {
-				ResetShaderBlending();
 			}
 
-			if (blendState.enabled) {
+			if (blendState.blendEnabled) {
 				keys_.blend.blendEnable = true;
 				keys_.blend.logicOpEnable = false;
 				keys_.blend.blendOpColor = d3d11BlendEqLookup[(size_t)blendState.eqColor];
@@ -178,7 +188,8 @@ void DrawEngineD3D11::ApplyDrawState(int prim) {
 				keys_.blend.srcAlpha = d3d11BlendFactorLookup[(size_t)blendState.srcAlpha];
 				keys_.blend.destColor = d3d11BlendFactorLookup[(size_t)blendState.dstColor];
 				keys_.blend.destAlpha = d3d11BlendFactorLookup[(size_t)blendState.dstAlpha];
-				if (blendState.dirtyShaderBlend) {
+				if (blendState.dirtyShaderBlendFixValues) {
+					dirtyRequiresRecheck_ |= DIRTY_SHADERBLEND;
 					gstate_c.Dirty(DIRTY_SHADERBLEND);
 				}
 				dynState_.useBlendColor = blendState.useBlendColor;
@@ -190,7 +201,7 @@ void DrawEngineD3D11::ApplyDrawState(int prim) {
 				dynState_.useBlendColor = false;
 			}
 
-			if (gstate_c.Supports(GPU_SUPPORTS_LOGIC_OP)) {
+			if (gstate_c.Use(GPU_USE_LOGIC_OP)) {
 				// Logic Ops
 				if (gstate.isLogicOpEnabled() && gstate.getLogicOp() != GE_LOGIC_COPY) {
 					keys_.blend.blendEnable = false;  // Can't have both blend & logic op - although I think the PSP can!
@@ -201,43 +212,152 @@ void DrawEngineD3D11::ApplyDrawState(int prim) {
 				}
 			}
 
-			// PSP color/alpha mask is per bit but we can only support per byte.
-			// But let's do that, at least. And let's try a threshold.
-			bool rmask = (gstate.pmskc & 0xFF) < 128;
-			bool gmask = ((gstate.pmskc >> 8) & 0xFF) < 128;
-			bool bmask = ((gstate.pmskc >> 16) & 0xFF) < 128;
-			bool amask = (gstate.pmska & 0xFF) < 128;
-
-#ifndef MOBILE_DEVICE
-			u8 abits = (gstate.pmska >> 0) & 0xFF;
-			u8 rbits = (gstate.pmskc >> 0) & 0xFF;
-			u8 gbits = (gstate.pmskc >> 8) & 0xFF;
-			u8 bbits = (gstate.pmskc >> 16) & 0xFF;
-			if ((rbits != 0 && rbits != 0xFF) || (gbits != 0 && gbits != 0xFF) || (bbits != 0 && bbits != 0xFF)) {
-				WARN_LOG_REPORT_ONCE(rgbmask, G3D, "Unsupported RGB mask: r=%02x g=%02x b=%02x", rbits, gbits, bbits);
-			}
-			if (abits != 0 && abits != 0xFF) {
-				// The stencil part of the mask is supported.
-				WARN_LOG_REPORT_ONCE(amask, G3D, "Unsupported alpha/stencil mask: %02x", abits);
-			}
-#endif
-
-			// Let's not write to alpha if stencil isn't enabled.
-			if (!gstate.isStencilTestEnabled()) {
-				amask = false;
-			} else {
-				// If the stencil type is set to KEEP, we shouldn't write to the stencil/alpha channel.
-				if (ReplaceAlphaWithStencilType() == STENCIL_VALUE_KEEP) {
-					amask = false;
-				}
-			}
-
-			keys_.blend.colorWriteMask = (rmask ? 1 : 0) | (gmask ? 2 : 0) | (bmask ? 4 : 0) | (amask ? 8 : 0);
+			keys_.blend.colorWriteMask = maskState.channelMask;
 		}
+	}
 
+	if (gstate_c.IsDirty(DIRTY_RASTER_STATE)) {
+		keys_.raster.value = 0;
+		bool wantCull = !gstate.isModeClear() && prim != GE_PRIM_RECTANGLES && prim > GE_PRIM_LINE_STRIP && gstate.isCullEnabled();
+		keys_.raster.cullMode = wantCull ? (gstate.getCullMode() ? D3D11_CULL_FRONT : D3D11_CULL_BACK) : D3D11_CULL_NONE;
+
+		if (gstate.isModeClear() || gstate.isModeThrough()) {
+			// TODO: Might happen in clear mode if not through...
+			keys_.raster.depthClipEnable = 1;
+		} else {
+			if (gstate.getDepthRangeMin() == 0 || gstate.getDepthRangeMax() == 65535) {
+				// TODO: Still has a bug where we clamp to depth range if one is not the full range.
+				// But the alternate is not clamping in either direction...
+				keys_.raster.depthClipEnable = !gstate.isDepthClampEnabled() || !gstate_c.Use(GPU_USE_DEPTH_CLAMP);
+			} else {
+				// We just want to clip in this case, the clamp would be clipped anyway.
+				keys_.raster.depthClipEnable = 1;
+			}
+		}
+	}
+
+	if (gstate_c.IsDirty(DIRTY_DEPTHSTENCIL_STATE)) {
+		GenericStencilFuncState stencilState;
+		ConvertStencilFuncState(stencilState);
+
+		if (gstate.isModeClear()) {
+			keys_.depthStencil.value = 0;
+			keys_.depthStencil.depthTestEnable = true;
+			keys_.depthStencil.depthCompareOp = D3D11_COMPARISON_ALWAYS;
+			keys_.depthStencil.depthWriteEnable = gstate.isClearModeDepthMask();
+
+			// Stencil Test
+			bool alphaMask = gstate.isClearModeAlphaMask();
+			if (alphaMask) {
+				keys_.depthStencil.stencilTestEnable = true;
+				keys_.depthStencil.stencilCompareFunc = D3D11_COMPARISON_ALWAYS;
+				keys_.depthStencil.stencilPassOp = D3D11_STENCIL_OP_REPLACE;
+				keys_.depthStencil.stencilFailOp = D3D11_STENCIL_OP_REPLACE;
+				keys_.depthStencil.stencilDepthFailOp = D3D11_STENCIL_OP_REPLACE;
+				dynState_.useStencil = true;
+				// In clear mode, the stencil value is set to the alpha value of the vertex.
+				// A normal clear will be 2 points, the second point has the color.
+				// We override this value in the pipeline from software transform for clear rectangles.
+				dynState_.stencilRef = 0xFF;
+				// But we still apply the stencil write mask.
+				keys_.depthStencil.stencilWriteMask = stencilState.writeMask;
+			} else {
+				keys_.depthStencil.stencilTestEnable = false;
+				dynState_.useStencil = false;
+			}
+
+		} else {
+			keys_.depthStencil.value = 0;
+			// Depth Test
+			if (!IsDepthTestEffectivelyDisabled()) {
+				keys_.depthStencil.depthTestEnable = true;
+				keys_.depthStencil.depthCompareOp = compareOps[gstate.getDepthTestFunction()];
+				keys_.depthStencil.depthWriteEnable = gstate.isDepthWriteEnabled();
+				UpdateEverUsedEqualDepth(gstate.getDepthTestFunction());
+			} else {
+				keys_.depthStencil.depthTestEnable = false;
+				keys_.depthStencil.depthWriteEnable = false;
+				keys_.depthStencil.depthCompareOp = D3D11_COMPARISON_ALWAYS;
+			}
+
+			// Stencil Test
+			if (stencilState.enabled) {
+				keys_.depthStencil.stencilTestEnable = true;
+				keys_.depthStencil.stencilCompareFunc = compareOps[stencilState.testFunc];
+				keys_.depthStencil.stencilPassOp = stencilOps[stencilState.zPass];
+				keys_.depthStencil.stencilFailOp = stencilOps[stencilState.sFail];
+				keys_.depthStencil.stencilDepthFailOp = stencilOps[stencilState.zFail];
+				keys_.depthStencil.stencilCompareMask = stencilState.testMask;
+				keys_.depthStencil.stencilWriteMask = stencilState.writeMask;
+				dynState_.useStencil = true;
+				dynState_.stencilRef = stencilState.testRef;
+
+				// Nasty special case for Spongebob and similar where it tries to write zeros to alpha/stencil during
+				// depth-fail. We can't write to alpha then because the pixel is killed. However, we can invert the depth
+				// test and modify the alpha function...
+				if (SpongebobDepthInverseConditions(stencilState)) {
+					keys_.blend.blendEnable = true;
+					keys_.blend.blendOpAlpha = D3D11_BLEND_OP_ADD;
+					keys_.blend.blendOpColor = D3D11_BLEND_OP_ADD;
+					keys_.blend.srcColor = D3D11_BLEND_ZERO;
+					keys_.blend.destColor = D3D11_BLEND_ZERO;
+					keys_.blend.logicOpEnable = false;
+					keys_.blend.srcAlpha = D3D11_BLEND_ZERO;
+					keys_.blend.destAlpha = D3D11_BLEND_ZERO;
+					keys_.blend.colorWriteMask = D3D11_COLOR_WRITE_ENABLE_ALPHA;
+
+					keys_.depthStencil.depthCompareOp = D3D11_COMPARISON_LESS;  // Inverse of GREATER_EQUAL
+					keys_.depthStencil.stencilCompareFunc = D3D11_COMPARISON_ALWAYS;
+					// Invert
+					keys_.depthStencil.stencilPassOp = D3D11_STENCIL_OP_ZERO;
+					keys_.depthStencil.stencilFailOp = D3D11_STENCIL_OP_ZERO;
+					keys_.depthStencil.stencilDepthFailOp = D3D11_STENCIL_OP_KEEP;
+
+					dirtyRequiresRecheck_ |= DIRTY_BLEND_STATE | DIRTY_DEPTHSTENCIL_STATE;
+					gstate_c.Dirty(DIRTY_BLEND_STATE | DIRTY_DEPTHSTENCIL_STATE);
+				}
+			} else {
+				keys_.depthStencil.stencilTestEnable = false;
+				dynState_.useStencil = false;
+			}
+		}
+	}
+
+	if (gstate_c.IsDirty(DIRTY_VIEWPORTSCISSOR_STATE)) {
+		ViewportAndScissor vpAndScissor;
+		ConvertViewportAndScissor(useBufferedRendering,
+			framebufferManager_->GetRenderWidth(), framebufferManager_->GetRenderHeight(),
+			framebufferManager_->GetTargetBufferWidth(), framebufferManager_->GetTargetBufferHeight(),
+			vpAndScissor);
+		UpdateCachedViewportState(vpAndScissor);
+
+		float depthMin = vpAndScissor.depthRangeMin;
+		float depthMax = vpAndScissor.depthRangeMax;
+
+		if (depthMin < 0.0f) depthMin = 0.0f;
+		if (depthMax > 1.0f) depthMax = 1.0f;
+
+		Draw::Viewport &vp = dynState_.viewport;
+		vp.TopLeftX = vpAndScissor.viewportX;
+		vp.TopLeftY = vpAndScissor.viewportY;
+		vp.Width = vpAndScissor.viewportW;
+		vp.Height = vpAndScissor.viewportH;
+		vp.MinDepth = depthMin;
+		vp.MaxDepth = depthMax;
+
+		D3D11_RECT &scissor = dynState_.scissor;
+		scissor.left = vpAndScissor.scissorX;
+		scissor.top = vpAndScissor.scissorY;
+		scissor.right = vpAndScissor.scissorX + std::max(0, vpAndScissor.scissorW);
+		scissor.bottom = vpAndScissor.scissorY + std::max(0, vpAndScissor.scissorH);
+	}
+
+	// Actually create/set the state objects only after we're done mapping all the state.
+	// There might have been interactions between depth and blend above.
+	if (gstate_c.IsDirty(DIRTY_BLEND_STATE)) {
 		if (!device1_) {
-			ID3D11BlendState *bs = blendCache_.Get(keys_.blend.value);
-			if (bs == nullptr) {
+			ID3D11BlendState *bs = nullptr;
+			if (!blendCache_.Get(keys_.blend.value, &bs) || !bs) {
 				D3D11_BLEND_DESC desc{};
 				D3D11_RENDER_TARGET_BLEND_DESC &rt = desc.RenderTarget[0];
 				rt.BlendEnable = keys_.blend.blendEnable;
@@ -253,8 +373,8 @@ void DrawEngineD3D11::ApplyDrawState(int prim) {
 			}
 			blendState_ = bs;
 		} else {
-			ID3D11BlendState1 *bs1 = blendCache1_.Get(keys_.blend.value);
-			if (bs1 == nullptr) {
+			ID3D11BlendState1 *bs1 = nullptr;
+			if (!blendCache1_.Get(keys_.blend.value, &bs1) || !bs1) {
 				D3D11_BLEND_DESC1 desc1{};
 				D3D11_RENDER_TARGET_BLEND_DESC1 &rt = desc1.RenderTarget[0];
 				rt.BlendEnable = keys_.blend.blendEnable;
@@ -275,26 +395,8 @@ void DrawEngineD3D11::ApplyDrawState(int prim) {
 	}
 
 	if (gstate_c.IsDirty(DIRTY_RASTER_STATE)) {
-		keys_.raster.value = 0;
-		if (gstate.isModeClear() || gstate.isModeThrough()) {
-			keys_.raster.cullMode = D3D11_CULL_NONE;
-			// TODO: Might happen in clear mode if not through...
-			keys_.raster.depthClipEnable = 1;
-		} else {
-			// Set cull
-			bool wantCull = prim != GE_PRIM_RECTANGLES && gstate.isCullEnabled();
-			keys_.raster.cullMode = wantCull ? (gstate.getCullMode() ? D3D11_CULL_FRONT : D3D11_CULL_BACK) : D3D11_CULL_NONE;
-			if (gstate.getDepthRangeMin() == 0 || gstate.getDepthRangeMax() == 65535) {
-				// TODO: Still has a bug where we clamp to depth range if one is not the full range.
-				// But the alternate is not clamping in either direction...
-				keys_.raster.depthClipEnable = !gstate.isDepthClampEnabled() || !gstate_c.Supports(GPU_SUPPORTS_DEPTH_CLAMP);
-			} else {
-				// We just want to clip in this case, the clamp would be clipped anyway.
-				keys_.raster.depthClipEnable = 1;
-			}
-		}
-		ID3D11RasterizerState *rs = rasterCache_.Get(keys_.raster.value);
-		if (rs == nullptr) {
+		ID3D11RasterizerState *rs;
+		if (!rasterCache_.Get(keys_.raster.value, &rs) || !rs) {
 			D3D11_RASTERIZER_DESC desc{};
 			desc.CullMode = (D3D11_CULL_MODE)(keys_.raster.cullMode);
 			desc.FillMode = D3D11_FILL_SOLID;
@@ -308,71 +410,8 @@ void DrawEngineD3D11::ApplyDrawState(int prim) {
 	}
 
 	if (gstate_c.IsDirty(DIRTY_DEPTHSTENCIL_STATE)) {
-		if (gstate.isModeClear()) {
-			keys_.depthStencil.value = 0;
-			keys_.depthStencil.depthTestEnable = true;
-			keys_.depthStencil.depthCompareOp = D3D11_COMPARISON_ALWAYS;
-			keys_.depthStencil.depthWriteEnable = gstate.isClearModeDepthMask();
-			if (gstate.isClearModeDepthMask()) {
-				framebufferManager_->SetDepthUpdated();
-			}
-
-			// Stencil Test
-			bool alphaMask = gstate.isClearModeAlphaMask();
-			if (alphaMask) {
-				keys_.depthStencil.stencilTestEnable = true;
-				keys_.depthStencil.stencilCompareFunc = D3D11_COMPARISON_ALWAYS;
-				keys_.depthStencil.stencilPassOp = D3D11_STENCIL_OP_REPLACE;
-				keys_.depthStencil.stencilFailOp = D3D11_STENCIL_OP_REPLACE;
-				keys_.depthStencil.stencilDepthFailOp = D3D11_STENCIL_OP_REPLACE;
-				dynState_.useStencil = true;
-				// In clear mode, the stencil value is set to the alpha value of the vertex.
-				// A normal clear will be 2 points, the second point has the color.
-				// We override this value in the pipeline from software transform for clear rectangles.
-				dynState_.stencilRef = 0xFF;
-				keys_.depthStencil.stencilWriteMask = 0xFF;
-			} else {
-				keys_.depthStencil.stencilTestEnable = false;
-				dynState_.useStencil = false;
-			}
-
-		} else {
-			keys_.depthStencil.value = 0;
-			// Depth Test
-			if (gstate.isDepthTestEnabled()) {
-				keys_.depthStencil.depthTestEnable = true;
-				keys_.depthStencil.depthCompareOp = compareOps[gstate.getDepthTestFunction()];
-				keys_.depthStencil.depthWriteEnable = gstate.isDepthWriteEnabled();
-				if (gstate.isDepthWriteEnabled()) {
-					framebufferManager_->SetDepthUpdated();
-				}
-			} else {
-				keys_.depthStencil.depthTestEnable = false;
-				keys_.depthStencil.depthWriteEnable = false;
-				keys_.depthStencil.depthCompareOp = D3D11_COMPARISON_ALWAYS;
-			}
-
-			GenericStencilFuncState stencilState;
-			ConvertStencilFuncState(stencilState);
-
-			// Stencil Test
-			if (stencilState.enabled) {
-				keys_.depthStencil.stencilTestEnable = true;
-				keys_.depthStencil.stencilCompareFunc = compareOps[stencilState.testFunc];
-				keys_.depthStencil.stencilPassOp = stencilOps[stencilState.zPass];
-				keys_.depthStencil.stencilFailOp = stencilOps[stencilState.sFail];
-				keys_.depthStencil.stencilDepthFailOp = stencilOps[stencilState.zFail];
-				keys_.depthStencil.stencilCompareMask = stencilState.testMask;
-				keys_.depthStencil.stencilWriteMask = stencilState.writeMask;
-				dynState_.useStencil = true;
-				dynState_.stencilRef = stencilState.testRef;
-			} else {
-				keys_.depthStencil.stencilTestEnable = false;
-				dynState_.useStencil = false;
-			}
-		}
-		ID3D11DepthStencilState *ds = depthStencilCache_.Get(keys_.depthStencil.value);
-		if (ds == nullptr) {
+		ID3D11DepthStencilState *ds;
+		if (!depthStencilCache_.Get(keys_.depthStencil.value, &ds) || !ds) {
 			D3D11_DEPTH_STENCIL_DESC desc{};
 			desc.DepthEnable = keys_.depthStencil.depthTestEnable;
 			desc.DepthWriteMask = keys_.depthStencil.depthWriteEnable ? D3D11_DEPTH_WRITE_MASK_ALL : D3D11_DEPTH_WRITE_MASK_ZERO;
@@ -391,78 +430,26 @@ void DrawEngineD3D11::ApplyDrawState(int prim) {
 		depthStencilState_ = ds;
 	}
 
-	if (gstate_c.IsDirty(DIRTY_VIEWPORTSCISSOR_STATE)) {
-		ViewportAndScissor vpAndScissor;
-		ConvertViewportAndScissor(useBufferedRendering,
-			framebufferManager_->GetRenderWidth(), framebufferManager_->GetRenderHeight(),
-			framebufferManager_->GetTargetBufferWidth(), framebufferManager_->GetTargetBufferHeight(),
-			vpAndScissor);
-
-		float depthMin = vpAndScissor.depthRangeMin;
-		float depthMax = vpAndScissor.depthRangeMax;
-
-		if (depthMin < 0.0f) depthMin = 0.0f;
-		if (depthMax > 1.0f) depthMax = 1.0f;
-		if (vpAndScissor.dirtyDepth) {
-			gstate_c.Dirty(DIRTY_DEPTHRANGE);
-		}
-
-		Draw::Viewport &vp = dynState_.viewport;
-		vp.TopLeftX = vpAndScissor.viewportX;
-		vp.TopLeftY = vpAndScissor.viewportY;
-		vp.Width = vpAndScissor.viewportW;
-		vp.Height = vpAndScissor.viewportH;
-		vp.MinDepth = depthMin;
-		vp.MaxDepth = depthMax;
-
-		if (vpAndScissor.dirtyProj) {
-			gstate_c.Dirty(DIRTY_PROJMATRIX);
-		}
-
-		D3D11_RECT &scissor = dynState_.scissor;
-		if (vpAndScissor.scissorEnable) {
-			scissor.left = vpAndScissor.scissorX;
-			scissor.top = vpAndScissor.scissorY;
-			scissor.right = vpAndScissor.scissorX + std::max(0, vpAndScissor.scissorW);
-			scissor.bottom = vpAndScissor.scissorY + std::max(0, vpAndScissor.scissorH);
-		} else {
-			scissor.left = 0;
-			scissor.top = 0;
-			scissor.right = framebufferManager_->GetRenderWidth();
-			scissor.bottom = framebufferManager_->GetRenderHeight();
-		}
-	}
-
 	if (gstate_c.IsDirty(DIRTY_TEXTURE_IMAGE | DIRTY_TEXTURE_PARAMS) && !gstate.isModeClear() && gstate.isTextureMapEnabled()) {
 		textureCache_->SetTexture();
 		gstate_c.Clean(DIRTY_TEXTURE_IMAGE | DIRTY_TEXTURE_PARAMS);
-	} else if (gstate.getTextureAddress(0) == ((gstate.getFrameBufRawAddress() | 0x04000000) & 0x3FFFFFFF)) {
+	} else if (gstate.getTextureAddress(0) == (gstate.getFrameBufRawAddress() | 0x04000000)) {
 		// This catches the case of clearing a texture.
 		gstate_c.Dirty(DIRTY_TEXTURE_IMAGE);
 	}
 }
 
 void DrawEngineD3D11::ApplyDrawStateLate(bool applyStencilRef, uint8_t stencilRef) {
-	if (!gstate.isModeClear()) {
-		if (fboTexNeedBind_) {
-			framebufferManager_->BindFramebufferAsColorTexture(1, framebufferManager_->GetCurrentRenderVFB(), BINDFBCOLOR_MAY_COPY);
-			// No sampler required, we do a plain Load in the pixel shader.
-			fboTexBound_ = true;
-			fboTexNeedBind_ = false;
-		}
-		textureCache_->ApplyTexture();
-	}
-
 	// we go through Draw here because it automatically handles screen rotation, as needed in UWP on mobiles.
 	if (gstate_c.IsDirty(DIRTY_VIEWPORTSCISSOR_STATE)) {
-		draw_->SetViewports(1, &dynState_.viewport);
+		draw_->SetViewport(dynState_.viewport);
 		draw_->SetScissorRect(dynState_.scissor.left, dynState_.scissor.top, dynState_.scissor.right - dynState_.scissor.left, dynState_.scissor.bottom - dynState_.scissor.top);
 	}
 	if (gstate_c.IsDirty(DIRTY_RASTER_STATE)) {
 		context_->RSSetState(rasterState_);
 	}
 	if (gstate_c.IsDirty(DIRTY_BLEND_STATE)) {
-		// Need to do this AFTER ApplyTexture because the process of depallettization can ruin the blend state.
+		// Need to do this AFTER ApplyTexture because the process of depalettization can ruin the blend state.
 		float blendColor[4];
 		Uint8x4ToFloat4(blendColor, dynState_.blendColor);
 		if (device1_) {
@@ -475,8 +462,6 @@ void DrawEngineD3D11::ApplyDrawStateLate(bool applyStencilRef, uint8_t stencilRe
 		context_->OMSetDepthStencilState(depthStencilState_, applyStencilRef ? stencilRef : dynState_.stencilRef);
 	}
 	gstate_c.Clean(DIRTY_VIEWPORTSCISSOR_STATE | DIRTY_DEPTHSTENCIL_STATE | DIRTY_RASTER_STATE | DIRTY_BLEND_STATE);
-
-	// Must dirty blend state here so we re-copy next time.  Example: Lunar's spell effects.
-	if (fboTexBound_)
-		gstate_c.Dirty(DIRTY_BLEND_STATE);
+	gstate_c.Dirty(dirtyRequiresRecheck_);
+	dirtyRequiresRecheck_ = 0;
 }
