@@ -151,7 +151,12 @@ static void DirtyDrawnVRAM() {
 	DirtyVRAM(gstate.getFrameBufAddress(), bytes, DirtyVRAMFlag::DRAWN);
 }
 
-static void BeginRecording() {
+static bool BeginRecording() {
+	if (PSP_CoreParameter().fileType == IdentifiedFileType::PPSSPP_GE_DUMP) {
+		// Can't record a GE dump.
+		return false;
+	}
+
 	active = true;
 	nextFrame = false;
 	lastTextures.clear();
@@ -178,6 +183,7 @@ static void BeginRecording() {
 	}
 
 	DirtyAllVRAM(DirtyVRAMFlag::DIRTY);
+	return true;
 }
 
 static void WriteCompressed(FILE *fp, const void *p, size_t sz) {
@@ -197,7 +203,7 @@ static Path WriteRecording() {
 
 	const Path filename = GenRecordingFilename();
 
-	NOTICE_LOG(G3D, "Recording filename: %s", filename.c_str());
+	NOTICE_LOG(Log::G3D, "Recording filename: %s", filename.c_str());
 
 	FILE *fp = File::OpenCFile(filename, "wb");
 	Header header{};
@@ -619,19 +625,24 @@ void ClearCallback() {
 
 static void FinishRecording() {
 	// We're done - this was just to write the result out.
+	if (!active) {
+		return;
+	}
+
 	Path filename = WriteRecording();
 	commands.clear();
 	pushbuf.clear();
 	lastVRAM.clear();
 
-	NOTICE_LOG(SYSTEM, "Recording finished");
+	NOTICE_LOG(Log::System, "Recording finished");
 	active = false;
 	flipLastAction = gpuStats.numFlips;
 	flipFinishAt = -1;
 	lastEdramTrans = 0x400;
 
-	if (writeCallback)
+	if (writeCallback) {
 		writeCallback(filename);
+	}
 	writeCallback = nullptr;
 }
 
@@ -783,8 +794,8 @@ void NotifyDisplay(u32 framebuf, int stride, int fmt) {
 		writePending = true;
 	}
 	if (!active && nextFrame && (gstate_c.skipDrawReason & SKIPDRAW_SKIPFRAME) == 0) {
-		NOTICE_LOG(SYSTEM, "Recording starting on display...");
-		BeginRecording();
+		NOTICE_LOG(Log::System, "Recording starting on display...");
+		BeginRecording();  // TODO: Handle return value.
 	}
 	if (!active) {
 		return;
@@ -807,7 +818,7 @@ void NotifyDisplay(u32 framebuf, int stride, int fmt) {
 	commands.push_back({ CommandType::DISPLAY, sz, ptr });
 
 	if (writePending) {
-		NOTICE_LOG(SYSTEM, "Recording complete on display");
+		NOTICE_LOG(Log::System, "Recording complete on display");
 		FinishRecording();
 	}
 }
@@ -816,7 +827,7 @@ void NotifyBeginFrame() {
 	const bool noDisplayAction = flipLastAction + 4 < gpuStats.numFlips;
 	// We do this only to catch things that don't call NotifyDisplay.
 	if (active && HasDrawCommands() && (noDisplayAction || gpuStats.numFlips == flipFinishAt)) {
-		NOTICE_LOG(SYSTEM, "Recording complete on frame");
+		NOTICE_LOG(Log::System, "Recording complete on frame");
 
 		CheckEdramTrans();
 		struct DisplayBufData {
@@ -838,7 +849,7 @@ void NotifyBeginFrame() {
 		FinishRecording();
 	}
 	if (!active && nextFrame && (gstate_c.skipDrawReason & SKIPDRAW_SKIPFRAME) == 0 && noDisplayAction) {
-		NOTICE_LOG(SYSTEM, "Recording starting on frame...");
+		NOTICE_LOG(Log::System, "Recording starting on frame...");
 		BeginRecording();
 		// If we began on a BeginFrame, end on a BeginFrame.
 		flipFinishAt = gpuStats.numFlips + 1;
