@@ -379,7 +379,7 @@ static inline bool GuessVRDrawingHUD(bool is2D, bool flatScreen) {
 	return hud;
 }
 
-void LinkedShader::use(const ShaderID &VSID) {
+void LinkedShader::use(const ShaderID &VSID) const {
 	render_->BindProgram(program);
 	// Note that we no longer track attr masks here - we do it for the input layouts instead.
 }
@@ -412,7 +412,8 @@ void LinkedShader::UpdateUniforms(const ShaderID &vsid, bool useBufferedRenderin
 	// Set HUD mode
 	if (gstate_c.Use(GPU_USE_VIRTUAL_REALITY)) {
 		if (GuessVRDrawingHUD(is2D, flatScreen)) {
-			render_->SetUniformF1(&u_scaleX, g_Config.fHeadUpDisplayScale * 480.0f / 272.0f);
+			float aspect = 480.0f / 272.0f * (IsImmersiveVRMode() ? 0.5f : 1.0f);
+			render_->SetUniformF1(&u_scaleX, g_Config.fHeadUpDisplayScale * aspect);
 			render_->SetUniformF1(&u_scaleY, g_Config.fHeadUpDisplayScale);
 		} else {
 			render_->SetUniformF1(&u_scaleX, 1.0f);
@@ -423,21 +424,18 @@ void LinkedShader::UpdateUniforms(const ShaderID &vsid, bool useBufferedRenderin
 	// Update any dirty uniforms before we draw
 	if (dirty & DIRTY_PROJMATRIX) {
 		if (gstate_c.Use(GPU_USE_VIRTUAL_REALITY)) {
-			Matrix4x4 leftEyeMatrix, rightEyeMatrix;
+			Matrix4x4 vrProjection;
 			if (flatScreen || is2D) {
-				memcpy(&leftEyeMatrix, gstate.projMatrix, 16 * sizeof(float));
-				memcpy(&rightEyeMatrix, gstate.projMatrix, 16 * sizeof(float));
+				memcpy(&vrProjection, gstate.projMatrix, 16 * sizeof(float));
 			} else {
-				UpdateVRProjection(gstate.projMatrix, leftEyeMatrix.m, rightEyeMatrix.m);
+				UpdateVRProjection(gstate.projMatrix, vrProjection.m);
 			}
 			UpdateVRParams(gstate.projMatrix);
 
-			FlipProjMatrix(leftEyeMatrix, useBufferedRendering);
-			FlipProjMatrix(rightEyeMatrix, useBufferedRendering);
-			ScaleProjMatrix(leftEyeMatrix, useBufferedRendering);
-			ScaleProjMatrix(rightEyeMatrix, useBufferedRendering);
+			FlipProjMatrix(vrProjection, useBufferedRendering);
+			ScaleProjMatrix(vrProjection, useBufferedRendering);
 
-			render_->SetUniformM4x4Stereo("u_proj_lens", &u_proj_lens, leftEyeMatrix.m, rightEyeMatrix.m);
+			render_->SetUniformM4x4(&u_proj_lens, vrProjection.m);
 		}
 
 		Matrix4x4 flippedMatrix;
@@ -484,7 +482,7 @@ void LinkedShader::UpdateUniforms(const ShaderID &vsid, bool useBufferedRenderin
 	}
 	if (dirty & DIRTY_FOGCOLOR) {
 		SetColorUniform3(render_, &u_fogcolor, gstate.fogcolor);
-		if (IsVREnabled()) {
+		if (gstate_c.Use(GPU_USE_VIRTUAL_REALITY)) {
 			SetVRCompat(VR_COMPAT_FOG_COLOR, gstate.fogcolor);
 		}
 	}
@@ -591,8 +589,8 @@ void LinkedShader::UpdateUniforms(const ShaderID &vsid, bool useBufferedRenderin
 		float vpZCenter = gstate.getViewportZCenter();
 
 		// These are just the reverse of the formulas in GPUStateUtils.
-		float halfActualZRange = gstate_c.vpDepthScale != 0.0f ? vpZScale / gstate_c.vpDepthScale : 0.0f;
-		float inverseDepthScale = gstate_c.vpDepthScale != 0.0f ? 1.0f / gstate_c.vpDepthScale : 0.0f;
+		float halfActualZRange = InfToZero(gstate_c.vpDepthScale != 0.0f ? vpZScale / gstate_c.vpDepthScale : 0.0f);
+		float inverseDepthScale = InfToZero(gstate_c.vpDepthScale != 0.0f ? 1.0f / gstate_c.vpDepthScale : 0.0f);
 		float minz = -((gstate_c.vpZOffset * halfActualZRange) - vpZCenter) - halfActualZRange;
 		float viewZScale = halfActualZRange;
 		float viewZCenter = minz + halfActualZRange;
@@ -752,7 +750,7 @@ Shader *ShaderManagerGLES::CompileFragmentShader(FShaderID FSID) {
 	std::string errorString;
 	FragmentShaderFlags flags;
 	if (!GenerateFragmentShader(FSID, codeBuffer_, draw_->GetShaderLanguageDesc(), draw_->GetBugs(), &uniformMask, &flags, &errorString)) {
-		ERROR_LOG_REPORT(G3D, "FS shader gen error: %s (%s: %08x:%08x)", errorString.c_str(), "GLES", FSID.d[0], FSID.d[1]);
+		ERROR_LOG_REPORT(Log::G3D, "FS shader gen error: %s (%s: %08x:%08x)", errorString.c_str(), "GLES", FSID.d[0], FSID.d[1]);
 		return nullptr;
 	}
 	_assert_msg_(strlen(codeBuffer_) < CODE_BUFFER_SIZE, "FS length error: %d", (int)strlen(codeBuffer_));
@@ -770,7 +768,7 @@ Shader *ShaderManagerGLES::CompileVertexShader(VShaderID VSID) {
 	std::string errorString;
 	VertexShaderFlags flags;
 	if (!GenerateVertexShader(VSID, codeBuffer_, draw_->GetShaderLanguageDesc(), draw_->GetBugs(), &attrMask, &uniformMask, &flags, &errorString)) {
-		ERROR_LOG_REPORT(G3D, "VS shader gen error: %s (%s: %08x:%08x)", errorString.c_str(), "GLES", VSID.d[0], VSID.d[1]);
+		ERROR_LOG_REPORT(Log::G3D, "VS shader gen error: %s (%s: %08x:%08x)", errorString.c_str(), "GLES", VSID.d[0], VSID.d[1]);
 		return nullptr;
 	}
 	_assert_msg_(strlen(codeBuffer_) < CODE_BUFFER_SIZE, "VS length error: %d", (int)strlen(codeBuffer_));
@@ -804,9 +802,9 @@ Shader *ShaderManagerGLES::ApplyVertexShader(bool useHWTransform, bool useHWTess
 	// Vertex shader not in cache. Let's compile it.
 	vs = CompileVertexShader(*VSID);
 	if (!vs) {
-		auto gr = GetI18NCategory(I18NCat::GRAPHICS);
-		ERROR_LOG(G3D, "Vertex shader generation failed, falling back to software transform");
+		ERROR_LOG(Log::G3D, "Vertex shader generation failed, falling back to software transform");
 		if (!g_Config.bHideSlowWarnings) {
+			auto gr = GetI18NCategory(I18NCat::GRAPHICS);
 			g_OSD.Show(OSDType::MESSAGE_ERROR, gr->T("hardware transform error - falling back to software"), 2.5f);
 		}
 
@@ -821,7 +819,6 @@ Shader *ShaderManagerGLES::ApplyVertexShader(bool useHWTransform, bool useHWTess
 	}
 
 	vsCache_.Insert(*VSID, vs);
-	diskCacheDirty_ = true;
 	return vs;
 }
 
@@ -856,11 +853,10 @@ LinkedShader *ShaderManagerGLES::ApplyFragmentShader(VShaderID VSID, Shader *vs,
 		// Could fail to generate, in which case we're kinda screwed.
 		fs = CompileFragmentShader(FSID);
 		if (!fs) {
-			ERROR_LOG(G3D, "Failed to generate fragment shader with ID %08x:%08x", FSID.d[0], FSID.d[1]);
+			ERROR_LOG(Log::G3D, "Failed to generate fragment shader with ID %08x:%08x", FSID.d[0], FSID.d[1]);
 			// Still insert it so we don't end up spamming generation.
 		}
 		fsCache_.Insert(FSID, fs);
-		diskCacheDirty_ = true;
 	}
 
 	// Okay, we have both shaders. Let's see if there's a linked one.
@@ -916,22 +912,18 @@ std::vector<std::string> ShaderManagerGLES::DebugGetShaderIDs(DebugShaderType ty
 	std::vector<std::string> ids;
 	switch (type) {
 	case SHADER_TYPE_VERTEX:
-		{
-			vsCache_.Iterate([&](const VShaderID &id, Shader *shader) {
-				std::string idstr;
-				id.ToString(&idstr);
-				ids.push_back(idstr);
-			});
-		}
+		vsCache_.Iterate([&](const VShaderID &id, Shader *shader) {
+			std::string idstr;
+			id.ToString(&idstr);
+			ids.push_back(idstr);
+		});
 		break;
 	case SHADER_TYPE_FRAGMENT:
-		{
-			fsCache_.Iterate([&](const FShaderID &id, Shader *shader) {
-				std::string idstr;
-				id.ToString(&idstr);
-				ids.push_back(idstr);
-			});
-		}
+		fsCache_.Iterate([&](const FShaderID &id, Shader *shader) {
+			std::string idstr;
+			id.ToString(&idstr);
+			ids.push_back(idstr);
+		});
 		break;
 	default:
 		break;
@@ -983,7 +975,7 @@ enum class CacheDetectFlags {
 };
 
 #define CACHE_HEADER_MAGIC 0x83277592
-#define CACHE_VERSION 33
+#define CACHE_VERSION 37
 
 struct CacheHeader {
 	uint32_t magic;
@@ -1012,6 +1004,31 @@ bool ShaderManagerGLES::LoadCacheFlags(File::IOFile &f, DrawEngineGLES *drawEngi
 }
 
 bool ShaderManagerGLES::LoadCache(File::IOFile &f) {
+	// TODO: Get rid of this struct.
+	struct {
+		std::vector<VShaderID> vert;
+		std::vector<FShaderID> frag;
+		std::vector<std::pair<VShaderID, FShaderID>> link;
+
+		size_t vertPos = 0;
+		size_t fragPos = 0;
+		size_t linkPos = 0;
+		double start;
+
+		void Clear() {
+			vert.clear();
+			frag.clear();
+			link.clear();
+			vertPos = 0;
+			fragPos = 0;
+			linkPos = 0;
+		}
+
+		bool Done() const {
+			return vertPos >= vert.size() && fragPos >= frag.size() && linkPos >= link.size();
+		}
+	} diskCachePending_;
+
 	u64 sz = f.GetSize();
 	f.Seek(0, SEEK_SET);
 	CacheHeader header;
@@ -1027,7 +1044,7 @@ bool ShaderManagerGLES::LoadCache(File::IOFile &f) {
 
 	// Sanity check the file contents
 	if (header.numFragmentShaders > 1000 || header.numVertexShaders > 1000 || header.numLinkedPrograms > 1000) {
-		ERROR_LOG(G3D, "Corrupt shader cache file header, aborting.");
+		ERROR_LOG(Log::G3D, "Corrupt shader cache file header, aborting.");
 		return false;
 	}
 
@@ -1037,7 +1054,7 @@ bool ShaderManagerGLES::LoadCache(File::IOFile &f) {
 	expectedSize += header.numFragmentShaders * sizeof(FShaderID);
 	expectedSize += header.numLinkedPrograms * (sizeof(VShaderID) + sizeof(FShaderID));
 	if (sz != expectedSize) {
-		ERROR_LOG(G3D, "Shader cache file is wrong size: %lld instead of %lld", sz, expectedSize);
+		ERROR_LOG(Log::G3D, "Shader cache file is wrong size: %lld instead of %lld", sz, expectedSize);
 		return false;
 	}
 
@@ -1066,12 +1083,6 @@ bool ShaderManagerGLES::LoadCache(File::IOFile &f) {
 		diskCachePending_.link.emplace_back(vsid, fsid);
 	}
 
-	// Actual compilation happens in ContinuePrecompile(), called by GPU_GLES's IsReady.
-	diskCacheDirty_ = false;
-	return true;
-}
-
-bool ShaderManagerGLES::ContinuePrecompile(float sliceTime) {
 	auto &pending = diskCachePending_;
 	if (pending.Done()) {
 		return true;
@@ -1080,20 +1091,13 @@ bool ShaderManagerGLES::ContinuePrecompile(float sliceTime) {
 	PSP_SetLoading("Compiling shaders...");
 
 	double start = time_now_d();
-	// Let's try to keep it under sliceTime if possible.
-	double end = start + sliceTime;
 
 	for (size_t &i = pending.vertPos; i < pending.vert.size(); i++) {
-		if (time_now_d() >= end) {
-			// We'll finish later.
-			return false;
-		}
-
 		const VShaderID &id = pending.vert[i];
 		if (!vsCache_.ContainsKey(id)) {
 			if (id.Bit(VS_BIT_IS_THROUGH) && id.Bit(VS_BIT_USE_HW_TRANSFORM)) {
 				// Clearly corrupt, bailing.
-				ERROR_LOG_REPORT(G3D, "Corrupt shader cache: Both IS_THROUGH and USE_HW_TRANSFORM set.");
+				ERROR_LOG_REPORT(Log::G3D, "Corrupt shader cache: Both IS_THROUGH and USE_HW_TRANSFORM set.");
 				pending.Clear();
 				return false;
 			}
@@ -1102,46 +1106,35 @@ bool ShaderManagerGLES::ContinuePrecompile(float sliceTime) {
 			if (!vs) {
 				// Give up on using the cache, just bail. We can't safely create the fallback shaders here
 				// without trying to deduce the vertType from the VSID.
-				ERROR_LOG(G3D, "Failed to compile a vertex shader loading from cache. Skipping rest of shader cache.");
-				delete vs;
+				ERROR_LOG(Log::G3D, "Failed to compile a vertex shader loading from cache. Skipping rest of shader cache.");
 				pending.Clear();
 				return false;
 			}
 			vsCache_.Insert(id, vs);
 		} else {
-			WARN_LOG(G3D, "Duplicate vertex shader found in GL shader cache, ignoring");
+			WARN_LOG(Log::G3D, "Duplicate vertex shader found in GL shader cache, ignoring");
 		}
 	}
 
 	for (size_t &i = pending.fragPos; i < pending.frag.size(); i++) {
-		if (time_now_d() >= end) {
-			// We'll finish later.
-			return false;
-		}
-
 		const FShaderID &id = pending.frag[i];
 		if (!fsCache_.ContainsKey(id)) {
 			Shader *fs = CompileFragmentShader(id);
 			if (!fs) {
 				// Give up on using the cache - something went wrong.
 				// We'll still keep the shaders we generated so far around.
-				ERROR_LOG(G3D, "Failed to compile a fragment shader loading from cache. Skipping rest of shader cache.");
-				delete fs;
+				ERROR_LOG(Log::G3D, "Failed to compile a fragment shader loading from cache. Skipping rest of shader cache.");
 				pending.Clear();
 				return false;
 			}
 			fsCache_.Insert(id, fs);
 		} else {
-			WARN_LOG(G3D, "Duplicate fragment shader found in GL shader cache, ignoring");
+			WARN_LOG(Log::G3D, "Duplicate fragment shader found in GL shader cache, ignoring");
 		}
 	}
 
+	linkedShaderCache_.reserve(pending.link.size() - pending.linkPos);
 	for (size_t &i = pending.linkPos; i < pending.link.size(); i++) {
-		if (time_now_d() >= end) {
-			// We'll finish later.
-			return false;
-		}
-
 		const VShaderID &vsid = pending.link[i].first;
 		const FShaderID &fsid = pending.link[i].second;
 		Shader *vs = nullptr;
@@ -1158,28 +1151,20 @@ bool ShaderManagerGLES::ContinuePrecompile(float sliceTime) {
 	// Okay, finally done.  Time to report status.
 	double finish = time_now_d();
 
-	NOTICE_LOG(G3D, "Precompile: Compiled and linked %d programs (%d vertex, %d fragment) in %0.1f milliseconds", (int)pending.link.size(), (int)pending.vert.size(), (int)pending.frag.size(), 1000 * (finish - pending.start));
+	NOTICE_LOG(Log::G3D, "Precompile: Compiled and linked %d programs (%d vertex, %d fragment) in %0.1f milliseconds", (int)pending.link.size(), (int)pending.vert.size(), (int)pending.frag.size(), 1000 * (finish - pending.start));
 	pending.Clear();
 
 	return true;
 }
 
-void ShaderManagerGLES::CancelPrecompile() {
-	diskCachePending_.Clear();
-}
-
 void ShaderManagerGLES::SaveCache(const Path &filename, DrawEngineGLES *drawEngine) {
-	if (!diskCacheDirty_) {
-		return;
-	}
 	if (linkedShaderCache_.empty()) {
 		return;
 	}
-	INFO_LOG(G3D, "Saving the shader cache to '%s'", filename.c_str());
+	INFO_LOG(Log::G3D, "Saving the shader cache to '%s'", filename.c_str());
 	FILE *f = File::OpenCFile(filename, "wb");
 	if (!f) {
 		// Can't save, give up for now.
-		diskCacheDirty_ = false;
 		return;
 	}
 	CacheHeader header;
@@ -1199,7 +1184,7 @@ void ShaderManagerGLES::SaveCache(const Path &filename, DrawEngineGLES *drawEngi
 	fsCache_.Iterate([&](const ShaderID &id, Shader *shader) {
 		fwrite(&id, 1, sizeof(id), f);
 	});
-	for (auto iter : linkedShaderCache_) {
+	for (const auto &iter : linkedShaderCache_) {
 		ShaderID vsid, fsid;
 		vsCache_.Iterate([&](const ShaderID &id, Shader *shader) {
 			if (iter.vs == shader)
@@ -1213,5 +1198,4 @@ void ShaderManagerGLES::SaveCache(const Path &filename, DrawEngineGLES *drawEngi
 		fwrite(&fsid, 1, sizeof(fsid), f);
 	}
 	fclose(f);
-	diskCacheDirty_ = false;
 }
