@@ -210,9 +210,9 @@ u32 GPUCommon::DrawSync(int mode) {
 
 	// If there's no current list, it must be complete.
 	DisplayList *top = NULL;
-	for (auto it = dlQueue.begin(), end = dlQueue.end(); it != end; ++it) {
-		if (dls[*it].state != PSP_GE_DL_STATE_COMPLETED) {
-			top = &dls[*it];
+	for (int i : dlQueue) {
+		if (dls[i].state != PSP_GE_DL_STATE_COMPLETED) {
+			top = &dls[i];
 			break;
 		}
 	}
@@ -363,13 +363,13 @@ u32 GPUCommon::EnqueueList(u32 listpc, u32 stall, int subIntrBase, PSPPointer<Ps
 	// Check alignment
 	// TODO Check the context and stack alignement too
 	if (((listpc | stall) & 3) != 0 || !Memory::IsValidAddress(listpc)) {
-		ERROR_LOG_REPORT(G3D, "sceGeListEnqueue: invalid address %08x", listpc);
+		ERROR_LOG_REPORT(Log::G3D, "sceGeListEnqueue: invalid address %08x", listpc);
 		return SCE_KERNEL_ERROR_INVALID_POINTER;
 	}
 
 	// If args->size is below 16, it's the old struct without stack info.
 	if (args.IsValid() && args->size >= 16 && args->numStacks >= 256) {
-		return hleLogError(G3D, SCE_KERNEL_ERROR_INVALID_SIZE, "invalid stack depth %d", args->numStacks);
+		return hleLogError(Log::G3D, SCE_KERNEL_ERROR_INVALID_SIZE, "invalid stack depth %d", args->numStacks);
 	}
 
 	int id = -1;
@@ -384,10 +384,10 @@ u32 GPUCommon::EnqueueList(u32 listpc, u32 stall, int subIntrBase, PSPPointer<Ps
 				// Logically, if the CPU has not interrupted yet, it hasn't seen the latest pc either.
 				// Exit enqueues right after an END, which fails without ignoring pendingInterrupt lists.
 				if (dls[i].pc == listpc && !dls[i].pendingInterrupt) {
-					ERROR_LOG(G3D, "sceGeListEnqueue: can't enqueue, list address %08X already used", listpc);
+					ERROR_LOG(Log::G3D, "sceGeListEnqueue: can't enqueue, list address %08X already used", listpc);
 					return 0x80000021;
 				} else if (stackAddr != 0 && dls[i].stackAddr == stackAddr && !dls[i].pendingInterrupt) {
-					ERROR_LOG(G3D, "sceGeListEnqueue: can't enqueue, stack address %08X already used", stackAddr);
+					ERROR_LOG(Log::G3D, "sceGeListEnqueue: can't enqueue, stack address %08X already used", stackAddr);
 					return 0x80000021;
 				}
 			}
@@ -411,10 +411,10 @@ u32 GPUCommon::EnqueueList(u32 listpc, u32 stall, int subIntrBase, PSPPointer<Ps
 		}
 	}
 	if (id < 0) {
-		ERROR_LOG_REPORT(G3D, "No DL ID available to enqueue");
-		for (auto it = dlQueue.begin(); it != dlQueue.end(); ++it) {
-			DisplayList &dl = dls[*it];
-			DEBUG_LOG(G3D, "DisplayList %d status %d pc %08x stall %08x", *it, dl.state, dl.pc, dl.stall);
+		ERROR_LOG_REPORT(Log::G3D, "No DL ID available to enqueue");
+		for (int i : dlQueue) {
+			DisplayList &dl = dls[i];
+			DEBUG_LOG(Log::G3D, "DisplayList %d status %d pc %08x stall %08x", i, dl.state, dl.pc, dl.stall);
 		}
 		return SCE_KERNEL_ERROR_OUT_OF_MEMORY;
 	}
@@ -583,7 +583,7 @@ u32 GPUCommon::Break(int mode) {
 		{
 			if (currentList->signal == PSP_GE_SIGNAL_HANDLER_PAUSE)
 			{
-				ERROR_LOG_REPORT(G3D, "sceGeBreak: can't break signal-pausing list");
+				ERROR_LOG_REPORT(Log::G3D, "sceGeBreak: can't break signal-pausing list");
 			}
 			else
 				return SCE_KERNEL_ERROR_ALREADY;
@@ -620,7 +620,7 @@ void GPUCommon::NotifySteppingEnter() {
 void GPUCommon::NotifySteppingExit() {
 	if (coreCollectDebugStats) {
 		if (timeSteppingStarted_ <= 0.0) {
-			ERROR_LOG(G3D, "Mismatched stepping enter/exit.");
+			ERROR_LOG(Log::G3D, "Mismatched stepping enter/exit.");
 		}
 		double total = time_now_d() - timeSteppingStarted_;
 		_dbg_assert_msg_(total >= 0.0, "Time spent stepping became negative");
@@ -648,7 +648,7 @@ bool GPUCommon::InterpretList(DisplayList &list) {
 	gstate_c.offsetAddr = list.offsetAddr;
 
 	if (!Memory::IsValidAddress(list.pc)) {
-		ERROR_LOG_REPORT(G3D, "DL PC = %08x WTF!!!!", list.pc);
+		ERROR_LOG_REPORT(Log::G3D, "DL PC = %08x WTF!!!!", list.pc);
 		return true;
 	}
 
@@ -709,10 +709,10 @@ bool GPUCommon::InterpretList(DisplayList &list) {
 	return gpuState == GPUSTATE_DONE || gpuState == GPUSTATE_ERROR;
 }
 
-void GPUCommon::BeginFrame() {
+void GPUCommon::PSPFrame() {
 	immCount_ = 0;
 	if (dumpNextFrame_) {
-		NOTICE_LOG(G3D, "DUMPING THIS FRAME");
+		NOTICE_LOG(Log::G3D, "DUMPING THIS FRAME");
 		dumpThisFrame_ = true;
 		dumpNextFrame_ = false;
 	} else if (dumpThisFrame_) {
@@ -742,7 +742,7 @@ void GPUCommon::SlowRunLoop(DisplayList &list) {
 					prev = 0;
 				}
 				GeDisassembleOp(list.pc, op, prev, temp, 256);
-				NOTICE_LOG(G3D, "%08x: %s", op, temp);
+				NOTICE_LOG(Log::G3D, "%08x: %s", op, temp);
 			}
 			gstate.cmdmem[cmd] = op;
 
@@ -835,13 +835,13 @@ void GPUCommon::ProcessDLQueue() {
 
 	// Seems to be correct behaviour to process the list anyway?
 	if (startingTicks < busyTicks) {
-		DEBUG_LOG(G3D, "Can't execute a list yet, still busy for %lld ticks", busyTicks - startingTicks);
+		DEBUG_LOG(Log::G3D, "Can't execute a list yet, still busy for %lld ticks", busyTicks - startingTicks);
 		//return;
 	}
 
 	for (int listIndex = GetNextListIndex(); listIndex != -1; listIndex = GetNextListIndex()) {
 		DisplayList &l = dls[listIndex];
-		DEBUG_LOG(G3D, "Starting DL execution at %08x - stall = %08x", l.pc, l.stall);
+		DEBUG_LOG(Log::G3D, "Starting DL execution at %08x - stall = %08x", l.pc, l.stall);
 		if (!InterpretList(l)) {
 			return;
 		} else {
@@ -885,7 +885,7 @@ void GPUCommon::Execute_Origin(u32 op, u32 diff) {
 void GPUCommon::Execute_Jump(u32 op, u32 diff) {
 	const u32 target = gstate_c.getRelativeAddress(op & 0x00FFFFFC);
 	if (!Memory::IsValidAddress(target)) {
-		ERROR_LOG(G3D, "JUMP to illegal address %08x - ignoring! data=%06x", target, op & 0x00FFFFFF);
+		ERROR_LOG(Log::G3D, "JUMP to illegal address %08x - ignoring! data=%06x", target, op & 0x00FFFFFF);
 		UpdateState(GPUSTATE_ERROR);
 		return;
 	}
@@ -902,7 +902,7 @@ void GPUCommon::Execute_BJump(u32 op, u32 diff) {
 			UpdatePC(currentList->pc, target - 4);
 			currentList->pc = target - 4; // pc will be increased after we return, counteract that
 		} else {
-			ERROR_LOG(G3D, "BJUMP to illegal address %08x - ignoring! data=%06x", target, op & 0x00FFFFFF);
+			ERROR_LOG(Log::G3D, "BJUMP to illegal address %08x - ignoring! data=%06x", target, op & 0x00FFFFFF);
 			UpdateState(GPUSTATE_ERROR);
 		}
 	}
@@ -913,7 +913,10 @@ void GPUCommon::Execute_Call(u32 op, u32 diff) {
 
 	const u32 target = gstate_c.getRelativeAddress(op & 0x00FFFFFC);
 	if (!Memory::IsValidAddress(target)) {
-		ERROR_LOG(G3D, "CALL to illegal address %08x - ignoring! data=%06x", target, op & 0x00FFFFFF);
+		ERROR_LOG(Log::G3D, "CALL to illegal address %08x - ignoring! data=%06x", target, op & 0x00FFFFFF);
+		if (g_Config.bIgnoreBadMemAccess) {
+			return;
+		}
 		UpdateState(GPUSTATE_ERROR);
 		return;
 	}
@@ -921,9 +924,6 @@ void GPUCommon::Execute_Call(u32 op, u32 diff) {
 }
 
 void GPUCommon::DoExecuteCall(u32 target) {
-	// Saint Seiya needs correct support for relative calls.
-	const u32 retval = currentList->pc + 4;
-
 	// Bone matrix optimization - many games will CALL a bone matrix (!).
 	// We don't optimize during recording - so the matrix data gets recorded.
 	if (!debugRecording_ && Memory::IsValidRange(target, 13 * 4) && (Memory::ReadUnchecked_U32(target) >> 24) == GE_CMD_BONEMATRIXDATA) {
@@ -940,11 +940,11 @@ void GPUCommon::DoExecuteCall(u32 target) {
 	}
 
 	if (currentList->stackptr == ARRAY_SIZE(currentList->stack)) {
-		ERROR_LOG(G3D, "CALL: Stack full!");
+		ERROR_LOG(Log::G3D, "CALL: Stack full!");
 		// TODO: UpdateState(GPUSTATE_ERROR) ?
 	} else {
 		auto &stackEntry = currentList->stack[currentList->stackptr++];
-		stackEntry.pc = retval;
+		stackEntry.pc = currentList->pc + 4;
 		stackEntry.offsetAddr = gstate_c.offsetAddr;
 		// The base address is NOT saved/restored for a regular call.
 		UpdatePC(currentList->pc, target - 4);
@@ -954,7 +954,7 @@ void GPUCommon::DoExecuteCall(u32 target) {
 
 void GPUCommon::Execute_Ret(u32 op, u32 diff) {
 	if (currentList->stackptr == 0) {
-		DEBUG_LOG(G3D, "RET: Stack empty!");
+		DEBUG_LOG(Log::G3D, "RET: Stack empty!");
 	} else {
 		auto &stackEntry = currentList->stack[--currentList->stackptr];
 		gstate_c.offsetAddr = stackEntry.offsetAddr;
@@ -964,7 +964,7 @@ void GPUCommon::Execute_Ret(u32 op, u32 diff) {
 		currentList->pc = target - 4;
 #ifdef _DEBUG
 		if (!Memory::IsValidAddress(currentList->pc)) {
-			ERROR_LOG_REPORT(G3D, "Invalid DL PC %08x on return", currentList->pc);
+			ERROR_LOG_REPORT(Log::G3D, "Invalid DL PC %08x on return", currentList->pc);
 			UpdateState(GPUSTATE_ERROR);
 		}
 #endif
@@ -997,12 +997,12 @@ void GPUCommon::Execute_End(u32 op, u32 diff) {
 				if (sceKernelGetCompiledSdkVersion() <= 0x02000010)
 					currentList->state = PSP_GE_DL_STATE_PAUSED;
 				currentList->signal = behaviour;
-				DEBUG_LOG(G3D, "Signal with wait. signal/end: %04x %04x", signal, enddata);
+				DEBUG_LOG(Log::G3D, "Signal with wait. signal/end: %04x %04x", signal, enddata);
 				break;
 			case PSP_GE_SIGNAL_HANDLER_CONTINUE:
 				// Resume the list right away, then call the handler.
 				currentList->signal = behaviour;
-				DEBUG_LOG(G3D, "Signal without wait. signal/end: %04x %04x", signal, enddata);
+				DEBUG_LOG(Log::G3D, "Signal without wait. signal/end: %04x %04x", signal, enddata);
 				break;
 			case PSP_GE_SIGNAL_HANDLER_PAUSE:
 				// Pause the list instead of ending at the next FINISH.
@@ -1011,7 +1011,7 @@ void GPUCommon::Execute_End(u32 op, u32 diff) {
 				// But right now, signal is always reset by interrupts, so that causes pause to not work.
 				trigger = false;
 				currentList->signal = behaviour;
-				DEBUG_LOG(G3D, "Signal with Pause. signal/end: %04x %04x", signal, enddata);
+				DEBUG_LOG(Log::G3D, "Signal with Pause. signal/end: %04x %04x", signal, enddata);
 				break;
 			case PSP_GE_SIGNAL_SYNC:
 				// Acts as a memory barrier, never calls any user code.
@@ -1020,7 +1020,7 @@ void GPUCommon::Execute_End(u32 op, u32 diff) {
 				// However, this is likely a bug in how GE signal interrupts are handled.
 				trigger = false;
 				currentList->signal = behaviour;
-				DEBUG_LOG(G3D, "Signal with Sync. signal/end: %04x %04x", signal, enddata);
+				DEBUG_LOG(Log::G3D, "Signal with Sync. signal/end: %04x %04x", signal, enddata);
 				break;
 			case PSP_GE_SIGNAL_JUMP:
 			case PSP_GE_SIGNAL_RJUMP:
@@ -1040,12 +1040,12 @@ void GPUCommon::Execute_End(u32 op, u32 diff) {
 					}
 
 					if (!Memory::IsValidAddress(target)) {
-						ERROR_LOG_REPORT(G3D, "Signal with Jump (%s): bad address. signal/end: %04x %04x", targetType, signal, enddata);
+						ERROR_LOG_REPORT(Log::G3D, "Signal with Jump (%s): bad address. signal/end: %04x %04x", targetType, signal, enddata);
 						UpdateState(GPUSTATE_ERROR);
 					} else {
 						UpdatePC(currentList->pc, target);
 						currentList->pc = target;
-						DEBUG_LOG(G3D, "Signal with Jump (%s). signal/end: %04x %04x", targetType, signal, enddata);
+						DEBUG_LOG(Log::G3D, "Signal with Jump (%s). signal/end: %04x %04x", targetType, signal, enddata);
 					}
 				}
 				break;
@@ -1067,9 +1067,9 @@ void GPUCommon::Execute_End(u32 op, u32 diff) {
 					}
 
 					if (currentList->stackptr == ARRAY_SIZE(currentList->stack)) {
-						ERROR_LOG_REPORT(G3D, "Signal with Call (%s): stack full. signal/end: %04x %04x", targetType, signal, enddata);
+						ERROR_LOG_REPORT(Log::G3D, "Signal with Call (%s): stack full. signal/end: %04x %04x", targetType, signal, enddata);
 					} else if (!Memory::IsValidAddress(target)) {
-						ERROR_LOG_REPORT(G3D, "Signal with Call (%s): bad address. signal/end: %04x %04x", targetType, signal, enddata);
+						ERROR_LOG_REPORT(Log::G3D, "Signal with Call (%s): bad address. signal/end: %04x %04x", targetType, signal, enddata);
 						UpdateState(GPUSTATE_ERROR);
 					} else {
 						// TODO: This might save/restore other state...
@@ -1079,7 +1079,7 @@ void GPUCommon::Execute_End(u32 op, u32 diff) {
 						stackEntry.baseAddr = gstate.base;
 						UpdatePC(currentList->pc, target);
 						currentList->pc = target;
-						DEBUG_LOG(G3D, "Signal with Call (%s). signal/end: %04x %04x", targetType, signal, enddata);
+						DEBUG_LOG(Log::G3D, "Signal with Call (%s). signal/end: %04x %04x", targetType, signal, enddata);
 					}
 				}
 				break;
@@ -1088,7 +1088,7 @@ void GPUCommon::Execute_End(u32 op, u32 diff) {
 					trigger = false;
 					currentList->signal = behaviour;
 					if (currentList->stackptr == 0) {
-						ERROR_LOG_REPORT(G3D, "Signal with Return: stack empty. signal/end: %04x %04x", signal, enddata);
+						ERROR_LOG_REPORT(Log::G3D, "Signal with Return: stack empty. signal/end: %04x %04x", signal, enddata);
 					} else {
 						// TODO: This might save/restore other state...
 						auto &stackEntry = currentList->stack[--currentList->stackptr];
@@ -1096,12 +1096,12 @@ void GPUCommon::Execute_End(u32 op, u32 diff) {
 						gstate.base = stackEntry.baseAddr;
 						UpdatePC(currentList->pc, stackEntry.pc);
 						currentList->pc = stackEntry.pc;
-						DEBUG_LOG(G3D, "Signal with Return. signal/end: %04x %04x", signal, enddata);
+						DEBUG_LOG(Log::G3D, "Signal with Return. signal/end: %04x %04x", signal, enddata);
 					}
 				}
 				break;
 			default:
-				ERROR_LOG_REPORT(G3D, "UNKNOWN Signal UNIMPLEMENTED %i ! signal/end: %04x %04x", behaviour, signal, enddata);
+				ERROR_LOG_REPORT(Log::G3D, "UNKNOWN Signal UNIMPLEMENTED %i ! signal/end: %04x %04x", behaviour, signal, enddata);
 				break;
 			}
 			// TODO: Technically, jump/call/ret should generate an interrupt, but before the pc change maybe?
@@ -1154,7 +1154,7 @@ void GPUCommon::Execute_End(u32 op, u32 diff) {
 		}
 		break;
 	default:
-		DEBUG_LOG(G3D,"Ah, not finished: %06x", prev & 0xFFFFFF);
+		DEBUG_LOG(Log::G3D,"Ah, not finished: %06x", prev & 0xFFFFFF);
 		break;
 	}
 }
@@ -1177,7 +1177,7 @@ void GPUCommon::Execute_BoundingBox(u32 op, u32 diff) {
 	if (Memory::IsValidRange(gstate_c.vertexAddr, bytesRead)) {
 		const void *control_points = Memory::GetPointerUnchecked(gstate_c.vertexAddr);
 		if (!control_points) {
-			ERROR_LOG_REPORT_ONCE(boundingbox, G3D, "Invalid verts in bounding box check");
+			ERROR_LOG_REPORT_ONCE(boundingbox, Log::G3D, "Invalid verts in bounding box check");
 			currentList->bboxResult = true;
 			return;
 		}
@@ -1187,7 +1187,7 @@ void GPUCommon::Execute_BoundingBox(u32 op, u32 diff) {
 			int indexShift = ((gstate.vertType & GE_VTYPE_IDX_MASK) >> GE_VTYPE_IDX_SHIFT) - 1;
 			inds = Memory::GetPointerUnchecked(gstate_c.indexAddr);
 			if (!inds || !Memory::IsValidRange(gstate_c.indexAddr, count << indexShift)) {
-				ERROR_LOG_REPORT_ONCE(boundingboxInds, G3D, "Invalid inds in bounding box check");
+				ERROR_LOG_REPORT_ONCE(boundingboxInds, Log::G3D, "Invalid inds in bounding box check");
 				currentList->bboxResult = true;
 				return;
 			}
@@ -1207,7 +1207,7 @@ void GPUCommon::Execute_BoundingBox(u32 op, u32 diff) {
 		}
 		AdvanceVerts(gstate.vertType, count, bytesRead);
 	} else {
-		ERROR_LOG_REPORT_ONCE(boundingbox, G3D, "Bad bounding box data: %06x", count);
+		ERROR_LOG_REPORT_ONCE(boundingbox, Log::G3D, "Bad bounding box data: %06x", count);
 		// Data seems invalid. Let's assume the box test passed.
 		currentList->bboxResult = true;
 	}
@@ -1222,7 +1222,7 @@ void GPUCommon::Execute_ImmVertexAlphaPrim(u32 op, u32 diff) {
 	if (immCount_ >= MAX_IMMBUFFER_SIZE) {
 		// Only print once for each overrun.
 		if (immCount_ == MAX_IMMBUFFER_SIZE) {
-			ERROR_LOG_REPORT_ONCE(exceed_imm_buffer, G3D, "Exceeded immediate draw buffer size. gstate.imm_ap=%06x , prim=%d", gstate.imm_ap & 0xFFFFFF, (int)immPrim_);
+			ERROR_LOG_REPORT_ONCE(exceed_imm_buffer, Log::G3D, "Exceeded immediate draw buffer size. gstate.imm_ap=%06x , prim=%d", gstate.imm_ap & 0xFFFFFF, (int)immPrim_);
 		}
 		if (immCount_ < 0x7fffffff)  // Paranoia :)
 			immCount_++;
@@ -1269,7 +1269,7 @@ void GPUCommon::Execute_ImmVertexAlphaPrim(u32 op, u32 diff) {
 		if (immCount_ == flushPrimCount[immPrim_ & 7])
 			FlushImm();
 	} else {
-		ERROR_LOG_REPORT_ONCE(imm_draw_prim, G3D, "Immediate draw: Unexpected primitive %d at count %d", prim, immCount_);
+		ERROR_LOG_REPORT_ONCE(imm_draw_prim, Log::G3D, "Immediate draw: Unexpected primitive %d at count %d", prim, immCount_);
 	}
 }
 
@@ -1277,16 +1277,26 @@ void GPUCommon::FlushImm() {
 	if (immCount_ == 0 || immPrim_ == GE_PRIM_INVALID)
 		return;
 
-	SetDrawType(DRAW_PRIM, immPrim_);
-	VirtualFramebuffer *vfb = nullptr;
-	if (framebufferManager_)
-		vfb = framebufferManager_->SetRenderFrameBuffer(gstate_c.IsDirty(DIRTY_FRAMEBUF), gstate_c.skipDrawReason);
 	if (gstate_c.skipDrawReason & (SKIPDRAW_SKIPFRAME | SKIPDRAW_NON_DISPLAYED_FB)) {
 		// No idea how many cycles to skip, heh.
 		immCount_ = 0;
 		return;
 	}
-	UpdateUVScaleOffset();
+
+	// Ignore immediate point primitives with a X/Y of 0.
+	// These are accidentally created when games clear the graphics state.
+	if (immCount_ == 1 && immPrim_ == GE_PRIM_POINTS && immBuffer_[0].x == 0 && immBuffer_[0].y == 0 && immBuffer_[0].z == 0 && immBuffer_[0].color0_32 == 0) {
+		immCount_ = 0;
+		return;
+	}
+
+	SetDrawType(DRAW_PRIM, immPrim_);
+
+	gstate_c.UpdateUVScaleOffset();
+
+	VirtualFramebuffer *vfb = nullptr;
+	if (framebufferManager_)
+		vfb = framebufferManager_->SetRenderFrameBuffer(gstate_c.IsDirty(DIRTY_FRAMEBUF), gstate_c.skipDrawReason);
 	if (vfb) {
 		CheckDepthUsage(vfb);
 	}
@@ -1306,7 +1316,7 @@ void GPUCommon::FlushImm() {
 	bool prevDither = gstate.isDitherEnabled();
 
 	if ((immFlags_ & GE_IMM_CLIPMASK) != 0) {
-		WARN_LOG_REPORT_ONCE(geimmclipvalue, G3D, "Imm vertex used clip value, flags=%06x", immFlags_);
+		WARN_LOG_REPORT_ONCE(geimmclipvalue, Log::G3D, "Imm vertex used clip value, flags=%06x", immFlags_);
 	}
 
 	bool changed = texturing != prevTexturing || cullEnable != prevCullEnable || dither != prevDither;
@@ -1339,8 +1349,7 @@ void GPUCommon::FlushImm() {
 }
 
 void GPUCommon::Execute_Unknown(u32 op, u32 diff) {
-	if ((op & 0xFFFFFF) != 0)
-		WARN_LOG_REPORT_ONCE(unknowncmd, G3D, "Unknown GE command : %08x ", op);
+	// Do nothing. We used to report here, but we're confident we have them all so no need to report unknown.
 }
 
 void GPUCommon::FastLoadBoneMatrix(u32 target) {
@@ -1545,8 +1554,8 @@ bool GPUCommon::GetCurrentDisplayList(DisplayList &list) {
 std::vector<DisplayList> GPUCommon::ActiveDisplayLists() {
 	std::vector<DisplayList> result;
 
-	for (auto it = dlQueue.begin(), end = dlQueue.end(); it != end; ++it) {
-		result.push_back(dls[*it]);
+	for (int it : dlQueue) {
+		result.push_back(dls[it]);
 	}
 
 	return result;
@@ -1605,6 +1614,7 @@ std::vector<GPUDebugOp> GPUCommon::DissassembleOpRange(u32 startpc, u32 endpc) {
 
 	// Don't trigger a pause.
 	u32 prev = Memory::IsValidAddress(startpc - 4) ? Memory::Read_U32(startpc - 4) : 0;
+	result.reserve((endpc - startpc) / 4);
 	for (u32 pc = startpc; pc < endpc; pc += 4) {
 		u32 op = Memory::IsValidAddress(pc) ? Memory::Read_U32(pc) : 0;
 		GeDisassembleOp(pc, op, prev, buffer, sizeof(buffer));
@@ -1664,7 +1674,8 @@ void GPUCommon::DoBlockTransfer(u32 skipDrawReason) {
 
 	int bpp = gstate.getTransferBpp();
 
-	DEBUG_LOG(G3D, "Block transfer: %08x/%x -> %08x/%x, %ix%ix%i (%i,%i)->(%i,%i)", srcBasePtr, srcStride, dstBasePtr, dstStride, width, height, bpp, srcX, srcY, dstX, dstY);
+	DEBUG_LOG(Log::G3D, "Block transfer: %08x/%x -> %08x/%x, %ix%ix%i (%i,%i)->(%i,%i)", srcBasePtr, srcStride, dstBasePtr, dstStride, width, height, bpp, srcX, srcY, dstX, dstY);
+	gpuStats.numBlockTransfers++;
 
 	// For VRAM, we wrap around when outside valid memory (mirrors still work.)
 	if ((srcBasePtr & 0x04800000) == 0x04800000)
@@ -1672,11 +1683,12 @@ void GPUCommon::DoBlockTransfer(u32 skipDrawReason) {
 	if ((dstBasePtr & 0x04800000) == 0x04800000)
 		dstBasePtr &= ~0x00800000;
 
-	// Use height less one to account for width, which can be greater or less than stride.
+	// Use height less one to account for width, which can be greater or less than stride, and then add it on for the last line.
+	// NOTE: The sizes are only used for validity checks and memory info tracking.
 	const uint32_t src = srcBasePtr + (srcY * srcStride + srcX) * bpp;
-	const uint32_t srcSize = (height - 1) * (srcStride + width) * bpp;
 	const uint32_t dst = dstBasePtr + (dstY * dstStride + dstX) * bpp;
-	const uint32_t dstSize = (height - 1) * (dstStride + width) * bpp;
+	const uint32_t srcSize = ((height - 1) * srcStride) + width * bpp;
+	const uint32_t dstSize = ((height - 1) * dstStride) + width * bpp;
 
 	bool srcDstOverlap = src + srcSize > dst && dst + dstSize > src;
 	bool srcValid = Memory::IsValidRange(src, srcSize);
@@ -1704,9 +1716,7 @@ void GPUCommon::DoBlockTransfer(u32 skipDrawReason) {
 			memcpy(dstp, srcp, bytesToCopy);
 
 			if (MemBlockInfoDetailed(bytesToCopy)) {
-				tagSize = FormatMemWriteTagAt(tag, sizeof(tag), "GPUBlockTransfer/", src, bytesToCopy);
-				NotifyMemInfo(MemBlockFlags::READ, src, bytesToCopy, tag, tagSize);
-				NotifyMemInfo(MemBlockFlags::WRITE, dst, bytesToCopy, tag, tagSize);
+				NotifyMemInfoCopy(dst, src, bytesToCopy, "GPUBlockTransfer/");
 			}
 		} else if ((srcDstOverlap || srcWraps || dstWraps) && (srcValid || srcWraps) && (dstValid || dstWraps)) {
 			// This path means we have either src/dst overlap, OR one or both of src and dst wrap.
@@ -1841,7 +1851,7 @@ void GPUCommon::DoBlockTransfer(u32 skipDrawReason) {
 		} else {
 			// This seems to cause the GE to require a break/reset on a PSP.
 			// TODO: Handle that and figure out which bytes are still copied?
-			ERROR_LOG_REPORT_ONCE(invalidtransfer, G3D, "Block transfer invalid: %08x/%x -> %08x/%x, %ix%ix%i (%i,%i)->(%i,%i)", srcBasePtr, srcStride, dstBasePtr, dstStride, width, height, bpp, srcX, srcY, dstX, dstY);
+			ERROR_LOG_REPORT_ONCE(invalidtransfer, Log::G3D, "Block transfer invalid: %08x/%x -> %08x/%x, %ix%ix%i (%i,%i)->(%i,%i)", srcBasePtr, srcStride, dstBasePtr, dstStride, width, height, bpp, srcX, srcY, dstX, dstY);
 		}
 
 		if (framebufferManager_) {
@@ -1856,18 +1866,29 @@ void GPUCommon::DoBlockTransfer(u32 skipDrawReason) {
 }
 
 bool GPUCommon::PerformMemoryCopy(u32 dest, u32 src, int size, GPUCopyFlag flags) {
+	/*
+	// TODO: Should add this. But let's do it after the 1.18 release.
+	if (dest == 0 || src == 0) {
+		_dbg_assert_msg_(false, "Bad PerformMemoryCopy: %08x -> %08x, size %d (flag: %d)", src, dest, size, (int)flags);
+		return false;
+	}
+	*/
+	if (size == 0) {
+		_dbg_assert_msg_(false, "Zero-sized PerformMemoryCopy: %08x -> %08x, size %d (flag: %d)", src, dest, size, (int)flags);
+		// Let's not ignore this yet but if we hit this, we should investigate.
+	}
+
 	// Track stray copies of a framebuffer in RAM. MotoGP does this.
-	if (framebufferManager_->MayIntersectFramebuffer(src) || framebufferManager_->MayIntersectFramebuffer(dest)) {
+	if (framebufferManager_->MayIntersectFramebufferColor(src) || framebufferManager_->MayIntersectFramebufferColor(dest)) {
 		if (!framebufferManager_->NotifyFramebufferCopy(src, dest, size, flags, gstate_c.skipDrawReason)) {
 			// We use matching values in PerformReadbackToMemory/PerformWriteColorFromMemory.
 			// Since they're identical we don't need to copy.
 			if (dest != src) {
+				if (Memory::IsValidRange(dest, size) && Memory::IsValidRange(src, size)) {
+					memcpy(Memory::GetPointerWriteUnchecked(dest), Memory::GetPointerUnchecked(src), size);
+				}
 				if (MemBlockInfoDetailed(size)) {
-					char tag[128];
-					size_t tagSize = FormatMemWriteTagAt(tag, sizeof(tag), "GPUMemcpy/", src, size);
-					Memory::Memcpy(dest, src, size, tag, tagSize);
-				} else {
-					Memory::Memcpy(dest, src, size, "GPUMemcpy");
+					NotifyMemInfoCopy(dest, src, size, "GPUMemcpy/");
 				}
 			}
 		}
@@ -1876,10 +1897,7 @@ bool GPUCommon::PerformMemoryCopy(u32 dest, u32 src, int size, GPUCopyFlag flags
 	}
 
 	if (MemBlockInfoDetailed(size)) {
-		char tag[128];
-		size_t tagSize = FormatMemWriteTagAt(tag, sizeof(tag), "GPUMemcpy/", src, size);
-		NotifyMemInfo(MemBlockFlags::READ, src, size, tag, tagSize);
-		NotifyMemInfo(MemBlockFlags::WRITE, dest, size, tag, tagSize);
+		NotifyMemInfoCopy(dest, src, size, "GPUMemcpy/");
 	}
 	InvalidateCache(dest, size, GPU_INVALIDATE_HINT);
 	if (!(flags & GPUCopyFlag::DEBUG_NOTIFIED))
@@ -1889,7 +1907,7 @@ bool GPUCommon::PerformMemoryCopy(u32 dest, u32 src, int size, GPUCopyFlag flags
 
 bool GPUCommon::PerformMemorySet(u32 dest, u8 v, int size) {
 	// This may indicate a memset, usually to 0, of a framebuffer.
-	if (framebufferManager_->MayIntersectFramebuffer(dest)) {
+	if (framebufferManager_->MayIntersectFramebufferColor(dest)) {
 		Memory::Memset(dest, v, size, "GPUMemset");
 		if (!framebufferManager_->NotifyFramebufferCopy(dest, dest, size, GPUCopyFlag::MEMSET, gstate_c.skipDrawReason)) {
 			InvalidateCache(dest, size, GPU_INVALIDATE_HINT);
@@ -1928,7 +1946,7 @@ void GPUCommon::PerformWriteFormattedFromMemory(u32 addr, int size, int frameWid
 }
 
 bool GPUCommon::PerformWriteStencilFromMemory(u32 dest, int size, WriteStencil flags) {
-	if (framebufferManager_->MayIntersectFramebuffer(dest)) {
+	if (framebufferManager_->MayIntersectFramebufferColor(dest)) {
 		framebufferManager_->PerformWriteStencilFromMemory(dest, size, flags);
 		return true;
 	}
@@ -1936,29 +1954,12 @@ bool GPUCommon::PerformWriteStencilFromMemory(u32 dest, int size, WriteStencil f
 }
 
 bool GPUCommon::GetCurrentSimpleVertices(int count, std::vector<GPUDebugVertex> &vertices, std::vector<u16> &indices) {
-	UpdateUVScaleOffset();
+	gstate_c.UpdateUVScaleOffset();
 	return drawEngineCommon_->GetCurrentSimpleVertices(count, vertices, indices);
 }
 
 bool GPUCommon::DescribeCodePtr(const u8 *ptr, std::string &name) {
-	if (drawEngineCommon_->IsCodePtrVertexDecoder(ptr)) {
-		name = "VertexDecoderJit";
-		return true;
-	}
-	return false;
-}
-
-void GPUCommon::UpdateUVScaleOffset() {
-#ifdef _M_SSE
-	__m128i values = _mm_slli_epi32(_mm_load_si128((const __m128i *) & gstate.texscaleu), 8);
-	_mm_storeu_si128((__m128i *)&gstate_c.uv, values);
-#elif PPSSPP_ARCH(ARM_NEON)
-	const uint32x4_t values = vshlq_n_u32(vld1q_u32((const u32 *)&gstate.texscaleu), 8);
-	vst1q_u32((u32 *)&gstate_c.uv, values);
-#else
-	gstate_c.uv.uScale = getFloat24(gstate.texscaleu);
-	gstate_c.uv.vScale = getFloat24(gstate.texscalev);
-	gstate_c.uv.uOff = getFloat24(gstate.texoffsetu);
-	gstate_c.uv.vOff = getFloat24(gstate.texoffsetv);
-#endif
+	// The only part of GPU emulation (other than software) that jits is the vertex decoder, currently,
+	// which is owned by the drawengine.
+	return drawEngineCommon_->DescribeCodePtr(ptr, name);
 }
