@@ -17,12 +17,25 @@
 
 #pragma once
 
+#include "ppsspp_config.h"
+
 #include "Common/CommonTypes.h"
 #include "Common/Swap.h"
 #include "GPU/GPU.h"
 #include "GPU/ge_constants.h"
 #include "GPU/Common/ShaderCommon.h"
 
+#if defined(_M_SSE)
+#include <emmintrin.h>
+#endif
+#if PPSSPP_ARCH(ARM_NEON)
+#if defined(_MSC_VER) && PPSSPP_ARCH(ARM64)
+#include <arm64_neon.h>
+#else
+#include <arm_neon.h>
+#endif
+#endif
+
 class PointerWrap;
 
 struct GPUgstate {
@@ -443,7 +456,7 @@ struct GPUgstate {
 
 	// Real data in the context ends here
 
-	void Reset();
+	static void Reset();
 	void Save(u32_le *ptr);
 	void Restore(const u32_le *ptr);
 };
@@ -523,6 +536,8 @@ enum class SubmitType {
 	HW_SPLINE,
 };
 
+extern GPUgstate gstate;
+
 struct GPUStateCache {
 	bool Use(u32 flags) const { return (useFlags_ & flags) != 0; } // Return true if ANY of flags are true.
 	bool UseAll(u32 flags) const { return (useFlags_ & flags) == flags; } // Return true if ALL flags are true.
@@ -574,6 +589,9 @@ struct GPUStateCache {
 			Dirty(DIRTY_FRAGMENTSHADER_STATE);
 		}
 	}
+	void SetTextureIsVideo(bool isVideo) {
+		textureIsVideo = isVideo;
+	}
 	void SetTextureIsBGRA(bool isBGRA) {
 		if (bgraTexture != isBGRA) {
 			bgraTexture = isBGRA;
@@ -604,6 +622,21 @@ struct GPUStateCache {
 		return useFlags_;
 	}
 
+	void UpdateUVScaleOffset() {
+#if defined(_M_SSE)
+		__m128i values = _mm_slli_epi32(_mm_load_si128((const __m128i *)&gstate.texscaleu), 8);
+		_mm_storeu_si128((__m128i *)&uv, values);
+#elif PPSSPP_ARCH(ARM_NEON)
+		const uint32x4_t values = vshlq_n_u32(vld1q_u32((const u32 *)&gstate.texscaleu), 8);
+		vst1q_u32((u32 *)&uv, values);
+#else
+		uv.uScale = getFloat24(gstate.texscaleu);
+		uv.vScale = getFloat24(gstate.texscalev);
+		uv.uOff = getFloat24(gstate.texoffsetu);
+		uv.vOff = getFloat24(gstate.texoffsetv);
+#endif
+	}
+
 private:
 	u32 useFlags_;
 public:
@@ -627,6 +660,7 @@ public:
 	bool needShaderTexClamp;
 	bool textureIsArray;
 	bool textureIsFramebuffer;
+	bool textureIsVideo;
 	bool useFlagsChanged;
 
 	float morphWeights[8];
@@ -659,6 +693,9 @@ public:
 	// We detect this case and go into a special drawing mode.
 	bool blueToAlpha;
 
+	// U/V is 1:1 to pixels. Can influence texture sampling.
+	bool pixelMapped;
+
 	// TODO: These should be accessed from the current VFB object directly.
 	u32 curRTWidth;
 	u32 curRTHeight;
@@ -683,14 +720,13 @@ public:
 	GEBufferFormat depalFramebufferFormat;
 
 	u32 getRelativeAddress(u32 data) const;
-	void Reset();
+	static void Reset();
 	void DoState(PointerWrap &p);
 };
 
 class GPUInterface;
 class GPUDebugInterface;
 
-extern GPUgstate gstate;
 extern GPUStateCache gstate_c;
 
 inline u32 GPUStateCache::getRelativeAddress(u32 data) const {
