@@ -34,7 +34,7 @@ void GeDescribeVertexType(u32 op, char *buffer, int len) {
 	int morphCount = (op & GE_VTYPE_MORPHCOUNT_MASK) >> GE_VTYPE_MORPHCOUNT_SHIFT;
 	int idx = (op & GE_VTYPE_IDX_MASK) >> GE_VTYPE_IDX_SHIFT;
 
-	static const char *colorNames[] = {
+	static const char * const colorNames[] = {
 		NULL,
 		"unsupported1",
 		"unsupported2",
@@ -44,19 +44,19 @@ void GeDescribeVertexType(u32 op, char *buffer, int len) {
 		"ABGR 4444",
 		"ABGR 8888",
 	};
-	static const char *typeNames[] = {
+	static const char * const typeNames[] = {
 		NULL,
 		"u8",
 		"u16",
 		"float",
 	};
-	static const char *typeNamesI[] = {
+	static const char * const typeNamesI[] = {
 		NULL,
 		"u8",
 		"u16",
 		"u32",
 	};
-	static const char *typeNamesS[] = {
+	static const char * const typeNamesS[] = {
 		NULL,
 		"s8",
 		"s16",
@@ -474,13 +474,13 @@ void GeDisassembleOp(u32 pc, u32 op, u32 prev, char *buffer, int bufsize) {
 
 	case GE_CMD_TEXMAPMODE:
 		{
-			static const char *uvgen[] = {
+			static const char * const uvgen[] = {
 				"texcoords",
 				"texgen matrix",
 				"env map",
 				"invalid"
 			};
-			static const char *uvproj[] = {
+			static const char * const uvproj[] = {
 				"pos",
 				"uv",
 				"normalized normal",
@@ -1006,7 +1006,7 @@ void GeDisassembleOp(u32 pc, u32 op, u32 prev, char *buffer, int bufsize) {
 
 	case GE_CMD_TEXFUNC:
 		{
-			const char *texfuncs[] = {
+			const char * const texfuncs[] = {
 				"modulate",
 				"decal",
 				"blend",
@@ -1025,7 +1025,7 @@ void GeDisassembleOp(u32 pc, u32 op, u32 prev, char *buffer, int bufsize) {
 
 	case GE_CMD_TEXFILTER:
 		{
-			static const char *textureFilters[] = {
+			static const char * const textureFilters[] = {
 				"nearest",
 				"linear",
 				"nearest, invalid",
@@ -1057,7 +1057,7 @@ void GeDisassembleOp(u32 pc, u32 op, u32 prev, char *buffer, int bufsize) {
 
 	case GE_CMD_TEXFORMAT:
 		{
-			static const char *texformats[] = {
+			static const char * const texformats[] = {
 				"5650",
 				"5551",
 				"4444",
@@ -1105,7 +1105,7 @@ void GeDisassembleOp(u32 pc, u32 op, u32 prev, char *buffer, int bufsize) {
 
 	case GE_CMD_TEXLEVEL:
 		{
-			static const char *mipLevelModes[] = {
+			static const char * const mipLevelModes[] = {
 				"auto + bias",
 				"bias",
 				"slope + bias",
@@ -1149,14 +1149,14 @@ void GeDisassembleOp(u32 pc, u32 op, u32 prev, char *buffer, int bufsize) {
 
 	case GE_CMD_STENCILOP:
 		{
-			static const char *stencilOps[] = { "KEEP", "ZERO", "REPLACE", "INVERT", "INCREMENT", "DECREMENT", "unsupported1", "unsupported2" };
+			static const char * const stencilOps[] = { "KEEP", "ZERO", "REPLACE", "INVERT", "INCREMENT", "DECREMENT", "unsupported1", "unsupported2" };
 			snprintf(buffer, bufsize, "Stencil op: fail=%s, pass/depthfail=%s, pass=%s", stencilOps[data & 7], stencilOps[(data >> 8) & 7], stencilOps[(data >> 16) & 7]);
 		}
 		break;
 
 	case GE_CMD_STENCILTEST:
 		{
-			static const char *zTestFuncs[] = { " NEVER ", " ALWAYS ", " == ", " != ", " < ", " <= ", " > ", " >= " };
+			static const char * const zTestFuncs[] = { " NEVER ", " ALWAYS ", " == ", " != ", " < ", " <= ", " > ", " >= " };
 			if (data & ~0xFFFF07)
 				snprintf(buffer, bufsize, "Stencil test: %02x%s(dst.a & %02x) (extra %06x)", (data >> 8) & 0xFF, zTestFuncs[data & 7], (data >> 16) & 0xFF, data & ~0xFFFF07);
 			else
@@ -1170,7 +1170,7 @@ void GeDisassembleOp(u32 pc, u32 op, u32 prev, char *buffer, int bufsize) {
 
 	case GE_CMD_ZTEST:
 		{
-			static const char *zTestFuncs[] = { "NEVER", "ALWAYS", "==", "!=", "<", "<=", ">", ">=" };
+			static const char * const zTestFuncs[] = { "NEVER", "ALWAYS", "==", "!=", "<", "<=", ">", ">=" };
 			if (data & ~7)
 				snprintf(buffer, bufsize, "Z test mode: %s (extra %06x)", zTestFuncs[data & 7], data & ~7);
 			else
@@ -1202,7 +1202,7 @@ void GeDisassembleOp(u32 pc, u32 op, u32 prev, char *buffer, int bufsize) {
 
 	case GE_CMD_LOGICOP:
 		{
-			const char *logicOps[] = {
+			const char * const logicOps[] = {
 				"clear",
 				"and",
 				"reverse and",
