@@ -17,10 +17,11 @@
 
 #pragma once
 
-#include "thin3d/thin3d.h"
+#include "Common/GPU/thin3d.h"
 
 class GPUInterface;
 class UIContext;
 
 // gpu MUST be an instance of GPU_Vulkan. If not, will definitely crash.
-void DrawAllocatorVis(UIContext *ui, GPUInterface *gpu);
+void DrawGPUMemoryVis(UIContext *ui, GPUInterface *gpu);
+void DrawGPUProfilerVis(UIContext *ui, GPUInterface *gpu);
