@@ -81,7 +81,7 @@ GPU_Vulkan::GPU_Vulkan(GraphicsContext *gfxCtx, Draw::DrawContext *draw)
 
 	// Sanity check gstate
 	if ((int *)&gstate.transferstart - (int *)&gstate != 0xEA) {
-		ERROR_LOG(G3D, "gstate has drifted out of sync!");
+		ERROR_LOG(Log::G3D, "gstate has drifted out of sync!");
 	}
 
 	BuildReportingInfo();
@@ -99,7 +99,7 @@ GPU_Vulkan::GPU_Vulkan(GraphicsContext *gfxCtx, Draw::DrawContext *draw)
 
 void GPU_Vulkan::LoadCache(const Path &filename) {
 	if (!g_Config.bShaderCache) {
-		WARN_LOG(G3D, "Shader cache disabled. Not loading.");
+		WARN_LOG(Log::G3D, "Shader cache disabled. Not loading.");
 		return;
 	}
 
@@ -114,7 +114,7 @@ void GPU_Vulkan::LoadCache(const Path &filename) {
 	// it can just memcpy the finished shader binaries out of the pipeline cache file.
 	bool result = shaderManagerVulkan_->LoadCacheFlags(f, &drawEngine_);
 	if (!result) {
-		WARN_LOG(G3D, "ShaderManagerVulkan failed to load cache header.");
+		WARN_LOG(Log::G3D, "ShaderManagerVulkan failed to load cache header.");
 	}
 	if (result) {
 		// Reload use flags in case LoadCacheFlags() changed them.
@@ -124,7 +124,7 @@ void GPU_Vulkan::LoadCache(const Path &filename) {
 		gstate_c.SetUseFlags(CheckGPUFeatures());
 		result = shaderManagerVulkan_->LoadCache(f);
 		if (!result) {
-			WARN_LOG(G3D, "ShaderManagerVulkan failed to load cache.");
+			WARN_LOG(Log::G3D, "ShaderManagerVulkan failed to load cache.");
 		}
 	}
 	if (result) {
@@ -134,23 +134,23 @@ void GPU_Vulkan::LoadCache(const Path &filename) {
 	fclose(f);
 
 	if (!result) {
-		WARN_LOG(G3D, "Incompatible Vulkan pipeline cache - rebuilding.");
+		WARN_LOG(Log::G3D, "Incompatible Vulkan pipeline cache - rebuilding.");
 		// Bad cache file for this GPU/Driver/etc. Delete it.
 		File::Delete(filename);
 	} else {
-		INFO_LOG(G3D, "Loaded Vulkan pipeline cache.");
+		INFO_LOG(Log::G3D, "Loaded Vulkan pipeline cache.");
 	}
 }
 
 void GPU_Vulkan::SaveCache(const Path &filename) {
 	if (!g_Config.bShaderCache) {
-		INFO_LOG(G3D, "Shader cache disabled. Not saving.");
+		INFO_LOG(Log::G3D, "Shader cache disabled. Not saving.");
 		return;
 	}
 
 	if (!draw_) {
 		// Already got the lost message, we're in shutdown.
-		WARN_LOG(G3D, "Not saving shaders - shutting down from in-game.");
+		WARN_LOG(Log::G3D, "Not saving shaders - shutting down from in-game.");
 		return;
 	}
 
@@ -160,7 +160,7 @@ void GPU_Vulkan::SaveCache(const Path &filename) {
 	shaderManagerVulkan_->SaveCache(f, &drawEngine_);
 	// WARNING: See comment in LoadCache if you are tempted to flip the second parameter to true.
 	pipelineManager_->SavePipelineCache(f, false, shaderManagerVulkan_, draw_);
-	INFO_LOG(G3D, "Saved Vulkan pipeline cache");
+	INFO_LOG(Log::G3D, "Saved Vulkan pipeline cache");
 	fclose(f);
 }
 
@@ -168,12 +168,13 @@ GPU_Vulkan::~GPU_Vulkan() {
 	if (draw_) {
 		VulkanRenderManager *rm = (VulkanRenderManager *)draw_->GetNativeObject(Draw::NativeObject::RENDER_MANAGER);
 		// This now also does a hard sync with the render thread, so that we can safely delete our pipeline layout below.
-		rm->DrainAndBlockCompileQueue();
+		rm->StopThreads();
+		rm->CheckNothingPending();
 	}
 
 	SaveCache(shaderCachePath_);
 
-	// Super important to delete pipeline manager FIRST, before clearing shaders, so we wait for all pending pipelines to finish compiling.
+	// StopThreads should have ensured that no pipelines are queued to compile at this point. So we can tear it down.
 	delete pipelineManager_;
 	pipelineManager_ = nullptr;
 
@@ -185,7 +186,7 @@ GPU_Vulkan::~GPU_Vulkan() {
 	// other managers are deleted in ~GPUCommonHW.
 	if (draw_) {
 		VulkanRenderManager *rm = (VulkanRenderManager *)draw_->GetNativeObject(Draw::NativeObject::RENDER_MANAGER);
-		rm->ReleaseCompileQueue();
+		rm->StartThreads();
 	}
 }
 
@@ -254,18 +255,18 @@ u32 GPU_Vulkan::CheckGPUFeatures() const {
 		}
 	}
 
-	// These are VULKAN_4444_FORMAT and friends.
-	uint32_t fmt4444 = draw_->GetDataFormatSupport(Draw::DataFormat::B4G4R4A4_UNORM_PACK16);
-	uint32_t fmt1555 = draw_->GetDataFormatSupport(Draw::DataFormat::A1R5G5B5_UNORM_PACK16);
-
-	// Note that we are (accidentally) using B5G6R5 instead of the mandatory R5G6B5.
-	// Support is almost as widespread, but not quite. So let's just not use any 16-bit formats
-	// if it's not available, for simplicity.
-	uint32_t fmt565 = draw_->GetDataFormatSupport(Draw::DataFormat::B5G6R5_UNORM_PACK16);
-	if ((fmt4444 & Draw::FMT_TEXTURE) && (fmt565 & Draw::FMT_TEXTURE) && (fmt1555 & Draw::FMT_TEXTURE)) {
-		features |= GPU_USE_16BIT_FORMATS;
-	} else {
-		INFO_LOG(G3D, "Deficient texture format support: 4444: %d  1555: %d  565: %d", fmt4444, fmt1555, fmt565);
+	if (!draw_->GetBugs().Has(Draw::Bugs::PVR_BAD_16BIT_TEXFORMATS)) {
+		// These are VULKAN_4444_FORMAT and friends.
+		// Note that we are now using the correct set of formats - the only cases where some may be missing
+		// are non-conformant implementations like MoltenVK.
+		uint32_t fmt4444 = draw_->GetDataFormatSupport(Draw::DataFormat::B4G4R4A4_UNORM_PACK16);
+		uint32_t fmt1555 = draw_->GetDataFormatSupport(Draw::DataFormat::A1R5G5B5_UNORM_PACK16);
+		uint32_t fmt565 = draw_->GetDataFormatSupport(Draw::DataFormat::R5G6B5_UNORM_PACK16);
+		if ((fmt4444 & Draw::FMT_TEXTURE) && (fmt565 & Draw::FMT_TEXTURE) && (fmt1555 & Draw::FMT_TEXTURE)) {
+			features |= GPU_USE_16BIT_FORMATS;
+		} else {
+			INFO_LOG(Log::G3D, "Deficient texture format support: 4444: %d  1555: %d  565: %d", fmt4444, fmt1555, fmt565);
+		}
 	}
 
 	if (g_Config.bStereoRendering && draw_->GetDeviceCaps().multiViewSupported) {
@@ -305,7 +306,7 @@ void GPU_Vulkan::BeginHostFrame() {
 	if (gstate_c.useFlagsChanged) {
 		// TODO: It'd be better to recompile them in the background, probably?
 		// This most likely means that saw equal depth changed.
-		WARN_LOG(G3D, "Shader use flags changed, clearing all shaders and depth buffers");
+		WARN_LOG(Log::G3D, "Shader use flags changed, clearing all shaders and depth buffers");
 		// TODO: Not all shaders need to be recompiled. In fact, quite few? Of course, depends on
 		// the use flag change.. This is a major frame rate hitch in the start of a race in Outrun.
 		shaderManager_->ClearShaders();
@@ -315,7 +316,7 @@ void GPU_Vulkan::BeginHostFrame() {
 	}
 
 	if (dumpNextFrame_) {
-		NOTICE_LOG(G3D, "DUMPING THIS FRAME");
+		NOTICE_LOG(Log::G3D, "DUMPING THIS FRAME");
 		dumpThisFrame_ = true;
 		dumpNextFrame_ = false;
 	} else if (dumpThisFrame_) {
@@ -387,7 +388,7 @@ void GPU_Vulkan::FinishDeferred() {
 }
 
 void GPU_Vulkan::InitDeviceObjects() {
-	INFO_LOG(G3D, "GPU_Vulkan::InitDeviceObjects");
+	INFO_LOG(Log::G3D, "GPU_Vulkan::InitDeviceObjects");
 
 	uint32_t hacks = 0;
 	if (PSP_CoreParameter().compat.flags().MGS2AcidHack)
@@ -405,7 +406,7 @@ void GPU_Vulkan::InitDeviceObjects() {
 }
 
 void GPU_Vulkan::DestroyDeviceObjects() {
-	INFO_LOG(G3D, "GPU_Vulkan::DestroyDeviceObjects");
+	INFO_LOG(Log::G3D, "GPU_Vulkan::DestroyDeviceObjects");
 	// Need to turn off hacks when shutting down the GPU. Don't want them running in the menu.
 	if (draw_) {
 		VulkanRenderManager *rm = (VulkanRenderManager *)draw_->GetNativeObject(Draw::NativeObject::RENDER_MANAGER);
@@ -423,16 +424,12 @@ void GPU_Vulkan::CheckRenderResized() {
 }
 
 void GPU_Vulkan::DeviceLost() {
-	CancelReady();
-	while (!IsReady()) {
-		sleep_ms(10);
-	}
 	// draw_ is normally actually still valid here in Vulkan. But we null it out in GPUCommonHW::DeviceLost so we don't try to use it again.
 	// So, we have to save it here to be able to call ReleaseCompileQueue().
 	Draw::DrawContext *draw = draw_;
 	if (draw) {
 		VulkanRenderManager *rm = (VulkanRenderManager *)draw->GetNativeObject(Draw::NativeObject::RENDER_MANAGER);
-		rm->DrainAndBlockCompileQueue();
+		rm->StopThreads();
 	}
 
 	if (shaderCachePath_.Valid()) {
@@ -445,7 +442,7 @@ void GPU_Vulkan::DeviceLost() {
 
 	if (draw) {
 		VulkanRenderManager *rm = (VulkanRenderManager *)draw->GetNativeObject(Draw::NativeObject::RENDER_MANAGER);
-		rm->ReleaseCompileQueue();
+		rm->StartThreads();
 	}
 }
 
