@@ -27,6 +27,7 @@
 #include "Common/GPU/thin3d.h"
 #include "Common/Data/Encoding/Utf8.h"
 #include "Common/TimeUtil.h"
+#include "Common/MemoryUtil.h"
 
 #include "Common/StringUtils.h"
 #include "Common/GPU/Vulkan/VulkanContext.h"
@@ -59,13 +60,13 @@ static Promise<VkShaderModule> *CompileShaderModuleAsync(VulkanContext *vulkan,
 
 		if (!errorMessage.empty()) {
 			if (success) {
-				ERROR_LOG(G3D, "Warnings in shader compilation!");
+				ERROR_LOG(Log::G3D, "Warnings in shader compilation!");
 			} else {
-				ERROR_LOG(G3D, "Error in shader compilation!");
+				ERROR_LOG(Log::G3D, "Error in shader compilation!");
 			}
 			std::string numberedSource = LineNumberString(code);
-			ERROR_LOG(G3D, "Messages: %s", errorMessage.c_str());
-			ERROR_LOG(G3D, "Shader source:\n%s", numberedSource.c_str());
+			ERROR_LOG(Log::G3D, "Messages: %s", errorMessage.c_str());
+			ERROR_LOG(Log::G3D, "Shader source:\n%s", numberedSource.c_str());
 #if PPSSPP_PLATFORM(WINDOWS)
 			OutputDebugStringA("Error messages:\n");
 			OutputDebugStringA(errorMessage.c_str());
@@ -91,8 +92,7 @@ static Promise<VkShaderModule> *CompileShaderModuleAsync(VulkanContext *vulkan,
 #ifdef SHADERLOG
 			OutputDebugStringA("OK");
 #endif
-			if (tag)
-				delete tag;
+			delete tag;
 		}
 		return shaderModule;
 	};
@@ -116,7 +116,7 @@ VulkanFragmentShader::VulkanFragmentShader(VulkanContext *vulkan, FShaderID id,
 	_assert_(!id.is_invalid());
 	source_ = code;
 	module_ = CompileShaderModuleAsync(vulkan, VK_SHADER_STAGE_FRAGMENT_BIT, source_.c_str(), new std::string(FragmentShaderDesc(id)));
-	VERBOSE_LOG(G3D, "Compiled fragment shader:\n%s\n", (const char *)code);
+	VERBOSE_LOG(Log::G3D, "Compiled fragment shader:\n%s\n", (const char *)code);
 }
 
 VulkanFragmentShader::~VulkanFragmentShader() {
@@ -148,7 +148,7 @@ VulkanVertexShader::VulkanVertexShader(VulkanContext *vulkan, VShaderID id, Vert
 	_assert_(!id.is_invalid());
 	source_ = code;
 	module_ = CompileShaderModuleAsync(vulkan, VK_SHADER_STAGE_VERTEX_BIT, source_.c_str(), new std::string(VertexShaderDesc(id)));
-	VERBOSE_LOG(G3D, "Compiled vertex shader:\n%s\n", (const char *)code);
+	VERBOSE_LOG(Log::G3D, "Compiled vertex shader:\n%s\n", (const char *)code);
 }
 
 VulkanVertexShader::~VulkanVertexShader() {
@@ -180,7 +180,7 @@ VulkanGeometryShader::VulkanGeometryShader(VulkanContext *vulkan, GShaderID id,
 	_assert_(!id.is_invalid());
 	source_ = code;
 	module_ = CompileShaderModuleAsync(vulkan, VK_SHADER_STAGE_GEOMETRY_BIT, source_.c_str(), new std::string(GeometryShaderDesc(id).c_str()));
-	VERBOSE_LOG(G3D, "Compiled geometry shader:\n%s\n", (const char *)code);
+	VERBOSE_LOG(Log::G3D, "Compiled geometry shader:\n%s\n", (const char *)code);
 }
 
 VulkanGeometryShader::~VulkanGeometryShader() {
@@ -214,16 +214,17 @@ ShaderManagerVulkan::ShaderManagerVulkan(Draw::DrawContext *draw)
 	codeBuffer_ = new char[CODE_BUFFER_SIZE];
 	VulkanContext *vulkan = (VulkanContext *)draw->GetNativeObject(Draw::NativeObject::CONTEXT);
 	uboAlignment_ = vulkan->GetPhysicalDeviceProperties().properties.limits.minUniformBufferOffsetAlignment;
-	memset(&ub_base, 0, sizeof(ub_base));
-	memset(&ub_lights, 0, sizeof(ub_lights));
-	memset(&ub_bones, 0, sizeof(ub_bones));
 
-	static_assert(sizeof(ub_base) <= 512, "ub_base grew too big");
-	static_assert(sizeof(ub_lights) <= 512, "ub_lights grew too big");
-	static_assert(sizeof(ub_bones) <= 384, "ub_bones grew too big");
+	uniforms_ = (Uniforms *)AllocateAlignedMemory(sizeof(Uniforms), 16);
+	_assert_(uniforms_);
+
+	static_assert(sizeof(uniforms_->ub_base) <= 512, "ub_base grew too big");
+	static_assert(sizeof(uniforms_->ub_lights) <= 512, "ub_lights grew too big");
+	static_assert(sizeof(uniforms_->ub_bones) <= 384, "ub_bones grew too big");
 }
 
 ShaderManagerVulkan::~ShaderManagerVulkan() {
+	FreeAlignedMemory(uniforms_);
 	Clear();
 	delete[] codeBuffer_;
 }
@@ -279,11 +280,11 @@ uint64_t ShaderManagerVulkan::UpdateUniforms(bool useBufferedRendering) {
 	uint64_t dirty = gstate_c.GetDirtyUniforms();
 	if (dirty != 0) {
 		if (dirty & DIRTY_BASE_UNIFORMS)
-			BaseUpdateUniforms(&ub_base, dirty, false, useBufferedRendering);
+			BaseUpdateUniforms(&uniforms_->ub_base, dirty, false, useBufferedRendering);
 		if (dirty & DIRTY_LIGHT_UNIFORMS)
-			LightUpdateUniforms(&ub_lights, dirty);
+			LightUpdateUniforms(&uniforms_->ub_lights, dirty);
 		if (dirty & DIRTY_BONE_UNIFORMS)
-			BoneUpdateUniforms(&ub_bones, dirty);
+			BoneUpdateUniforms(&uniforms_->ub_bones, dirty);
 	}
 	gstate_c.CleanUniforms();
 	return dirty;
@@ -393,32 +394,26 @@ std::vector<std::string> ShaderManagerVulkan::DebugGetShaderIDs(DebugShaderType
 	std::vector<std::string> ids;
 	switch (type) {
 	case SHADER_TYPE_VERTEX:
-	{
 		vsCache_.Iterate([&](const VShaderID &id, VulkanVertexShader *shader) {
 			std::string idstr;
 			id.ToString(&idstr);
 			ids.push_back(idstr);
 		});
 		break;
-	}
 	case SHADER_TYPE_FRAGMENT:
-	{
 		fsCache_.Iterate([&](const FShaderID &id, VulkanFragmentShader *shader) {
 			std::string idstr;
 			id.ToString(&idstr);
 			ids.push_back(idstr);
 		});
 		break;
-	}
 	case SHADER_TYPE_GEOMETRY:
-	{
 		gsCache_.Iterate([&](const GShaderID &id, VulkanGeometryShader *shader) {
 			std::string idstr;
 			id.ToString(&idstr);
 			ids.push_back(idstr);
 		});
 		break;
-	}
 	default:
 		break;
 	}
@@ -507,7 +502,7 @@ enum class VulkanCacheDetectFlags {
 };
 
 #define CACHE_HEADER_MAGIC 0xff51f420 
-#define CACHE_VERSION 48
+#define CACHE_VERSION 52
 
 struct VulkanCacheHeader {
 	uint32_t magic;
@@ -526,11 +521,11 @@ bool ShaderManagerVulkan::LoadCacheFlags(FILE *f, DrawEngineVulkan *drawEngine)
 	// We'll read it again later, this is just to check the flags.
 	success = success && fseek(f, pos, SEEK_SET) == 0;
 	if (!success || header.magic != CACHE_HEADER_MAGIC) {
-		WARN_LOG(G3D, "Shader cache magic mismatch");
+		WARN_LOG(Log::G3D, "Shader cache magic mismatch");
 		return false;
 	}
 	if (header.version != CACHE_VERSION) {
-		WARN_LOG(G3D, "Shader cache version mismatch, %d, expected %d", header.version, CACHE_VERSION);
+		WARN_LOG(Log::G3D, "Shader cache version mismatch, %d, expected %d", header.version, CACHE_VERSION);
 		return false;
 	}
 
@@ -549,7 +544,7 @@ bool ShaderManagerVulkan::LoadCache(FILE *f) {
 	if (header.useFlags != gstate_c.GetUseFlags()) {
 		// This can simply be a result of sawExactEqualDepth_ having been flipped to true in the previous run.
 		// Let's just keep going.
-		WARN_LOG(G3D, "Shader cache useFlags mismatch, %08x, expected %08x", header.useFlags, gstate_c.GetUseFlags());
+		WARN_LOG(Log::G3D, "Shader cache useFlags mismatch, %08x, expected %08x", header.useFlags, gstate_c.GetUseFlags());
 	} else {
 		// We're compiling shaders now, so they haven't changed anymore.
 		gstate_c.useFlagsChanged = false;
@@ -561,7 +556,7 @@ bool ShaderManagerVulkan::LoadCache(FILE *f) {
 	for (int i = 0; i < header.numVertexShaders; i++) {
 		VShaderID id;
 		if (fread(&id, sizeof(id), 1, f) != 1) {
-			ERROR_LOG(G3D, "Vulkan shader cache truncated (in VertexShaders)");
+			ERROR_LOG(Log::G3D, "Vulkan shader cache truncated (in VertexShaders)");
 			return false;
 		}
 		bool useHWTransform = id.Bit(VS_BIT_USE_HW_TRANSFORM);
@@ -570,7 +565,7 @@ bool ShaderManagerVulkan::LoadCache(FILE *f) {
 		uint64_t uniformMask = 0;
 		VertexShaderFlags flags;
 		if (!GenerateVertexShader(id, codeBuffer_, compat_, draw_->GetBugs(), &attributeMask, &uniformMask, &flags, &genErrorString)) {
-			ERROR_LOG(G3D, "Failed to generate vertex shader during cache load");
+			ERROR_LOG(Log::G3D, "Failed to generate vertex shader during cache load");
 			// We just ignore this one and carry on.
 			failCount++;
 			continue;
@@ -587,14 +582,14 @@ bool ShaderManagerVulkan::LoadCache(FILE *f) {
 	for (int i = 0; i < header.numFragmentShaders; i++) {
 		FShaderID id;
 		if (fread(&id, sizeof(id), 1, f) != 1) {
-			ERROR_LOG(G3D, "Vulkan shader cache truncated (in FragmentShaders)");
+			ERROR_LOG(Log::G3D, "Vulkan shader cache truncated (in FragmentShaders)");
 			return false;
 		}
 		std::string genErrorString;
 		uint64_t uniformMask = 0;
 		FragmentShaderFlags flags;
 		if (!GenerateFragmentShader(id, codeBuffer_, compat_, draw_->GetBugs(), &uniformMask, &flags, &genErrorString)) {
-			ERROR_LOG(G3D, "Failed to generate fragment shader during cache load");
+			ERROR_LOG(Log::G3D, "Failed to generate fragment shader during cache load");
 			// We just ignore this one and carry on.
 			failCount++;
 			continue;
@@ -611,12 +606,12 @@ bool ShaderManagerVulkan::LoadCache(FILE *f) {
 		for (int i = 0; i < header.numGeometryShaders; i++) {
 			GShaderID id;
 			if (fread(&id, sizeof(id), 1, f) != 1) {
-				ERROR_LOG(G3D, "Vulkan shader cache truncated (in GeometryShaders)");
+				ERROR_LOG(Log::G3D, "Vulkan shader cache truncated (in GeometryShaders)");
 				return false;
 			}
 			std::string genErrorString;
 			if (!GenerateGeometryShader(id, codeBuffer_, compat_, draw_->GetBugs(), &genErrorString)) {
-				ERROR_LOG(G3D, "Failed to generate geometry shader during cache load");
+				ERROR_LOG(Log::G3D, "Failed to generate geometry shader during cache load");
 				// We just ignore this one and carry on.
 				failCount++;
 				continue;
@@ -629,7 +624,7 @@ bool ShaderManagerVulkan::LoadCache(FILE *f) {
 		}
 	}
 
-	NOTICE_LOG(G3D, "ShaderCache: Loaded %d vertex, %d fragment shaders and %d geometry shaders (failed %d)", header.numVertexShaders, header.numFragmentShaders, header.numGeometryShaders, failCount);
+	NOTICE_LOG(Log::G3D, "ShaderCache: Loaded %d vertex, %d fragment shaders and %d geometry shaders (failed %d)", header.numVertexShaders, header.numFragmentShaders, header.numGeometryShaders, failCount);
 	return true;
 }
 
@@ -655,8 +650,8 @@ void ShaderManagerVulkan::SaveCache(FILE *f, DrawEngineVulkan *drawEngine) {
 		writeFailed = writeFailed || fwrite(&id, sizeof(id), 1, f) != 1;
 	});
 	if (writeFailed) {
-		ERROR_LOG(G3D, "Failed to write Vulkan shader cache, disk full?");
+		ERROR_LOG(Log::G3D, "Failed to write Vulkan shader cache, disk full?");
 	} else {
-		NOTICE_LOG(G3D, "Saved %d vertex and %d fragment shaders", header.numVertexShaders, header.numFragmentShaders);
+		NOTICE_LOG(Log::G3D, "Saved %d vertex and %d fragment shaders", header.numVertexShaders, header.numFragmentShaders);
 	}
 }
