@@ -45,6 +45,8 @@
 #include "Common/Thread/ThreadUtil.h"
 #include "Common/Data/Encoding/Utf8.h"
 #include "Common/StringUtils.h"
+#include "Common/TimeUtil.h"
+
 #include "Core/Config.h"
 #include "Core/ConfigValues.h"
 #include "Core/HW/Camera.h"
@@ -53,7 +55,7 @@
 #include <signal.h>
 #include <string.h>
 
-// Audio
+// AUDIO
 #define AUDIO_FREQ 44100
 #define AUDIO_CHANNELS 2
 #define AUDIO_SAMPLES 2048
@@ -92,23 +94,23 @@ static void InitSDLAudioDevice() {
 	if (!g_Config.sAudioDevice.empty()) {
 		audioDev = SDL_OpenAudioDevice(g_Config.sAudioDevice.c_str(), 0, &fmt, &g_retFmt, SDL_AUDIO_ALLOW_FREQUENCY_CHANGE);
 		if (audioDev <= 0) {
-			WARN_LOG(AUDIO, "Failed to open preferred audio device %s", g_Config.sAudioDevice.c_str());
+			WARN_LOG(Log::Audio, "Failed to open preferred audio device %s", g_Config.sAudioDevice.c_str());
 		}
 	}
 	if (audioDev <= 0) {
 		audioDev = SDL_OpenAudioDevice(nullptr, 0, &fmt, &g_retFmt, SDL_AUDIO_ALLOW_FREQUENCY_CHANGE);
 	}
 	if (audioDev <= 0) {
-		ERROR_LOG(AUDIO, "Failed to open audio: %s", SDL_GetError());
+		ERROR_LOG(Log::Audio, "Failed to open audio: %s", SDL_GetError());
 	} else {
 		if (g_retFmt.samples != fmt.samples) // Notify, but still use it
-			ERROR_LOG(AUDIO, "Output audio samples: %d (requested: %d)", g_retFmt.samples, fmt.samples);
+			ERROR_LOG(Log::Audio, "Output audio samples: %d (requested: %d)", g_retFmt.samples, fmt.samples);
 		if (g_retFmt.format != fmt.format || g_retFmt.channels != fmt.channels) {
-			ERROR_LOG(AUDIO, "Sound buffer format does not match requested format.");
-			ERROR_LOG(AUDIO, "Output audio freq: %d (requested: %d)", g_retFmt.freq, fmt.freq);
-			ERROR_LOG(AUDIO, "Output audio format: %d (requested: %d)", g_retFmt.format, fmt.format);
-			ERROR_LOG(AUDIO, "Output audio channels: %d (requested: %d)", g_retFmt.channels, fmt.channels);
-			ERROR_LOG(AUDIO, "Provided output format does not match requirement, turning audio off");
+			ERROR_LOG(Log::Audio, "Sound buffer format does not match requested format.");
+			ERROR_LOG(Log::Audio, "Output audio freq: %d (requested: %d)", g_retFmt.freq, fmt.freq);
+			ERROR_LOG(Log::Audio, "Output audio format: %d (requested: %d)", g_retFmt.format, fmt.format);
+			ERROR_LOG(Log::Audio, "Output audio channels: %d (requested: %d)", g_retFmt.channels, fmt.channels);
+			ERROR_LOG(Log::Audio, "Provided output format does not match requirement, turning audio off");
 			SDL_CloseAudioDevice(audioDev);
 		}
 		SDL_PauseAudioDevice(audioDev, 0);
@@ -185,7 +187,7 @@ std::vector<std::string> System_GetPropertyStringVec(SystemProperty prop) {
 	}
 }
 
-int System_GetPropertyInt(SystemProperty prop) {
+int64_t System_GetPropertyInt(SystemProperty prop) {
 	switch (prop) {
 #if defined(SDL)
 	case SYSPROP_AUDIO_SAMPLE_RATE:
@@ -245,6 +247,8 @@ float System_GetPropertyFloat(SystemProperty prop) {
 
 bool System_GetPropertyBool(SystemProperty prop) {
 	switch (prop) {
+	case SYSPROP_HAS_TEXT_CLIPBOARD:
+		return true;
 	case SYSPROP_HAS_BACK_BUTTON:
 		return true;
 	case SYSPROP_HAS_IMAGE_BROWSER:
@@ -304,7 +308,7 @@ bool MainUI::HandleCustomEvent(QEvent *e) {
 		const char *filter = "All files (*.*)";
 		switch (fileType) {
 		case BrowseFileType::BOOTABLE:
-			filter = "PSP ROMs (*.iso *.cso *.pbp *.elf *.zip *.ppdmp)";
+			filter = "PSP ROMs (*.iso *.cso *.chd *.pbp *.elf *.zip *.ppdmp)";
 			break;
 		case BrowseFileType::IMAGE:
 			filter = "Pictures (*.jpg *.png)";
@@ -316,7 +320,10 @@ bool MainUI::HandleCustomEvent(QEvent *e) {
 			filter = "DB files (*.db)";
 			break;
 		case BrowseFileType::SOUND_EFFECT:
-			filter = "WAVE files (*.wav)";
+			filter = "WAVE files (*.wav *.mp3)";
+			break;
+		case BrowseFileType::ZIP:
+			filter = "ZIP files (*.zip)";
 			break;
 		case BrowseFileType::ANY:
 			break;
@@ -351,7 +358,7 @@ bool MainUI::HandleCustomEvent(QEvent *e) {
 	return true;
 }
 
-bool System_MakeRequest(SystemRequestType type, int requestId, const std::string &param1, const std::string &param2, int param3) {
+bool System_MakeRequest(SystemRequestType type, int requestId, const std::string &param1, const std::string &param2, int64_t param3, int64_t param4) {
 	switch (type) {
 	case SystemRequestType::EXIT_APP:
 		qApp->exit(0);
@@ -385,7 +392,7 @@ bool System_MakeRequest(SystemRequestType type, int requestId, const std::string
 	}
 	case SystemRequestType::BROWSE_FOR_IMAGE:
 		// Fall back to file browser.
-		return System_MakeRequest(SystemRequestType::BROWSE_FOR_FILE, requestId, param1, param2, (int)BrowseFileType::IMAGE);
+		return System_MakeRequest(SystemRequestType::BROWSE_FOR_FILE, requestId, param1, param2, (int)BrowseFileType::IMAGE, 0);
 	case SystemRequestType::BROWSE_FOR_FILE:
 		g_requestId = requestId;
 		g_param1 = param1;
@@ -416,7 +423,7 @@ bool System_MakeRequest(SystemRequestType type, int requestId, const std::string
 	}
 }
 
-void System_Toast(const char *text) {}
+void System_Toast(std::string_view text) {}
 
 void System_AskForPermission(SystemPermission permission) {}
 PermissionStatus System_GetPermissionStatus(SystemPermission permission) { return PERMISSION_STATUS_GRANTED; }
@@ -514,9 +521,9 @@ MainUI::MainUI(QWidget *parent)
 }
 
 MainUI::~MainUI() {
-	INFO_LOG(SYSTEM, "MainUI::Destructor");
+	INFO_LOG(Log::System, "MainUI::Destructor");
 	if (emuThreadState != (int)EmuThreadState::DISABLED) {
-		INFO_LOG(SYSTEM, "EmuThreadStop");
+		INFO_LOG(Log::System, "EmuThreadStop");
 		EmuThreadStop();
 		while (graphicsContext->ThreadFrame()) {
 			// Need to keep eating frames to allow the EmuThread to exit correctly.
@@ -542,7 +549,7 @@ QString MainUI::InputBoxGetQString(QString title, QString defaultValue) {
 
 void MainUI::resizeGL(int w, int h) {
 	if (UpdateScreenScale(w, h)) {
-		System_PostUIMessage("gpu_displayResized", "");
+		System_PostUIMessage(UIMessage::GPU_RENDER_RESIZED);
 	}
 	xscale = w / this->width();
 	yscale = h / this->height();
@@ -658,7 +665,7 @@ bool MainUI::event(QEvent *e) {
 			default:
 				if (str.size()) {
 					int pos = 0;
-					int unicode = u8_nextchar(str.c_str(), &pos);
+					int unicode = u8_nextchar(str.c_str(), &pos, str.size());
 					NativeKey(KeyInput(DEVICE_ID_KEYBOARD, unicode));
 				}
 				break;
@@ -681,7 +688,7 @@ bool MainUI::event(QEvent *e) {
 
 void MainUI::initializeGL() {
 	if (g_Config.iGPUBackend != (int)GPUBackend::OPENGL) {
-		INFO_LOG(SYSTEM, "Only GL supported under Qt - switching.");
+		INFO_LOG(Log::System, "Only GL supported under Qt - switching.");
 		g_Config.iGPUBackend = (int)GPUBackend::OPENGL;
 	}
 
@@ -703,12 +710,12 @@ void MainUI::initializeGL() {
 #endif
 	if (g_Config.iGPUBackend == (int)GPUBackend::OPENGL) {
 		// OpenGL uses a background thread to do the main processing and only renders on the gl thread.
-		INFO_LOG(SYSTEM, "Initializing GL graphics context");
+		INFO_LOG(Log::System, "Initializing GL graphics context");
 		graphicsContext = new QtGLGraphicsContext();
-		INFO_LOG(SYSTEM, "Using thread, starting emu thread");
+		INFO_LOG(Log::System, "Using thread, starting emu thread");
 		EmuThreadStart();
 	} else {
-		INFO_LOG(SYSTEM, "Not using thread, backend=%d", (int)g_Config.iGPUBackend);
+		INFO_LOG(Log::System, "Not using thread, backend=%d", (int)g_Config.iGPUBackend);
 	}
 	graphicsContext->ThreadStart();
 }
@@ -796,6 +803,8 @@ Q_DECL_EXPORT
 #endif
 int main(int argc, char *argv[])
 {
+	TimeInit();
+
 	for (int i = 1; i < argc; i++) {
 		if (!strcmp(argv[i], "--version")) {
 			printf("%s\n", PPSSPP_GIT_VERSION);
@@ -861,7 +870,7 @@ int main(int argc, char *argv[])
 	g_Config.iGPUBackend = (int)GPUBackend::OPENGL;
 
 	int ret = mainInternal(a);
-	INFO_LOG(SYSTEM, "Left mainInternal here.");
+	INFO_LOG(Log::System, "Left mainInternal here.");
 
 #ifdef SDL
 	if (audioDev > 0) {
