@@ -1,6 +1,8 @@
 #include <string>
 #include <mutex>
 
+#include "ext/minimp3/minimp3_ex.h"
+
 #include "Common/File/VFS/VFS.h"
 #include "Common/UI/Root.h"
 
@@ -34,22 +36,23 @@ struct WavData {
 	int raw_bytes_per_frame = 0;
 	uint8_t *raw_data = nullptr;
 	int raw_data_size = 0;
-	u8 at3_extradata[16];
+	u8 at3_extradata[16]{};
 
-	void Read(RIFFReader &riff);
+	bool Read(RIFFReader &riff);
 
 	~WavData() {
 		free(raw_data);
 		raw_data = nullptr;
 	}
 
+	[[nodiscard]]
 	bool IsSimpleWAV() const {
 		bool isBad = raw_bytes_per_frame > sizeof(int16_t) * num_channels;
 		return !isBad && num_channels > 0 && sample_rate >= 8000 && codec == 0;
 	}
 };
 
-void WavData::Read(RIFFReader &file_) {
+bool WavData::Read(RIFFReader &file_) {
 	// If we have no loop start info, we'll just loop the entire audio.
 	raw_offset_loop_start = 0;
 	raw_offset_loop_end = 0;
@@ -71,8 +74,8 @@ void WavData::Read(RIFFReader &file_) {
 				codec = 0;
 				break;
 			default:
-				ERROR_LOG(SCEAUDIO, "Unexpected wave format %04x", format);
-				return;
+				ERROR_LOG(Log::sceAudio, "Unexpected wave format %04x", format);
+				return false;
 			}
 
 			num_channels = temp >> 16;
@@ -93,11 +96,11 @@ void WavData::Read(RIFFReader &file_) {
 				}
 			}
 			file_.Ascend();
-			// INFO_LOG(AUDIO, "got fmt data: %i", samplesPerSec);
+			// INFO_LOG(Log::AUDIO, "got fmt data: %i", samplesPerSec);
 		} else {
-			ERROR_LOG(AUDIO, "Error - no format chunk in wav");
+			ERROR_LOG(Log::Audio, "Error - no format chunk in wav");
 			file_.Ascend();
-			return;
+			return false;
 		}
 
 		if (file_.Descend('smpl')) {
@@ -146,29 +149,32 @@ void WavData::Read(RIFFReader &file_) {
 			int numBytes = file_.GetCurrentChunkSize();
 			numFrames = numBytes / raw_bytes_per_frame;  // numFrames
 
-			raw_data = (uint8_t *)malloc(numBytes);
+			// It seems the atrac3 codec likes to read a little bit outside.
+			const int padding = 32;  // 32 is the value FFMPEG uses.
+			raw_data = (uint8_t *)malloc(numBytes + padding);
 			raw_data_size = numBytes;
 
 			if (num_channels == 1 || num_channels == 2) {
 				file_.ReadData(raw_data, numBytes);
 			} else {
-				ERROR_LOG(AUDIO, "Error - bad blockalign or channels");
+				ERROR_LOG(Log::Audio, "Error - bad blockalign or channels");
 				free(raw_data);
 				raw_data = nullptr;
-				return;
+				return false;
 			}
 			file_.Ascend();
 		} else {
-			ERROR_LOG(AUDIO, "Error - no data chunk in wav");
+			ERROR_LOG(Log::Audio, "Error - no data chunk in wav");
 			file_.Ascend();
-			return;
+			return false;
 		}
 		file_.Ascend();
 	} else {
-		ERROR_LOG(AUDIO, "Could not descend into RIFF file.");
-		return;
+		ERROR_LOG(Log::Audio, "Could not descend into RIFF file.");
+		return false;
 	}
 	sample_rate = samplesPerSec;
+	return true;
 }
 
 // Really simple looping in-memory AT3 player that also takes care of reading the file format.
@@ -184,11 +190,18 @@ public:
 
 		wave_.Read(file_);
 
-		decoder_ = new SimpleAudio(wave_.codec, wave_.sample_rate, wave_.num_channels);
+		uint8_t *extraData = nullptr;
+		size_t extraDataSize = 0;
+		size_t blockSize = 0;
 		if (wave_.codec == PSP_CODEC_AT3) {
-			decoder_->SetExtraData(&wave_.at3_extradata[2], 14, wave_.raw_bytes_per_frame);
+			extraData = &wave_.at3_extradata[2];
+			extraDataSize = 14;
+			blockSize = wave_.raw_bytes_per_frame;
+		} else if (wave_.codec == PSP_CODEC_AT3PLUS) {
+			blockSize = wave_.raw_bytes_per_frame;
 		}
-		INFO_LOG(AUDIO, "read ATRAC, frames: %d, rate %d", wave_.numFrames, wave_.sample_rate);
+		decoder_ = CreateAudioDecoder((PSPAudioType)wave_.codec, wave_.sample_rate, wave_.num_channels, blockSize, extraData, extraDataSize);
+		INFO_LOG(Log::Audio, "read ATRAC, frames: %d, rate %d", wave_.numFrames, wave_.sample_rate);
 	}
 
 	~AT3PlusReader() {
@@ -198,17 +211,19 @@ public:
 		decoder_ = nullptr;
 	}
 
-	bool IsOK() { return wave_.raw_data != nullptr; }
+	bool IsOK() const { return wave_.raw_data != nullptr; }
 
 	bool Read(int *buffer, int len) {
 		if (!wave_.raw_data)
 			return false;
 
 		while (bgQueue.size() < (size_t)(len * 2)) {
-			int outBytes = 0;
-			decoder_->Decode(wave_.raw_data + raw_offset_, wave_.raw_bytes_per_frame, (uint8_t *)buffer_, &outBytes);
-			if (!outBytes)
+			int outSamples = 0;
+			int inbytesConsumed = 0;
+			bool result = decoder_->Decode(wave_.raw_data + raw_offset_, wave_.raw_bytes_per_frame, &inbytesConsumed, 2, (int16_t *)buffer_, &outSamples);
+			if (!result || !outSamples)
 				return false;
+			int outBytes = outSamples * 2 * sizeof(int16_t);
 
 			if (wave_.raw_offset_loop_end != 0 && raw_offset_ == wave_.raw_offset_loop_end) {
 				// Only take the remaining bytes, but convert to stereo s16.
@@ -251,7 +266,7 @@ private:
 	int skip_next_samples_ = 0;
 	FixedSizeQueue<s16, 128 * 1024> bgQueue;
 	short *buffer_ = nullptr;
-	SimpleAudio *decoder_ = nullptr;
+	AudioDecoder *decoder_ = nullptr;
 };
 
 BackgroundAudio g_BackgroundAudio;
@@ -302,10 +317,6 @@ void BackgroundAudio::SetGame(const Path &path) {
 }
 
 bool BackgroundAudio::Play() {
-	if (GetUIState() == UISTATE_INGAME) {
-		return false;
-	}
-
 	std::lock_guard<std::mutex> lock(mutex_);
 
 	// Immediately stop the sound if it is turned off while playing.
@@ -324,11 +335,13 @@ bool BackgroundAudio::Play() {
 	if (at3Reader_) {
 		if (at3Reader_->Read(buffer, sz)) {
 			if (fadingOut_) {
+                float vol = volume_;
 				for (int i = 0; i < sz*2; i += 2) {
-					buffer[i] *= volume_;
-					buffer[i + 1] *= volume_;
-					volume_ += delta_;
+					buffer[i] = (int)((float)buffer[i] * vol);
+					buffer[i + 1] = (int)((float)buffer[i + 1] * vol);
+					vol += delta_;
 				}
+                volume_ = vol;
 			}
 		}
 	} else {
@@ -361,8 +374,8 @@ void BackgroundAudio::Update() {
 			return;
 
 		// Grab some audio from the current game and play it.
-		std::shared_ptr<GameInfo> gameInfo = g_gameInfoCache->GetInfo(nullptr, bgGamePath_, GAMEINFO_WANTSND);
-		if (!gameInfo || gameInfo->pending) {
+		std::shared_ptr<GameInfo> gameInfo = g_gameInfoCache->GetInfo(nullptr, bgGamePath_, GameInfoFlags::SND);
+		if (!gameInfo->Ready(GameInfoFlags::SND)) {
 			// Should try again shortly..
 			return;
 		}
@@ -382,40 +395,67 @@ inline int16_t ConvertU8ToI16(uint8_t value) {
 }
 
 Sample *Sample::Load(const std::string &path) {
-	size_t bytes;
-	uint8_t *data = g_VFS.ReadFile(path.c_str(), &bytes);
-	if (!data) {
-		WARN_LOG(AUDIO, "Failed to load sample '%s'", path.c_str());
+	size_t data_size = 0;
+	uint8_t *data = g_VFS.ReadFile(path.c_str(), &data_size);
+	if (!data || data_size > 100000000) {
+		WARN_LOG(Log::Audio, "Failed to load sample '%s'", path.c_str());
 		return nullptr;
 	}
 
-	RIFFReader reader(data, (int)bytes);
-
-	WavData wave;
-	wave.Read(reader);
-
-	delete[] data;
-
-	if (!wave.IsSimpleWAV()) {
-		ERROR_LOG(AUDIO, "Wave format not supported for mixer playback. Must be 8-bit or 16-bit raw mono or stereo. '%s'", path.c_str());
-		return nullptr;
-	}
-
-	int16_t *samples = new int16_t[wave.num_channels * wave.numFrames];
-	if (wave.raw_bytes_per_frame == wave.num_channels * 2) {
-		// 16-bit
-		memcpy(samples, wave.raw_data, wave.numFrames * wave.raw_bytes_per_frame);
-	} else if (wave.raw_bytes_per_frame == wave.num_channels) {
-		// 8-bit. Convert.
-		for (int i = 0; i < wave.num_channels * wave.numFrames; i++) {
-			samples[i] = ConvertU8ToI16(wave.raw_data[i]);
+	const char *mp3_magic = "ID3\03";
+	const char *wav_magic = "RIFF";
+	if (!memcmp(data, wav_magic, 4)) {
+		RIFFReader reader(data, (int)data_size);
+		WavData wave;
+		if (!wave.Read(reader)) {
+			delete[] data;
+			return nullptr;
 		}
+		// A wav file.
+		delete[] data;
+
+		if (!wave.IsSimpleWAV()) {
+			ERROR_LOG(Log::Audio, "Wave format not supported for mixer playback. Must be 8-bit or 16-bit raw mono or stereo. '%s'", path.c_str());
+			return nullptr;
+		}
+
+		int16_t *samples = new int16_t[wave.num_channels * wave.numFrames];
+		if (wave.raw_bytes_per_frame == wave.num_channels * 2) {
+			// 16-bit
+			memcpy(samples, wave.raw_data, wave.numFrames * wave.raw_bytes_per_frame);
+		} else if (wave.raw_bytes_per_frame == wave.num_channels) {
+			// 8-bit. Convert.
+			for (int i = 0; i < wave.num_channels * wave.numFrames; i++) {
+				samples[i] = ConvertU8ToI16(wave.raw_data[i]);
+			}
+		}
+
+		// Protect against bad metadata.
+		int actualFrames = std::min(wave.numFrames, wave.raw_data_size / wave.raw_bytes_per_frame);
+
+		return new Sample(samples, wave.num_channels, actualFrames, wave.sample_rate);
 	}
 
-	// Protect against bad metadata.
-	int actualFrames = std::min(wave.numFrames, wave.raw_data_size / wave.raw_bytes_per_frame);
+	// Something else.
+	// Let's see if minimp3 can read it.
+	mp3dec_t mp3d;
+	mp3dec_init(&mp3d);
+	mp3dec_file_info_t mp3_info;
+	int retval = mp3dec_load_buf(&mp3d, data, data_size, &mp3_info, nullptr, nullptr);
 
-	return new Sample(samples, wave.num_channels, actualFrames, wave.sample_rate);
+	if (retval < 0 || mp3_info.samples == 0) {
+		ERROR_LOG(Log::Audio, "Couldn't load MP3 for sound effect from %s", path.c_str());
+		return nullptr;
+	}
+
+	// mp3_info contains the decoded data.
+	int16_t *sample_data = new int16_t[mp3_info.samples];
+	memcpy(sample_data, mp3_info.buffer, mp3_info.samples * sizeof(int16_t));
+
+	Sample *sample = new Sample(sample_data, mp3_info.channels, (int)mp3_info.samples / mp3_info.channels, mp3_info.hz);
+	free(mp3_info.buffer);
+	delete[] data;
+	return sample;
 }
 
 static inline int16_t Clamp16(int32_t sample) {
@@ -518,14 +558,38 @@ void SoundEffectMixer::LoadDefaultSample(UI::UISound sound) {
 	}
 	Sample *sample = Sample::Load(filename);
 	if (!sample) {
-		ERROR_LOG(SYSTEM, "Failed to load the default sample for UI sound %d", (int)sound);
+		ERROR_LOG(Log::System, "Failed to load the default sample for UI sound %d", (int)sound);
 	}
 	std::lock_guard<std::mutex> guard(mutex_);
 	samples_[(size_t)sound] = std::unique_ptr<Sample>(sample);
 }
 
-void SoundEffectMixer::LoadSamples() {
+class SampleLoadTask : public Task {
+public:
+	SampleLoadTask(SoundEffectMixer *mixer) : mixer_(mixer) {}
+	TaskType Type() const override { return TaskType::IO_BLOCKING; }
+	TaskPriority Priority() const override {
+		return TaskPriority::NORMAL;
+	}
+	virtual void Run() override {
+		mixer_->LoadSamplesOnThread();
+	}
+private:
+	SoundEffectMixer *mixer_;
+};
+
+void SoundEffectMixer::Init() {
 	samples_.resize((size_t)UI::UISound::COUNT);
+	UI::SetSoundCallback([](UI::UISound sound, float volume) {
+		g_BackgroundAudio.SFX().Play(sound, volume);
+	});
+
+	// Load samples in the background.
+
+	g_threadManager.EnqueueTask(new SampleLoadTask(this));
+}
+
+void SoundEffectMixer::LoadSamplesOnThread() {
 	LoadDefaultSample(UI::UISound::BACK);
 	LoadDefaultSample(UI::UISound::SELECT);
 	LoadDefaultSample(UI::UISound::CONFIRM);
@@ -542,8 +606,4 @@ void SoundEffectMixer::LoadSamples() {
 	} else {
 		LoadDefaultSample(UI::UISound::LEADERBOARD_SUBMITTED);
 	}
-
-	UI::SetSoundCallback([](UI::UISound sound, float volume) {
-		g_BackgroundAudio.SFX().Play(sound, volume);
-	});
 }
