@@ -47,15 +47,16 @@ CwCheatScreen::~CwCheatScreen() {
 	delete engine_;
 }
 
-void CwCheatScreen::LoadCheatInfo() {
-	std::shared_ptr<GameInfo> info = g_gameInfoCache->GetInfo(nullptr, gamePath_, 0);
+bool CwCheatScreen::TryLoadCheatInfo() {
+	std::shared_ptr<GameInfo> info = g_gameInfoCache->GetInfo(nullptr, gamePath_, GameInfoFlags::PARAM_SFO);
 	std::string gameID;
-	if (info && info->paramSFOLoaded) {
-		gameID = info->paramSFO.GetValueString("DISC_ID");
+	if (!info->Ready(GameInfoFlags::PARAM_SFO)) {
+		return false;
 	}
+	gameID = info->GetParamSFO().GetValueString("DISC_ID");
 	if ((info->id.empty() || !info->disc_total)
 		&& gamePath_.FilePathContainsNoCase("PSP/GAME/")) {
-		gameID = g_paramSFO.GenerateFakeID(gamePath_.ToString());
+		gameID = g_paramSFO.GenerateFakeID(gamePath_);
 	}
 
 	if (!engine_ || gameID != gameID_) {
@@ -67,7 +68,7 @@ void CwCheatScreen::LoadCheatInfo() {
 
 	// We won't parse this, just using it to detect changes to the file.
 	std::string str;
-	if (File::ReadFileToString(true, engine_->CheatFilename(), str)) {
+	if (File::ReadTextFileToString(engine_->CheatFilename(), &str)) {
 		fileCheckHash_ = XXH3_64bits(str.c_str(), str.size());
 	}
 	fileCheckCounter_ = 0;
@@ -76,6 +77,7 @@ void CwCheatScreen::LoadCheatInfo() {
 
 	// Let's also trigger a reload, in case it changed.
 	g_Config.bReloadCheats = true;
+	return true;
 }
 
 void CwCheatScreen::CreateViews() {
@@ -86,7 +88,7 @@ void CwCheatScreen::CreateViews() {
 
 	root_ = new AnchorLayout(new LayoutParams(FILL_PARENT, FILL_PARENT));
 
-	LoadCheatInfo();
+	TryLoadCheatInfo();  // in case the info is already in cache.
 	Margins actionMenuMargins(50, -15, 15, 0);
 
 	LinearLayout *leftColumn = new LinearLayout(ORIENT_VERTICAL, new LinearLayoutParams(400, FILL_PARENT));
@@ -97,7 +99,7 @@ void CwCheatScreen::CreateViews() {
 
 	std::string root = GetSysDirectory(DIRECTORY_MEMSTICK_ROOT).ToString();
 
-	std::string title = StringFromFormat(cw->T("Import from %s"), "PSP/Cheats/cheat.db");
+	std::string title = StringFromFormat(cw->T_cstr("Import from %s"), "PSP/Cheats/cheat.db");
 
 	leftColumn->Add(new Choice(title.c_str()))->OnClick.Handle(this, &CwCheatScreen::OnImportCheat);
 	leftColumn->Add(new Choice(mm->T("Browse"), ImageID("I_FOLDER_OPEN")))->OnClick.Handle(this, &CwCheatScreen::OnImportBrowse);
@@ -109,7 +111,8 @@ void CwCheatScreen::CreateViews() {
 	leftColumn->Add(new Choice(cw->T("Edit Cheat File")))->OnClick.Handle(this, &CwCheatScreen::OnEditCheatFile);
 #endif
 	leftColumn->Add(new Choice(di->T("Disable All")))->OnClick.Handle(this, &CwCheatScreen::OnDisableAll);
-	leftColumn->Add(new PopupSliderChoice(&g_Config.iCwCheatRefreshRate, 1, 1000, 77, cw->T("Refresh Rate"), 1, screenManager()));
+	leftColumn->Add(new PopupSliderChoice(&g_Config.iCwCheatRefreshIntervalMs, 1, 1000, 77, cw->T("Refresh interval"), 1, screenManager()))->SetFormat(di->T("%d ms"));
+
 
 	rightScroll_ = new ScrollView(ORIENT_VERTICAL, new LinearLayoutParams(FILL_PARENT, FILL_PARENT, 0.5f));
 	rightScroll_->SetTag("CwCheats");
@@ -133,10 +136,16 @@ void CwCheatScreen::CreateViews() {
 }
 
 void CwCheatScreen::update() {
+	if (gameID_.empty()) {
+		if (TryLoadCheatInfo()) {
+			RecreateViews();
+		}
+	}
+
 	if (fileCheckCounter_++ >= FILE_CHECK_FRAME_INTERVAL && engine_) {
 		// Check if the file has changed.  If it has, we'll reload.
 		std::string str;
-		if (File::ReadFileToString(true, engine_->CheatFilename(), str)) {
+		if (File::ReadTextFileToString(engine_->CheatFilename(), &str)) {
 			uint64_t newHash = XXH3_64bits(str.c_str(), str.size());
 			if (newHash != fileCheckHash_) {
 				// This will update the hash.
@@ -206,9 +215,9 @@ static char *GetLineNoNewline(char *temp, int sz, FILE *fp) {
 }
 
 UI::EventReturn CwCheatScreen::OnImportBrowse(UI::EventParams &params) {
-	System_BrowseForFile("Open cheat DB file", BrowseFileType::DB, [&](const std::string &value, int) {
+	System_BrowseForFile(GetRequesterToken(), "Open cheat DB file", BrowseFileType::DB, [&](const std::string &value, int) {
 		Path path(value);
-		INFO_LOG(SYSTEM, "Attempting to load cheats from: '%s'", path.ToVisualString().c_str());
+		INFO_LOG(Log::System, "Attempting to load cheats from: '%s'", path.ToVisualString().c_str());
 		if (ImportCheats(path)) {
 			g_Config.bReloadCheats = true;
 		} else {
@@ -233,7 +242,7 @@ UI::EventReturn CwCheatScreen::OnImportCheat(UI::EventParams &params) {
 
 bool CwCheatScreen::ImportCheats(const Path & cheatFile) {
 	if (gameID_.length() != 9 || !engine_) {
-		WARN_LOG(COMMON, "CWCHEAT: Incorrect ID(%s) - can't import cheats.", gameID_.c_str());
+		WARN_LOG(Log::Common, "CWCHEAT: Incorrect ID(%s) - can't import cheats.", gameID_.c_str());
 		return false;
 	}
 
@@ -241,7 +250,7 @@ bool CwCheatScreen::ImportCheats(const Path & cheatFile) {
 
 	FILE *in = File::OpenCFile(cheatFile, "rt");
 	if (!in) {
-		WARN_LOG(COMMON, "Unable to open %s\n", cheatFile.c_str());
+		WARN_LOG(Log::Common, "Unable to open %s\n", cheatFile.c_str());
 		return false;
 	}
 
@@ -306,7 +315,7 @@ bool CwCheatScreen::ImportCheats(const Path & cheatFile) {
 		}
 	}
 
-	NOTICE_LOG(COMMON, "Imported %u lines from %s.\n", (int)newList.size(), cheatFile.c_str());
+	NOTICE_LOG(Log::Common, "Imported %u lines from %s.\n", (int)newList.size(), cheatFile.c_str());
 	if (newList.size() != 0) {
 		fputc('\n', append);
 	}
