@@ -35,18 +35,26 @@ static const char *ppsspp_app_id = "423397985041383434";
 #ifdef ENABLE_DISCORD
 // No context argument? What?
 static void handleDiscordError(int errCode, const char *message) {
-	ERROR_LOG(SYSTEM, "Discord error code %d: '%s'", errCode, message);
+	ERROR_LOG(Log::System, "Discord error code %d: '%s'", errCode, message);
 }
 #endif
 
 Discord::~Discord() {
 	if (initialized_) {
-		ERROR_LOG(SYSTEM, "Discord destructor running though g_Discord.Shutdown() has not been called.");
+		ERROR_LOG(Log::System, "Discord destructor running though g_Discord.Shutdown() has not been called.");
 	}
 }
 
+bool Discord::IsAvailable() {
+#ifdef ENABLE_DISCORD
+	return true;
+#else
+	return false;
+#endif
+}
+
 bool Discord::IsEnabled() const {
-	return g_Config.bDiscordPresence;
+	return g_Config.bDiscordRichPresence;
 }
 
 void Discord::Init() {
@@ -57,7 +65,7 @@ void Discord::Init() {
 	DiscordEventHandlers eventHandlers{};
 	eventHandlers.errored = &handleDiscordError;
 	Discord_Initialize(ppsspp_app_id, &eventHandlers, 0, nullptr);
-	INFO_LOG(SYSTEM, "Discord connection initialized");
+	INFO_LOG(Log::System, "Discord connection initialized");
 #endif
 
 	initialized_ = true;
@@ -92,7 +100,7 @@ void Discord::Update() {
 #endif
 }
 
-void Discord::SetPresenceGame(const char *gameTitle) {
+void Discord::SetPresenceGame(std::string_view gameTitle) {
 	if (!IsEnabled())
 		return;
 	
@@ -102,11 +110,10 @@ void Discord::SetPresenceGame(const char *gameTitle) {
 
 #ifdef ENABLE_DISCORD
 	auto sc = GetI18NCategory(I18NCat::SCREEN);
-
+	std::string title(gameTitle);
 	DiscordRichPresence discordPresence{};
-	discordPresence.state = gameTitle;
-	std::string details = sc->T("Playing");
-	discordPresence.details = details.c_str();
+	discordPresence.state = title.c_str();
+	discordPresence.details = sc->T_cstr("Playing");
 	discordPresence.startTimestamp = time(0);
 	discordPresence.largeImageText = "PPSSPP is the best PlayStation Portable emulator around!";
 #ifdef GOLD
@@ -130,7 +137,7 @@ void Discord::SetPresenceMenu() {
 	auto sc = GetI18NCategory(I18NCat::SCREEN);
 
 	DiscordRichPresence discordPresence{};
-	discordPresence.state = sc->T("In menu");
+	discordPresence.state = sc->T_cstr("In menu");
 	discordPresence.details = "";
 	discordPresence.startTimestamp = time(0);
 	discordPresence.largeImageText = "PPSSPP is the best PlayStation Portable emulator around!";
