@@ -27,12 +27,13 @@
 #include "Common/File/VFS/VFS.h"
 #include "Common/File/FileUtil.h"
 #include "Common/File/Path.h"
+#include "Common/Render/ManagedTexture.h"
 #include "Common/StringUtils.h"
 #include "Common/TimeUtil.h"
-#include "Common/Render/ManagedTexture.h"
 #include "Core/FileSystems/ISOFileSystem.h"
 #include "Core/FileSystems/DirectoryFileSystem.h"
 #include "Core/FileSystems/VirtualDiscFileSystem.h"
+#include "Core/HLE/sceUtility.h"
 #include "Core/ELF/PBPReader.h"
 #include "Core/SaveState.h"
 #include "Core/System.h"
@@ -43,8 +44,21 @@
 
 GameInfoCache *g_gameInfoCache;
 
-GameInfo::GameInfo() : fileType(IdentifiedFileType::UNKNOWN) {
-	pending = true;
+void GameInfoTex::Clear() {
+	if (!data.empty()) {
+		data.clear();
+		dataLoaded = false;
+	}
+	if (texture) {
+		texture->Release();
+		texture = nullptr;
+	}
+	timeLoaded = 0.0;
+}
+
+GameInfo::GameInfo(const Path &gamePath) : filePath_(gamePath) {
+	// here due to a forward decl.
+	fileType = IdentifiedFileType::UNKNOWN;
 }
 
 GameInfo::~GameInfo() {
@@ -63,6 +77,7 @@ bool GameInfo::Delete() {
 		{
 			// Just delete the one file (TODO: handle two-disk games as well somehow).
 			Path fileToRemove = filePath_;
+			INFO_LOG(Log::System, "Deleting file %s", fileToRemove.c_str());
 			File::Delete(fileToRemove);
 			g_Config.RemoveRecent(filePath_.ToString());
 			return true;
@@ -72,9 +87,9 @@ bool GameInfo::Delete() {
 		{
 			// TODO: This could be handled by Core/Util/GameManager too somehow.
 			Path directoryToRemove = ResolvePBPDirectory(filePath_);
-			INFO_LOG(SYSTEM, "Deleting %s", directoryToRemove.c_str());
+			INFO_LOG(Log::System, "Deleting directory %s", directoryToRemove.c_str());
 			if (!File::DeleteDirRecursively(directoryToRemove)) {
-				ERROR_LOG(SYSTEM, "Failed to delete file");
+				ERROR_LOG(Log::System, "Failed to delete file");
 				return false;
 			}
 			g_Config.CleanRecent();
@@ -90,6 +105,7 @@ bool GameInfo::Delete() {
 	case IdentifiedFileType::PPSSPP_GE_DUMP:
 		{
 			const Path &fileToRemove = filePath_;
+			INFO_LOG(Log::System, "Deleting file %s", fileToRemove.c_str());
 			File::Delete(fileToRemove);
 			g_Config.RemoveRecent(filePath_.ToString());
 			return true;
@@ -98,6 +114,7 @@ bool GameInfo::Delete() {
 	case IdentifiedFileType::PPSSPP_SAVESTATE:
 		{
 			const Path &ppstPath = filePath_;
+			INFO_LOG(Log::System, "Deleting file %s", ppstPath.c_str());
 			File::Delete(ppstPath);
 			const Path screenshotPath = filePath_.WithReplacedExtension(".ppst", ".jpg");
 			if (File::Exists(screenshotPath)) {
@@ -107,23 +124,81 @@ bool GameInfo::Delete() {
 		}
 
 	default:
+		INFO_LOG(Log::System, "Don't know how to delete this type of file: %s", filePath_.c_str());
 		return false;
 	}
 }
 
-u64 GameInfo::GetGameSizeInBytes() {
+u64 GameInfo::GetSizeOnDiskInBytes() {
 	switch (fileType) {
 	case IdentifiedFileType::PSP_PBP_DIRECTORY:
 	case IdentifiedFileType::PSP_SAVEDATA_DIRECTORY:
 		return File::ComputeRecursiveDirectorySize(ResolvePBPDirectory(filePath_));
-
+	case IdentifiedFileType::PSP_DISC_DIRECTORY:
+		return File::ComputeRecursiveDirectorySize(GetFileLoader()->GetPath());
 	default:
 		return GetFileLoader()->FileSize();
 	}
 }
 
+u64 GameInfo::GetSizeUncompressedInBytes() {
+	switch (fileType) {
+	case IdentifiedFileType::PSP_PBP_DIRECTORY:
+	case IdentifiedFileType::PSP_SAVEDATA_DIRECTORY:
+		return File::ComputeRecursiveDirectorySize(ResolvePBPDirectory(filePath_));
+	case IdentifiedFileType::PSP_DISC_DIRECTORY:
+		return File::ComputeRecursiveDirectorySize(GetFileLoader()->GetPath());
+	default:
+	{
+		BlockDevice *blockDevice = constructBlockDevice(GetFileLoader().get());
+		if (blockDevice) {
+			u64 size = blockDevice->GetUncompressedSize();
+			delete blockDevice;
+			return size;
+		} else {
+			return GetFileLoader()->FileSize();
+		}
+	}
+	}
+}
+
+std::string GetFileDateAsString(const Path &filename) {
+	tm time;
+	if (File::GetModifTime(filename, time)) {
+		char buf[256];
+		switch (g_Config.iDateFormat) {
+		case PSP_SYSTEMPARAM_DATE_FORMAT_YYYYMMDD:
+			strftime(buf, sizeof(buf), "%Y-%m-%d %H:%M:%S", &time);
+			break;
+		case PSP_SYSTEMPARAM_DATE_FORMAT_MMDDYYYY:
+			strftime(buf, sizeof(buf), "%m-%d-%Y %H:%M:%S", &time);
+			break;
+		case PSP_SYSTEMPARAM_DATE_FORMAT_DDMMYYYY:
+			strftime(buf, sizeof(buf), "%d-%m-%Y %H:%M:%S", &time);
+			break;
+		default: // Should never happen
+			return "";
+		}
+		return std::string(buf);
+	}
+	return "";
+}
+
+std::string GameInfo::GetMTime() const {
+	switch (fileType) {
+	case IdentifiedFileType::PSP_SAVEDATA_DIRECTORY:
+		return GetFileDateAsString(GetFilePath() / "PARAM.SFO");
+	case IdentifiedFileType::PSP_PBP_DIRECTORY:
+		return GetFileDateAsString(GetFilePath() / "EBOOT.PBP");
+	default:
+		return GetFileDateAsString(GetFilePath());
+	}
+}
+
 // Not too meaningful if the object itself is a savedata directory...
+// Call this under lock.
 std::vector<Path> GameInfo::GetSaveDataDirectories() {
+	_dbg_assert_(hasFlags & GameInfoFlags::PARAM_SFO);  // so we know we have the ID.
 	Path memc = GetSysDirectory(DIRECTORY_SAVEDATA);
 
 	std::vector<File::FileInfo> dirs;
@@ -142,7 +217,7 @@ std::vector<Path> GameInfo::GetSaveDataDirectories() {
 	return directories;
 }
 
-u64 GameInfo::GetSaveDataSizeInBytes() {
+u64 GameInfo::GetGameSavedataSizeInBytes() {
 	if (fileType == IdentifiedFileType::PSP_SAVEDATA_DIRECTORY || fileType == IdentifiedFileType::PPSSPP_SAVESTATE) {
 		return 0;
 	}
@@ -193,22 +268,13 @@ u64 GameInfo::GetInstallDataSizeInBytes() {
 	return totalSize;
 }
 
-bool GameInfo::LoadFromPath(const Path &gamePath) {
-	std::lock_guard<std::mutex> guard(lock);
-	// No need to rebuild if we already have it loaded.
-	if (filePath_ != gamePath) {
-		{
-			std::lock_guard<std::mutex> guard(loaderLock);
-			fileLoader.reset(ConstructFileLoader(gamePath));
-			if (!fileLoader)
-				return false;
-		}
-		filePath_ = gamePath;
-
-		// This is a fallback title, while we're loading / if unable to load.
-		title = filePath_.GetFilename();
+bool GameInfo::CreateLoader() {
+	if (!fileLoader) {
+		std::lock_guard<std::mutex> guard(loaderLock);
+		fileLoader.reset(ConstructFileLoader(filePath_));
+		if (!fileLoader)
+			return false;
 	}
-
 	return true;
 }
 
@@ -283,13 +349,15 @@ void GameInfo::ParseParamSFO() {
 			region = GAMEREGION_CHINA;
 		}*/
 	}
-
-	paramSFOLoaded = true;
 }
 
 std::string GameInfo::GetTitle() {
 	std::lock_guard<std::mutex> guard(lock);
-	return title;
+	if ((hasFlags & GameInfoFlags::PARAM_SFO) && !title.empty()) {
+		return title;
+	} else {
+		return filePath_.GetFilename();
+	}
 }
 
 void GameInfo::SetTitle(const std::string &newTitle) {
@@ -297,6 +365,37 @@ void GameInfo::SetTitle(const std::string &newTitle) {
 	title = newTitle;
 }
 
+void GameInfo::FinishPendingTextureLoads(Draw::DrawContext *draw) {
+	if (draw && icon.dataLoaded && !icon.texture) {
+		SetupTexture(draw, icon);
+	}
+	if (draw && pic0.dataLoaded && !pic0.texture) {
+		SetupTexture(draw, pic0);
+	}
+	if (draw && pic1.dataLoaded && !pic1.texture) {
+		SetupTexture(draw, pic1);
+	}
+}
+
+void GameInfo::SetupTexture(Draw::DrawContext *thin3d, GameInfoTex &tex) {
+	if (tex.timeLoaded) {
+		// Failed before, skip.
+		return;
+	}
+	if (tex.data.empty()) {
+		tex.timeLoaded = time_now_d();
+		return;
+	}
+	using namespace Draw;
+	// TODO: Use TempImage to semi-load the image in the worker task, then here we
+	// could just call CreateTextureFromTempImage.
+	tex.texture = CreateTextureFromFileData(thin3d, (const uint8_t *)tex.data.data(), tex.data.size(), ImageFileType::DETECT, false, GetTitle().c_str());
+	tex.timeLoaded = time_now_d();
+	if (!tex.texture) {
+		ERROR_LOG(Log::G3D, "Failed creating texture (%s) from %d-byte file", GetTitle().c_str(), (int)tex.data.size());
+	}
+}
+
 static bool ReadFileToString(IFileSystem *fs, const char *filename, std::string *contents, std::mutex *mtx) {
 	PSPFileInfo info = fs->GetFileInfo(filename);
 	if (!info.exists) {
@@ -307,16 +406,38 @@ static bool ReadFileToString(IFileSystem *fs, const char *filename, std::string
 	if (handle < 0) {
 		return false;
 	}
-
 	if (mtx) {
+		std::string data;
+		data.resize(info.size);
+		size_t readSize = fs->ReadFile(handle, (u8 *)data.data(), info.size);
+		fs->CloseFile(handle);
+		if (readSize != info.size) {
+			return false;
+		}
 		std::lock_guard<std::mutex> lock(*mtx);
-		contents->resize(info.size);
-		fs->ReadFile(handle, (u8 *)contents->data(), info.size);
+		*contents = std::move(data);
 	} else {
 		contents->resize(info.size);
-		fs->ReadFile(handle, (u8 *)contents->data(), info.size);
+		size_t readSize = fs->ReadFile(handle, (u8 *)contents->data(), info.size);
+		fs->CloseFile(handle);
+		if (readSize != info.size) {
+			return false;
+		}
+	}
+	return true;
+}
+
+static bool ReadLocalFileToString(const Path &path, std::string *contents, std::mutex *mtx) {
+	std::string data;
+	if (!File::ReadBinaryFileToString(path, &data)) {
+		return false;
+	}
+	if (mtx) {
+		std::lock_guard<std::mutex> lock(*mtx);
+		*contents = std::move(data);
+	} else {
+		*contents = std::move(data);
 	}
-	fs->CloseFile(handle);
 	return true;
 }
 
@@ -330,23 +451,20 @@ static bool ReadVFSToString(const char *filename, std::string *contents, std::mu
 		} else {
 			*contents = std::string((const char *)data, sz);
 		}
+	} else {
+		return false;
 	}
 	delete [] data;
-	return data != nullptr;
+	return true;
 }
 
-
 class GameInfoWorkItem : public Task {
 public:
-	GameInfoWorkItem(const Path &gamePath, std::shared_ptr<GameInfo> &info)
-		: gamePath_(gamePath), info_(info) {
-	}
+	GameInfoWorkItem(const Path &gamePath, std::shared_ptr<GameInfo> &info, GameInfoFlags flags)
+		: gamePath_(gamePath), info_(info), flags_(flags) {}
 
 	~GameInfoWorkItem() {
-		info_->pending.store(false);
-		info_->working.store(false);
 		info_->DisposeFileLoader();
-		info_->readyEvent.Notify();
 	}
 
 	TaskType Type() const override {
@@ -368,19 +486,24 @@ public:
 	void Run() override {
 		// An early-return will result in the destructor running, where we can set
 		// flags like working and pending.
-		if (!info_->LoadFromPath(gamePath_)) {
-			return;
-		}
-
-		// In case of a remote file, check if it actually exists before locking.
-		if (!info_->GetFileLoader() || !info_->GetFileLoader()->Exists()) {
+		if (!info_->CreateLoader() || !info_->GetFileLoader() || !info_->GetFileLoader()->Exists()) {
+			// Mark everything requested as done, so 
+			std::unique_lock<std::mutex> lock(info_->lock);
+			info_->MarkReadyNoLock(flags_);
+			ERROR_LOG(Log::Loader, "Failed getting game info for %s", info_->GetFilePath().ToVisualString().c_str());
 			return;
 		}
 
 		std::string errorString;
 
-		info_->working = true;
-		info_->fileType = Identify_File(info_->GetFileLoader().get(), &errorString);
+		if (flags_ & GameInfoFlags::FILE_TYPE) {
+			info_->fileType = Identify_File(info_->GetFileLoader().get(), &errorString);
+		}
+
+		if (!info_->Ready(GameInfoFlags::FILE_TYPE) && !(flags_ & GameInfoFlags::FILE_TYPE)) {
+			_dbg_assert_(false);
+		}
+
 		switch (info_->fileType) {
 		case IdentifiedFileType::PSP_PBP:
 		case IdentifiedFileType::PSP_PBP_DIRECTORY:
@@ -398,61 +521,76 @@ public:
 					if (pbp.IsELF()) {
 						goto handleELF;
 					}
-					ERROR_LOG(LOADER, "invalid pbp '%s'\n", pbpLoader->GetPath().c_str());
+					ERROR_LOG(Log::Loader, "invalid pbp '%s'\n", pbpLoader->GetPath().c_str());
+					// We can't win here - just mark everything pending as fetched, and let the caller
+					// handle the missing data.
+					std::unique_lock<std::mutex> lock(info_->lock);
+					info_->MarkReadyNoLock(flags_);
 					return;
 				}
 
 				// First, PARAM.SFO.
-				std::vector<u8> sfoData;
-				if (pbp.GetSubFile(PBP_PARAM_SFO, &sfoData)) {
-					std::lock_guard<std::mutex> lock(info_->lock);
-					info_->paramSFO.ReadSFO(sfoData);
-					info_->ParseParamSFO();
+				if (flags_ & GameInfoFlags::PARAM_SFO) {
+					std::vector<u8> sfoData;
+					if (pbp.GetSubFile(PBP_PARAM_SFO, &sfoData)) {
+						std::lock_guard<std::mutex> lock(info_->lock);
+						info_->paramSFO.ReadSFO(sfoData);
+						info_->ParseParamSFO();
 
-					// Assuming PSP_PBP_DIRECTORY without ID or with disc_total < 1 in GAME dir must be homebrew
-					if ((info_->id.empty() || !info_->disc_total)
-						&& gamePath_.FilePathContainsNoCase("PSP/GAME/")
-						&& info_->fileType == IdentifiedFileType::PSP_PBP_DIRECTORY) {
-						info_->id = g_paramSFO.GenerateFakeID(gamePath_.ToString());
-						info_->id_version = info_->id + "_1.00";
-						info_->region = GAMEREGION_MAX + 1; // Homebrew
+						// Assuming PSP_PBP_DIRECTORY without ID or with disc_total < 1 in GAME dir must be homebrew
+						if ((info_->id.empty() || !info_->disc_total)
+							&& gamePath_.FilePathContainsNoCase("PSP/GAME/")
+							&& info_->fileType == IdentifiedFileType::PSP_PBP_DIRECTORY) {
+							info_->id = g_paramSFO.GenerateFakeID(gamePath_);
+							info_->id_version = info_->id + "_1.00";
+							info_->region = GAMEREGION_MAX + 1; // Homebrew
+						}
+						info_->MarkReadyNoLock(GameInfoFlags::PARAM_SFO);
 					}
 				}
 
 				// Then, ICON0.PNG.
-				if (pbp.GetSubFileSize(PBP_ICON0_PNG) > 0) {
-					std::lock_guard<std::mutex> lock(info_->lock);
-					pbp.GetSubFileAsString(PBP_ICON0_PNG, &info_->icon.data);
-				} else {
-					Path screenshot_jpg = GetSysDirectory(DIRECTORY_SCREENSHOT) / (info_->id + "_00000.jpg");
-					Path screenshot_png = GetSysDirectory(DIRECTORY_SCREENSHOT) / (info_->id + "_00000.png");
-					// Try using png/jpg screenshots first
-					if (File::Exists(screenshot_png))
-						File::ReadFileToString(false, screenshot_png, info_->icon.data);
-					else if (File::Exists(screenshot_jpg))
-						File::ReadFileToString(false, screenshot_jpg, info_->icon.data);
-					else
-						// Read standard icon
-						ReadVFSToString("unknown.png", &info_->icon.data, &info_->lock);
-				}
-				info_->icon.dataLoaded = true;
-
-				if (info_->wantFlags & GAMEINFO_WANTBG) {
-					if (pbp.GetSubFileSize(PBP_PIC0_PNG) > 0) {
+				if (flags_ & GameInfoFlags::ICON) {
+					if (pbp.GetSubFileSize(PBP_ICON0_PNG) > 0) {
 						std::lock_guard<std::mutex> lock(info_->lock);
-						pbp.GetSubFileAsString(PBP_PIC0_PNG, &info_->pic0.data);
+						pbp.GetSubFileAsString(PBP_ICON0_PNG, &info_->icon.data);
+					} else {
+						Path screenshot_jpg = GetSysDirectory(DIRECTORY_SCREENSHOT) / (info_->id + "_00000.jpg");
+						Path screenshot_png = GetSysDirectory(DIRECTORY_SCREENSHOT) / (info_->id + "_00000.png");
+						// Try using png/jpg screenshots first
+						if (File::Exists(screenshot_png))
+							ReadLocalFileToString(screenshot_png, &info_->icon.data, &info_->lock);
+						else if (File::Exists(screenshot_jpg))
+							ReadLocalFileToString(screenshot_jpg, &info_->icon.data, &info_->lock);
+						else
+							// Read standard icon
+							ReadVFSToString("unknown.png", &info_->icon.data, &info_->lock);
+					}
+					info_->icon.dataLoaded = true;
+				}
+
+				if (flags_ & GameInfoFlags::BG) {
+					if (pbp.GetSubFileSize(PBP_PIC0_PNG) > 0) {
+						std::string data;
+						pbp.GetSubFileAsString(PBP_PIC0_PNG, &data);
+						std::lock_guard<std::mutex> lock(info_->lock);
+						info_->pic0.data = std::move(data);
 						info_->pic0.dataLoaded = true;
 					}
 					if (pbp.GetSubFileSize(PBP_PIC1_PNG) > 0) {
+						std::string data;
+						pbp.GetSubFileAsString(PBP_PIC1_PNG, &data);
 						std::lock_guard<std::mutex> lock(info_->lock);
-						pbp.GetSubFileAsString(PBP_PIC1_PNG, &info_->pic1.data);
+						info_->pic1.data = std::move(data);
 						info_->pic1.dataLoaded = true;
 					}
 				}
-				if (info_->wantFlags & GAMEINFO_WANTSND) {
+				if (flags_ & GameInfoFlags::SND) {
 					if (pbp.GetSubFileSize(PBP_SND0_AT3) > 0) {
+						std::string data;
+						pbp.GetSubFileAsString(PBP_SND0_AT3, &data);
 						std::lock_guard<std::mutex> lock(info_->lock);
-						pbp.GetSubFileAsString(PBP_SND0_AT3, &info_->sndFileData);
+						info_->sndFileData = std::move(data);
 						info_->sndDataLoaded = true;
 					}
 				}
@@ -462,25 +600,25 @@ public:
 		case IdentifiedFileType::PSP_ELF:
 handleELF:
 			// An elf on its own has no usable information, no icons, no nothing.
-			{
-				std::lock_guard<std::mutex> lock(info_->lock);
-				info_->id = g_paramSFO.GenerateFakeID(gamePath_.ToString());
+			if (flags_ & GameInfoFlags::PARAM_SFO) {
+				info_->id = g_paramSFO.GenerateFakeID(gamePath_);
 				info_->id_version = info_->id + "_1.00";
 				info_->region = GAMEREGION_MAX + 1; // Homebrew
-
-				info_->paramSFOLoaded = true;
 			}
-			{
-				Path screenshot_jpg = GetSysDirectory(DIRECTORY_SCREENSHOT) / (info_->id + "_00000.jpg");
-				Path screenshot_png = GetSysDirectory(DIRECTORY_SCREENSHOT) / (info_->id + "_00000.png");
+
+			if (flags_ & GameInfoFlags::ICON) {
+				std::string id = g_paramSFO.GenerateFakeID(gamePath_);
+				// Due to the dependency of the BASIC info, we fetch it already here.
+				Path screenshot_jpg = GetSysDirectory(DIRECTORY_SCREENSHOT) / (id + "_00000.jpg");
+				Path screenshot_png = GetSysDirectory(DIRECTORY_SCREENSHOT) / (id + "_00000.png");
 				// Try using png/jpg screenshots first
 				if (File::Exists(screenshot_png)) {
-					File::ReadFileToString(false, screenshot_png, info_->icon.data);
+					ReadLocalFileToString(screenshot_png, &info_->icon.data, &info_->lock);
 				} else if (File::Exists(screenshot_jpg)) {
-					File::ReadFileToString(false, screenshot_jpg, info_->icon.data);
+					ReadLocalFileToString(screenshot_jpg, &info_->icon.data, &info_->lock);
 				} else {
 					// Read standard icon
-					VERBOSE_LOG(LOADER, "Loading unknown.png because there was an ELF");
+					VERBOSE_LOG(Log::Loader, "Loading unknown.png because there was an ELF");
 					ReadVFSToString("unknown.png", &info_->icon.data, &info_->lock);
 				}
 				info_->icon.dataLoaded = true;
@@ -492,17 +630,21 @@ handleELF:
 			SequentialHandleAllocator handles;
 			VirtualDiscFileSystem umd(&handles, gamePath_);
 
-			// Alright, let's fetch the PARAM.SFO.
-			std::string paramSFOcontents;
-			if (ReadFileToString(&umd, "/PARAM.SFO", &paramSFOcontents, 0)) {
-				std::lock_guard<std::mutex> lock(info_->lock);
-				info_->paramSFO.ReadSFO((const u8 *)paramSFOcontents.data(), paramSFOcontents.size());
-				info_->ParseParamSFO();
+			if (flags_ & GameInfoFlags::PARAM_SFO) {
+				// Alright, let's fetch the PARAM.SFO.
+				std::string paramSFOcontents;
+				if (ReadFileToString(&umd, "/PARAM.SFO", &paramSFOcontents, 0)) {
+					std::lock_guard<std::mutex> lock(info_->lock);
+					info_->paramSFO.ReadSFO((const u8 *)paramSFOcontents.data(), paramSFOcontents.size());
+					info_->ParseParamSFO();
+					info_->MarkReadyNoLock(GameInfoFlags::PARAM_SFO);
+				}
 			}
-
-			ReadFileToString(&umd, "/ICON0.PNG", &info_->icon.data, &info_->lock);
-			info_->icon.dataLoaded = true;
-			if (info_->wantFlags & GAMEINFO_WANTBG) {
+			if (flags_ & GameInfoFlags::ICON) {
+				ReadFileToString(&umd, "/ICON0.PNG", &info_->icon.data, &info_->lock);
+				info_->icon.dataLoaded = true;
+			}
+			if (flags_ & GameInfoFlags::BG) {
 				ReadFileToString(&umd, "/PIC1.PNG", &info_->pic1.data, &info_->lock);
 				info_->pic1.dataLoaded = true;
 			}
@@ -511,17 +653,19 @@ handleELF:
 
 		case IdentifiedFileType::PPSSPP_SAVESTATE:
 		{
-			info_->SetTitle(SaveState::GetTitle(gamePath_));
-
-			std::lock_guard<std::mutex> guard(info_->lock);
+			if (flags_ & GameInfoFlags::PARAM_SFO) {
+				info_->SetTitle(SaveState::GetTitle(gamePath_));
+				std::lock_guard<std::mutex> lock(info_->lock);
+				info_->MarkReadyNoLock(GameInfoFlags::PARAM_SFO);
+			}
 
 			// Let's use the screenshot as an icon, too.
-			Path screenshotPath = gamePath_.WithReplacedExtension(".ppst", ".jpg");
-			if (File::Exists(screenshotPath)) {
-				if (File::ReadFileToString(false, screenshotPath, info_->icon.data)) {
+			if (flags_ & GameInfoFlags::ICON) {
+				Path screenshotPath = gamePath_.WithReplacedExtension(".ppst", ".jpg");
+				if (ReadLocalFileToString(screenshotPath, &info_->icon.data, &info_->lock)) {
 					info_->icon.dataLoaded = true;
 				} else {
-					ERROR_LOG(G3D, "Error loading screenshot data: '%s'", screenshotPath.c_str());
+					ERROR_LOG(Log::G3D, "Error loading screenshot data: '%s'", screenshotPath.c_str());
 				}
 			}
 			break;
@@ -529,15 +673,11 @@ handleELF:
 
 		case IdentifiedFileType::PPSSPP_GE_DUMP:
 		{
-			std::lock_guard<std::mutex> guard(info_->lock);
-
-			// Let's use the comparison screenshot as an icon, if it exists.
-			Path screenshotPath = gamePath_.WithReplacedExtension(".ppdmp", ".png");
-			if (File::Exists(screenshotPath)) {
-				if (File::ReadFileToString(false, screenshotPath, info_->icon.data)) {
+			if (flags_ & GameInfoFlags::ICON) {
+				Path screenshotPath = gamePath_.WithReplacedExtension(".ppdmp", ".png");
+				// Let's use the comparison screenshot as an icon, if it exists.
+				if (ReadLocalFileToString(screenshotPath, &info_->icon.data, &info_->lock)) {
 					info_->icon.dataLoaded = true;
-				} else {
-					ERROR_LOG(G3D, "Error loading screenshot data: '%s'", screenshotPath.c_str());
 				}
 			}
 			break;
@@ -545,27 +685,30 @@ handleELF:
 
 		case IdentifiedFileType::PSP_DISC_DIRECTORY:
 			{
-				info_->fileType = IdentifiedFileType::PSP_ISO;
 				SequentialHandleAllocator handles;
 				VirtualDiscFileSystem umd(&handles, gamePath_);
 
 				// Alright, let's fetch the PARAM.SFO.
-				std::string paramSFOcontents;
-				if (ReadFileToString(&umd, "/PSP_GAME/PARAM.SFO", &paramSFOcontents, 0)) {
-					std::lock_guard<std::mutex> lock(info_->lock);
-					info_->paramSFO.ReadSFO((const u8 *)paramSFOcontents.data(), paramSFOcontents.size());
-					info_->ParseParamSFO();
+				if (flags_ & GameInfoFlags::PARAM_SFO) {
+					std::string paramSFOcontents;
+					if (ReadFileToString(&umd, "/PSP_GAME/PARAM.SFO", &paramSFOcontents, nullptr)) {
+						std::lock_guard<std::mutex> lock(info_->lock);
+						info_->paramSFO.ReadSFO((const u8 *)paramSFOcontents.data(), paramSFOcontents.size());
+						info_->ParseParamSFO();
+					}
 				}
 
-				ReadFileToString(&umd, "/PSP_GAME/ICON0.PNG", &info_->icon.data, &info_->lock);
-				info_->icon.dataLoaded = true;
-				if (info_->wantFlags & GAMEINFO_WANTBG) {
+				if (flags_ & GameInfoFlags::ICON) {
+					ReadFileToString(&umd, "/PSP_GAME/ICON0.PNG", &info_->icon.data, &info_->lock);
+					info_->icon.dataLoaded = true;
+				}
+				if (flags_ & GameInfoFlags::BG) {
 					ReadFileToString(&umd, "/PSP_GAME/PIC0.PNG", &info_->pic0.data, &info_->lock);
 					info_->pic0.dataLoaded = true;
 					ReadFileToString(&umd, "/PSP_GAME/PIC1.PNG", &info_->pic1.data, &info_->lock);
 					info_->pic1.dataLoaded = true;
 				}
-				if (info_->wantFlags & GAMEINFO_WANTSND) {
+				if (flags_ & GameInfoFlags::SND) {
 					ReadFileToString(&umd, "/PSP_GAME/SND0.AT3", &info_->sndFileData, &info_->lock);
 					info_->pic1.dataLoaded = true;
 				}
@@ -575,75 +718,88 @@ handleELF:
 		case IdentifiedFileType::PSP_ISO:
 		case IdentifiedFileType::PSP_ISO_NP:
 			{
-				info_->fileType = IdentifiedFileType::PSP_ISO;
 				SequentialHandleAllocator handles;
 				// Let's assume it's an ISO.
 				// TODO: This will currently read in the whole directory tree. Not really necessary for just a
 				// few files.
 				auto fl = info_->GetFileLoader();
 				if (!fl) {
+					// BAD! Can't win here.
+					ERROR_LOG(Log::Loader, "Failed getting game info for ISO %s", info_->GetFilePath().ToVisualString().c_str());
+					std::unique_lock<std::mutex> lock(info_->lock);
+					info_->MarkReadyNoLock(flags_);
 					return;
 				}
 				BlockDevice *bd = constructBlockDevice(info_->GetFileLoader().get());
 				if (!bd) {
+					ERROR_LOG(Log::Loader, "Failed constructing block device for ISO %s", info_->GetFilePath().ToVisualString().c_str());
+					std::unique_lock<std::mutex> lock(info_->lock);
+					info_->MarkReadyNoLock(flags_);
 					return;
 				}
 				ISOFileSystem umd(&handles, bd);
 
 				// Alright, let's fetch the PARAM.SFO.
-				std::string paramSFOcontents;
-				if (ReadFileToString(&umd, "/PSP_GAME/PARAM.SFO", &paramSFOcontents, nullptr)) {
-					std::lock_guard<std::mutex> lock(info_->lock);
-					info_->paramSFO.ReadSFO((const u8 *)paramSFOcontents.data(), paramSFOcontents.size());
-					info_->ParseParamSFO();
+				if (flags_ & GameInfoFlags::PARAM_SFO) {
+					std::string paramSFOcontents;
+					if (ReadFileToString(&umd, "/PSP_GAME/PARAM.SFO", &paramSFOcontents, nullptr)) {
+						{
+							std::lock_guard<std::mutex> lock(info_->lock);
+							info_->paramSFO.ReadSFO((const u8 *)paramSFOcontents.data(), paramSFOcontents.size());
+							info_->ParseParamSFO();
 
-					if (info_->wantFlags & GAMEINFO_WANTBG) {
-						info_->pic0.dataLoaded = ReadFileToString(&umd, "/PSP_GAME/PIC0.PNG", &info_->pic0.data, nullptr);
-						info_->pic1.dataLoaded = ReadFileToString(&umd, "/PSP_GAME/PIC1.PNG", &info_->pic1.data, nullptr);
-					}
-					if (info_->wantFlags & GAMEINFO_WANTSND) {
-						info_->sndDataLoaded = ReadFileToString(&umd, "/PSP_GAME/SND0.AT3", &info_->sndFileData, nullptr);
+							// quick-update the info while we have the lock, so we don't need to wait for the image load to display the title.
+							info_->MarkReadyNoLock(GameInfoFlags::PARAM_SFO);
+						}
 					}
 				}
 
+				if (flags_ & GameInfoFlags::BG) {
+					info_->pic0.dataLoaded = ReadFileToString(&umd, "/PSP_GAME/PIC0.PNG", &info_->pic0.data, &info_->lock);
+					info_->pic1.dataLoaded = ReadFileToString(&umd, "/PSP_GAME/PIC1.PNG", &info_->pic1.data, &info_->lock);
+				}
+
+				if (flags_ & GameInfoFlags::SND) {
+					info_->sndDataLoaded = ReadFileToString(&umd, "/PSP_GAME/SND0.AT3", &info_->sndFileData, &info_->lock);
+				}
+
 				// Fall back to unknown icon if ISO is broken/is a homebrew ISO, override is allowed though
-				if (!ReadFileToString(&umd, "/PSP_GAME/ICON0.PNG", &info_->icon.data, &info_->lock)) {
-					Path screenshot_jpg = GetSysDirectory(DIRECTORY_SCREENSHOT) / (info_->id + "_00000.jpg");
-					Path screenshot_png = GetSysDirectory(DIRECTORY_SCREENSHOT) / (info_->id + "_00000.png");
-					// Try using png/jpg screenshots first
-					if (File::Exists(screenshot_png))
-						info_->icon.dataLoaded = File::ReadFileToString(false, screenshot_png, info_->icon.data);
-					else if (File::Exists(screenshot_jpg))
-						info_->icon.dataLoaded = File::ReadFileToString(false, screenshot_jpg, info_->icon.data);
-					else {
-						DEBUG_LOG(LOADER, "Loading unknown.png because no icon was found");
-						info_->icon.dataLoaded = ReadVFSToString("unknown.png", &info_->icon.data, &info_->lock);
+				if (flags_ & GameInfoFlags::ICON) {
+					if (!ReadFileToString(&umd, "/PSP_GAME/ICON0.PNG", &info_->icon.data, &info_->lock)) {
+						Path screenshot_jpg = GetSysDirectory(DIRECTORY_SCREENSHOT) / (info_->id + "_00000.jpg");
+						Path screenshot_png = GetSysDirectory(DIRECTORY_SCREENSHOT) / (info_->id + "_00000.png");
+						// Try using png/jpg screenshots first
+						if (File::Exists(screenshot_png))
+							info_->icon.dataLoaded = ReadLocalFileToString(screenshot_png, &info_->icon.data, &info_->lock);
+						else if (File::Exists(screenshot_jpg))
+							info_->icon.dataLoaded = ReadLocalFileToString(screenshot_jpg, &info_->icon.data, &info_->lock);
+						else {
+							DEBUG_LOG(Log::Loader, "Loading unknown.png because no icon was found");
+							info_->icon.dataLoaded = ReadVFSToString("unknown.png", &info_->icon.data, &info_->lock);
+						}
+					} else {
+						info_->icon.dataLoaded = true;
 					}
-				} else {
-					info_->icon.dataLoaded = true;
 				}
 				break;
 			}
 
 			case IdentifiedFileType::ARCHIVE_ZIP:
-				info_->paramSFOLoaded = true;
-				{
+				if (flags_ & GameInfoFlags::ICON) {
 					ReadVFSToString("zip.png", &info_->icon.data, &info_->lock);
 					info_->icon.dataLoaded = true;
 				}
 				break;
 
 			case IdentifiedFileType::ARCHIVE_RAR:
-				info_->paramSFOLoaded = true;
-				{
+				if (flags_ & GameInfoFlags::ICON) {
 					ReadVFSToString("rargray.png", &info_->icon.data, &info_->lock);
 					info_->icon.dataLoaded = true;
 				}
 				break;
 
 			case IdentifiedFileType::ARCHIVE_7Z:
-				info_->paramSFOLoaded = true;
-				{
+				if (flags_ & GameInfoFlags::ICON) {
 					ReadVFSToString("7z.png", &info_->icon.data, &info_->lock);
 					info_->icon.dataLoaded = true;
 				}
@@ -651,25 +807,47 @@ handleELF:
 
 			case IdentifiedFileType::NORMAL_DIRECTORY:
 			default:
-				info_->paramSFOLoaded = true;
 				break;
 		}
 
-		info_->hasConfig = g_Config.hasGameConfig(info_->id);
-
-		if (info_->wantFlags & GAMEINFO_WANTSIZE) {
-			std::lock_guard<std::mutex> lock(info_->lock);
-			info_->gameSize = info_->GetGameSizeInBytes();
-			info_->saveDataSize = info_->GetSaveDataSizeInBytes();
-			info_->installDataSize = info_->GetInstallDataSizeInBytes();
+		if (flags_ & GameInfoFlags::PARAM_SFO) {
+			// We fetch the hasConfig together with the params, since that's what fills out the id.
+			info_->hasConfig = g_Config.hasGameConfig(info_->id);
 		}
 
-		// INFO_LOG(SYSTEM, "Completed writing info for %s", info_->GetTitle().c_str());
+		if (flags_ & GameInfoFlags::SIZE) {
+			std::lock_guard<std::mutex> lock(info_->lock);
+			info_->gameSizeOnDisk = info_->GetSizeOnDiskInBytes();
+			switch (info_->fileType) {
+			case IdentifiedFileType::PSP_ISO:
+			case IdentifiedFileType::PSP_ISO_NP:
+			case IdentifiedFileType::PSP_DISC_DIRECTORY:
+			case IdentifiedFileType::PSP_PBP:
+			case IdentifiedFileType::PSP_PBP_DIRECTORY:
+				info_->saveDataSize = info_->GetGameSavedataSizeInBytes();
+				info_->installDataSize = info_->GetInstallDataSizeInBytes();
+				break;
+			default:
+				info_->saveDataSize = 0;
+				info_->installDataSize = 0;
+				break;
+			}
+		}
+		if (flags_ & GameInfoFlags::UNCOMPRESSED_SIZE) {
+			info_->gameSizeUncompressed = info_->GetSizeUncompressedInBytes();
+		}
+
+		// Time to update the flags.
+		std::unique_lock<std::mutex> lock(info_->lock);
+		info_->MarkReadyNoLock(flags_);
+		// INFO_LOG(Log::System, "Completed writing info for %s", info_->GetTitle().c_str());
 	}
 
 private:
 	Path gamePath_;
 	std::shared_ptr<GameInfo> info_;
+	GameInfoFlags flags_{};
+
 	DISALLOW_COPY_AND_ASSIGN(GameInfoWorkItem);
 };
 
@@ -691,10 +869,12 @@ void GameInfoCache::Shutdown() {
 void GameInfoCache::Clear() {
 	CancelAll();
 
+	std::lock_guard<std::mutex> lock(mapLock_);
 	info_.clear();
 }
 
 void GameInfoCache::CancelAll() {
+	std::lock_guard<std::mutex> lock(mapLock_);
 	for (auto info : info_) {
 		// GetFileLoader will create one if there isn't one already.
 		// Avoid that by checking.
@@ -708,6 +888,7 @@ void GameInfoCache::CancelAll() {
 }
 
 void GameInfoCache::FlushBGs() {
+	std::lock_guard<std::mutex> lock(mapLock_);
 	for (auto iter = info_.begin(); iter != info_.end(); iter++) {
 		std::lock_guard<std::mutex> lock(iter->second->lock);
 		iter->second->pic0.Clear();
@@ -716,93 +897,92 @@ void GameInfoCache::FlushBGs() {
 			iter->second->sndFileData.clear();
 			iter->second->sndDataLoaded = false;
 		}
-		iter->second->wantFlags &= ~(GAMEINFO_WANTBG | GAMEINFO_WANTSND | GAMEINFO_WANTBGDATA);
+		iter->second->hasFlags &= ~(GameInfoFlags::BG | GameInfoFlags::SND);
 	}
 }
 
 void GameInfoCache::PurgeType(IdentifiedFileType fileType) {
-	for (auto iter = info_.begin(); iter != info_.end();) {
-		auto &info = iter->second;
-		info->readyEvent.Wait();
-		if (info->fileType == fileType) {
-			iter = info_.erase(iter);
-		} else {
-			iter++;
+	bool retry = false;
+	int retryCount = 10;
+	// Trickery to avoid sleeping with the lock held.
+	do {
+		if (retry) {
+			retryCount--;
+			if (retryCount == 0) {
+				break;
+			}
 		}
-	}
+		retry = false;
+		{
+			std::lock_guard<std::mutex> lock(mapLock_);
+			for (auto iter = info_.begin(); iter != info_.end();) {
+				auto &info = iter->second;
+				if (!(info->hasFlags & GameInfoFlags::FILE_TYPE)) {
+					iter++;
+					continue;
+				}
+				if (info->fileType != fileType) {
+					iter++;
+					continue;
+				}
+				// TODO: Find a better way to wait here.
+				if (info->pendingFlags != (GameInfoFlags)0) {
+					INFO_LOG(Log::Loader, "%s: pending flags %08x, retrying", info->GetTitle().c_str(), (int)info->pendingFlags);
+					retry = true;
+					break;
+				}
+				iter = info_.erase(iter);
+			}
+		}
+
+		sleep_ms(10);
+	} while (retry);
 }
 
-void GameInfoCache::WaitUntilDone(std::shared_ptr<GameInfo> &info) {
-	info->readyEvent.Wait();
-}
+// Call on the main thread ONLY - that is from stuff called from NativeFrame.
+// Can also be called from the audio thread for menu background music, but that cannot request images!
+std::shared_ptr<GameInfo> GameInfoCache::GetInfo(Draw::DrawContext *draw, const Path &gamePath, GameInfoFlags wantFlags) {
+	const std::string &pathStr = gamePath.ToString();
 
-// Runs on the main thread. Only call from render() and similar, not update()!
-// Can also be called from the audio thread for menu background music.
-std::shared_ptr<GameInfo> GameInfoCache::GetInfo(Draw::DrawContext *draw, const Path &gamePath, int wantFlags) {
-	std::shared_ptr<GameInfo> info;
+	// _dbg_assert_(gamePath != GetSysDirectory(DIRECTORY_SAVEDATA));
 
-	std::string pathStr = gamePath.ToString();
+	// This is always needed to determine the method to get the other info, so make sure it's computed first.
+	wantFlags |= GameInfoFlags::FILE_TYPE;
+
+	mapLock_.lock();
 
 	auto iter = info_.find(pathStr);
 	if (iter != info_.end()) {
-		info = iter->second;
-	}
+		// There's already a structure about this game. Let's check.
+		std::shared_ptr<GameInfo> info = iter->second;
+		mapLock_.unlock();
 
-	// If wantFlags don't match, we need to start over.  We'll just queue the work item again.
-	if (info && (info->wantFlags & wantFlags) == wantFlags) {
-		if (draw && info->icon.dataLoaded && !info->icon.texture) {
-			SetupTexture(info, draw, info->icon);
-		}
-		if (draw && info->pic0.dataLoaded && !info->pic0.texture) {
-			SetupTexture(info, draw, info->pic0);
-		}
-		if (draw && info->pic1.dataLoaded && !info->pic1.texture) {
-			SetupTexture(info, draw, info->pic1);
-		}
+		info->FinishPendingTextureLoads(draw);
 		info->lastAccessedTime = time_now_d();
+		GameInfoFlags wanted = (GameInfoFlags)0;
+		{
+			// Careful now!
+			std::unique_lock<std::mutex> lock(info->lock);
+			GameInfoFlags hasFlags = info->hasFlags | info->pendingFlags;  // We don't want to re-fetch data that we have, so or in pendingFlags.
+			wanted = (GameInfoFlags)((int)wantFlags & ~(int)hasFlags);  // & is reserved for testing. ugh.
+			info->pendingFlags |= wanted;
+		}
+		if (wanted != (GameInfoFlags)0) {
+			// We're missing info that we want. Go get it!
+			GameInfoWorkItem *item = new GameInfoWorkItem(gamePath, info, wanted);
+			g_threadManager.EnqueueTask(item);
+		}
 		return info;
 	}
 
-	if (!info) {
-		info = std::make_shared<GameInfo>();
-	}
+	std::shared_ptr<GameInfo> info = std::make_shared<GameInfo>(gamePath);
+	info->pendingFlags = wantFlags;
+	info->lastAccessedTime = time_now_d();
+	info_.insert(std::make_pair(pathStr, info));
+	mapLock_.unlock();
 
-	if (info->working) {
-		// Uh oh, it's currently in process.  It could mark pending = false with the wrong wantFlags.
-		// Let's wait it out, then queue.
-		// NOTE: This is bad because we're likely on the UI thread....
-		WaitUntilDone(info);
-	}
-
-	{
-		std::lock_guard<std::mutex> lock(info->lock);
-		info->wantFlags |= wantFlags;
-		info->pending = true;
-	}
-
-	GameInfoWorkItem *item = new GameInfoWorkItem(gamePath, info);
+	// Just get all the stuff we wanted.
+	GameInfoWorkItem *item = new GameInfoWorkItem(gamePath, info, wantFlags);
 	g_threadManager.EnqueueTask(item);
-
-	// Don't re-insert if we already have it.
-	if (info_.find(pathStr) == info_.end())
-		info_[pathStr] = info;
 	return info;
 }
-
-void GameInfoCache::SetupTexture(std::shared_ptr<GameInfo> &info, Draw::DrawContext *thin3d, GameInfoTex &tex) {
-	using namespace Draw;
-	if (tex.data.size()) {
-		if (!tex.texture) {
-			tex.texture = CreateTextureFromFileData(thin3d, (const uint8_t *)tex.data.data(), (int)tex.data.size(), ImageFileType::DETECT, false, info->GetTitle().c_str());
-			if (tex.texture) {
-				tex.timeLoaded = time_now_d();
-			} else {
-				ERROR_LOG(G3D, "Failed creating texture (%s) from %d-byte file", info->GetTitle().c_str(), (int)tex.data.size());
-			}
-		}
-		if ((info->wantFlags & GAMEINFO_WANTBGDATA) == 0) {
-			tex.data.clear();
-			tex.dataLoaded = false;
-		}
-	}
-}
