@@ -26,7 +26,6 @@
 #include "Common/Thread/Event.h"
 #include "Core/ELF/ParamSFO.h"
 #include "Common/File/Path.h"
-#include "Common/Render/ManagedTexture.h"
 
 namespace Draw {
 	class DrawContext;
@@ -51,41 +50,44 @@ enum GameRegion {
 	GAMEREGION_MAX,
 };
 
-enum GameInfoWantFlags {
-	GAMEINFO_WANTBG = 0x01,
-	GAMEINFO_WANTSIZE = 0x02,
-	GAMEINFO_WANTSND = 0x04,
-	GAMEINFO_WANTBGDATA = 0x08, // Use with WANTBG.
+enum class GameInfoFlags {
+	FILE_TYPE = 0x01,  // Don't need to specify this, always included.
+	PARAM_SFO = 0x02,
+	ICON = 0x04,
+	BG = 0x08,
+	SND = 0x10,
+	SIZE = 0x20,
+	UNCOMPRESSED_SIZE = 0x40,
 };
+ENUM_CLASS_BITOPS(GameInfoFlags);
 
 class FileLoader;
 enum class IdentifiedFileType;
 
 struct GameInfoTex {
 	std::string data;
-	std::unique_ptr<ManagedTexture> texture;
+	Draw::Texture *texture = nullptr;
 	// The time at which the Icon and the BG were loaded.
 	// Can be useful to fade them in smoothly once they appear.
+	// Also, timeLoaded != 0 && texture == nullptr means that the load failed.
 	double timeLoaded = 0.0;
 	std::atomic<bool> dataLoaded{};
 
-	void Clear() {
-		if (!data.empty()) {
-			data.clear();
-			dataLoaded = false;
-		}
-		texture.reset(nullptr);
+	// Can ONLY be called from the main thread!
+	void Clear();
+	bool Failed() const {
+		return timeLoaded != 0.0 && !texture;
 	}
 };
 
 class GameInfo {
 public:
-	GameInfo();
+	GameInfo(const Path &gamePath);
 	~GameInfo();
 
 	bool Delete();  // Better be sure what you're doing when calling this.
 	bool DeleteAllSaveData();
-	bool LoadFromPath(const Path &gamePath);
+	bool CreateLoader();
 
 	bool HasFileLoader() const {
 		return fileLoader.get() != nullptr;
@@ -94,17 +96,42 @@ public:
 	std::shared_ptr<FileLoader> GetFileLoader();
 	void DisposeFileLoader();
 
-	u64 GetGameSizeInBytes();
-	u64 GetSaveDataSizeInBytes();
+	u64 GetSizeUncompressedInBytes();  // NOTE: More expensive than GetGameSizeOnDiskInBytes().
+	u64 GetSizeOnDiskInBytes();
+	u64 GetGameSavedataSizeInBytes();  // For games
 	u64 GetInstallDataSizeInBytes();
 
+	// For various kinds of savedata, mainly.
+	// NOTE: This one actually performs I/O directly, not cached.
+	std::string GetMTime() const;
+
 	void ParseParamSFO();
+	const ParamSFOData &GetParamSFO() const {
+		_dbg_assert_(hasFlags & GameInfoFlags::PARAM_SFO);
+		return paramSFO;
+	}
+	void FinishPendingTextureLoads(Draw::DrawContext *draw);
 
 	std::vector<Path> GetSaveDataDirectories();
 
 	std::string GetTitle();
 	void SetTitle(const std::string &newTitle);
 
+	const Path &GetFilePath() const {
+		return filePath_;
+	}
+
+	bool Ready(GameInfoFlags flags) {
+		std::unique_lock<std::mutex> guard(lock);
+		// Avoid the operator, we want to check all the bits.
+		return ((int)hasFlags & (int)flags) == (int)flags;
+	}
+
+	void MarkReadyNoLock(GameInfoFlags flags) {
+		hasFlags |= flags;
+		pendingFlags &= ~flags;
+	}
+
 	GameInfoTex *GetBGPic() {
 		if (pic1.texture)
 			return &pic1;
@@ -122,14 +149,17 @@ public:
 	// Controls access to the fileLoader pointer.
 	std::mutex loaderLock;
 
+	// Keep track of what we have, or what we're processing.
+	// These are protected by the mutex. While pendingFlags != 0, something is being loaded.
+	GameInfoFlags hasFlags{};
+	GameInfoFlags pendingFlags{};
+
 	std::string id;
 	std::string id_version;
 	int disc_total = 0;
 	int disc_number = 0;
 	int region = -1;
 	IdentifiedFileType fileType;
-	ParamSFOData paramSFO;
-	bool paramSFOLoaded = false;
 	bool hasConfig = false;
 
 	// Pre read the data, create a texture the next time (GL thread..)
@@ -140,20 +170,15 @@ public:
 	std::string sndFileData;
 	std::atomic<bool> sndDataLoaded{};
 
-	int wantFlags = 0;
-
 	double lastAccessedTime = 0.0;
 
-	u64 gameSize = 0;
+	u64 gameSizeUncompressed = 0;
+	u64 gameSizeOnDisk = 0;  // compressed size, in case of CSO
 	u64 saveDataSize = 0;
 	u64 installDataSize = 0;
 
-	std::atomic<bool> pending{};
-	std::atomic<bool> working{};
-
-	Event readyEvent;
-
 protected:
+	ParamSFOData paramSFO;
 	// Note: this can change while loading, use GetTitle().
 	std::string title;
 
@@ -161,8 +186,11 @@ protected:
 	std::shared_ptr<FileLoader> fileLoader;
 	Path filePath_;
 
+	void SetupTexture(Draw::DrawContext *draw, GameInfoTex &tex);
+
 private:
 	DISALLOW_COPY_AND_ASSIGN(GameInfo);
+	friend class GameInfoWorkItem;
 };
 
 class GameInfoCache {
@@ -178,20 +206,20 @@ public:
 	// but filled in later asynchronously in the background. So keep calling this,
 	// redrawing the UI often. Only set flags to GAMEINFO_WANTBG or WANTSND if you really want them 
 	// because they're big. bgTextures and sound may be discarded over time as well.
-	std::shared_ptr<GameInfo> GetInfo(Draw::DrawContext *draw, const Path &gamePath, int wantFlags);
+	// NOTE: This never returns null, so you don't need to check for that. Do check Ready() flags though.
+	std::shared_ptr<GameInfo> GetInfo(Draw::DrawContext *draw, const Path &gamePath, GameInfoFlags wantFlags);
 	void FlushBGs();  // Gets rid of all BG textures. Also gets rid of bg sounds.
 
 	void CancelAll();
-	void WaitUntilDone(std::shared_ptr<GameInfo> &info);
 
 private:
 	void Init();
 	void Shutdown();
-	void SetupTexture(std::shared_ptr<GameInfo> &info, Draw::DrawContext *draw, GameInfoTex &tex);
 
 	// Maps ISO path to info. Need to use shared_ptr as we can return these pointers - 
 	// and if they get destructed while being in use, that's bad.
 	std::map<std::string, std::shared_ptr<GameInfo> > info_;
+	std::mutex mapLock_;
 };
 
 // This one can be global, no good reason not to.
