@@ -19,11 +19,15 @@
 
 #include <functional>
 
+#include "Common/File/Path.h"
+
 #include "Common/UI/View.h"
 #include "Common/UI/UIScreen.h"
 
 #include "UI/MiscScreens.h"
 
+class SavedataView;
+
 class InstallZipScreen : public UIDialogScreenWithBackground {
 public:
 	InstallZipScreen(const Path &zipPath);
@@ -40,9 +44,13 @@ private:
 
 	UI::Choice *installChoice_ = nullptr;
 	UI::Choice *backChoice_ = nullptr;
-	UI::ProgressBar *progressBar_ = nullptr;
 	UI::TextView *doneView_ = nullptr;
+	SavedataView *existingSaveView_ = nullptr;
+	Path savedataToOverwrite_;
 	Path zipPath_;
+	std::vector<Path> destFolders_;
+	int destFolderChoice_ = 0;
+	ZipFileInfo zipFileInfo_{};
 	bool returnToHomebrew_ = true;
 	bool installStarted_ = false;
 	bool deleteZipFile_ = false;
