@@ -44,65 +44,13 @@
 #include "Core/Reporting.h"
 #include "Core/System.h"
 #include "Core/Util/GameManager.h"
+#include "Core/Util/MemStick.h"
 
 #include "UI/MemStickScreen.h"
 #include "UI/MainScreen.h"
 #include "UI/MiscScreens.h"
 #include "UI/OnScreenDisplay.h"
 
-static bool FolderSeemsToBeUsed(Path newMemstickFolder) {
-	// Inspect the potential new folder, quickly.
-	if (File::Exists(newMemstickFolder / "PSP/SAVEDATA") || File::Exists(newMemstickFolder / "SAVEDATA")) {
-		// Does seem likely. We could add more criteria like checking for actual savegames or something.
-		return true;
-	} else {
-		return false;
-	}
-}
-
-static bool SwitchMemstickFolderTo(Path newMemstickFolder) {
-	Path testWriteFile = newMemstickFolder / ".write_verify_file";
-
-	// Doesn't already exist, create.
-	// Should this ever happen?
-	if (newMemstickFolder.Type() == PathType::NATIVE) {
-		if (!File::Exists(newMemstickFolder)) {
-			File::CreateFullPath(newMemstickFolder);
-		}
-		if (!File::WriteDataToFile(true, "1", 1, testWriteFile)) {
-			return false;
-		}
-		File::Delete(testWriteFile);
-	} else {
-		// TODO: Do the same but with scoped storage? Not really necessary, right? If it came from a browse
-		// for folder, we can assume it exists and is writable, barring wacky race conditions like the user
-		// being connected by USB and deleting it.
-	}
-
-	Path memStickDirFile = g_Config.internalDataDirectory / "memstick_dir.txt";
-#if PPSSPP_PLATFORM(UWP)
-	File::Delete(memStickDirFile);
-	if (newMemstickFolder != g_Config.internalDataDirectory) {
-#endif
-
-	std::string str = newMemstickFolder.ToString();
-	if (!File::WriteDataToFile(true, str.c_str(), (unsigned int)str.size(), memStickDirFile)) {
-		ERROR_LOG(SYSTEM, "Failed to write memstick path '%s' to '%s'", newMemstickFolder.c_str(), memStickDirFile.c_str());
-		// Not sure what to do if this file can't be written.  Disk full?
-	}
-
-#if PPSSPP_PLATFORM(UWP)
-	}
-#endif
-
-	// Save so the settings, at least, are transferred.
-	g_Config.memStickDirectory = newMemstickFolder;
-	g_Config.SetSearchPath(GetSysDirectory(DIRECTORY_SYSTEM));
-	g_Config.UpdateIniLocation();
-
-	return true;
-}
-
 static std::string FormatSpaceString(int64_t space) {
 	if (space >= 0) {
 		char buffer[50];
@@ -129,7 +77,7 @@ MemStickScreen::MemStickScreen(bool initialSetup)
 		if (System_GetPropertyBool(SYSPROP_ANDROID_SCOPED_STORAGE)) {
 			choice_ = CHOICE_BROWSE_FOLDER;
 		} else {
-			WARN_LOG_REPORT(SYSTEM, "Scoped storage not enabled - shouldn't be in MemStickScreen at initial setup");
+			WARN_LOG_REPORT(Log::System, "Scoped storage not enabled - shouldn't be in MemStickScreen at initial setup");
 			choice_ = CHOICE_STORAGE_ROOT;
 			// Shouldn't really be here in initial setup.
 		}
@@ -209,7 +157,7 @@ void MemStickScreen::CreateViews() {
 	using namespace UI;
 
 	auto di = GetI18NCategory(I18NCat::DIALOG);
-	auto iz = GetI18NCategory(I18NCat::MEMSTICK);
+	auto ms = GetI18NCategory(I18NCat::MEMSTICK);
 
 	Margins actionMenuMargins(15, 0, 15, 0);
 
@@ -226,12 +174,12 @@ void MemStickScreen::CreateViews() {
 
 	if (initialSetup_) {
 		mainColumn->Add(new Spacer(new LinearLayoutParams(FILL_PARENT, 12.0f, 0.0f)));
-		mainColumn->Add(new TextView(iz->T("Welcome to PPSSPP!"), ALIGN_LEFT, false));
+		mainColumn->Add(new TextView(ms->T("Welcome to PPSSPP!"), ALIGN_LEFT, false));
 	}
 
 	mainColumn->Add(new Spacer(new LinearLayoutParams(FILL_PARENT, 18.0f, 0.0f)));
 
-	mainColumn->Add(new TextView(iz->T("MemoryStickDescription", "Choose where to keep PSP data (Memory Stick)"), ALIGN_LEFT, false));
+	mainColumn->Add(new TextView(ms->T("MemoryStickDescription", "Choose where to keep PSP data (Memory Stick)"), ALIGN_LEFT, false));
 	mainColumn->Add(new Spacer(new LinearLayoutParams(FILL_PARENT, 18.0f, 0.0f)));
 
 	ViewGroup *subColumns = new LinearLayoutList(ORIENT_HORIZONTAL);
@@ -247,7 +195,7 @@ void MemStickScreen::CreateViews() {
 	// Trying to avoid needing a scroll view, so only showing the explanation for one option at a time.
 #if !PPSSPP_PLATFORM(UWP)
 	if (!System_GetPropertyBool(SYSPROP_ANDROID_SCOPED_STORAGE)) {
-		leftColumn->Add(new RadioButton(&choice_, CHOICE_STORAGE_ROOT, iz->T("Use PSP folder at root of storage")))->OnClick.Handle(this, &MemStickScreen::OnChoiceClick);
+		leftColumn->Add(new RadioButton(&choice_, CHOICE_STORAGE_ROOT, ms->T("Use PSP folder at root of storage")))->OnClick.Handle(this, &MemStickScreen::OnChoiceClick);
 		if (choice_ == CHOICE_STORAGE_ROOT) {
 			AddExplanation(leftColumn, (MemStickScreen::Choice)choice_);
 		}
@@ -255,30 +203,30 @@ void MemStickScreen::CreateViews() {
 #endif
 
 	if (storageBrowserWorking_) {
-		leftColumn->Add(new RadioButton(&choice_, CHOICE_BROWSE_FOLDER, iz->T("Create or Choose a PSP folder")))->OnClick.Handle(this, &MemStickScreen::OnChoiceClick);
+		leftColumn->Add(new RadioButton(&choice_, CHOICE_BROWSE_FOLDER, ms->T("Create or Choose a PSP folder")))->OnClick.Handle(this, &MemStickScreen::OnChoiceClick);
 
 		// TODO: Show current folder here if we have one set.
 	} else {
-		leftColumn->Add(new RadioButton(&choice_, CHOICE_SET_MANUAL, iz->T("Manually specify PSP folder")))->OnClick.Handle(this, &MemStickScreen::OnChoiceClick);
-		leftColumn->Add(new TextView(iz->T("DataWillStay", "Data will stay even if you uninstall PPSSPP.")))->SetBullet(true);
-		leftColumn->Add(new TextView(iz->T("DataCanBeShared", "Data can be shared between PPSSPP regular/Gold.")))->SetBullet(true);
+		leftColumn->Add(new RadioButton(&choice_, CHOICE_SET_MANUAL, ms->T("Manually specify PSP folder")))->OnClick.Handle(this, &MemStickScreen::OnChoiceClick);
+		leftColumn->Add(new TextView(ms->T("DataWillStay", "Data will stay even if you uninstall PPSSPP.")))->SetBullet(true);
+		leftColumn->Add(new TextView(ms->T("DataCanBeShared", "Data can be shared between PPSSPP regular/Gold.")))->SetBullet(true);
 		// TODO: Show current folder here if we have one set.
 	}
-	errorNoticeView_ = leftColumn->Add(new NoticeView(NoticeLevel::WARN, iz->T("Cancelled - try again"), ""));
+	errorNoticeView_ = leftColumn->Add(new NoticeView(NoticeLevel::WARN, ms->T("Cancelled - try again"), ""));
 	errorNoticeView_->SetVisibility(UI::V_GONE);
 
 	if (choice_ == CHOICE_BROWSE_FOLDER || choice_ == CHOICE_SET_MANUAL) {
 		UI::View *extraView = nullptr;
 		if (!g_Config.memStickDirectory.empty()) {
-			extraView = new TextView(StringFromFormat("    %s: %s", iz->T("Current"), g_Config.memStickDirectory.ToVisualString().c_str()), ALIGN_LEFT, false);
+			extraView = new TextView(StringFromFormat("    %s: %s", ms->T_cstr("Current"), g_Config.memStickDirectory.ToVisualString().c_str()), ALIGN_LEFT, false);
 		}
 		AddExplanation(leftColumn, (MemStickScreen::Choice)choice_, extraView);
 	}
 
-	std::string privateString = iz->T("Use App Private Data");
+	std::string privateString(ms->T("Use App Private Data"));
 
 	if (initialSetup_) {
-		privateString = StringFromFormat("%s (%s)", iz->T("Skip for now"), privateString.c_str());
+		privateString = StringFromFormat("%s (%s)", ms->T_cstr("Skip for now"), privateString.c_str());
 	}
 
 	leftColumn->Add(new RadioButton(&choice_, CHOICE_PRIVATE_DIRECTORY, privateString))->OnClick.Handle(this, &MemStickScreen::OnChoiceClick);
@@ -288,7 +236,7 @@ void MemStickScreen::CreateViews() {
 
 	leftColumn->Add(new Spacer(new LinearLayoutParams(FILL_PARENT, 12.0f, 0.0f)));
 
-	const char *confirmButtonText = nullptr;
+	std::string_view confirmButtonText;
 	ImageID confirmButtonImage = ImageID::invalid();
 	switch (choice_) {
 	case CHOICE_BROWSE_FOLDER:
@@ -317,10 +265,10 @@ void MemStickScreen::CreateViews() {
 		rightColumnItems->Add(new UI::Choice(di->T("Back")))->OnClick.Handle<UIScreen>(this, &UIScreen::OnBack);
 	}
 	if (System_GetPropertyInt(SYSPROP_DEVICE_TYPE) != DEVICE_TYPE_TV) {
-		rightColumnItems->Add(new UI::Choice(iz->T("WhatsThis", "What's this?")))->OnClick.Handle<MemStickScreen>(this, &MemStickScreen::OnHelp);
+		rightColumnItems->Add(new UI::Choice(ms->T("WhatsThis", "What's this?")))->OnClick.Handle<MemStickScreen>(this, &MemStickScreen::OnHelp);
 	}
 
-	INFO_LOG(SYSTEM, "MemStickScreen: initialSetup=%d", (int)initialSetup_);
+	INFO_LOG(Log::System, "MemStickScreen: initialSetup=%d", (int)initialSetup_);
 }
 
 UI::EventReturn MemStickScreen::OnHelp(UI::EventParams &params) {
@@ -354,17 +302,17 @@ UI::EventReturn MemStickScreen::OnConfirmClick(UI::EventParams &params) {
 }
 
 UI::EventReturn MemStickScreen::SetFolderManually(UI::EventParams &params) {
-	// The old way, from before scoped storage.
+	// The old way, from before scoped storage. Write in the full path.
 #if PPSSPP_PLATFORM(ANDROID) || PPSSPP_PLATFORM(SWITCH)
 	auto sy = GetI18NCategory(I18NCat::SYSTEM);
-	System_InputBoxGetString(sy->T("Memory Stick Folder"), g_Config.memStickDirectory.ToString(), [&](const std::string &value, int) {
+	System_InputBoxGetString(GetRequesterToken(), sy->T("Memory Stick Folder"), g_Config.memStickDirectory.ToString(), false, [&](const std::string &value, int) {
 		auto sy = GetI18NCategory(I18NCat::SYSTEM);
 		auto di = GetI18NCategory(I18NCat::DIALOG);
 
 		std::string newPath = value;
-		size_t pos = newPath.find_last_not_of("/");
+		size_t pos = newPath.find_last_not_of('/');
 		// Gotta have at least something but a /, and also needs to start with a /.
-		if (newPath.empty() || pos == newPath.npos || newPath[0] != '/') {
+		if (newPath.empty() || pos == std::string::npos || newPath[0] != '/') {
 			settingInfo_->Show(sy->T("ChangingMemstickPathInvalid", "That path couldn't be used to save Memory Stick files."), nullptr);
 			return;
 		}
@@ -394,10 +342,10 @@ UI::EventReturn MemStickScreen::SetFolderManually(UI::EventParams &params) {
 			std::string oldNewPath = newPath;
 			FixPathCase(Path(""), newPath, FixPathCaseBehavior::FPC_FILE_MUST_EXIST);
 			if (oldNewPath != newPath) {
-				NOTICE_LOG(IO, "Fixed path case: %s -> %s", oldNewPath.c_str(), newPath.c_str());
+				NOTICE_LOG(Log::IO, "Fixed path case: %s -> %s", oldNewPath.c_str(), newPath.c_str());
 				pendingMemStickFolder = Path(newPath);
 			} else {
-				NOTICE_LOG(IO, "Failed to fix case of path %s (result: %s)", newPath.c_str(), oldNewPath.c_str());
+				NOTICE_LOG(Log::IO, "Failed to fix case of path %s (result: %s)", newPath.c_str(), oldNewPath.c_str());
 			}
 		}
 
@@ -432,7 +380,7 @@ UI::EventReturn MemStickScreen::UseInternalStorage(UI::EventParams &params) {
 			TriggerFinish(DialogResult::DR_OK);
 		} else {
 			// This can't really happen?? Not worth making an error message.
-			ERROR_LOG_REPORT(SYSTEM, "Could not switch memstick path in setup (internal)");
+			ERROR_LOG_REPORT(Log::System, "Could not switch memstick path in setup (internal)");
 		}
 		// Don't have a confirmation dialog that would otherwise do it for us, need to just switch directly to the main screen.
 		screenManager()->switchScreen(new MainScreen());
@@ -456,7 +404,7 @@ UI::EventReturn MemStickScreen::UseStorageRoot(UI::EventParams &params) {
 			TriggerFinish(DialogResult::DR_OK);
 		} else {
 			// This can't really happen?? Not worth making an error message.
-			ERROR_LOG_REPORT(SYSTEM, "Could not switch memstick path in setup");
+			ERROR_LOG_REPORT(Log::System, "Could not switch memstick path in setup");
 		}
 	} else if (pendingMemStickFolder != g_Config.memStickDirectory) {
 		// Always ask for confirmation when called from the UI. Likely there's already some data.
@@ -470,18 +418,19 @@ UI::EventReturn MemStickScreen::UseStorageRoot(UI::EventParams &params) {
 
 UI::EventReturn MemStickScreen::Browse(UI::EventParams &params) {
 	auto mm = GetI18NCategory(I18NCat::MAINMENU);
-	System_BrowseForFolder(mm->T("Choose folder"), [=](const std::string &value, int) {
+	System_BrowseForFolder(GetRequesterToken(), mm->T("Choose folder"), g_Config.memStickDirectory, [=](const std::string &value, int) {
 		Path pendingMemStickFolder = Path(value);
-		INFO_LOG(SYSTEM, "Got folder: '%s'", pendingMemStickFolder.c_str());
+		INFO_LOG(Log::System, "Got folder: '%s' (old: %s)", pendingMemStickFolder.c_str(), g_Config.memStickDirectory.c_str());
 		// Browse finished. Let's pop up the confirmation dialog.
 		if (!pendingMemStickFolder.empty() && pendingMemStickFolder == g_Config.memStickDirectory && File::IsDirectory(pendingMemStickFolder)) {
-			auto iz = GetI18NCategory(I18NCat::MEMSTICK);
+			auto di = GetI18NCategory(I18NCat::DIALOG);
 			// Not sure how this could happen, but let's go with it.
-			g_OSD.Show(OSDType::MESSAGE_SUCCESS, iz->T("Done!"));
+			g_OSD.Show(OSDType::MESSAGE_SUCCESS, di->T("Done!"));
 			done_ = true;
 			return;
 		}
 		errorNoticeView_->SetVisibility(UI::V_GONE);
+
 		screenManager()->push(new ConfirmMemstickMoveScreen(pendingMemStickFolder, initialSetup_));
 	}, [=]() {
 		errorNoticeView_->SetVisibility(UI::V_VISIBLE);
@@ -491,7 +440,7 @@ UI::EventReturn MemStickScreen::Browse(UI::EventParams &params) {
 
 void MemStickScreen::dialogFinished(const Screen *dialog, DialogResult result) {
 	if (result == DialogResult::DR_OK) {
-		INFO_LOG(SYSTEM, "Confirmation screen done - moving on.");
+		INFO_LOG(Log::System, "Confirmation screen done - moving on.");
 		// There's a screen manager bug if we call TriggerFinish directly.
 		// Can't be bothered right now, so we pick this up in update().
 		done_ = true;
@@ -507,45 +456,8 @@ void MemStickScreen::update() {
 	}
 }
 
-// Keep the size with the file, so we can skip overly large ones in the move.
-// The user will have to take care of them afterwards, it'll just take too long probably.
-struct FileSuffix {
-	std::string suffix;
-	u64 fileSize;
-};
-
-static bool ListFileSuffixesRecursively(const Path &root, Path folder, std::vector<std::string> &dirSuffixes, std::vector<FileSuffix> &fileSuffixes) {
-	std::vector<File::FileInfo> files;
-	if (!File::GetFilesInDir(folder, &files)) {
-		return false;
-	}
-
-	for (auto &file : files) {
-		if (file.isDirectory) {
-			std::string dirSuffix;
-			if (root.ComputePathTo(file.fullName, dirSuffix)) {
-				if (!dirSuffix.empty()) {
-					dirSuffixes.push_back(dirSuffix);
-					ListFileSuffixesRecursively(root, folder / file.name, dirSuffixes, fileSuffixes);
-				}
-			} else {
-				ERROR_LOG_REPORT(SYSTEM, "Failed to compute PathTo from '%s' to '%s'", root.c_str(), folder.c_str());
-			}
-		} else {
-			std::string fileSuffix;
-			if (root.ComputePathTo(file.fullName, fileSuffix)) {
-				if (!fileSuffix.empty()) {
-					fileSuffixes.push_back(FileSuffix{ fileSuffix, file.size });
-				}
-			}
-		}
-	}
-
-	return true;
-}
-
-ConfirmMemstickMoveScreen::ConfirmMemstickMoveScreen(Path newMemstickFolder, bool initialSetup)
-	: newMemstickFolder_(newMemstickFolder), initialSetup_(initialSetup) {
+ConfirmMemstickMoveScreen::ConfirmMemstickMoveScreen(const Path &newMemstickFolder, bool initialSetup)
+	: newMemstickFolder_(newMemstickFolder), initialSetup_(initialSetup), progressReporter_() {
 	existingFilesInNewFolder_ = FolderSeemsToBeUsed(newMemstickFolder);
 	if (initialSetup_) {
 		moveData_ = false;
@@ -554,21 +466,28 @@ ConfirmMemstickMoveScreen::ConfirmMemstickMoveScreen(Path newMemstickFolder, boo
 
 ConfirmMemstickMoveScreen::~ConfirmMemstickMoveScreen() {
 	if (moveDataTask_) {
-		INFO_LOG(SYSTEM, "Move Data task still running, blocking on it");
+		INFO_LOG(Log::System, "Move Data task still running, blocking on it");
 		moveDataTask_->BlockUntilReady();
 		delete moveDataTask_;
 	}
+	if (oldSpaceTask_) {
+		oldSpaceTask_->BlockUntilReady();
+		delete oldSpaceTask_;
+	}
+	if (newSpaceTask_) {
+		newSpaceTask_->BlockUntilReady();
+		delete newSpaceTask_;
+	}
 }
 
 void ConfirmMemstickMoveScreen::CreateViews() {
 	using namespace UI;
-	auto di = GetI18NCategory(I18NCat::DIALOG);
 	auto sy = GetI18NCategory(I18NCat::SYSTEM);
-	auto iz = GetI18NCategory(I18NCat::MEMSTICK);
+	auto ms = GetI18NCategory(I18NCat::MEMSTICK);
 
 	root_ = new LinearLayout(ORIENT_HORIZONTAL);
 
-	Path oldMemstickFolder = g_Config.memStickDirectory;
+	Path &oldMemstickFolder = g_Config.memStickDirectory;
 
 	Spacer *spacerColumn = new Spacer(new LinearLayoutParams(20.0, FILL_PARENT, 0.0f));
 	ViewGroup *leftColumn = new LinearLayout(ORIENT_VERTICAL, new LinearLayoutParams(1.0));
@@ -577,22 +496,24 @@ void ConfirmMemstickMoveScreen::CreateViews() {
 	root_->Add(leftColumn);
 	root_->Add(rightColumn);
 
-	int64_t freeSpaceNew;
-	int64_t freeSpaceOld;
-	free_disk_space(newMemstickFolder_, freeSpaceNew);
-	free_disk_space(oldMemstickFolder, freeSpaceOld);
-
-	leftColumn->Add(new TextView(iz->T("Selected PSP Data Folder"), ALIGN_LEFT, false));
+	leftColumn->Add(new TextView(ms->T("Selected PSP Data Folder"), ALIGN_LEFT, false));
 	if (!initialSetup_) {
-		leftColumn->Add(new NoticeView(NoticeLevel::WARN, iz->T("PPSSPP will restart after the change"), ""));
+		leftColumn->Add(new NoticeView(NoticeLevel::WARN, ms->T("PPSSPP will restart after the change"), ""));
 	}
 	leftColumn->Add(new TextView(newMemstickFolder_.ToVisualString(), ALIGN_LEFT, false));
-	std::string newFreeSpaceText = std::string(iz->T("Free space")) + ": " + FormatSpaceString(freeSpaceNew);
-	leftColumn->Add(new TextView(newFreeSpaceText, ALIGN_LEFT, false));
+
+	newFreeSpaceView_ = leftColumn->Add(new TextView(ms->T("Free space"), ALIGN_LEFT, false));
+
+	newSpaceTask_ = Promise<SpaceResult *>::Spawn(&g_threadManager, [&]() -> SpaceResult * {
+		int64_t freeSpaceNew;
+		free_disk_space(newMemstickFolder_, freeSpaceNew);
+		return new SpaceResult{ freeSpaceNew };
+	}, TaskType::IO_BLOCKING, TaskPriority::HIGH);
+
 	if (existingFilesInNewFolder_) {
-		leftColumn->Add(new NoticeView(NoticeLevel::SUCCESS, iz->T("Already contains PSP data"), ""));
+		leftColumn->Add(new NoticeView(NoticeLevel::SUCCESS, ms->T("Already contains PSP data"), ""));
 		if (!moveData_) {
-			leftColumn->Add(new NoticeView(NoticeLevel::INFO, iz->T("No data will be changed"), ""));
+			leftColumn->Add(new NoticeView(NoticeLevel::INFO, ms->T("No data will be changed"), ""));
 		}
 	}
 	if (!error_.empty()) {
@@ -600,24 +521,29 @@ void ConfirmMemstickMoveScreen::CreateViews() {
 	}
 
 	if (!oldMemstickFolder.empty()) {
-		std::string oldFreeSpaceText = std::string(iz->T("Free space")) + ": " + FormatSpaceString(freeSpaceOld);
+		oldSpaceTask_ = Promise<SpaceResult *>::Spawn(&g_threadManager, [&]() -> SpaceResult * {
+			int64_t freeSpaceOld;
+			free_disk_space(oldMemstickFolder, freeSpaceOld);
+			return new SpaceResult{ freeSpaceOld };
+		}, TaskType::IO_BLOCKING, TaskPriority::HIGH);
 
-		rightColumn->Add(new TextView(std::string(iz->T("Current")) + ":", ALIGN_LEFT, false));
+		rightColumn->Add(new TextView(std::string(ms->T("Current")) + ":", ALIGN_LEFT, false));
 		rightColumn->Add(new TextView(oldMemstickFolder.ToVisualString(), ALIGN_LEFT, false));
-		rightColumn->Add(new TextView(oldFreeSpaceText, ALIGN_LEFT, false));
+		oldFreeSpaceView_ = rightColumn->Add(new TextView(ms->T("Free space"), ALIGN_LEFT, false));
 	}
 
 	if (moveDataTask_) {
-		progressView_ = leftColumn->Add(new TextView(progressReporter_.Get()));
+		progressView_ = leftColumn->Add(new TextView(progressReporter_.Format()));
 	} else {
 		progressView_ = nullptr;
 	}
 
 	if (!moveDataTask_) {
 		if (!initialSetup_) {
-			leftColumn->Add(new CheckBox(&moveData_, iz->T("Move Data")))->OnClick.Handle(this, &ConfirmMemstickMoveScreen::OnMoveDataClick);
+			leftColumn->Add(new CheckBox(&moveData_, ms->T("Move Data")))->OnClick.Handle(this, &ConfirmMemstickMoveScreen::OnMoveDataClick);
 		}
 
+		auto di = GetI18NCategory(I18NCat::DIALOG);
 		leftColumn->Add(new Choice(di->T("OK")))->OnClick.Handle(this, &ConfirmMemstickMoveScreen::OnConfirm);
 		leftColumn->Add(new Choice(di->T("Back")))->OnClick.Handle<UIScreen>(this, &UIScreen::OnBack);
 	}
@@ -630,24 +556,24 @@ UI::EventReturn ConfirmMemstickMoveScreen::OnMoveDataClick(UI::EventParams &para
 
 void ConfirmMemstickMoveScreen::update() {
 	UIDialogScreenWithBackground::update();
-	auto iz = GetI18NCategory(I18NCat::MEMSTICK);
+	auto ms = GetI18NCategory(I18NCat::MEMSTICK);
 
 	if (moveDataTask_) {
 		if (progressView_) {
-			progressView_->SetText(progressReporter_.Get());
+			progressView_->SetText(progressReporter_.Format());
 		}
 
 		MoveResult *result = moveDataTask_->Poll();
 
 		if (result) {
 			if (result->success) {
-				progressReporter_.Set(iz->T("Done!"));
-				INFO_LOG(SYSTEM, "Move data task finished successfully!");
+				progressReporter_.SetProgress(ms->T("Done!"));
+				INFO_LOG(Log::System, "Move data task finished successfully!");
 				// Succeeded!
 				FinishFolderMove();
 			} else {
-				progressReporter_.Set(iz->T("Failed to move some files!"));
-				INFO_LOG(SYSTEM, "Move data task failed!");
+				progressReporter_.SetProgress(ms->T("Failed to move some files!"));
+				INFO_LOG(Log::System, "Move data task failed!");
 				// What do we do here? We might be in the middle of a move... Bad.
 				RecreateViews();
 			}
@@ -655,6 +581,23 @@ void ConfirmMemstickMoveScreen::update() {
 			moveDataTask_ = nullptr;
 		}
 	}
+
+	if (newSpaceTask_ && newFreeSpaceView_) {
+		SpaceResult *result = newSpaceTask_->Poll();
+		if (result) {
+			newFreeSpaceView_->SetText(std::string(ms->T("Free space")) + ": " + FormatSpaceString(result->bytesFree));
+			delete newSpaceTask_;
+			newSpaceTask_ = nullptr;
+		}
+	}
+	if (oldSpaceTask_ && oldFreeSpaceView_) {
+		SpaceResult *result = oldSpaceTask_->Poll();
+		if (result) {
+			oldFreeSpaceView_->SetText(std::string(ms->T("Free space")) + ": " + FormatSpaceString(result->bytesFree));
+			delete oldSpaceTask_;
+			oldSpaceTask_ = nullptr;
+		}
+	}
 }
 
 UI::EventReturn ConfirmMemstickMoveScreen::OnConfirm(UI::EventParams &params) {
@@ -664,111 +607,12 @@ UI::EventReturn ConfirmMemstickMoveScreen::OnConfirm(UI::EventParams &params) {
 	// If the directory itself is called PSP, don't go below.
 
 	if (moveData_) {
-		progressReporter_.Set(T(I18NCat::MEMSTICK, "Starting move..."));
+		progressReporter_.SetProgress(T(I18NCat::MEMSTICK, "Starting move..."));
 
 		moveDataTask_ = Promise<MoveResult *>::Spawn(&g_threadManager, [&]() -> MoveResult * {
-			auto ms = GetI18NCategory(I18NCat::MEMSTICK);
 			Path moveSrc = g_Config.memStickDirectory;
 			Path moveDest = newMemstickFolder_;
-			if (moveSrc.GetFilename() != "PSP") {
-				moveSrc = moveSrc / "PSP";
-			}
-			if (moveDest.GetFilename() != "PSP") {
-				moveDest = moveDest / "PSP";
-				File::CreateDir(moveDest);
-			}
-
-			INFO_LOG(SYSTEM, "About to move PSP data from '%s' to '%s'", moveSrc.c_str(), moveDest.c_str());
-
-			// Search through recursively, listing the files to move and also summing their sizes.
-			std::vector<FileSuffix> fileSuffixesToMove;
-			std::vector<std::string> directorySuffixesToCreate;
-
-			// NOTE: It's correct to pass moveSrc twice here, it's to keep the root in the recursion.
-			if (!ListFileSuffixesRecursively(moveSrc, moveSrc, directorySuffixesToCreate, fileSuffixesToMove)) {
-				// TODO: Handle failure listing files.
-				std::string error = "Failed to read old directory";
-				INFO_LOG(SYSTEM, "%s", error.c_str());
-				progressReporter_.Set(ms->T(error.c_str()));
-				return new MoveResult{ false, error };
-			}
-
-			bool dryRun = false;  // Useful for debugging.
-
-			size_t failedFiles = 0;
-			size_t skippedFiles = 0;
-
-			// We're not moving huge files like ISOs during this process, unless
-			// they can be directly moved, without rewriting the file.
-			const uint64_t BIG_FILE_THRESHOLD = 24 * 1024 * 1024;
-
-			if (!moveSrc.empty()) {
-				// Better not interrupt the app while this is happening!
-
-				// Create all the necessary directories.
-				for (auto &dirSuffix : directorySuffixesToCreate) {
-					Path dir = moveDest / dirSuffix;
-					if (dryRun) {
-						INFO_LOG(SYSTEM, "dry run: Would have created dir '%s'", dir.c_str());
-					} else {
-						INFO_LOG(SYSTEM, "Creating dir '%s'", dir.c_str());
-						progressReporter_.Set(dirSuffix);
-						// Just ignore already-exists errors.
-						File::CreateDir(dir);
-					}
-				}
-
-				for (auto &fileSuffix : fileSuffixesToMove) {
-					progressReporter_.Set(StringFromFormat("%s (%s)", fileSuffix.suffix.c_str(), NiceSizeFormat(fileSuffix.fileSize).c_str()));
-
-					Path from = moveSrc / fileSuffix.suffix;
-					Path to = moveDest / fileSuffix.suffix;
-
-					if (fileSuffix.fileSize > BIG_FILE_THRESHOLD) {
-						// We only move big files if it's fast to do so.
-						if (dryRun) {
-							INFO_LOG(SYSTEM, "dry run: Would have moved '%s' to '%s' (%d bytes) if fast", from.c_str(), to.c_str(), (int)fileSuffix.fileSize);
-						} else {
-							if (!File::MoveIfFast(from, to)) {
-								INFO_LOG(SYSTEM, "Skipped moving file '%s' to '%s' (%s)", from.c_str(), to.c_str(), NiceSizeFormat(fileSuffix.fileSize).c_str());
-								skippedFiles++;
-							} else {
-								INFO_LOG(SYSTEM, "Moved file '%s' to '%s'", from.c_str(), to.c_str());
-							}
-						}
-					} else {
-						if (dryRun) {
-							INFO_LOG(SYSTEM, "dry run: Would have moved '%s' to '%s' (%d bytes)", from.c_str(), to.c_str(), (int)fileSuffix.fileSize);
-						} else {
-							// Remove the "from" prefix from the path.
-							// We have to drop down to string operations for this.
-							if (!File::Move(from, to)) {
-								ERROR_LOG(SYSTEM, "Failed to move file '%s' to '%s'", from.c_str(), to.c_str());
-								failedFiles++;
-								// Should probably just bail?
-							} else {
-								INFO_LOG(SYSTEM, "Moved file '%s' to '%s'", from.c_str(), to.c_str());
-							}
-						}
-					}
-				}
-
-				// Delete all the old, now hopefully empty, directories.
-				for (auto &dirSuffix : directorySuffixesToCreate) {
-					Path dir = moveSrc / dirSuffix;
-					if (dryRun) {
-						INFO_LOG(SYSTEM, "dry run: Would have deleted dir '%s'", dir.c_str());
-					} else {
-						INFO_LOG(SYSTEM, "Deleting dir '%s'", dir.c_str());
-						progressReporter_.Set(dirSuffix);
-						if (File::Exists(dir)) {
-							File::DeleteDir(dir);
-						}
-					}
-				}
-			}
-
-			return new MoveResult{ true, "", failedFiles };
+			return MoveDirectoryContentsSafe(moveSrc, moveDest, progressReporter_);
 		}, TaskType::IO_BLOCKING, TaskPriority::HIGH);
 
 		RecreateViews();
@@ -782,6 +626,8 @@ UI::EventReturn ConfirmMemstickMoveScreen::OnConfirm(UI::EventParams &params) {
 void ConfirmMemstickMoveScreen::FinishFolderMove() {
 	auto ms = GetI18NCategory(I18NCat::MEMSTICK);
 
+	Path oldMemstickFolder = g_Config.memStickDirectory;
+
 	// Successful so far, switch the memstick folder.
 	if (!SwitchMemstickFolderTo(newMemstickFolder_)) {
 		// TODO: More precise errors.
@@ -791,6 +637,12 @@ void ConfirmMemstickMoveScreen::FinishFolderMove() {
 
 	// If the chosen folder already had a config, reload it!
 	g_Config.Load();
+
+	// If the current browser directory is the old memstick folder, drop it.
+	if (g_Config.currentDirectory == oldMemstickFolder) {
+		g_Config.currentDirectory = g_Config.defaultCurrentDirectory;
+	}
+
 	PostLoadConfig();
 
 	if (!initialSetup_) {
