@@ -28,7 +28,7 @@
 // Background worker threads should be spawned in NativeInit and joined
 // in NativeShutdown.
 
-#include <locale.h>
+#include <clocale>
 #include <algorithm>
 #include <cstdlib>
 #include <memory>
@@ -78,7 +78,7 @@
 #include "Common/File/FileUtil.h"
 #include "Common/TimeUtil.h"
 #include "Common/StringUtils.h"
-#include "Common/LogManager.h"
+#include "Common/Log/LogManager.h"
 #include "Common/MemArena.h"
 #include "Common/GraphicsContext.h"
 #include "Common/OSVersion.h"
@@ -114,9 +114,11 @@
 #include "UI/AudioCommon.h"
 #include "UI/BackgroundAudio.h"
 #include "UI/ControlMappingScreen.h"
+#include "UI/DevScreens.h"
 #include "UI/DiscordIntegration.h"
 #include "UI/EmuScreen.h"
 #include "UI/GameInfoCache.h"
+#include "UI/GameSettingsScreen.h"
 #include "UI/GPUDriverTestScreen.h"
 #include "UI/MiscScreens.h"
 #include "UI/MemStickScreen.h"
@@ -151,9 +153,14 @@
 #include "UI/DarwinFileSystemServices.h"
 #endif
 
+#if !defined(__LIBRETRO__)
+#include "Core/Util/GameDB.h"
+#endif
+
 #include <Core/HLE/Plugins.h>
 
-bool HandleGlobalMessage(const std::string &msg, const std::string &value);
+bool HandleGlobalMessage(UIMessage message, const std::string &value);
+static void ProcessWheelRelease(InputKeyCode keyCode, double now, bool keyPress);
 
 ScreenManager *g_screenManager;
 std::string config_filename;
@@ -167,7 +174,7 @@ static bool restarting = false;
 static int renderCounter = 0;
 
 struct PendingMessage {
-	std::string msg;
+	UIMessage message;
 	std::string value;
 };
 
@@ -177,6 +184,7 @@ static Draw::DrawContext *g_draw;
 static Draw::Pipeline *colorPipeline;
 static Draw::Pipeline *texColorPipeline;
 static UIContext *uiContext;
+static int g_restartGraphics;
 
 #ifdef _WIN32
 WindowsAudioBackend *winAudioBackend;
@@ -212,44 +220,6 @@ public:
 static LogListener *logger = nullptr;
 Path boot_filename;
 
-std::string NativeQueryConfig(std::string query) {
-	char temp[128];
-	if (query == "screenRotation") {
-		INFO_LOG(G3D, "g_Config.screenRotation = %d", g_Config.iScreenRotation);
-		snprintf(temp, sizeof(temp), "%d", g_Config.iScreenRotation);
-		return std::string(temp);
-	} else if (query == "immersiveMode") {
-		return std::string(g_Config.bImmersiveMode ? "1" : "0");
-	} else if (query == "hwScale") {
-		int scale = g_Config.iAndroidHwScale;
-		// Override hw scale for TV type devices.
-		if (System_GetPropertyInt(SYSPROP_DEVICE_TYPE) == DEVICE_TYPE_TV)
-			scale = 0;
-
-		if (scale == 1) {
-			// If g_Config.iInternalResolution is also set to Auto (1), we fall back to "Device resolution" (0). It works out.
-			scale = g_Config.iInternalResolution;
-		} else if (scale >= 2) {
-			scale -= 1;
-		}
-
-		int max_res = std::max(System_GetPropertyInt(SYSPROP_DISPLAY_XRES), System_GetPropertyInt(SYSPROP_DISPLAY_YRES)) / 480 + 1;
-		snprintf(temp, sizeof(temp), "%d", std::min(scale, max_res));
-		return std::string(temp);
-	} else if (query == "sustainedPerformanceMode") {
-		return std::string(g_Config.bSustainedPerformanceMode ? "1" : "0");
-	} else if (query == "androidJavaGL") {
-		// If we're using Vulkan, we say no... need C++ to use Vulkan.
-		if (GetGPUBackend() == GPUBackend::VULKAN) {
-			return "false";
-		}
-		// Otherwise, some devices prefer the Java init so play it safe.
-		return "true";
-	} else {
-		return "";
-	}
-}
-
 int NativeMix(short *audio, int numSamples, int sampleRateHz) {
 	return __AudioMix(audio, numSamples, sampleRateHz);
 }
@@ -314,7 +284,7 @@ void PostLoadConfig() {
 static void CheckFailedGPUBackends() {
 #ifdef _DEBUG
 	// If you're in debug mode, you probably don't want a fallback. If you're in release mode, use IGNORE below.
-	NOTICE_LOG(LOADER, "Not checking for failed graphics backends in debug mode");
+	NOTICE_LOG(Log::Loader, "Not checking for failed graphics backends in debug mode");
 	return;
 #endif
 
@@ -339,7 +309,7 @@ static void CheckFailedGPUBackends() {
 
 	if (System_GetPropertyBool(SYSPROP_SUPPORTS_PERMISSIONS)) {
 		std::string data;
-		if (File::ReadFileToString(true, cache, data))
+		if (File::ReadTextFileToString(cache, &data))
 			g_Config.sFailedGPUBackends = data;
 	}
 
@@ -347,14 +317,14 @@ static void CheckFailedGPUBackends() {
 	if (g_Config.sFailedGPUBackends == "IGNORE")
 		return;
 	else if (!g_Config.sFailedGPUBackends.empty())
-		ERROR_LOG(LOADER, "Failed graphics backends: %s", g_Config.sFailedGPUBackends.c_str());
+		ERROR_LOG(Log::Loader, "Failed graphics backends: %s", g_Config.sFailedGPUBackends.c_str());
 
 	// Okay, let's not try a backend in the failed list.
 	g_Config.iGPUBackend = g_Config.NextValidBackend();
 	if (lastBackend != g_Config.iGPUBackend) {
 		std::string param = GPUBackendToString((GPUBackend)lastBackend) + " -> " + GPUBackendToString((GPUBackend)g_Config.iGPUBackend);
 		System_GraphicsBackendFailedAlert(param);
-		WARN_LOG(LOADER, "Failed graphics backend switched from %s (%d to %d)", param.c_str(), lastBackend, g_Config.iGPUBackend);
+		WARN_LOG(Log::Loader, "Failed graphics backend switched from %s (%d to %d)", param.c_str(), lastBackend, g_Config.iGPUBackend);
 	}
 	// And then let's - for now - add the current to the failed list.
 	if (g_Config.sFailedGPUBackends.empty()) {
@@ -461,24 +431,24 @@ void NativeInit(int argc, const char *argv[], const char *savegame_dir, const ch
 
 	Path memstickDirFile = g_Config.internalDataDirectory / "memstick_dir.txt";
 	if (File::Exists(memstickDirFile)) {
-		INFO_LOG(SYSTEM, "Reading '%s' to find memstick dir.", memstickDirFile.c_str());
+		INFO_LOG(Log::System, "Reading '%s' to find memstick dir.", memstickDirFile.c_str());
 		std::string memstickDir;
-		if (File::ReadFileToString(true, memstickDirFile, memstickDir)) {
+		if (File::ReadTextFileToString(memstickDirFile, &memstickDir)) {
 			Path memstickPath(memstickDir);
 			if (!memstickPath.empty() && File::Exists(memstickPath)) {
 				g_Config.memStickDirectory = memstickPath;
-				INFO_LOG(SYSTEM, "Memstick Directory from memstick_dir.txt: '%s'", g_Config.memStickDirectory.c_str());
+				INFO_LOG(Log::System, "Memstick Directory from memstick_dir.txt: '%s'", g_Config.memStickDirectory.c_str());
 			} else {
-				ERROR_LOG(SYSTEM, "Couldn't read directory '%s' specified by memstick_dir.txt.", memstickDir.c_str());
+				ERROR_LOG(Log::System, "Couldn't read directory '%s' specified by memstick_dir.txt.", memstickDir.c_str());
 				if (System_GetPropertyBool(SYSPROP_ANDROID_SCOPED_STORAGE)) {
 					// Ask the user to configure a memstick directory.
-					INFO_LOG(SYSTEM, "Asking the user.");
+					INFO_LOG(Log::System, "Asking the user.");
 					g_Config.memStickDirectory.clear();
 				}
 			}
 		}
 	} else {
-		INFO_LOG(SYSTEM, "No memstick directory file found (tried to open '%s')", memstickDirFile.c_str());
+		INFO_LOG(Log::System, "No memstick directory file found (tried to open '%s')", memstickDirFile.c_str());
 	}
 
 	// Attempt to create directories after reading the path.
@@ -488,23 +458,23 @@ void NativeInit(int argc, const char *argv[], const char *savegame_dir, const ch
 #elif PPSSPP_PLATFORM(UWP) && !defined(__LIBRETRO__)
 	Path memstickDirFile = g_Config.internalDataDirectory / "memstick_dir.txt";
 	if (File::Exists(memstickDirFile)) {
-		INFO_LOG(SYSTEM, "Reading '%s' to find memstick dir.", memstickDirFile.c_str());
+		INFO_LOG(Log::System, "Reading '%s' to find memstick dir.", memstickDirFile.c_str());
 		std::string memstickDir;
-		if (File::ReadFileToString(true, memstickDirFile, memstickDir)) {
+		if (File::ReadTextFileToString(memstickDirFile, &memstickDir)) {
 			Path memstickPath(memstickDir);
 			if (!memstickPath.empty() && File::Exists(memstickPath)) {
 				g_Config.memStickDirectory = memstickPath;
 				g_Config.SetSearchPath(GetSysDirectory(DIRECTORY_SYSTEM));
 				g_Config.Reload();
-				INFO_LOG(SYSTEM, "Memstick Directory from memstick_dir.txt: '%s'", g_Config.memStickDirectory.c_str());
+				INFO_LOG(Log::System, "Memstick Directory from memstick_dir.txt: '%s'", g_Config.memStickDirectory.c_str());
 			} else {
-				ERROR_LOG(SYSTEM, "Couldn't read directory '%s' specified by memstick_dir.txt.", memstickDir.c_str());
+				ERROR_LOG(Log::System, "Couldn't read directory '%s' specified by memstick_dir.txt.", memstickDir.c_str());
 				g_Config.memStickDirectory.clear();
 			}
 		}
 	}
 	else {
-		INFO_LOG(SYSTEM, "No memstick directory file found (tried to open '%s')", memstickDirFile.c_str());
+		INFO_LOG(Log::System, "No memstick directory file found (tried to open '%s')", memstickDirFile.c_str());
 	}
 #elif PPSSPP_PLATFORM(IOS)
 	g_Config.defaultCurrentDirectory = g_Config.internalDataDirectory;
@@ -559,12 +529,13 @@ void NativeInit(int argc, const char *argv[], const char *savegame_dir, const ch
 
 	LogManager *logman = LogManager::GetInstance();
 
-	const char *fileToLog = 0;
+	const char *fileToLog = nullptr;
 	Path stateToLoad;
 
 	bool gotBootFilename = false;
 	bool gotoGameSettings = false;
 	bool gotoTouchScreenTest = false;
+	bool gotoDeveloperTools = false;
 	boot_filename.clear();
 
 	// Parse command line
@@ -618,10 +589,8 @@ void NativeInit(int argc, const char *argv[], const char *savegame_dir, const ch
 					fileToLog = argv[i] + strlen("--log=");
 				if (!strncmp(argv[i], "--state=", strlen("--state=")) && strlen(argv[i]) > strlen("--state="))
 					stateToLoad = Path(std::string(argv[i] + strlen("--state=")));
-#if !defined(MOBILE_DEVICE)
 				if (!strncmp(argv[i], "--escape-exit", strlen("--escape-exit")))
 					g_Config.bPauseExitsEmulator = true;
-#endif
 				if (!strncmp(argv[i], "--pause-menu-exit", strlen("--pause-menu-exit")))
 					g_Config.bPauseMenuExitsEmulator = true;
 				if (!strcmp(argv[i], "--fullscreen")) {
@@ -636,6 +605,8 @@ void NativeInit(int argc, const char *argv[], const char *savegame_dir, const ch
 					gotoTouchScreenTest = true;
 				if (!strcmp(argv[i], "--gamesettings"))
 					gotoGameSettings = true;
+				if (!strcmp(argv[i], "--developertools"))
+					gotoDeveloperTools = true;
 				if (!strncmp(argv[i], "--appendconfig=", strlen("--appendconfig=")) && strlen(argv[i]) > strlen("--appendconfig=")) {
 					g_Config.SetAppendedConfigIni(Path(std::string(argv[i] + strlen("--appendconfig="))));
 					g_Config.LoadAppendedConfig();
@@ -647,21 +618,21 @@ void NativeInit(int argc, const char *argv[], const char *savegame_dir, const ch
 			// don't already have one.
 			if (!gotBootFilename) {
 				gotBootFilename = true;
-				INFO_LOG(SYSTEM, "Boot filename found in args: '%s'", argv[i]);
+				INFO_LOG(Log::System, "Boot filename found in args: '%s'", argv[i]);
 
 				bool okToLoad = true;
 				bool okToCheck = true;
 				if (System_GetPropertyBool(SYSPROP_SUPPORTS_PERMISSIONS)) {
 					PermissionStatus status = System_GetPermissionStatus(SYSTEM_PERMISSION_STORAGE);
 					if (status == PERMISSION_STATUS_DENIED) {
-						ERROR_LOG(IO, "Storage permission denied. Launching without argument.");
+						ERROR_LOG(Log::IO, "Storage permission denied. Launching without argument.");
 						okToLoad = false;
 						okToCheck = false;
 					} else if (status != PERMISSION_STATUS_GRANTED) {
-						ERROR_LOG(IO, "Storage permission not granted. Launching without argument check.");
+						ERROR_LOG(Log::IO, "Storage permission not granted. Launching without argument check.");
 						okToCheck = false;
 					} else {
-						INFO_LOG(IO, "Storage permission granted.");
+						INFO_LOG(Log::IO, "Storage permission granted.");
 					}
 				}
 				if (okToLoad) {
@@ -669,7 +640,7 @@ void NativeInit(int argc, const char *argv[], const char *savegame_dir, const ch
 					// Handle file:/// URIs, since you get those when creating shortcuts on some Android systems.
 					if (startsWith(str, "file:///")) {
 						str = UriDecode(str.substr(7));
-						INFO_LOG(IO, "Decoding '%s' to '%s'", argv[i], str.c_str());
+						INFO_LOG(Log::IO, "Decoding '%s' to '%s'", argv[i], str.c_str());
 					}
 
 					boot_filename = Path(str);
@@ -688,7 +659,7 @@ void NativeInit(int argc, const char *argv[], const char *savegame_dir, const ch
 					}
 				}
 			} else {
-				fprintf(stderr, "Can only boot one file");
+				fprintf(stderr, "Syntax error: Can only boot one file.\nNote: Many command line args need a =, like --appendconfig=FILENAME.ini.\n");
 #if defined(_WIN32) || defined(__ANDROID__)
 				// Ignore and proceed.
 #else
@@ -758,11 +729,10 @@ void NativeInit(int argc, const char *argv[], const char *savegame_dir, const ch
 	}
 #endif
 
-	// TODO: Load these in the background instead of synchronously.
-	g_BackgroundAudio.SFX().LoadSamples();
+	g_BackgroundAudio.SFX().Init();
 
 	if (!boot_filename.empty() && stateToLoad.Valid()) {
-		SaveState::Load(stateToLoad, -1, [](SaveState::Status status, const std::string &message, void *) {
+		SaveState::Load(stateToLoad, -1, [](SaveState::Status status, std::string_view message, void *) {
 			if (!message.empty() && (!g_Config.bDumpFrames || !g_Config.bDumpVideoOutput)) {
 				g_OSD.Show(status == SaveState::Status::SUCCESS ? OSDType::MESSAGE_SUCCESS : OSDType::MESSAGE_ERROR,
 					message, status == SaveState::Status::SUCCESS ? 2.0 : 5.0);
@@ -778,23 +748,26 @@ void NativeInit(int argc, const char *argv[], const char *savegame_dir, const ch
 		}
 	}
 
-	DEBUG_LOG(SYSTEM, "ScreenManager!");
+	DEBUG_LOG(Log::System, "ScreenManager!");
 	g_screenManager = new ScreenManager();
 	if (g_Config.memStickDirectory.empty()) {
-		INFO_LOG(SYSTEM, "No memstick directory! Asking for one to be configured.");
+		INFO_LOG(Log::System, "No memstick directory! Asking for one to be configured.");
 		g_screenManager->switchScreen(new LogoScreen(AfterLogoScreen::MEMSTICK_SCREEN_INITIAL_SETUP));
 	} else if (gotoGameSettings) {
 		g_screenManager->switchScreen(new LogoScreen(AfterLogoScreen::TO_GAME_SETTINGS));
 	} else if (gotoTouchScreenTest) {
 		g_screenManager->switchScreen(new MainScreen());
 		g_screenManager->push(new TouchTestScreen(Path()));
+	} else if (gotoDeveloperTools) {
+		g_screenManager->switchScreen(new MainScreen());
+		g_screenManager->push(new DeveloperToolsScreen(Path()));
 	} else if (skipLogo) {
 		g_screenManager->switchScreen(new EmuScreen(boot_filename));
 	} else {
 		g_screenManager->switchScreen(new LogoScreen(AfterLogoScreen::DEFAULT));
 	}
 
-	g_screenManager->SetOverlayScreen(new OSDOverlayScreen());
+	g_screenManager->SetBackgroundOverlayScreens(new BackgroundScreen(), new OSDOverlayScreen());
 
 	// Easy testing
 	// screenManager->push(new GPUDriverTestScreen());
@@ -826,7 +799,7 @@ void CallbackPostRender(UIContext *dc, void *userdata);
 bool CreateGlobalPipelines();
 
 bool NativeInitGraphics(GraphicsContext *graphicsContext) {
-	INFO_LOG(SYSTEM, "NativeInitGraphics");
+	INFO_LOG(Log::System, "NativeInitGraphics");
 
 	_assert_msg_(g_screenManager, "No screenmanager, bad init order. Backend = %d", g_Config.iGPUBackend);
 
@@ -839,23 +812,20 @@ bool NativeInitGraphics(GraphicsContext *graphicsContext) {
 	_assert_(g_draw);
 
 	if (!CreateGlobalPipelines()) {
-		ERROR_LOG(G3D, "Failed to create global pipelines");
+		ERROR_LOG(Log::G3D, "Failed to create global pipelines");
 		return false;
 	}
 
 	ui_draw2d.SetAtlas(GetUIAtlas());
 	ui_draw2d.SetFontAtlas(GetFontAtlas());
-	ui_draw2d_front.SetAtlas(GetUIAtlas());
-	ui_draw2d_front.SetFontAtlas(GetFontAtlas());
 
 	uiContext = new UIContext();
 	uiContext->theme = GetTheme();
 	UpdateTheme(uiContext);
 
 	ui_draw2d.Init(g_draw, texColorPipeline);
-	ui_draw2d_front.Init(g_draw, texColorPipeline);
 
-	uiContext->Init(g_draw, texColorPipeline, colorPipeline, &ui_draw2d, &ui_draw2d_front);
+	uiContext->Init(g_draw, texColorPipeline, colorPipeline, &ui_draw2d);
 	if (uiContext->Text())
 		uiContext->Text()->SetFont("Tahoma", 20, 0);
 
@@ -877,7 +847,7 @@ bool NativeInitGraphics(GraphicsContext *graphicsContext) {
 	if (IsWin7OrHigher()) {
 		winCamera = new WindowsCaptureDevice(CAPTUREDEVIDE_TYPE::VIDEO);
 		winCamera->sendMessage({ CAPTUREDEVIDE_COMMAND::INITIALIZE, nullptr });
-		winMic = new WindowsCaptureDevice(CAPTUREDEVIDE_TYPE::AUDIO);
+		winMic = new WindowsCaptureDevice(CAPTUREDEVIDE_TYPE::Audio);
 		winMic->sendMessage({ CAPTUREDEVIDE_COMMAND::INITIALIZE, nullptr });
 	}
 #endif
@@ -890,7 +860,7 @@ bool NativeInitGraphics(GraphicsContext *graphicsContext) {
 		gpu->DeviceRestore(g_draw);
 	}
 
-	INFO_LOG(SYSTEM, "NativeInitGraphics completed");
+	INFO_LOG(Log::System, "NativeInitGraphics completed");
 
 	return true;
 }
@@ -904,7 +874,7 @@ bool CreateGlobalPipelines() {
 	ShaderModule *fs_texture_color_2d = g_draw->GetFshaderPreset(FS_TEXTURE_COLOR_2D);
 
 	if (!vs_color_2d || !fs_color_2d || !vs_texture_color_2d || !fs_texture_color_2d) {
-		ERROR_LOG(G3D, "Failed to get shader preset");
+		ERROR_LOG(Log::G3D, "Failed to get shader preset");
 		return false;
 	}
 
@@ -946,12 +916,14 @@ bool CreateGlobalPipelines() {
 }
 
 void NativeShutdownGraphics() {
-	INFO_LOG(SYSTEM, "NativeShutdownGraphics");
+	INFO_LOG(Log::System, "NativeShutdownGraphics");
 
 	if (g_screenManager) {
 		g_screenManager->deviceLost();
 	}
+	g_iconCache.ClearTextures();
 
+	// TODO: This is not really necessary with Vulkan on Android - could keep shaders etc in memory
 	if (gpu)
 		gpu->DeviceLost();
 
@@ -982,7 +954,6 @@ void NativeShutdownGraphics() {
 	uiContext = nullptr;
 
 	ui_draw2d.Shutdown();
-	ui_draw2d_front.Shutdown();
 
 	if (colorPipeline) {
 		colorPipeline->Release();
@@ -993,10 +964,10 @@ void NativeShutdownGraphics() {
 		texColorPipeline = nullptr;
 	}
 
-	INFO_LOG(SYSTEM, "NativeShutdownGraphics done");
+	INFO_LOG(Log::System, "NativeShutdownGraphics done");
 }
 
-static void TakeScreenshot() {
+static void TakeScreenshot(Draw::DrawContext *draw) {
 	Path path = GetSysDirectory(DIRECTORY_SCREENSHOT);
 	if (!File::Exists(path)) {
 		File::CreateDir(path);
@@ -1019,9 +990,19 @@ static void TakeScreenshot() {
 		i++;
 	}
 
-	bool success = TakeGameScreenshot(filename, g_Config.bScreenshotsAsPNG ? ScreenshotFormat::PNG : ScreenshotFormat::JPG, SCREENSHOT_OUTPUT);
+	bool success = TakeGameScreenshot(draw, filename, g_Config.bScreenshotsAsPNG ? ScreenshotFormat::PNG : ScreenshotFormat::JPG, SCREENSHOT_OUTPUT);
 	if (success) {
-		g_OSD.Show(OSDType::MESSAGE_FILE_LINK, filename.ToString());
+		g_OSD.Show(OSDType::MESSAGE_FILE_LINK, filename.ToString(), 0.0f, "screenshot_link");
+		if (System_GetPropertyBool(SYSPROP_CAN_SHOW_FILE)) {
+			g_OSD.SetClickCallback("screenshot_link", [](bool clicked, void *data) -> void {
+				Path *path = reinterpret_cast<Path *>(data);
+				if (clicked) {
+					System_ShowFileInFolder(*path);
+				} else {
+					delete path;
+				}
+			}, new Path(filename));
+		}
 	} else {
 		auto err = GetI18NCategory(I18NCat::ERRORS);
 		g_OSD.Show(OSDType::MESSAGE_ERROR, err->T("Could not save screenshot file"));
@@ -1030,7 +1011,7 @@ static void TakeScreenshot() {
 
 void CallbackPostRender(UIContext *dc, void *userdata) {
 	if (g_TakeScreenshot) {
-		TakeScreenshot();
+		TakeScreenshot(dc->GetDrawContext());
 		g_TakeScreenshot = false;
 	}
 }
@@ -1067,40 +1048,34 @@ static Matrix4x4 ComputeOrthoMatrix(float xres, float yres) {
 	return ortho;
 }
 
+static void SendMouseDeltaAxis();
+
 void NativeFrame(GraphicsContext *graphicsContext) {
 	PROFILE_END_FRAME();
 
-	bool menuThrottle = (GetUIState() != UISTATE_INGAME || !PSP_IsInited()) && GetUIState() != UISTATE_EXIT;
-
-	double startTime;
-	if (menuThrottle) {
-		startTime = time_now_d();
+	// This can only be accessed from Windows currently, and causes linking errors with headless etc.
+	if (g_restartGraphics == 1) {
+		// Used for debugging only.
+		NativeShutdownGraphics();
+		g_restartGraphics++;
+		return;
+	}
+	else if (g_restartGraphics == 2) {
+		NativeInitGraphics(graphicsContext);
+		g_restartGraphics = 0;
 	}
 
-	std::vector<PendingMessage> toProcess;
-	{
-		std::lock_guard<std::mutex> lock(pendingMutex);
-		toProcess = std::move(pendingMessages);
-		pendingMessages.clear();
-	}
+	double startTime = time_now_d();
 
-	for (const auto &item : toProcess) {
-		if (HandleGlobalMessage(item.msg, item.value)) {
-			INFO_LOG(SYSTEM, "Handled global message: %s / %s", item.msg.c_str(), item.value.c_str());
-		}
-		g_screenManager->sendMessage(item.msg.c_str(), item.value.c_str());
-	}
-
-	g_requestManager.ProcessRequests();
+	ProcessWheelRelease(NKCODE_EXT_MOUSEWHEEL_UP, startTime, false);
+	ProcessWheelRelease(NKCODE_EXT_MOUSEWHEEL_DOWN, startTime, false);
 
 	// it's ok to call this redundantly with DoFrame from EmuScreen
 	Achievements::Idle();
 
 	g_DownloadManager.Update();
-	g_screenManager->update();
 
 	g_Discord.Update();
-	g_BackgroundAudio.Play();
 
 	g_OSD.Update();
 
@@ -1117,6 +1092,28 @@ void NativeFrame(GraphicsContext *graphicsContext) {
 		g_BackgroundAudio.Update();
 	}
 
+	g_iconCache.FrameUpdate();
+
+	g_screenManager->update();
+
+	// Do this after g_screenManager.update() so we can receive setting changes before rendering.
+	std::vector<PendingMessage> toProcess;
+	{
+		std::lock_guard<std::mutex> lock(pendingMutex);
+		toProcess = std::move(pendingMessages);
+		pendingMessages.clear();
+	}
+
+	for (const auto &item : toProcess) {
+		if (HandleGlobalMessage(item.message, item.value)) {
+			// TODO: Add a to-string thingy.
+			INFO_LOG(Log::System, "Handled global message: %d / %s", (int)item.message, item.value.c_str());
+		}
+		g_screenManager->sendMessage(item.message, item.value.c_str());
+	}
+
+	g_requestManager.ProcessRequests();
+
 	// Apply the UIContext bounds as a 2D transformation matrix.
 	// TODO: This should be moved into the draw context...
 	Matrix4x4 ortho = ComputeOrthoMatrix(g_display.dp_xres, g_display.dp_yres);
@@ -1132,23 +1129,22 @@ void NativeFrame(GraphicsContext *graphicsContext) {
 	g_draw->BeginFrame(debugFlags);
 
 	ui_draw2d.PushDrawMatrix(ortho);
-	ui_draw2d_front.PushDrawMatrix(ortho);
 
 	g_screenManager->getUIContext()->SetTintSaturation(g_Config.fUITint, g_Config.fUISaturation);
 
 	// All actual rendering happen in here.
-	g_screenManager->render();
+	ScreenRenderFlags renderFlags = g_screenManager->render();
 	if (g_screenManager->getUIContext()->Text()) {
 		g_screenManager->getUIContext()->Text()->OncePerFrame();
 	}
 
 	ui_draw2d.PopDrawMatrix();
-	ui_draw2d_front.PopDrawMatrix();
 
 	g_draw->EndFrame();
 
 	// This, between EndFrame and Present, is where we should actually wait to do present time management.
 	// There might not be a meaningful distinction here for all backends..
+	g_frameTiming.PostSubmit();
 
 	if (renderCounter < 10 && ++renderCounter == 10) {
 		// We're rendering fine, clear out failure info.
@@ -1160,7 +1156,7 @@ void NativeFrame(GraphicsContext *graphicsContext) {
 	g_draw->Present(presentMode, interval);
 
 	if (resized) {
-		INFO_LOG(G3D, "Resized flag set - recalculating bounds");
+		INFO_LOG(Log::G3D, "Resized flag set - recalculating bounds");
 		resized = false;
 
 		if (uiContext) {
@@ -1184,14 +1180,19 @@ void NativeFrame(GraphicsContext *graphicsContext) {
 #if !PPSSPP_PLATFORM(WINDOWS) && !defined(ANDROID)
 		PSP_CoreParameter().pixelWidth = g_display.pixel_xres;
 		PSP_CoreParameter().pixelHeight = g_display.pixel_yres;
-		System_PostUIMessage("gpu_displayResized", "");
+		System_PostUIMessage(UIMessage::GPU_DISPLAY_RESIZED);
 #endif
 	} else {
-		// INFO_LOG(G3D, "Polling graphics context");
+		// INFO_LOG(Log::G3D, "Polling graphics context");
 		graphicsContext->Poll();
 	}
 
-	if (menuThrottle) {
+	SendMouseDeltaAxis();
+
+	if (!(renderFlags & ScreenRenderFlags::HANDLED_THROTTLING)) {
+		// TODO: We should ideally mix this with game audio.
+		g_BackgroundAudio.Play();
+
 		float refreshRate = System_GetPropertyFloat(SYSPROP_DISPLAY_REFRESH_RATE);
 		// Simple throttling to not burn the GPU in the menu.
 		// TODO: This should move into NativeFrame. Also, it's only necessary in MAILBOX or IMMEDIATE presentation modes.
@@ -1202,34 +1203,37 @@ void NativeFrame(GraphicsContext *graphicsContext) {
 	}
 }
 
-bool HandleGlobalMessage(const std::string &msg, const std::string &value) {
-	if (msg == "savestate_displayslot") {
+bool HandleGlobalMessage(UIMessage message, const std::string &value) {
+	if (message == UIMessage::RESTART_GRAPHICS) {
+		g_restartGraphics = 1;
+		return true;
+	} else if (message == UIMessage::SAVESTATE_DISPLAY_SLOT) {
 		auto sy = GetI18NCategory(I18NCat::SYSTEM);
-		std::string msg = StringFromFormat("%s: %d", sy->T("Savestate Slot"), SaveState::GetCurrentSlot() + 1);
+		std::string msg = StringFromFormat("%s: %d", sy->T_cstr("Savestate Slot"), SaveState::GetCurrentSlot() + 1);
 		// Show for the same duration as the preview.
 		g_OSD.Show(OSDType::MESSAGE_INFO, msg, 2.0f, "savestate_slot");
 		return true;
 	}
-	else if (msg == "gpu_displayResized") {
+	else if (message == UIMessage::GPU_DISPLAY_RESIZED) {
 		if (gpu) {
 			gpu->NotifyDisplayResized();
 		}
 		return true;
 	}
-	else if (msg == "gpu_renderResized") {
+	else if (message == UIMessage::GPU_RENDER_RESIZED) {
 		if (gpu) {
 			gpu->NotifyRenderResized();
 		}
 		return true;
 	}
-	else if (msg == "gpu_configChanged") {
+	else if (message == UIMessage::GPU_CONFIG_CHANGED) {
 		if (gpu) {
 			gpu->NotifyConfigChanged();
 		}
 		Reporting::UpdateConfig();
 		return true;
 	}
-	else if (msg == "core_powerSaving") {
+	else if (message == UIMessage::POWER_SAVING) {
 		if (value != "false") {
 			auto sy = GetI18NCategory(I18NCat::SYSTEM);
 #if PPSSPP_PLATFORM(ANDROID)
@@ -1241,7 +1245,7 @@ bool HandleGlobalMessage(const std::string &msg, const std::string &value) {
 		Core_SetPowerSaving(value != "false");
 		return true;
 	}
-	else if (msg == "permission_granted" && value == "storage") {
+	else if (message == UIMessage::PERMISSION_GRANTED && value == "storage") {
 		CreateSysDirectories();
 		// We must have failed to load the config before, so load it now to avoid overwriting the old config
 		// with a freshly generated one.
@@ -1249,12 +1253,12 @@ bool HandleGlobalMessage(const std::string &msg, const std::string &value) {
 		// and then getting permission), it will get out of sync. So we save and restore g_Config.iGPUBackend.
 		// Ideally we should simply reinitialize graphics to the mode from the config, but there are potential issues.
 		int gpuBackend = g_Config.iGPUBackend;
-		INFO_LOG(IO, "Reloading config after storage permission grant.");
+		INFO_LOG(Log::IO, "Reloading config after storage permission grant.");
 		g_Config.Reload();
 		PostLoadConfig();
 		g_Config.iGPUBackend = gpuBackend;
 		return true;
-	} else if (msg == "app_resumed" || msg == "got_focus") {
+	} else if (message == UIMessage::APP_RESUMED || message == UIMessage::GOT_FOCUS) {
 		// Assume that the user may have modified things.
 		MemoryStick_NotifyWrite();
 		return true;
@@ -1266,16 +1270,16 @@ bool HandleGlobalMessage(const std::string &msg, const std::string &value) {
 bool NativeIsAtTopLevel() {
 	// This might need some synchronization?
 	if (!g_screenManager) {
-		ERROR_LOG(SYSTEM, "No screen manager active");
+		ERROR_LOG(Log::System, "No screen manager active");
 		return false;
 	}
 	Screen *currentScreen = g_screenManager->topScreen();
 	if (currentScreen) {
 		bool top = currentScreen->isTopLevel();
-		INFO_LOG(SYSTEM, "Screen toplevel: %i", (int)top);
+		INFO_LOG(Log::System, "Screen toplevel: %i", (int)top);
 		return currentScreen->isTopLevel();
 	} else {
-		ERROR_LOG(SYSTEM, "No current screen");
+		ERROR_LOG(Log::System, "No current screen");
 		return false;
 	}
 }
@@ -1293,9 +1297,31 @@ void NativeTouch(const TouchInput &touch) {
 	g_screenManager->touch(touch);
 }
 
+// up, down
+static double g_wheelReleaseTime[2]{};
+
+static void ProcessWheelRelease(InputKeyCode keyCode, double now, bool keyPress) {
+	int dir = keyCode - NKCODE_EXT_MOUSEWHEEL_UP;
+	if (g_wheelReleaseTime[dir] != 0.0 && (keyPress || now >= g_wheelReleaseTime[dir])) {
+		g_wheelReleaseTime[dir] = 0.0;
+		KeyInput key{};
+		key.deviceId = DEVICE_ID_MOUSE;
+		key.keyCode = keyCode;
+		key.flags = KEY_UP;
+		NativeKey(key);
+	}
+
+	if (keyPress) {
+		float releaseTime = (float)g_Config.iMouseWheelUpDelayMs * (1.0f / 1000.0f);
+		g_wheelReleaseTime[dir] = now + releaseTime;
+	}
+}
+
 bool NativeKey(const KeyInput &key) {
+	double now = time_now_d();
+
 	// VR actions
-	if (IsVREnabled() && !UpdateVRKeys(key)) {
+	if ((IsVREnabled() || g_Config.bForceVR) && !UpdateVRKeys(key)) {
 		return false;
 	}
 
@@ -1307,7 +1333,7 @@ bool NativeKey(const KeyInput &key) {
 	}
 #endif
 
-	// INFO_LOG(SYSTEM, "Key code: %i flags: %i", key.keyCode, key.flags);
+	// INFO_LOG(Log::System, "Key code: %i flags: %i", key.keyCode, key.flags);
 #if !defined(MOBILE_DEVICE)
 	if (g_Config.bPauseExitsEmulator) {
 		std::vector<int> pspKeys;
@@ -1321,12 +1347,19 @@ bool NativeKey(const KeyInput &key) {
 	}
 #endif
 
-	bool retval = false;
-	if (g_screenManager) {
-		HLEPlugins::SetKey(key.keyCode, (key.flags & KEY_DOWN) ? 1 : 0);
-		retval = g_screenManager->key(key);
+	if (!g_screenManager) {
+		return false;
 	}
 
+	// Handle releases of mousewheel keys.
+	if ((key.flags & KEY_DOWN) && key.deviceId == DEVICE_ID_MOUSE && (key.keyCode == NKCODE_EXT_MOUSEWHEEL_UP || key.keyCode == NKCODE_EXT_MOUSEWHEEL_DOWN)) {
+		ProcessWheelRelease(key.keyCode, now, true);
+	}
+
+	HLEPlugins::SetKey(key.keyCode, (key.flags & KEY_DOWN) ? 1 : 0);
+	// Dispatch the key event.
+	bool retval = g_screenManager->key(key);
+
 	// The Mode key can have weird consequences on some devices, see #17245.
 	if (key.keyCode == NKCODE_BUTTON_MODE) {
 		// Tell the caller that we handled the key.
@@ -1336,9 +1369,9 @@ bool NativeKey(const KeyInput &key) {
 	return retval;
 }
 
-static void ProcessOneAxisEvent(const AxisInput &axis) {
+void NativeAxis(const AxisInput *axes, size_t count) {
 	// VR actions
-	if (IsVREnabled() && !UpdateVRAxis(axis)) {
+	if ((IsVREnabled() || g_Config.bForceVR) && !UpdateVRAxis(axes, count)) {
 		return;
 	}
 
@@ -1347,17 +1380,48 @@ static void ProcessOneAxisEvent(const AxisInput &axis) {
 		return;
 	}
 
-	// only do special handling of tilt events if tilt is enabled.
-	HLEPlugins::PluginDataAxis[axis.axisId] = axis.value;
-	g_screenManager->axis(axis);
+	g_screenManager->axis(axes, count);
+
+	for (size_t i = 0; i < count; i++) {
+		const AxisInput &axis = axes[i];
+		HLEPlugins::PluginDataAxis[axis.axisId] = axis.value;
+	}
 }
 
-void NativeAxis(const AxisInput *axes, size_t count) {
-	for (size_t i = 0; i < count; i++) {
-		ProcessOneAxisEvent(axes[i]);
+// Called from NativeFrame and from NativeMouseDelta.
+static void SendMouseDeltaAxis() {
+	float mx, my;
+	MouseEventProcessor::MouseDeltaToAxes(time_now_d(), &mx, &my);
+
+	AxisInput axis[2];
+	axis[0].axisId = JOYSTICK_AXIS_MOUSE_REL_X;
+	axis[0].deviceId = DEVICE_ID_MOUSE;
+	axis[0].value = mx;
+	axis[1].axisId = JOYSTICK_AXIS_MOUSE_REL_Y;
+	axis[1].deviceId = DEVICE_ID_MOUSE;
+	axis[1].value = my;
+
+	HLEPlugins::PluginDataAxis[JOYSTICK_AXIS_MOUSE_REL_X] = mx;
+	HLEPlugins::PluginDataAxis[JOYSTICK_AXIS_MOUSE_REL_Y] = my;
+
+	//NOTICE_LOG(Log::System, "delta: %0.2f %0.2f    mx/my: %0.2f %0.2f   dpi: %f  sens: %f ",
+	//	g_mouseDeltaX, g_mouseDeltaY, mx, my, g_display.dpi_scale_x, g_Config.fMouseSensitivity);
+
+	if (GetUIState() == UISTATE_INGAME || g_Config.bMapMouse) {
+		NativeAxis(axis, 2);
 	}
 }
 
+void NativeMouseDelta(float dx, float dy) {
+	// Remap, shared code. Then send it as a regular axis event.
+	if (!g_Config.bMouseControl)
+		return;
+
+	MouseEventProcessor::ProcessDelta(time_now_d(), dx, dy);
+
+	SendMouseDeltaAxis();
+}
+
 void NativeAccelerometer(float tiltX, float tiltY, float tiltZ) {
 	if (g_Config.iTiltInputType == TILT_NULL) {
 		// if tilt events are disabled, don't do anything special.
@@ -1390,17 +1454,17 @@ void NativeAccelerometer(float tiltX, float tiltY, float tiltZ) {
 	HLEPlugins::PluginDataAxis[JOYSTICK_AXIS_ACCELEROMETER_Z] = tiltZ;
 }
 
-void System_PostUIMessage(const std::string &message, const std::string &value) {
+void System_PostUIMessage(UIMessage message, const std::string &value) {
 	std::lock_guard<std::mutex> lock(pendingMutex);
 	PendingMessage pendingMessage;
-	pendingMessage.msg = message;
+	pendingMessage.message = message;
 	pendingMessage.value = value;
 	pendingMessages.push_back(pendingMessage);
 }
 
 void NativeResized() {
 	// NativeResized can come from any thread so we just set a flag, then process it later.
-	VERBOSE_LOG(G3D, "NativeResized - setting flag");
+	VERBOSE_LOG(Log::G3D, "NativeResized - setting flag");
 	resized = true;
 }
 
@@ -1431,13 +1495,13 @@ void NativeShutdown() {
 
 	g_Config.Save("NativeShutdown");
 
-	INFO_LOG(SYSTEM, "NativeShutdown called");
+	INFO_LOG(Log::System, "NativeShutdown called");
 
 	g_i18nrepo.LogMissingKeys();
 
 	ShutdownWebServer();
 
-#if PPSSPP_PLATFORM(ANDROID) || PPSSPP_PLATFORM(IOS)
+#if PPSSPP_PLATFORM(ANDROID)
 	System_ExitApp();
 #endif
 
@@ -1471,25 +1535,27 @@ void NativeShutdown() {
 // In the future, we might make this more sophisticated, such as storing in the app private directory on Android.
 // Right now we just store secrets in separate files next to ppsspp.ini. The important thing is keeping them out of it
 // since we often ask people to post or send the ini for debugging.
-static Path GetSecretPath(const char *nameOfSecret) {
+static Path GetSecretPath(std::string_view nameOfSecret) {
 	return GetSysDirectory(DIRECTORY_SYSTEM) / ("ppsspp_" + std::string(nameOfSecret) + ".dat");
 }
 
 // name should be simple alphanumerics to avoid problems on Windows.
-bool NativeSaveSecret(const char *nameOfSecret, const std::string &data) {
+bool NativeSaveSecret(std::string_view nameOfSecret, std::string_view data) {
 	Path path = GetSecretPath(nameOfSecret);
-	if (!File::WriteDataToFile(false, data.data(), (unsigned int)data.size(), path)) {
-		WARN_LOG(SYSTEM, "Failed to write secret '%s' to path '%s'", nameOfSecret, path.c_str());
+	if (data.empty() && File::Exists(path)) {
+		return File::Delete(path);
+	} else if (!File::WriteDataToFile(false, data.data(), data.size(), path)) {
+		WARN_LOG(Log::System, "Failed to write secret '%.*s' to path '%s'", (int)nameOfSecret.size(), nameOfSecret.data(), path.c_str());
 		return false;
 	}
 	return true;
 }
 
 // On failure, returns an empty string. Good enough since any real secret is non-empty.
-std::string NativeLoadSecret(const char *nameOfSecret) {
+std::string NativeLoadSecret(std::string_view nameOfSecret) {
 	Path path = GetSecretPath(nameOfSecret);
 	std::string data;
-	if (!File::ReadFileToString(false, path, data)) {
+	if (!File::ReadBinaryFileToString(path, &data)) {
 		data.clear();  // just to be sure.
 	}
 	return data;
