@@ -28,12 +28,13 @@
 	NSURL *firstURL = urls.firstObject;
 	if (!firstURL) return; // No URLs, don't do anything
 	
-	System_PostUIMessage("boot", firstURL.fileSystemRepresentation);
+	System_PostUIMessage(UIMessage::REQUEST_GAME_BOOT, firstURL.fileSystemRepresentation);
 }
 
 - (NSMenu *)applicationDockMenu:(NSApplication *)sender {
+	// TODO: Actually implement a dock menu thingy.
 	for (std::string iso : g_Config.RecentIsos()) {
-		printf("%s\n", iso.c_str());
+		// printf("%s\n", iso.c_str());
 	}
 	
 	return nil;
