@@ -32,9 +32,11 @@
 #include "Common/VR/PPSSPPVR.h"
 #include "Common/UI/AsyncImageFileView.h"
 
+#include "Core/KeyMap.h"
 #include "Core/Reporting.h"
 #include "Core/SaveState.h"
 #include "Core/System.h"
+#include "Core/Core.h"
 #include "Core/Config.h"
 #include "Core/RetroAchievements.h"
 #include "Core/ELF/ParamSFO.h"
@@ -49,12 +51,14 @@
 #include "UI/ReportScreen.h"
 #include "UI/CwCheatScreen.h"
 #include "UI/MainScreen.h"
+#include "UI/GameScreen.h"
 #include "UI/OnScreenDisplay.h"
 #include "UI/GameInfoCache.h"
 #include "UI/DisplayLayoutScreen.h"
 #include "UI/RetroAchievementScreens.h"
+#include "UI/BackgroundAudio.h"
 
-static void AfterSaveStateAction(SaveState::Status status, const std::string &message, void *) {
+static void AfterSaveStateAction(SaveState::Status status, std::string_view message, void *) {
 	if (!message.empty() && (!g_Config.bDumpFrames || !g_Config.bDumpVideoOutput)) {
 		g_OSD.Show(status == SaveState::Status::SUCCESS ? OSDType::MESSAGE_SUCCESS : OSDType::MESSAGE_ERROR,
 			message, status == SaveState::Status::SUCCESS ? 2.0 : 5.0);
@@ -200,8 +204,10 @@ SaveSlotView::SaveSlotView(const Path &gameFilename, int slot, bool vertical, UI
 	fv->OnClick.Handle(this, &SaveSlotView::OnScreenshotClick);
 
 	if (SaveState::HasSaveInSlot(gamePath_, slot)) {
-		loadStateButton_ = buttons->Add(new Button(pa->T("Load State"), new LinearLayoutParams(0.0, G_VCENTER)));
-		loadStateButton_->OnClick.Handle(this, &SaveSlotView::OnLoadState);
+		if (!Achievements::HardcoreModeActive()) {
+			loadStateButton_ = buttons->Add(new Button(pa->T("Load State"), new LinearLayoutParams(0.0, G_VCENTER)));
+			loadStateButton_->OnClick.Handle(this, &SaveSlotView::OnLoadState);
+		}
 
 		std::string dateStr = SaveState::GetSlotDateAsString(gamePath_, slot_);
 		if (!dateStr.empty()) {
@@ -254,7 +260,7 @@ void GamePauseScreen::update() {
 	UIScreen::update();
 
 	if (finishNextFrame_) {
-		TriggerFinish(DR_CANCEL);
+		TriggerFinish(finishNextFrameResult_);
 		finishNextFrame_ = false;
 	}
 
@@ -263,12 +269,33 @@ void GamePauseScreen::update() {
 
 GamePauseScreen::GamePauseScreen(const Path &filename)
 	: UIDialogScreenWithGameBackground(filename) {
+	// So we can tell if something blew up while on the pause screen.
+	std::string assertStr = "PauseScreen: " + filename.GetFilename();
+	SetExtraAssertInfo(assertStr.c_str());
 }
 
 GamePauseScreen::~GamePauseScreen() {
 	__DisplaySetWasPaused();
 }
 
+bool GamePauseScreen::key(const KeyInput &key) {
+	if (!UIScreen::key(key) && (key.flags & KEY_DOWN)) {
+		// Special case to be able to unpause with a bound pause key.
+		// Normally we can't bind keys used in the UI.
+		InputMapping mapping(key.deviceId, key.keyCode);
+		std::vector<int> pspButtons;
+		KeyMap::InputMappingToPspButton(mapping, &pspButtons);
+		for (auto button : pspButtons) {
+			if (button == VIRTKEY_PAUSE) {
+				TriggerFinish(DR_CANCEL);
+				return true;
+			}
+		}
+		return false;
+	}
+	return false;
+}
+
 void GamePauseScreen::CreateSavestateControls(UI::LinearLayout *leftColumnItems, bool vertical) {
 	auto pa = GetI18NCategory(I18NCat::PAUSE);
 
@@ -286,7 +313,7 @@ void GamePauseScreen::CreateSavestateControls(UI::LinearLayout *leftColumnItems,
 	leftColumnItems->Add(new Spacer(0.0));
 
 	LinearLayout *buttonRow = leftColumnItems->Add(new LinearLayout(ORIENT_HORIZONTAL));
-	if (g_Config.bEnableStateUndo) {
+	if (g_Config.bEnableStateUndo && !Achievements::HardcoreModeActive()) {
 		UI::Choice *loadUndoButton = buttonRow->Add(new Choice(pa->T("Undo last load")));
 		loadUndoButton->SetEnabled(SaveState::HasUndoLoad(gamePath_));
 		loadUndoButton->OnClick.Handle(this, &GamePauseScreen::OnLoadUndo);
@@ -296,7 +323,7 @@ void GamePauseScreen::CreateSavestateControls(UI::LinearLayout *leftColumnItems,
 		saveUndoButton->OnClick.Handle(this, &GamePauseScreen::OnLastSaveUndo);
 	}
 
-	if (g_Config.iRewindSnapshotInterval > 0) {
+	if (g_Config.iRewindSnapshotInterval > 0 && !Achievements::HardcoreModeActive()) {
 		UI::Choice *rewindButton = buttonRow->Add(new Choice(pa->T("Rewind")));
 		rewindButton->SetEnabled(SaveState::CanRewind());
 		rewindButton->OnClick.Handle(this, &GamePauseScreen::OnRewind);
@@ -322,13 +349,18 @@ void GamePauseScreen::CreateViews() {
 	LinearLayout *leftColumnItems = new LinearLayoutList(ORIENT_VERTICAL, new LayoutParams(FILL_PARENT, WRAP_CONTENT));
 	leftColumn->Add(leftColumnItems);
 
-	leftColumnItems->Add(new Spacer(0.0));
+	leftColumnItems->SetSpacing(5.0f);
 	if (Achievements::IsActive()) {
 		leftColumnItems->Add(new GameAchievementSummaryView());
-		leftColumnItems->Add(new Spacer(5.0));
+
+		char buf[512];
+		size_t sz = Achievements::GetRichPresenceMessage(buf, sizeof(buf));
+		if (sz != (size_t)-1) {
+			leftColumnItems->Add(new TextView(std::string_view(buf, sz), new UI::LinearLayoutParams(Margins(5, 5))))->SetSmall(true);
+		}
 	}
 
-	if (!Achievements::ChallengeModeActive()) {
+	if (!Achievements::HardcoreModeActive() || g_Config.bAchievementsSaveStateInHardcoreMode) {
 		CreateSavestateControls(leftColumnItems, vertical);
 	} else {
 		// Let's show the active challenges.
@@ -342,10 +374,21 @@ void GamePauseScreen::CreateViews() {
 				leftColumnItems->Add(new AchievementView(achievement));
 			}
 		}
+
+		// And tack on an explanation for why savestate options are not available.
+		std::string_view notAvailable = ac->T("Save states not available in Hardcore Mode");
+		leftColumnItems->Add(new NoticeView(NoticeLevel::INFO, notAvailable, ""));
 	}
 
-	ViewGroup *rightColumn = new ScrollView(ORIENT_VERTICAL, new LinearLayoutParams(vertical ? 200 : 300, FILL_PARENT, actionMenuMargins));
-	root_->Add(rightColumn);
+	LinearLayout *middleColumn = new LinearLayout(ORIENT_VERTICAL, new LinearLayoutParams(64, FILL_PARENT, Margins(0, 10, 0, 15)));
+	root_->Add(middleColumn);
+	middleColumn->SetSpacing(0.0f);
+	ViewGroup *rightColumnHolder = new LinearLayout(ORIENT_VERTICAL, new LinearLayoutParams(vertical ? 200 : 300, FILL_PARENT, actionMenuMargins));
+
+	ViewGroup *rightColumn = new ScrollView(ORIENT_VERTICAL, new LinearLayoutParams(1.0f));
+	rightColumnHolder->Add(rightColumn);
+
+	root_->Add(rightColumnHolder);
 
 	LinearLayout *rightColumnItems = new LinearLayout(ORIENT_VERTICAL);
 	rightColumn->Add(rightColumnItems);
@@ -361,7 +404,7 @@ void GamePauseScreen::CreateViews() {
 	root_->SetDefaultFocusView(continueChoice);
 	continueChoice->OnClick.Handle<UIScreen>(this, &UIScreen::OnBack);
 
-	rightColumnItems->Add(new Spacer(25.0));
+	rightColumnItems->Add(new Spacer(20.0));
 
 	std::string gameId = g_paramSFO.GetDiscID();
 	if (g_Config.hasGameConfig(gameId)) {
@@ -371,7 +414,7 @@ void GamePauseScreen::CreateViews() {
 		rightColumnItems->Add(new Choice(pa->T("Settings")))->OnClick.Handle(this, &GamePauseScreen::OnGameSettings);
 		rightColumnItems->Add(new Choice(pa->T("Create Game Config")))->OnClick.Handle(this, &GamePauseScreen::OnCreateConfig);
 	}
-	UI::Choice *displayEditor_ = rightColumnItems->Add(new Choice(gr->T("Display Layout && Effects")));
+	UI::Choice *displayEditor_ = rightColumnItems->Add(new Choice(gr->T("Display layout & effects")));
 	displayEditor_->OnClick.Add([&](UI::EventParams &) -> UI::EventReturn {
 		screenManager()->push(new DisplayLayoutScreen(gamePath_));
 		return UI::EVENT_DONE;
@@ -395,13 +438,32 @@ void GamePauseScreen::CreateViews() {
 		auto rp = GetI18NCategory(I18NCat::REPORTING);
 		rightColumnItems->Add(new Choice(rp->T("ReportButton", "Report Feedback")))->OnClick.Handle(this, &GamePauseScreen::OnReportFeedback);
 	}
-	rightColumnItems->Add(new Spacer(25.0));
+	rightColumnItems->Add(new Spacer(20.0));
 	if (g_Config.bPauseMenuExitsEmulator) {
 		auto mm = GetI18NCategory(I18NCat::MAINMENU);
 		rightColumnItems->Add(new Choice(mm->T("Exit")))->OnClick.Handle(this, &GamePauseScreen::OnExitToMenu);
 	} else {
 		rightColumnItems->Add(new Choice(pa->T("Exit to menu")))->OnClick.Handle(this, &GamePauseScreen::OnExitToMenu);
 	}
+
+	if (!Core_MustRunBehind()) {
+		playButton_ = middleColumn->Add(new Button("", g_Config.bRunBehindPauseMenu ? ImageID("I_PAUSE") : ImageID("I_PLAY"), new LinearLayoutParams(64, 64)));
+		playButton_->OnClick.Add([=](UI::EventParams &e) {
+			g_Config.bRunBehindPauseMenu = !g_Config.bRunBehindPauseMenu;
+			playButton_->SetImageID(g_Config.bRunBehindPauseMenu ? ImageID("I_PAUSE") : ImageID("I_PLAY"));
+			return UI::EVENT_DONE;
+		});
+		middleColumn->Add(new Spacer(20.0));
+		Button *infoButton = middleColumn->Add(new Button("", ImageID("I_INFO"), new LinearLayoutParams(64, 64)));
+		infoButton->OnClick.Add([=](UI::EventParams &e) {
+			screenManager()->push(new GameScreen(gamePath_, true));
+			return UI::EVENT_DONE;
+		});
+	} else {
+		auto nw = GetI18NCategory(I18NCat::NETWORKING);
+		rightColumnHolder->Add(new TextView(nw->T("Network connected")));
+	}
+	rightColumnHolder->Add(new Spacer(10.0f));
 }
 
 UI::EventReturn GamePauseScreen::OnGameSettings(UI::EventParams &e) {
@@ -419,6 +481,9 @@ void GamePauseScreen::dialogFinished(const Screen *dialog, DialogResult dr) {
 	if (tag == "ScreenshotView" && dr == DR_OK) {
 		finishNextFrame_ = true;
 	} else {
+		if (tag == "Game") {
+			g_BackgroundAudio.SetGame(Path());
+		}
 		// There may have been changes to our savestates, so let's recreate.
 		RecreateViews();
 	}
@@ -438,10 +503,26 @@ UI::EventReturn GamePauseScreen::OnScreenshotClicked(UI::EventParams &e) {
 }
 
 UI::EventReturn GamePauseScreen::OnExitToMenu(UI::EventParams &e) {
-	if (g_Config.bPauseMenuExitsEmulator) {
-		System_ExitApp();
+	// If RAIntegration has dirty info, ask for confirmation.
+	if (Achievements::RAIntegrationDirty()) {
+		auto ac = GetI18NCategory(I18NCat::ACHIEVEMENTS);
+		auto di = GetI18NCategory(I18NCat::DIALOG);
+		screenManager()->push(new PromptScreen(gamePath_, ac->T("You have unsaved RAIntegration changes. Exit?"), di->T("Yes"), di->T("No"), [=](bool result) {
+			if (result) {
+				if (g_Config.bPauseMenuExitsEmulator) {
+					System_ExitApp();
+				} else {
+					finishNextFrameResult_ = DR_OK;  // exit game
+					finishNextFrame_ = true;
+				}
+			}
+		}));
 	} else {
-		TriggerFinish(DR_OK);
+		if (g_Config.bPauseMenuExitsEmulator) {
+			System_ExitApp();
+		} else {
+			TriggerFinish(DR_OK);
+		}
 	}
 	return UI::EVENT_DONE;
 }
@@ -475,26 +556,29 @@ UI::EventReturn GamePauseScreen::OnLastSaveUndo(UI::EventParams &e) {
 void GamePauseScreen::CallbackDeleteConfig(bool yes)
 {
 	if (yes) {
-		std::shared_ptr<GameInfo> info = g_gameInfoCache->GetInfo(NULL, gamePath_, 0);
-		g_Config.unloadGameConfig();
-		g_Config.deleteGameConfig(info->id);
-		info->hasConfig = false;
-		screenManager()->RecreateAllViews();
+		std::shared_ptr<GameInfo> info = g_gameInfoCache->GetInfo(NULL, gamePath_, GameInfoFlags::PARAM_SFO);
+		if (info->Ready(GameInfoFlags::PARAM_SFO)) {
+			g_Config.unloadGameConfig();
+			g_Config.deleteGameConfig(info->id);
+			info->hasConfig = false;
+			screenManager()->RecreateAllViews();
+		}
 	}
 }
 
 UI::EventReturn GamePauseScreen::OnCreateConfig(UI::EventParams &e)
 {
-	std::shared_ptr<GameInfo> info = g_gameInfoCache->GetInfo(NULL, gamePath_, 0);
-	std::string gameId = g_paramSFO.GetDiscID();
-	g_Config.createGameConfig(gameId);
-	g_Config.changeGameSpecific(gameId, info->GetTitle());
-	g_Config.saveGameConfig(gameId, info->GetTitle());
-	if (info) {
-		info->hasConfig = true;
+	std::shared_ptr<GameInfo> info = g_gameInfoCache->GetInfo(NULL, gamePath_, GameInfoFlags::PARAM_SFO);
+	if (info->Ready(GameInfoFlags::PARAM_SFO)) {
+		std::string gameId = g_paramSFO.GetDiscID();
+		g_Config.createGameConfig(gameId);
+		g_Config.changeGameSpecific(gameId, info->GetTitle());
+		g_Config.saveGameConfig(gameId, info->GetTitle());
+		if (info) {
+			info->hasConfig = true;
+		}
+		screenManager()->topScreen()->RecreateViews();
 	}
-
-	screenManager()->topScreen()->RecreateViews();
 	return UI::EVENT_DONE;
 }
 
