@@ -24,11 +24,7 @@
 #include "Common/UI/UIScreen.h"
 #include "Common/UI/ViewGroup.h"
 #include "UI/MiscScreens.h"
-
-enum class PauseScreenMode {
-	MAIN,
-	DISPLAY_SETTINGS,
-};
+#include "UI/Screen.h"
 
 class GamePauseScreen : public UIDialogScreenWithGameBackground {
 public:
@@ -36,6 +32,7 @@ public:
 	~GamePauseScreen();
 
 	void dialogFinished(const Screen *dialog, DialogResult dr) override;
+	bool key(const KeyInput &key) override;
 
 	const char *tag() const override { return "GamePause"; }
 
@@ -64,5 +61,7 @@ private:
 
 	// hack
 	bool finishNextFrame_ = false;
-	PauseScreenMode mode_ = PauseScreenMode::MAIN;
+	DialogResult finishNextFrameResult_ = DR_CANCEL;
+
+	UI::Button *playButton_ = nullptr;
 };
