@@ -3,6 +3,7 @@
 #include "ppsspp_config.h"
 
 #include "Common/UI/UIScreen.h"
+#include "Common/System/System.h"
 #include "Core/ConfigValues.h"
 #include "UI/MiscScreens.h"
 
@@ -10,7 +11,7 @@ class TabbedUIDialogScreenWithGameBackground : public UIDialogScreenWithGameBack
 public:
 	TabbedUIDialogScreenWithGameBackground(const Path &gamePath) : UIDialogScreenWithGameBackground(gamePath) {}
 
-	UI::LinearLayout *AddTab(const char *tag, const std::string &title, bool isSearch = false);
+	UI::LinearLayout *AddTab(const char *tag, std::string_view title, bool isSearch = false);
 	void CreateViews() override;
 
 
@@ -21,7 +22,7 @@ protected:
 	virtual bool ShowSearchControls() const { return true; }
 
 	void RecreateViews() override;
-	void sendMessage(const char *message, const char *value) override;
+	void sendMessage(UIMessage message, const char *value) override;
 
 	SettingInfoMessage *settingInfo_ = nullptr;
 
