@@ -39,3 +39,6 @@ private:
 	JoystickHistoryView *tilt_ = nullptr;
 	GamepadView *gpView_ = nullptr;
 };
+
+extern const char *g_tiltTypes[];
+extern const size_t g_numTiltTypes;
