@@ -15,7 +15,7 @@
 #include "Common/Input/InputState.h"
 #include "Common/System/NativeApp.h"
 #include "Common/System/System.h"
-#include "Common/LogManager.h"
+#include "Common/Log/LogManager.h"
 #include "Core/System.h"
 #include "Core/Config.h"
 #include "Core/Core.h"
@@ -331,7 +331,7 @@ void App::OnWindowSizeChanged(CoreWindow^ sender, WindowSizeChangedEventArgs^ ar
 	PSP_CoreParameter().pixelWidth = (int)(width * scale);
 	PSP_CoreParameter().pixelHeight = (int)(height * scale);
 	if (UpdateScreenScale((int)width, (int)height)) {
-		System_PostUIMessage("gpu_displayResized", "");
+		System_PostUIMessage(UIMessage::GPU_DISPLAY_RESIZED);
 	}
 }
 
