@@ -4,19 +4,18 @@
 
 #include "WindowsAudio.h"
 #include <mmreg.h>
-#include <dsound.h>
+
+struct IDirectSound8;
+struct IDirectSoundBuffer;
 
 class DSoundAudioBackend : public WindowsAudioBackend {
 public:
-	DSoundAudioBackend();
-	~DSoundAudioBackend() override;
+	~DSoundAudioBackend();
 
 	bool Init(HWND window, StreamCallback callback, int sampleRate) override;  // If fails, can safely delete the object
-	void Update() override;
-	int GetSampleRate() override { return sampleRate_; }
+	int GetSampleRate() const override { return sampleRate_; }
 
 private:
-	inline int ModBufferSize(int x) { return (x + bufferSize_) % bufferSize_; }
 	int RunThread();
 	static unsigned int WINAPI soundThread(void *param);
 	bool CreateBuffer();
@@ -25,8 +24,7 @@ private:
 		DWORD soundBytes); // Size of block to copy.
 
 	CRITICAL_SECTION soundCriticalSection;
-	HWND window_;
-	HANDLE soundSyncEvent_ = nullptr;
+	HWND window_ = nullptr;
 	HANDLE hThread_ = nullptr;
 
 	StreamCallback callback_;
@@ -34,18 +32,18 @@ private:
 	IDirectSound8 *ds_ = nullptr;
 	IDirectSoundBuffer *dsBuffer_ = nullptr;
 
-	int bufferSize_; // bytes
-	int totalRenderedBytes_;
-	int sampleRate_;
+	int bufferSize_ = 0; // bytes
+	int totalRenderedBytes_ = 0;
+	int sampleRate_ = 0;
 
-	volatile int threadData_;
+	volatile int threadData_ = 0;
 
 	enum {
 		BUFSIZE = 0x4000,
 		MAXWAIT = 20,   //ms
 	};
 
-	int currentPos_;
-	int lastPos_;
+	int currentPos_ = 0;
+	int lastPos_ = 0;
 	short realtimeBuffer_[BUFSIZE * 2];
 };
