@@ -277,24 +277,25 @@ std::string trimString(std::string input)
 	return input;
 }
 
-void CtrlDisAsmView::assembleOpcode(u32 address, std::string defaultText)
+void CtrlDisAsmView::assembleOpcode(u32 address, const std::string &defaultText)
 {
 	auto memLock = Memory::Lock();
-	if (Core_IsStepping() == false) {
+	if (!Core_IsStepping()) {
 		MessageBox(wnd,L"Cannot change code while the core is running!",L"Error",MB_OK);
 		return;
 	}
 	std::string op;
-	bool result = InputBox_GetString(MainWindow::GetHInstance(),wnd,L"Assemble opcode",defaultText, op, false);
-	if (!result)
+	bool result = InputBox_GetString(MainWindow::GetHInstance(), wnd, L"Assemble opcode", defaultText, op, InputBoxFlags::Default);
+	if (!result) {
 		return;
+	}
 
 	// check if it changes registers first
-	auto seperator = op.find('=');
-	if (seperator != std::string::npos)
+	auto separator = op.find('=');
+	if (separator != std::string::npos)
 	{
-		std::string registerName = trimString(op.substr(0,seperator));
-		std::string expression = trimString(op.substr(seperator+1));
+		std::string registerName = trimString(op.substr(0,separator));
+		std::string expression = trimString(op.substr(separator+1));
 
 		u32 value;
 		if (parseExpression(expression.c_str(),debugger,value) == true)
@@ -434,12 +435,12 @@ std::set<std::string> CtrlDisAsmView::getSelectedLineArguments() {
 		manager.getLine(addr, displaySymbols, line);
 		size_t p = 0, nextp = line.params.find(',');
 		while (nextp != line.params.npos) {
-			args.insert(line.params.substr(p, nextp - p));
+			args.emplace(line.params.substr(p, nextp - p));
 			p = nextp + 1;
 			nextp = line.params.find(',', p);
 		}
 		if (p < line.params.size()) {
-			args.insert(line.params.substr(p));
+			args.emplace(line.params.substr(p));
 		}
 	}
 
@@ -954,6 +955,12 @@ void CtrlDisAsmView::onMouseUp(WPARAM wParam, LPARAM lParam, int button)
 	}
 	else if (button == 2)
 	{
+		// We don't want to let the users play with deallocated or uninitialized debugging objects
+		GlobalUIState state = GetUIState();
+		if (state != UISTATE_INGAME && state != UISTATE_PAUSEMENU) {
+			return;
+		}
+		
 		switch (TriggerContextMenu(ContextMenuID::DISASM, wnd, ContextPoint::FromEvent(lParam)))
 		{
 		case ID_DISASM_GOTOINMEMORYVIEW:
