@@ -67,7 +67,7 @@ class CtrlDisAsmView
 		ADDRESSES,
 	};
 
-	void assembleOpcode(u32 address, std::string defaultText);
+	void assembleOpcode(u32 address, const std::string &defaultText);
 	std::string disassembleRange(u32 start, u32 size);
 	void disassembleToFile();
 	void search(bool continueSearch);
