@@ -77,8 +77,7 @@ LRESULT CALLBACK CtrlDisplayListView::wndProc(HWND hwnd, UINT msg, WPARAM wParam
 {
 	CtrlDisplayListView *win = CtrlDisplayListView::getFrom(hwnd);
 
-	switch(msg)
-	{
+	switch(msg) {
 	case WM_NCCREATE:
 		// Allocate a new CustCtrl structure for this window.
 		win = new CtrlDisplayListView(hwnd);
@@ -86,6 +85,7 @@ LRESULT CALLBACK CtrlDisplayListView::wndProc(HWND hwnd, UINT msg, WPARAM wParam
 		// Continue with window creation.
 		return win != NULL;
 	case WM_NCDESTROY:
+		SetWindowLongPtr(hwnd, GWLP_USERDATA, 0);
 		delete win;
 		break;
 	case WM_SIZE:
@@ -280,6 +280,10 @@ void CtrlDisplayListView::PromptBreakpointCond() {
 
 void CtrlDisplayListView::onMouseDown(WPARAM wParam, LPARAM lParam, int button)
 {
+	if (!validDisplayList || !gpuDebug) {
+		return;
+	}
+
 	int y = HIWORD(lParam);
 
 	int line = y/rowHeight;
@@ -307,6 +311,10 @@ void CtrlDisplayListView::onMouseDown(WPARAM wParam, LPARAM lParam, int button)
 
 void CtrlDisplayListView::onMouseUp(WPARAM wParam, LPARAM lParam, int button)
 {
+	if (!validDisplayList || !gpuDebug) {
+		return;
+	}
+
 	if (button == 2)
 	{
 		HMENU menu = GetContextMenu(ContextMenuID::DISPLAYLISTVIEW);
@@ -389,7 +397,7 @@ void CtrlDisplayListView::onMouseUp(WPARAM wParam, LPARAM lParam, int button)
 		case ID_GEDBG_GOTOADDR:
 			{
 				std::string expression = StringFromFormat("%08x", curAddress);
-				if (!InputBox_GetString(GetModuleHandle(NULL), wnd, L"Address", expression, expression, true)) {
+				if (!InputBox_GetString(GetModuleHandle(NULL), wnd, L"Address", expression, expression, InputBoxFlags::Selected)) {
 					break;
 				}
 				uint32_t newAddress = curAddress;
