@@ -19,8 +19,8 @@
 #include "Common/System/Request.h"
 #include "Common/File/FileUtil.h"
 #include "Common/Log.h"
-#include "Common/LogManager.h"
-#include "Common/ConsoleListener.h"
+#include "Common/Log/LogManager.h"
+#include "Common/Log/ConsoleListener.h"
 #include "Common/OSVersion.h"
 #include "Common/GPU/Vulkan/VulkanLoader.h"
 #include "Common/StringUtils.h"
@@ -53,6 +53,10 @@
 #include "Core/Core.h"
 #include "Core/RetroAchievements.h"
 
+#ifdef RC_CLIENT_SUPPORTS_RAINTEGRATION
+#include "ext/rcheevos/include/rc_client_raintegration.h"
+#endif
+
 extern bool g_TakeScreenshot;
 
 namespace MainWindow {
@@ -72,21 +76,26 @@ namespace MainWindow {
 	void SetIngameMenuItemStates(HMENU menu, const GlobalUIState state) {
 		bool menuEnableBool = state == UISTATE_INGAME || state == UISTATE_EXCEPTION;
 
+		bool loadStateEnableBool = menuEnableBool;
 		bool saveStateEnableBool = menuEnableBool;
-		if (Achievements::ChallengeModeActive()) {
-			saveStateEnableBool = false;
+		if (Achievements::HardcoreModeActive()) {
+			loadStateEnableBool = false;
+			if (!g_Config.bAchievementsSaveStateInHardcoreMode) {
+				saveStateEnableBool = false;
+			}
 		}
 
 		UINT menuEnable = menuEnableBool ? MF_ENABLED : MF_GRAYED;
+		UINT loadStateEnable = loadStateEnableBool ? MF_ENABLED : MF_GRAYED;
 		UINT saveStateEnable = saveStateEnableBool ? MF_ENABLED : MF_GRAYED;
 		UINT menuInGameEnable = state == UISTATE_INGAME ? MF_ENABLED : MF_GRAYED;
 		UINT umdSwitchEnable = state == UISTATE_INGAME && getUMDReplacePermit() ? MF_ENABLED : MF_GRAYED;
 
 		EnableMenuItem(menu, ID_FILE_SAVESTATE_SLOT_MENU, saveStateEnable);
 		EnableMenuItem(menu, ID_FILE_SAVESTATEFILE, saveStateEnable);
-		EnableMenuItem(menu, ID_FILE_LOADSTATEFILE, saveStateEnable);
+		EnableMenuItem(menu, ID_FILE_LOADSTATEFILE, loadStateEnable);
 		EnableMenuItem(menu, ID_FILE_QUICKSAVESTATE, saveStateEnable);
-		EnableMenuItem(menu, ID_FILE_QUICKLOADSTATE, saveStateEnable);
+		EnableMenuItem(menu, ID_FILE_QUICKLOADSTATE, loadStateEnable);
 		EnableMenuItem(menu, ID_EMULATION_PAUSE, menuEnable);
 		EnableMenuItem(menu, ID_EMULATION_STOP, menuEnable);
 		EnableMenuItem(menu, ID_EMULATION_RESET, menuEnable);
@@ -98,7 +107,6 @@ namespace MainWindow {
 		EnableMenuItem(menu, ID_DEBUG_LOADSYMFILE, menuEnable);
 		EnableMenuItem(menu, ID_DEBUG_SAVESYMFILE, menuEnable);
 		EnableMenuItem(menu, ID_DEBUG_RESETSYMBOLTABLE, menuEnable);
-		EnableMenuItem(menu, ID_DEBUG_TAKESCREENSHOT, menuEnable);
 		EnableMenuItem(menu, ID_DEBUG_SHOWDEBUGSTATISTICS, menuInGameEnable);
 		EnableMenuItem(menu, ID_DEBUG_EXTRACTFILE, menuEnable);
 		EnableMenuItem(menu, ID_DEBUG_MEMORYBASE, menuInGameEnable);
@@ -228,6 +236,7 @@ namespace MainWindow {
 		TranslateMenuItem(menu, ID_DEBUG_TAKESCREENSHOT, g_Config.bSystemControls ? L"\tF12" : L"");
 		TranslateMenuItem(menu, ID_DEBUG_DUMPNEXTFRAME);
 		TranslateMenuItem(menu, ID_DEBUG_SHOWDEBUGSTATISTICS);
+		TranslateMenuItem(menu, ID_DEBUG_RESTARTGRAPHICS);
 		TranslateMenuItem(menu, ID_DEBUG_DISASSEMBLY, g_Config.bSystemControls ? L"\tCtrl+D" : L"");
 		TranslateMenuItem(menu, ID_DEBUG_GEDEBUGGER, g_Config.bSystemControls ? L"\tCtrl+G" : L"");
 		TranslateMenuItem(menu, ID_DEBUG_EXTRACTFILE);
@@ -278,6 +287,7 @@ namespace MainWindow {
 		TranslateMenuItem(menu, ID_OPTIONS_NEARESTFILTERING);
 		TranslateMenuItem(menu, ID_OPTIONS_LINEARFILTERING);
 		TranslateMenuItem(menu, ID_OPTIONS_AUTOMAXQUALITYFILTERING);
+		TranslateMenuItem(menu, ID_OPTIONS_SMART2DTEXTUREFILTERING);
 		TranslateMenuItem(menu, ID_OPTIONS_SCREENFILTER_MENU);
 		TranslateMenuItem(menu, ID_OPTIONS_BUFLINEARFILTER);
 		TranslateMenuItem(menu, ID_OPTIONS_BUFNEARESTFILTER);
@@ -290,7 +300,6 @@ namespace MainWindow {
 		TranslateMenuItem(menu, ID_TEXTURESCALING_HYBRID_BICUBIC);
 		TranslateMenuItem(menu, ID_TEXTURESCALING_DEPOSTERIZE);
 		TranslateMenuItem(menu, ID_OPTIONS_HARDWARETRANSFORM);
-		TranslateMenuItem(menu, ID_OPTIONS_VERTEXCACHE);
 		TranslateMenuItem(menu, ID_EMULATION_SOUND);
 		TranslateMenuItem(menu, ID_EMULATION_CHEATS, g_Config.bSystemControls ? L"\tCtrl+T" : L"");
 		TranslateMenuItem(menu, ID_EMULATION_CHAT, g_Config.bSystemControls ? L"\tCtrl+C" : L"");
@@ -316,7 +325,7 @@ namespace MainWindow {
 
 	void BrowseAndBootDone(std::string filename);
 
-	void BrowseAndBoot(std::string defaultPath, bool browseDirectory) {
+	void BrowseAndBoot(RequesterToken token, std::string defaultPath, bool browseDirectory) {
 		browsePauseAfter = false;
 		if (GetUIState() == UISTATE_INGAME) {
 			browsePauseAfter = Core_IsStepping();
@@ -328,11 +337,11 @@ namespace MainWindow {
 		W32Util::MakeTopMost(GetHWND(), false);
 
 		if (browseDirectory) {
-			System_BrowseForFolder(mm->T("Load"), [](const std::string &value, int) {
+			System_BrowseForFolder(token, mm->T("Load"), Path(), [](const std::string &value, int) {
 				BrowseAndBootDone(value);
 			});
 		} else {
-			System_BrowseForFile(mm->T("Load"), BrowseFileType::BOOTABLE, [](const std::string &value, int) {
+			System_BrowseForFile(token, mm->T("Load"), BrowseFileType::BOOTABLE, [](const std::string &value, int) {
 				BrowseAndBootDone(value);
 			});
 		}
@@ -343,18 +352,18 @@ namespace MainWindow {
 			Core_EnableStepping(false);
 		}
 		filename = ReplaceAll(filename, "\\", "/");
-		System_PostUIMessage("boot", filename);
+		System_PostUIMessage(UIMessage::REQUEST_GAME_BOOT, filename);
 		W32Util::MakeTopMost(GetHWND(), g_Config.bTopMost);
 	}
 
-	static void UmdSwitchAction() {
+	static void UmdSwitchAction(RequesterToken token) {
 		auto mm = GetI18NCategory(I18NCat::MAINMENU);
-		System_BrowseForFile(mm->T("Switch UMD"), BrowseFileType::BOOTABLE, [](const std::string &value, int) {
+		System_BrowseForFile(token, mm->T("Switch UMD"), BrowseFileType::BOOTABLE, [](const std::string &value, int) {
 			__UmdReplace(Path(value));
 		});
 	}
 
-	static void SaveStateActionFinished(SaveState::Status status, const std::string &message, void *userdata) {
+	static void SaveStateActionFinished(SaveState::Status status, std::string_view message, void *userdata) {
 		if (!message.empty() && (!g_Config.bDumpFrames || !g_Config.bDumpVideoOutput)) {
 			g_OSD.Show(status == SaveState::Status::SUCCESS ? OSDType::MESSAGE_SUCCESS : OSDType::MESSAGE_ERROR, message, status == SaveState::Status::SUCCESS ? 2.0 : 5.0);
 		}
@@ -364,17 +373,17 @@ namespace MainWindow {
 	// not static
 	void setTexScalingMultiplier(int level) {
 		g_Config.iTexScalingLevel = level;
-		System_PostUIMessage("gpu_configChanged", "");
+		System_PostUIMessage(UIMessage::GPU_CONFIG_CHANGED);
 	}
 
 	static void setTexScalingType(int type) {
 		g_Config.iTexScalingType = type;
-		System_PostUIMessage("gpu_configChanged", "");
+		System_PostUIMessage(UIMessage::GPU_CONFIG_CHANGED);
 	}
 
 	static void setSkipBufferEffects(bool skip) {
 		g_Config.bSkipBufferEffects = skip;
-		System_PostUIMessage("gpu_configChanged", "");
+		System_PostUIMessage(UIMessage::GPU_CONFIG_CHANGED);
 	}
 
 	static void setFrameSkipping(int framesToSkip = -1) {
@@ -419,7 +428,7 @@ namespace MainWindow {
 	}
 
 	static void RestartApp() {
-		if (IsDebuggerPresent()) {
+		if (System_GetPropertyBool(SYSPROP_DEBUGGER_PRESENT)) {
 			PostMessage(MainWindow::GetHWND(), WM_USER_RESTART_EMUTHREAD, 0, 0);
 		} else {
 			g_Config.bRestartRequired = true;
@@ -436,15 +445,15 @@ namespace MainWindow {
 		// Parse the menu selections:
 		switch (wmId) {
 		case ID_FILE_LOAD:
-			BrowseAndBoot("", false);
+			BrowseAndBoot(NON_EPHEMERAL_TOKEN, "", false);
 			break;
 
 		case ID_FILE_LOAD_DIR:
-			BrowseAndBoot("", true);
+			BrowseAndBoot(NON_EPHEMERAL_TOKEN, "", true);
 			break;
 
 		case ID_FILE_LOAD_MEMSTICK:
-			BrowseAndBoot(GetSysDirectory(DIRECTORY_GAME).ToString());
+			BrowseAndBoot(NON_EPHEMERAL_TOKEN, GetSysDirectory(DIRECTORY_GAME).ToString());
 			break;
 
 		case ID_FILE_OPEN_NEW_INSTANCE:
@@ -457,8 +466,8 @@ namespace MainWindow {
 
 		case ID_TOGGLE_BREAK:
 			if (GetUIState() == UISTATE_PAUSEMENU) {
-				// Causes hang
-				//System_PostUIMessage("run", "");
+				// Causes hang (outdated comment?)
+				// System_PostUIMessage(UIMessage::REQUEST_GAME_RUN, "");
 
 				if (disasmWindow)
 					SendMessage(disasmWindow->GetDlgHandle(), WM_COMMAND, IDC_STOPGO, 0);
@@ -477,7 +486,7 @@ namespace MainWindow {
 			break;
 
 		case ID_EMULATION_PAUSE:
-			System_PostUIMessage("pause", "");
+			System_PostUIMessage(UIMessage::REQUEST_GAME_PAUSE);
 			break;
 
 		case ID_EMULATION_STOP:
@@ -485,17 +494,17 @@ namespace MainWindow {
 				Core_EnableStepping(false);
 
 			Core_Stop();
-			System_PostUIMessage("stop", "");
+			System_PostUIMessage(UIMessage::REQUEST_GAME_STOP);
 			Core_WaitInactive();
 			break;
 
 		case ID_EMULATION_RESET:
-			System_PostUIMessage("reset", "");
+			System_PostUIMessage(UIMessage::REQUEST_GAME_RESET);
 			Core_EnableStepping(false);
 			break;
 
 		case ID_EMULATION_SWITCH_UMD:
-			UmdSwitchAction();
+			UmdSwitchAction(NON_EPHEMERAL_TOKEN);
 			break;
 
 		case ID_EMULATION_ROTATION_H:   g_Config.iInternalScreenRotation = ROTATION_LOCKED_HORIZONTAL; break;
@@ -510,11 +519,11 @@ namespace MainWindow {
 
 		case ID_EMULATION_CHAT:
 			if (GetUIState() == UISTATE_INGAME) {
-				System_PostUIMessage("chat screen", "");
+				System_PostUIMessage(UIMessage::SHOW_CHAT_SCREEN);
 			}
 			break;
 		case ID_FILE_LOADSTATEFILE:
-			if (!Achievements::WarnUserIfChallengeModeActive()) {
+			if (!Achievements::WarnUserIfHardcoreModeActive(false)) {
 				if (W32Util::BrowseForFileName(true, hWnd, L"Load state", 0, L"Save States (*.ppst)\0*.ppst\0All files\0*.*\0\0", L"ppst", fn)) {
 					SetCursor(LoadCursor(0, IDC_WAIT));
 					SaveState::Load(Path(fn), -1, SaveStateActionFinished);
@@ -522,7 +531,7 @@ namespace MainWindow {
 			}
 			break;
 		case ID_FILE_SAVESTATEFILE:
-			if (!Achievements::WarnUserIfChallengeModeActive()) {
+			if (!Achievements::WarnUserIfHardcoreModeActive(true)) {
 				if (W32Util::BrowseForFileName(false, hWnd, L"Save state", 0, L"Save States (*.ppst)\0*.ppst\0All files\0*.*\0\0", L"ppst", fn)) {
 					SetCursor(LoadCursor(0, IDC_WAIT));
 					SaveState::Save(Path(fn), -1, SaveStateActionFinished);
@@ -532,19 +541,19 @@ namespace MainWindow {
 
 		case ID_FILE_SAVESTATE_NEXT_SLOT:
 		{
-			if (!Achievements::WarnUserIfChallengeModeActive()) {
+			if (!Achievements::WarnUserIfHardcoreModeActive(true)) {
 				SaveState::NextSlot();
-				System_PostUIMessage("savestate_displayslot", "");
+				System_PostUIMessage(UIMessage::SAVESTATE_DISPLAY_SLOT);
 			}
 			break;
 		}
 
 		case ID_FILE_SAVESTATE_NEXT_SLOT_HC:
 		{
-			if (!Achievements::WarnUserIfChallengeModeActive()) {
+			if (!Achievements::WarnUserIfHardcoreModeActive(true)) {
 				if (!KeyMap::PspButtonHasMappings(VIRTKEY_NEXT_SLOT)) {
 					SaveState::NextSlot();
-					System_PostUIMessage("savestate_displayslot", "");
+					System_PostUIMessage(UIMessage::SAVESTATE_DISPLAY_SLOT);
 				}
 			}
 			break;
@@ -555,13 +564,13 @@ namespace MainWindow {
 		case ID_FILE_SAVESTATE_SLOT_3:
 		case ID_FILE_SAVESTATE_SLOT_4:
 		case ID_FILE_SAVESTATE_SLOT_5:
-			if (!Achievements::WarnUserIfChallengeModeActive()) {
+			if (!Achievements::WarnUserIfHardcoreModeActive(true)) {
 				g_Config.iCurrentStateSlot = wmId - ID_FILE_SAVESTATE_SLOT_1;
 			}
 			break;
 
 		case ID_FILE_QUICKLOADSTATE:
-			if (!Achievements::WarnUserIfChallengeModeActive()) {
+			if (!Achievements::WarnUserIfHardcoreModeActive(false)) {
 				SetCursor(LoadCursor(0, IDC_WAIT));
 				SaveState::LoadSlot(PSP_CoreParameter().fileToStart, g_Config.iCurrentStateSlot, SaveStateActionFinished);
 			}
@@ -569,7 +578,7 @@ namespace MainWindow {
 
 		case ID_FILE_QUICKLOADSTATE_HC:
 		{
-			if (!Achievements::WarnUserIfChallengeModeActive()) {
+			if (!Achievements::WarnUserIfHardcoreModeActive(false)) {
 				if (!KeyMap::PspButtonHasMappings(VIRTKEY_LOAD_STATE)) {
 					SetCursor(LoadCursor(0, IDC_WAIT));
 					SaveState::LoadSlot(PSP_CoreParameter().fileToStart, g_Config.iCurrentStateSlot, SaveStateActionFinished);
@@ -579,7 +588,7 @@ namespace MainWindow {
 		}
 		case ID_FILE_QUICKSAVESTATE:
 		{
-			if (!Achievements::WarnUserIfChallengeModeActive()) {
+			if (!Achievements::WarnUserIfHardcoreModeActive(true)) {
 				SetCursor(LoadCursor(0, IDC_WAIT));
 				SaveState::SaveSlot(PSP_CoreParameter().fileToStart, g_Config.iCurrentStateSlot, SaveStateActionFinished);
 			}
@@ -588,7 +597,7 @@ namespace MainWindow {
 
 		case ID_FILE_QUICKSAVESTATE_HC:
 		{
-			if (!Achievements::WarnUserIfChallengeModeActive()) {
+			if (!Achievements::WarnUserIfHardcoreModeActive(true)) {
 				if (!KeyMap::PspButtonHasMappings(VIRTKEY_SAVE_STATE))
 				{
 					SetCursor(LoadCursor(0, IDC_WAIT));
@@ -600,7 +609,7 @@ namespace MainWindow {
 		}
 
 		case ID_OPTIONS_LANGUAGE:
-			System_PostUIMessage("language screen", "");
+			System_PostUIMessage(UIMessage::SHOW_LANGUAGE_SCREEN);
 			break;
 
 		case ID_OPTIONS_IGNOREWINKEY:
@@ -645,7 +654,7 @@ namespace MainWindow {
 			g_Config.bAutoFrameSkip = !g_Config.bAutoFrameSkip;
 			if (g_Config.bAutoFrameSkip && g_Config.bSkipBufferEffects) {
 				g_Config.bSkipBufferEffects = false;
-				System_PostUIMessage("gpu_configChanged", "");
+				System_PostUIMessage(UIMessage::GPU_CONFIG_CHANGED);
 			}
 			break;
 
@@ -662,7 +671,7 @@ namespace MainWindow {
 
 		case ID_TEXTURESCALING_DEPOSTERIZE:
 			g_Config.bTexDeposterize = !g_Config.bTexDeposterize;
-			System_PostUIMessage("gpu_configChanged", "");
+			System_PostUIMessage(UIMessage::GPU_CONFIG_CHANGED);
 			break;
 
 		case ID_OPTIONS_DIRECT3D9:
@@ -691,7 +700,7 @@ namespace MainWindow {
 
 		case ID_OPTIONS_SKIP_BUFFER_EFFECTS:
 			g_Config.bSkipBufferEffects = !g_Config.bSkipBufferEffects;
-			System_PostUIMessage("gpu_renderResized", "");
+			System_PostUIMessage(UIMessage::GPU_RENDER_RESIZED);
 			g_OSD.ShowOnOff(gr->T("Skip Buffer Effects"), g_Config.bSkipBufferEffects);
 			break;
 
@@ -703,17 +712,17 @@ namespace MainWindow {
 			} else {
 				g_Config.iDebugOverlay = (int)DebugOverlay::DEBUG_STATS;
 			}
-			System_PostUIMessage("clear jit", "");
+			System_PostUIMessage(UIMessage::REQUEST_CLEAR_JIT);
 			break;
 
 		case ID_OPTIONS_HARDWARETRANSFORM:
 			g_Config.bHardwareTransform = !g_Config.bHardwareTransform;
-			System_PostUIMessage("gpu_configChanged", "");
+			System_PostUIMessage(UIMessage::GPU_CONFIG_CHANGED);
 			g_OSD.ShowOnOff(gr->T("Hardware Transform"), g_Config.bHardwareTransform);
 			break;
 
 		case ID_OPTIONS_DISPLAY_LAYOUT:
-			System_PostUIMessage("display layout editor", "");
+			System_PostUIMessage(UIMessage::SHOW_DISPLAY_LAYOUT_EDITOR);
 			break;
 
 
@@ -744,7 +753,7 @@ namespace MainWindow {
 			break;
 
 		case ID_DEBUG_DUMPNEXTFRAME:
-			System_PostUIMessage("gpu dump next frame", "");
+			System_PostUIMessage(UIMessage::REQUEST_GPU_DUMP_NEXT_FRAME);
 			break;
 
 		case ID_DEBUG_LOADMAPFILE:
@@ -859,14 +868,13 @@ namespace MainWindow {
 			SendToggleFullscreen(!g_Config.UseFullScreen());
 			break;
 
-		case ID_OPTIONS_VERTEXCACHE:
-			g_Config.bVertexCache = !g_Config.bVertexCache;
-			break;
 		case ID_OPTIONS_TEXTUREFILTERING_AUTO:   g_Config.iTexFiltering = TEX_FILTER_AUTO; break;
 		case ID_OPTIONS_NEARESTFILTERING:        g_Config.iTexFiltering = TEX_FILTER_FORCE_NEAREST; break;
 		case ID_OPTIONS_LINEARFILTERING:         g_Config.iTexFiltering = TEX_FILTER_FORCE_LINEAR; break;
 		case ID_OPTIONS_AUTOMAXQUALITYFILTERING: g_Config.iTexFiltering = TEX_FILTER_AUTO_MAX_QUALITY; break;
 
+		case ID_OPTIONS_SMART2DTEXTUREFILTERING: g_Config.bSmart2DTexFiltering = !g_Config.bSmart2DTexFiltering; break;
+
 		case ID_OPTIONS_BUFLINEARFILTER:  g_Config.iDisplayFilter = SCALE_LINEAR; break;
 		case ID_OPTIONS_BUFNEARESTFILTER: g_Config.iDisplayFilter = SCALE_NEAREST; break;
 
@@ -880,11 +888,11 @@ namespace MainWindow {
 			break;
 
 		case ID_OPTIONS_CONTROLS:
-			System_PostUIMessage("control mapping", "");
+			System_PostUIMessage(UIMessage::SHOW_CONTROL_MAPPING);
 			break;
 
 		case ID_OPTIONS_MORE_SETTINGS:
-			System_PostUIMessage("settings", "");
+			System_PostUIMessage(UIMessage::SHOW_SETTINGS);
 			break;
 
 		case ID_EMULATION_SOUND:
@@ -921,6 +929,10 @@ namespace MainWindow {
 			g_TakeScreenshot = true;
 			break;
 
+		case ID_DEBUG_RESTARTGRAPHICS:
+			System_PostUIMessage(UIMessage::RESTART_GRAPHICS);
+			break;
+
 		case ID_FILE_DUMPFRAMES:
 			g_Config.bDumpFrames = !g_Config.bDumpFrames;
 			break;
@@ -938,7 +950,12 @@ namespace MainWindow {
 			break;
 
 		default:
-			MessageBox(hWnd, L"Unimplemented", L"Sorry", 0);
+#ifdef RC_CLIENT_SUPPORTS_RAINTEGRATION
+			if (rc_client_raintegration_activate_menu_item(Achievements::GetClient(), LOWORD(wParam))) {
+				break;
+			}
+#endif
+			MessageBox(hWnd, L"Unhandled menu item", L"Sorry", 0);
 			break;
 		}
 	}
@@ -954,7 +971,6 @@ namespace MainWindow {
 		CHECKITEM(ID_DEBUG_SHOWDEBUGSTATISTICS, (DebugOverlay)g_Config.iDebugOverlay == DebugOverlay::DEBUG_STATS);
 		CHECKITEM(ID_OPTIONS_HARDWARETRANSFORM, g_Config.bHardwareTransform);
 		CHECKITEM(ID_DEBUG_BREAKONLOAD, !g_Config.bAutoRun);
-		CHECKITEM(ID_OPTIONS_VERTEXCACHE, g_Config.bVertexCache);
 		CHECKITEM(ID_OPTIONS_FRAMESKIP_AUTO, g_Config.bAutoFrameSkip);
 		CHECKITEM(ID_OPTIONS_FRAMESKIP, g_Config.iFrameSkip != FRAMESKIP_OFF);
 		CHECKITEM(ID_OPTIONS_FRAMESKIPTYPE_COUNT, g_Config.iFrameSkipType == FRAMESKIPTYPE_COUNT);
@@ -962,6 +978,7 @@ namespace MainWindow {
 		CHECKITEM(ID_OPTIONS_VSYNC, g_Config.bVSync);
 		CHECKITEM(ID_OPTIONS_TOPMOST, g_Config.bTopMost);
 		CHECKITEM(ID_OPTIONS_PAUSE_FOCUS, g_Config.bPauseOnLostFocus);
+		CHECKITEM(ID_OPTIONS_SMART2DTEXTUREFILTERING, g_Config.bSmart2DTexFiltering);
 		CHECKITEM(ID_EMULATION_SOUND, g_Config.bEnableSound);
 		CHECKITEM(ID_TEXTURESCALING_DEPOSTERIZE, g_Config.bTexDeposterize);
 		CHECKITEM(ID_EMULATION_CHEATS, g_Config.bEnableCheats);
@@ -988,13 +1005,6 @@ namespace MainWindow {
 			CheckMenuItem(menu, displayrotationitems[i], MF_BYCOMMAND | ((i + 1) == g_Config.iInternalScreenRotation ? MF_CHECKED : MF_UNCHECKED));
 		}
 
-		// Disable Vertex Cache when HW T&L is disabled.
-		if (!g_Config.bHardwareTransform) {
-			EnableMenuItem(menu, ID_OPTIONS_VERTEXCACHE, MF_GRAYED);
-		} else {
-			EnableMenuItem(menu, ID_OPTIONS_VERTEXCACHE, MF_ENABLED);
-		}
-
 		static const int zoomitems[11] = {
 			ID_OPTIONS_SCREENAUTO,
 			ID_OPTIONS_SCREEN1X,
