@@ -11,9 +11,11 @@ class TabControl
 public:
 	TabControl(HWND handle, bool noDisplayArea = false);
 	void HandleNotify(LPARAM lParam);
+	int HitTest(const POINT &screenPos);
 	HWND AddTabWindow(const wchar_t* className, const wchar_t* title, DWORD style = 0);
 	void AddTabDialog(Dialog* dialog, const wchar_t* title);
 	void AddTab(HWND hwnd, const wchar_t* title);
+	HWND RemoveTab(int index);
 	void ShowTab(int index, bool setControlIndex = true);
 	void ShowTab(HWND pageHandle);
 	void NextTab(bool cycle);
@@ -30,6 +32,10 @@ public:
 	bool GetShowTabTitles() { return showTabTitles; }
 	void SetMinTabWidth(int w);
 
+	int Count() {
+		return (int)tabs.size();
+	}
+
 private:
 	static LRESULT CALLBACK wndProc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam);
 	void OnResize();
@@ -47,9 +53,9 @@ private:
 	HWND hwnd;
 	WNDPROC oldProc;
 	std::vector<TabInfo> tabs;
-	bool showTabTitles;
-	bool ignoreBottomMargin;
-	int currentTab;
+	bool showTabTitles = true;
+	bool ignoreBottomMargin = false;
+	int currentTab = 0;
 	bool hasButtons;
 	bool noDisplayArea_;
 };
