@@ -7,6 +7,8 @@ import android.app.Activity;
 import android.app.ActivityManager;
 import android.app.AlertDialog;
 import android.app.UiModeManager;
+import android.content.ClipData;
+import android.content.ClipboardManager;
 import android.content.Context;
 import android.content.DialogInterface;
 import android.content.Intent;
@@ -26,6 +28,8 @@ import android.os.Environment;
 import android.os.PowerManager;
 import android.os.Vibrator;
 import android.provider.MediaStore;
+
+import androidx.annotation.NonNull;
 import androidx.documentfile.provider.DocumentFile;
 import android.text.InputType;
 import android.util.Log;
@@ -68,7 +72,6 @@ public abstract class NativeActivity extends Activity {
 	// Graphics and audio interfaces for Vulkan (javaGL = false)
 	private NativeSurfaceView mSurfaceView;
 	private Surface mSurface;
-	private Thread mRenderLoopThread = null;
 
 	// Graphics and audio interfaces for Java EGL (javaGL = true)
 	private NativeGLView mGLSurfaceView;
@@ -91,7 +94,6 @@ public abstract class NativeActivity extends Activity {
 	// audioFocusChangeListener to listen to changes in audio state
 	private AudioFocusChangeListener audioFocusChangeListener;
 	private AudioManager audioManager;
-	private PowerManager powerManager;
 
 	private Vibrator vibrator;
 
@@ -112,6 +114,7 @@ public abstract class NativeActivity extends Activity {
 	private PowerSaveModeReceiver mPowerSaveModeReceiver = null;
 	private SizeManager sizeManager = null;
 	private static LocationHelper mLocationHelper;
+	private static InfraredHelper mInfraredHelper;
 	private static CameraHelper mCameraHelper;
 
 	private static final String[] permissionsForStorage = {
@@ -211,7 +214,7 @@ public abstract class NativeActivity extends Activity {
 	}
 
 	@Override
-	public void onRequestPermissionsResult(int requestCode, String [] permissions, int[] grantResults) {
+	public void onRequestPermissionsResult(int requestCode, @NonNull String [] permissions, @NonNull int[] grantResults) {
 		switch (requestCode) {
 		case REQUEST_CODE_STORAGE_PERMISSION:
 			if (permissionsGranted(permissions, grantResults)) {
@@ -257,7 +260,7 @@ public abstract class NativeActivity extends Activity {
 			// Try another method.
 			String removableStoragePath;
 			list = new ArrayList<String>();
-			File fileList[] = new File("/storage/").listFiles();
+			File[] fileList = new File("/storage/").listFiles();
 			if (fileList != null) {
 				for (File file : fileList) {
 					if (!file.getAbsolutePath().equalsIgnoreCase(Environment.getExternalStorageDirectory().getAbsolutePath()) && file.isDirectory() && file.canRead()) {
@@ -355,7 +358,7 @@ public abstract class NativeActivity extends Activity {
 			// Get the optimal buffer sz
 			detectOptimalAudioSettings();
 		}
-		powerManager = (PowerManager) getSystemService(Context.POWER_SERVICE);
+		PowerManager powerManager = (PowerManager) getSystemService(Context.POWER_SERVICE);
 		if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.N) {
 			if (powerManager != null && powerManager.isSustainedPerformanceModeSupported()) {
 				sustainedPerfSupported = true;
@@ -402,6 +405,7 @@ public abstract class NativeActivity extends Activity {
 		String extStorageDir = Environment.getExternalStorageDirectory().getAbsolutePath();
 		File externalFiles = this.getExternalFilesDir(null);
 		String externalFilesDir = externalFiles == null ? "" : externalFiles.getAbsolutePath();
+		String nativeLibDir = getApplicationLibraryDir(appInfo);
 
 		Log.i(TAG, "Ext storage: " + extStorageState + " " + extStorageDir);
 		Log.i(TAG, "Ext files dir: " + externalFilesDir);
@@ -424,7 +428,7 @@ public abstract class NativeActivity extends Activity {
 		}
 		catch (Exception e) {
 			NativeApp.reportException(e, null);
-			Log.e(TAG, "Failed to get SD storage dirs: " + e.toString());
+			Log.e(TAG, "Failed to get SD storage dirs: " + e);
 		}
 
 		Log.i(TAG, "End of storage paths");
@@ -442,9 +446,10 @@ public abstract class NativeActivity extends Activity {
 		String languageRegion = Locale.getDefault().getLanguage() + "_" + Locale.getDefault().getCountry();
 		String shortcut = overrideShortcutParam == null ? shortcutParam : overrideShortcutParam;
 		overrideShortcutParam = null;
+		shortcutParam = null;
 
 		NativeApp.audioConfig(optimalFramesPerBuffer, optimalSampleRate);
-		NativeApp.init(model, deviceType, languageRegion, apkFilePath, dataDir, extStorageDir, externalFilesDir, additionalStorageDirs, cacheDir, shortcut, Build.VERSION.SDK_INT, Build.BOARD);
+		NativeApp.init(model, deviceType, languageRegion, apkFilePath, dataDir, extStorageDir, externalFilesDir, nativeLibDir, additionalStorageDirs, cacheDir, shortcut, Build.VERSION.SDK_INT, Build.BOARD);
 
 		// Allow C++ to tell us to use JavaGL or not.
 		javaGL = "true".equalsIgnoreCase(NativeApp.queryConfig("androidJavaGL"));
@@ -472,6 +477,14 @@ public abstract class NativeActivity extends Activity {
 		}
 
 		mLocationHelper = new LocationHelper(this);
+		if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.KITKAT) {
+			try {
+				mInfraredHelper = new InfraredHelper(this);
+			} catch (Exception e) {
+				mInfraredHelper = null;
+				Log.i(TAG, "InfraredHelper exception: " + e);
+			}
+		}
 		if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.HONEYCOMB) {
 			// android.graphics.SurfaceTexture is not available before version 11.
 			mCameraHelper = new CameraHelper(this);
@@ -492,7 +505,6 @@ public abstract class NativeActivity extends Activity {
 		}
 	}
 
-	@TargetApi(Build.VERSION_CODES.GINGERBREAD)
 	private void updateScreenRotation(String cause) {
 		// Query the native application on the desired rotation.
 		int rot;
@@ -568,19 +580,6 @@ public abstract class NativeActivity extends Activity {
 		}
 	}
 
-	private final Runnable mEmulationRunner = new Runnable() {
-		@Override
-		public void run() {
-			Log.i(TAG, "Starting the render loop: " + mSurface);
-			// Start emulation using the provided Surface.
-			if (!runVulkanRenderLoop(mSurface)) {
-				// Shouldn't happen.
-				Log.e(TAG, "Failed to start up OpenGL/Vulkan - runVulkanRenderLoop returned false");
-			}
-			Log.i(TAG, "Left the render loop: " + mSurface);
-		}
-	};
-
 	public native boolean runVulkanRenderLoop(Surface surface);
 	// Tells the render loop thread to exit, so we can restart it.
 	public native void requestExitVulkanRenderLoop();
@@ -598,6 +597,7 @@ public abstract class NativeActivity extends Activity {
 		// whether to start at 1x or 2x.
 		sizeManager.updateDisplayMeasurements();
 
+		boolean wasInitialized = initialized;
 		if (!initialized) {
 			Initialize();
 			initialized = true;
@@ -650,6 +650,7 @@ public abstract class NativeActivity extends Activity {
 					// mGLSurfaceView.setEGLConfigChooser(8, 8, 8, 8, 16, 8);
 				// }
 			}
+
 			mGLSurfaceView.setRenderer(nativeRenderer);
 			setContentView(mGLSurfaceView);
 		} else {
@@ -664,6 +665,13 @@ public abstract class NativeActivity extends Activity {
 			Log.i(TAG, "setcontentview after");
 			startRenderLoopThread();
 		}
+
+		if (shortcutParam != null && shortcutParam.length() > 0) {
+			Log.i(TAG, "Got shortcutParam in onCreate on secondary run: " + shortcutParam);
+			// Make sure we only send it once.
+			NativeApp.sendMessageFromJava("shortcutParam", shortcutParam);
+			shortcutParam = null;
+		}
 	}
 
 	@Override
@@ -675,6 +683,37 @@ public abstract class NativeActivity extends Activity {
 		}
 	}
 
+	private void applyFrameRate(Surface surface, float frameRateHz) {
+		if (Build.VERSION.SDK_INT < Build.VERSION_CODES.R)
+			return;
+		if (mSurface != null && Build.VERSION.SDK_INT >= Build.VERSION_CODES.R) {
+			try {
+				int method = NativeApp.getDisplayFramerateMode();
+				if (method > 0) {
+					Log.i(TAG, "Setting desired framerate to " + frameRateHz + " Hz method=" + method);
+					switch (method) {
+						case 1:
+							mSurface.setFrameRate(frameRateHz, Surface.FRAME_RATE_COMPATIBILITY_DEFAULT);
+							break;
+						case 2:
+							mSurface.setFrameRate(frameRateHz, Surface.FRAME_RATE_COMPATIBILITY_FIXED_SOURCE);
+							break;
+						case 3:
+							if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.S) {
+								mSurface.setFrameRate(frameRateHz, Surface.FRAME_RATE_COMPATIBILITY_FIXED_SOURCE, Surface.CHANGE_FRAME_RATE_ALWAYS);
+							}
+							break;
+						default:
+							break;
+					}
+
+				}
+			} catch (Exception e) {
+				Log.e(TAG, "Failed to set framerate: " + e);
+			}
+		}
+	}
+
 	public void notifySurface(Surface surface) {
 		mSurface = surface;
 
@@ -690,49 +729,38 @@ public abstract class NativeActivity extends Activity {
 			} else {
 				startRenderLoopThread();
 			}
+		} else if (mSurface != null) {
+			applyFrameRate(mSurface, 60.0f);
 		}
 		updateSustainedPerformanceMode();
 	}
 
-	// Invariants: After this, mRenderLoopThread will be set, and the thread will be running,
-	// if in Vulkan mode.
+	// The render loop thread (EmuThread) is now spawned from the native side.
 	protected synchronized void startRenderLoopThread() {
 		if (javaGL) {
-			Log.e(TAG, "JavaGL mode - should not get into ensureRenderLoop.");
+			Log.e(TAG, "JavaGL mode - should not get into startRenderLoopThread.");
 			return;
 		}
 		if (mSurface == null) {
-			Log.w(TAG, "ensureRenderLoop - not starting thread, needs surface");
+			Log.w(TAG, "startRenderLoopThread - not starting thread, needs surface");
 			return;
 		}
-		if (mRenderLoopThread == null) {
-			Log.w(TAG, "ensureRenderLoop: Starting thread");
-			mRenderLoopThread = new Thread(mEmulationRunner);
-			mRenderLoopThread.start();
-		}
+
+		Log.w(TAG, "startRenderLoopThread: Starting thread");
+
+		applyFrameRate(mSurface, 60.0f);
+		runVulkanRenderLoop(mSurface);
 	}
 
-	// Invariants: After this, mRenderLoopThread will be null, and the thread has exited.
 	private synchronized void joinRenderLoopThread() {
 		if (javaGL) {
 			Log.e(TAG, "JavaGL - should not get into joinRenderLoopThread.");
 			return;
 		}
 
-		if (mRenderLoopThread != null) {
-			// This will wait until the thread has exited.
-			Log.i(TAG, "requestExitVulkanRenderLoop");
-			requestExitVulkanRenderLoop();
-			try {
-				Log.i(TAG, "joining render loop thread...");
-				mRenderLoopThread.join();
-				Log.w(TAG, "Joined render loop thread.");
-				mRenderLoopThread = null;
-			} catch (InterruptedException e) {
-				e.printStackTrace();
-				mRenderLoopThread = null;
-			}
-		}
+		// This will wait until the thread has exited.
+		Log.i(TAG, "requestExitVulkanRenderLoop");
+		requestExitVulkanRenderLoop();
 	}
 
 	@TargetApi(Build.VERSION_CODES.KITKAT)
@@ -803,6 +831,7 @@ public abstract class NativeActivity extends Activity {
 		if (isVRDevice()) {
 			System.exit(0);
 		}
+		Log.i(TAG, "onDestroy");
 	}
 
 	@Override
@@ -890,7 +919,7 @@ public abstract class NativeActivity extends Activity {
 	}
 
 	@Override
-	public void onConfigurationChanged(Configuration newConfig) {
+	public void onConfigurationChanged(@NonNull Configuration newConfig) {
 		Log.i(TAG, "onConfigurationChanged");
 		super.onConfigurationChanged(newConfig);
 		if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.ICE_CREAM_SANDWICH) {
@@ -925,7 +954,6 @@ public abstract class NativeActivity extends Activity {
 		}
 	}
 
-	@TargetApi(Build.VERSION_CODES.GINGERBREAD)
 	private InputDeviceState getInputDeviceState(InputEvent event) {
 		InputDevice device = event.getDevice();
 		if (device == null) {
@@ -1031,6 +1059,11 @@ public abstract class NativeActivity extends Activity {
 		}
 	}
 
+	@TargetApi(Build.VERSION_CODES.N)
+	void sendMouseDelta(float dx, float dy) {
+		NativeApp.mouseDelta(dx, dy);
+	}
+
 	@Override
 	@TargetApi(Build.VERSION_CODES.HONEYCOMB_MR1)
 	public boolean onGenericMotionEvent(MotionEvent event) {
@@ -1048,12 +1081,20 @@ public abstract class NativeActivity extends Activity {
 		}
 
 		if ((event.getSource() & InputDevice.SOURCE_CLASS_POINTER) != 0) {
+			if ((event.getSource() & InputDevice.SOURCE_MOUSE) == InputDevice.SOURCE_MOUSE) {
+				if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.N) {
+					float dx = event.getAxisValue(MotionEvent.AXIS_RELATIVE_X);
+					float dy = event.getAxisValue(MotionEvent.AXIS_RELATIVE_Y);
+					sendMouseDelta(dx, dy);
+				}
+			}
+
 			switch (event.getAction()) {
 			case MotionEvent.ACTION_HOVER_MOVE:
 				// process the mouse hover movement...
 				return true;
 			case MotionEvent.ACTION_SCROLL:
-				NativeApp.mouseWheelEvent(event.getX(), event.getY());
+				NativeApp.mouseWheelEvent(event.getAxisValue(MotionEvent.AXIS_HSCROLL), event.getAxisValue(MotionEvent.AXIS_VSCROLL));
 				return true;
 			}
 		}
@@ -1188,12 +1229,12 @@ public abstract class NativeActivity extends Activity {
 							getContentResolver().takePersistableUriPermission(selectedFile, Intent.FLAG_GRANT_READ_URI_PERMISSION);
 						}
 					} catch (Exception e) {
-						Log.w(TAG, "Exception getting permissions for document: " + e.toString());
+						Log.w(TAG, "Exception getting permissions for document: " + e);
 						NativeApp.sendRequestResult(requestId, false, "", 0);
 						NativeApp.reportException(e, selectedFile.toString());
 						return;
 					}
-					Log.i(TAG, "Browse file finished:" + selectedFile.toString());
+					Log.i(TAG, "Browse file finished:" + selectedFile);
 					NativeApp.sendRequestResult(requestId, true, selectedFile.toString(), 0);
 				}
 			} else if (requestCode == RESULT_OPEN_DOCUMENT_TREE) {
@@ -1206,7 +1247,7 @@ public abstract class NativeActivity extends Activity {
 							getContentResolver().takePersistableUriPermission(selectedDirectoryUri, Intent.FLAG_GRANT_READ_URI_PERMISSION | Intent.FLAG_GRANT_WRITE_URI_PERMISSION);
 						}
 					} catch (Exception e) {
-						Log.w(TAG, "Exception getting permissions for document: " + e.toString());
+						Log.w(TAG, "Exception getting permissions for document: " + e);
 						NativeApp.reportException(e, selectedDirectoryUri.toString());
 						// Even if we got an exception getting permissions, continue and try to pass along the file. Maybe this version of Android
 						// doesn't need it. If we can't access it, we'll fail in some other way later.
@@ -1325,7 +1366,11 @@ public abstract class NativeActivity extends Activity {
 		AlertDialog dlg = builder.create();
 
 		dlg.setCancelable(true);
-		dlg.show();
+		try {
+			dlg.show();
+		} catch (Exception e) {
+			NativeApp.reportException(e, "AlertDialog");
+		}
 	}
 
 	public boolean processCommand(String command, String params) {
@@ -1386,7 +1431,7 @@ public abstract class NativeActivity extends Activity {
 				Log.e(TAG, e.toString());
 				return false;
 			}
-		} else if (command.equals("browse_file") || command.equals("browse_file_audio")) {
+		} else if (command.equals("browse_file") || command.equals("browse_file_audio") || command.equals("browse_file_zip")) {
 			try {
 				int requestId = Integer.parseInt(params);
 				int packedResultCode = packResultCode(RESULT_OPEN_DOCUMENT, requestId);
@@ -1394,14 +1439,16 @@ public abstract class NativeActivity extends Activity {
 				Intent intent = new Intent(Intent.ACTION_OPEN_DOCUMENT);
 				intent.addCategory(Intent.CATEGORY_OPENABLE);
 				if (command.equals("browse_file_audio")) {
-					intent.setType("audio/x-wav");
+					// Trickery for multiple mime types.
+					String [] mimeTypes = {"audio/x-wav", "audio/x-mpeg3", "audio/mpeg"};
+					intent.setType("*/*");
+					intent.putExtra(Intent.EXTRA_MIME_TYPES, mimeTypes);
+				} else if (command.equals("browse_file_zip")) {
+					intent.setType("application/zip");
 				} else {
 					intent.setType("*/*");
 				}
 				intent.addFlags(Intent.FLAG_GRANT_PERSISTABLE_URI_PERMISSION);
-				// Possible alternative approach:
-				// String[] mimeTypes = {"application/octet-stream", "/x-iso9660-image"};
-				// intent.putExtra(Intent.EXTRA_MIME_TYPES, mimeTypes);
 				startActivityForResult(intent, packedResultCode);
 				// intent.putExtra(DocumentsContract.EXTRA_INITIAL_URI, pickerInitialUri);
 			} catch (Exception e) {
@@ -1474,7 +1521,7 @@ public abstract class NativeActivity extends Activity {
 			return true;
 		} else if (command.equals("vibrate")) {
 			int milliseconds = -1;
-			if (!params.equals("")) {
+			if (!params.isEmpty()) {
 				try {
 					milliseconds = Integer.parseInt(params);
 				} catch (NumberFormatException e) {
@@ -1530,7 +1577,7 @@ public abstract class NativeActivity extends Activity {
 			recreate();
 		} else if (command.equals("graphics_restart")) {
 			Log.i(TAG, "graphics_restart");
-			if (params != null && !params.equals("")) {
+			if (params != null && !params.isEmpty()) {
 				overrideShortcutParam = params;
 			}
 			shuttingDown = true;
@@ -1549,7 +1596,25 @@ public abstract class NativeActivity extends Activity {
 			} else if (params.equals("close")) {
 				mLocationHelper.stopLocationUpdates();
 			}
+		} else if (command.equals("infrared_command")) {
+			if (mInfraredHelper == null) {
+				return false;
+			}
+			if (params.startsWith("sircs") && Build.VERSION.SDK_INT >= Build.VERSION_CODES.KITKAT) {
+				Pattern pattern = Pattern.compile("sircs_(\\d+)_(\\d+)_(\\d+)_(\\d+)");
+				Matcher matcher = pattern.matcher(params);
+				if (!matcher.matches())
+					return false;
+				int ir_version = Integer.parseInt(matcher.group(1));
+				int ir_command = Integer.parseInt(matcher.group(2));
+				int ir_address = Integer.parseInt(matcher.group(3));
+				int ir_count   = Integer.parseInt(matcher.group(4));
+				mInfraredHelper.sendSircCommand(ir_version, ir_command, ir_address, ir_count);
+			}
 		} else if (command.equals("camera_command")) {
+			if (mCameraHelper == null) {
+				return false;
+			}
 			if (params.startsWith("startVideo")) {
 				Pattern pattern = Pattern.compile("startVideo_(\\d+)x(\\d+)");
 				Matcher matcher = pattern.matcher(params);
@@ -1558,7 +1623,7 @@ public abstract class NativeActivity extends Activity {
 				int width = Integer.parseInt(matcher.group(1));
 				int height = Integer.parseInt(matcher.group(2));
 				mCameraHelper.setCameraSize(width, height);
-				if (mCameraHelper != null && !askForPermissions(permissionsForCamera, REQUEST_CODE_CAMERA_PERMISSION)) {
+				if (!askForPermissions(permissionsForCamera, REQUEST_CODE_CAMERA_PERMISSION)) {
 					mCameraHelper.startCamera();
 				}
 			} else if (mCameraHelper != null && params.equals("stopVideo")) {
@@ -1574,10 +1639,10 @@ public abstract class NativeActivity extends Activity {
 			} else if (params.equals("stopRecording")) {
 				NativeApp.audioRecording_Stop();
 			}
-		} else if (command.equals("uistate")) {
+		} else if (command.equals("set_keep_screen_bright")) {
 			Window window = this.getWindow();
-			if (params.equals("ingame")) {
-				// Keep the screen bright - very annoying if it goes dark when tilting away
+			if (params.equals("on")) {
+				// Keep the screen bright - very annoying if it goes dark when using tilt or a joystick
 				window.addFlags(WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON);
 				updateSustainedPerformanceMode();
 			} else {
@@ -1590,10 +1655,42 @@ public abstract class NativeActivity extends Activity {
 			} catch (Exception e) {
 				NativeApp.reportException(e, params);
 			}
+		} else if (command.equals("show_folder")) {
+			try {
+				Uri selectedUri = Uri.parse(params);
+				Intent intent = new Intent(Intent.ACTION_VIEW);
+				intent.setDataAndType(selectedUri, "resource/folder");
+				if (intent.resolveActivityInfo(getPackageManager(), 0) != null) {
+					startActivity(intent);
+					Log.i(TAG, "Started activity for " + params);
+					return true;
+				} else {
+					Log.w(TAG, "No file explorer installed");
+					// if you reach this place, it means there is no any file
+					// explorer app installed on your device
+					return false;
+				}
+			} catch (Exception e) {
+				NativeApp.reportException(e, params);
+				return false;
+			}
+		} else if (command.equals("copy_to_clipboard")) {
+			if (android.os.Build.VERSION.SDK_INT >= Build.VERSION_CODES.HONEYCOMB) {
+				copyStringToClipboard(params);
+			}
+		} else {
+			Log.w(TAG, "Unknown string command " + command);
 		}
 		return false;
 	}
 
+	@TargetApi(Build.VERSION_CODES.HONEYCOMB)
+	private void copyStringToClipboard(String text) {
+		ClipboardManager clipboard = (ClipboardManager) getSystemService(Context.CLIPBOARD_SERVICE);
+		ClipData clip = ClipData.newPlainText("Copied Text", text);
+		clipboard.setPrimaryClip(clip);
+	}
+
 	@SuppressLint("NewApi")
 	@Override
 	public void recreate() {
