@@ -35,6 +35,8 @@
 #include <sys/sysctl.h>
 #include <mach/machine.h>
 
+// This stuff reads directly from _gsEvent which is not allowed on the App Store.
+
 #ifndef IS_IOS7
 #define IS_IOS7 ([[UIDevice currentDevice].systemVersion floatValue]>=7.0)
 #endif
@@ -63,6 +65,8 @@
 
 @implementation PPSSPPUIApplication
 
+#if !PPSSPP_PLATFORM(IOS_APP_STORE)
+
 - (void)decodeKeyEvent:(NSInteger *)eventMem {
     NSInteger eventType = eventMem[GSEVENT_TYPE];
     NSInteger eventScanCode = eventMem[GSEVENTKEY_KEYCODE];
@@ -108,4 +112,6 @@
     }
 }
 
+#endif  // IOS_APP_STORE
+
 @end
