@@ -9,14 +9,14 @@
 
 #include "Common/CPUDetect.h"
 #include "Common/Log.h"
-#include "Common/LogManager.h"
+#include "Common/Log/LogManager.h"
 #include "Common/System/Display.h"
 #include "Common/System/NativeApp.h"
 #include "Common/System/System.h"
 #include "Common/TimeUtil.h"
 #include "Common/File/FileUtil.h"
 #include "Common/Serialize/Serializer.h"
-#include "Common/ConsoleListener.h"
+#include "Common/Log/StdioListener.h"
 #include "Common/Input/InputState.h"
 #include "Common/Thread/ThreadUtil.h"
 #include "Common/Thread/ThreadManager.h"
@@ -32,7 +32,6 @@
 #include "Core/HLE/sceUtility.h"
 #include "Core/HLE/__sceAudio.h"
 #include "Core/HW/MemoryStick.h"
-#include "Core/HW/StereoResampler.h"
 #include "Core/MemMap.h"
 #include "Core/System.h"
 #include "Core/CoreTiming.h"
@@ -69,6 +68,13 @@
 
 #define SAMPLERATE 44100
 
+/* AUDIO output buffer */
+static struct {
+   int16_t *data;
+   int32_t size;
+   int32_t capacity;
+} output_audio_buffer = {NULL, 0, 0};
+
 // Calculated swap interval is 'stable' if the same
 // value is recorded for a number of retro_run()
 // calls equal to VSYNC_SWAP_INTERVAL_FRAMES
@@ -94,12 +100,15 @@ namespace Libretro
 {
    LibretroGraphicsContext *ctx;
    retro_environment_t environ_cb;
+   retro_hw_context_type backend = RETRO_HW_CONTEXT_DUMMY;
    static retro_audio_sample_batch_t audio_batch_cb;
    static retro_input_poll_t input_poll_cb;
    static retro_input_state_t input_state_cb;
+   static retro_log_printf_t log_cb;
 
    static bool detectVsyncSwapInterval = false;
    static bool detectVsyncSwapIntervalOptShown = true;
+   static bool softwareRenderInitHack = false;
 
    static s64 expectedTimeUsPerRun = 0;
    static uint32_t vsyncSwapInterval = 1;
@@ -110,6 +119,59 @@ namespace Libretro
    static float runSpeed = 0.0f;
    static s64 runTicksLast = 0;
 
+   static void ensure_output_audio_buffer_capacity(int32_t capacity)
+   {
+      if (capacity <= output_audio_buffer.capacity) {
+         return;
+      }
+
+      output_audio_buffer.data = (int16_t*)realloc(output_audio_buffer.data, capacity * sizeof(*output_audio_buffer.data));
+      output_audio_buffer.capacity = capacity;
+      log_cb(RETRO_LOG_DEBUG, "Output audio buffer capacity set to %d\n", capacity);
+   }
+
+   static void init_output_audio_buffer(int32_t capacity)
+   {
+      output_audio_buffer.data = NULL;
+      output_audio_buffer.size = 0;
+      output_audio_buffer.capacity = 0;
+      ensure_output_audio_buffer_capacity(capacity);
+   }
+
+   static void free_output_audio_buffer()
+   {
+      free(output_audio_buffer.data);
+      output_audio_buffer.data = NULL;
+      output_audio_buffer.size = 0;
+      output_audio_buffer.capacity = 0;
+   }
+
+   static void upload_output_audio_buffer()
+   {
+      audio_batch_cb(output_audio_buffer.data, output_audio_buffer.size / 2);
+      output_audio_buffer.size = 0;
+   }
+
+
+   /**
+    * Clamp a value to a given range.
+    *
+    * This implementation was taken from `RGBAUtil.cpp` to allow building when `std::clamp()` is unavailable.
+    *
+    * @param f The value to clamp.
+    * @param low The lower bound of the range.
+    * @param high The upper bound of the range.
+    * @return The clamped value.
+    */
+   template <typename T>
+   static T clamp(T f, T low, T high) {
+      if (f < low)
+         return low;
+      if (f > high)
+         return high;
+      return f;
+   }
+
    static void VsyncSwapIntervalReset()
    {
       expectedTimeUsPerRun = (s64)(1000000.0f / (60.0f / 1.001f));
@@ -425,9 +487,12 @@ static std::string map_psp_language_to_i18n_locale(int val)
 
 static void check_variables(CoreParameter &coreParam)
 {
-   bool isFastForwarding;
-   if (environ_cb(RETRO_ENVIRONMENT_GET_FASTFORWARDING, &isFastForwarding))
-       coreParam.fastForward = isFastForwarding;
+   if (g_Config.bForceLagSync)
+   {
+      bool isFastForwarding;
+      if (environ_cb(RETRO_ENVIRONMENT_GET_FASTFORWARDING, &isFastForwarding))
+          coreParam.fastForward = isFastForwarding;
+   }
 
    bool updated = false;
 
@@ -442,6 +507,7 @@ static void check_variables(CoreParameter &coreParam)
    int iTexScalingType_prev;
    int iTexScalingLevel_prev;
    int iMultiSampleLevel_prev;
+   bool bDisplayCropTo16x9_prev;
 
    var.key = "ppsspp_language";
    if (environ_cb(RETRO_ENVIRONMENT_GET_VARIABLE, &var) && var.value)
@@ -523,6 +589,8 @@ static void check_variables(CoreParameter &coreParam)
          g_Config.iIOTimingMethod = IOTIMING_HOST;
       else if (!strcmp(var.value, "Simulate UMD delays"))
          g_Config.iIOTimingMethod = IOTIMING_REALISTIC;
+      else if (!strcmp(var.value, "Simulate UMD slow reading speed"))
+         g_Config.iIOTimingMethod = IOTIMING_UMDSLOWREALISTIC;
    }
 
    var.key = "ppsspp_force_lag_sync";
@@ -583,6 +651,22 @@ static void check_variables(CoreParameter &coreParam)
          g_Config.bAnalogIsCircular = true;
    }
 
+   var.key = "ppsspp_analog_deadzone";
+   if (environ_cb(RETRO_ENVIRONMENT_GET_VARIABLE, &var) && var.value)
+      g_Config.fAnalogDeadzone = atof(var.value);
+
+   var.key = "ppsspp_analog_sensitivity";
+   if (environ_cb(RETRO_ENVIRONMENT_GET_VARIABLE, &var) && var.value)
+      g_Config.fAnalogSensitivity = atof(var.value);
+   
+   var.key = "ppsspp_memstick_inserted";
+   if (environ_cb(RETRO_ENVIRONMENT_GET_VARIABLE, &var) && var.value)
+   {
+      if (!strcmp(var.value, "disabled"))
+         g_Config.bMemStickInserted = false;
+      else
+         g_Config.bMemStickInserted = true;
+   }
 
    var.key = "ppsspp_internal_resolution";
    if (environ_cb(RETRO_ENVIRONMENT_GET_VARIABLE, &var) && var.value)
@@ -609,6 +693,26 @@ static void check_variables(CoreParameter &coreParam)
          g_Config.iInternalResolution = 9;
       else if (!strcmp(var.value, "4800x2720"))
          g_Config.iInternalResolution = 10;
+
+      // Force resolution to 1x without hardware context
+      if (backend == RETRO_HW_CONTEXT_NONE)
+         g_Config.iInternalResolution = 1;
+   }
+
+   var.key = "ppsspp_software_rendering";
+   if (environ_cb(RETRO_ENVIRONMENT_GET_VARIABLE, &var) && var.value)
+   {
+      if (!PSP_IsInited())
+      {
+         if (!strcmp(var.value, "disabled") && backend != RETRO_HW_CONTEXT_NONE)
+            g_Config.bSoftwareRendering = false;
+         else
+            g_Config.bSoftwareRendering = true;
+      }
+
+      // Force resolution to 1x with software rendering
+      if (g_Config.bSoftwareRendering)
+         g_Config.iInternalResolution = 1;
    }
 
 #if 0 // see issue #16786
@@ -628,13 +732,15 @@ static void check_variables(CoreParameter &coreParam)
    }
 #endif
 
-   var.key = "ppsspp_skip_gpu_readbacks";
+   var.key = "ppsspp_cropto16x9";
    if (environ_cb(RETRO_ENVIRONMENT_GET_VARIABLE, &var) && var.value)
    {
+      bDisplayCropTo16x9_prev = g_Config.bDisplayCropTo16x9;
+
       if (!strcmp(var.value, "disabled"))
-         g_Config.bSkipGPUReadbacks = false;
+         g_Config.bDisplayCropTo16x9 = false;
       else
-         g_Config.bSkipGPUReadbacks = true;
+         g_Config.bDisplayCropTo16x9 = true;
    }
 
    var.key = "ppsspp_frameskip";
@@ -688,31 +794,31 @@ static void check_variables(CoreParameter &coreParam)
          g_Config.iInflightFrames = 2;
    }
 
-   var.key = "ppsspp_gpu_hardware_transform";
+   var.key = "ppsspp_skip_buffer_effects";
    if (environ_cb(RETRO_ENVIRONMENT_GET_VARIABLE, &var) && var.value)
    {
       if (!strcmp(var.value, "disabled"))
-         g_Config.bHardwareTransform = false;
+         g_Config.bSkipBufferEffects = false;
       else
-         g_Config.bHardwareTransform = true;
+         g_Config.bSkipBufferEffects = true;
    }
 
-   var.key = "ppsspp_software_skinning";
+   var.key = "ppsspp_disable_range_culling";
    if (environ_cb(RETRO_ENVIRONMENT_GET_VARIABLE, &var) && var.value)
    {
       if (!strcmp(var.value, "disabled"))
-         g_Config.bSoftwareSkinning = false;
+         g_Config.bDisableRangeCulling = false;
       else
-         g_Config.bSoftwareSkinning = true;
+         g_Config.bDisableRangeCulling = true;
    }
 
-   var.key = "ppsspp_vertex_cache";
+   var.key = "ppsspp_skip_gpu_readbacks";
    if (environ_cb(RETRO_ENVIRONMENT_GET_VARIABLE, &var) && var.value)
    {
       if (!strcmp(var.value, "disabled"))
-         g_Config.bVertexCache = false;
+         g_Config.iSkipGPUReadbackMode = (int)SkipGPUReadbackMode::NO_SKIP;
       else
-         g_Config.bVertexCache = true;
+         g_Config.iSkipGPUReadbackMode = (int)SkipGPUReadbackMode::SKIP;
    }
 
    var.key = "ppsspp_lazy_texture_caching";
@@ -735,6 +841,24 @@ static void check_variables(CoreParameter &coreParam)
          g_Config.iSplineBezierQuality = 2;
    }
 
+   var.key = "ppsspp_gpu_hardware_transform";
+   if (environ_cb(RETRO_ENVIRONMENT_GET_VARIABLE, &var) && var.value)
+   {
+      if (!strcmp(var.value, "disabled"))
+         g_Config.bHardwareTransform = false;
+      else
+         g_Config.bHardwareTransform = true;
+   }
+
+   var.key = "ppsspp_software_skinning";
+   if (environ_cb(RETRO_ENVIRONMENT_GET_VARIABLE, &var) && var.value)
+   {
+      if (!strcmp(var.value, "disabled"))
+         g_Config.bSoftwareSkinning = false;
+      else
+         g_Config.bSoftwareSkinning = true;
+   }
+
    var.key = "ppsspp_hardware_tesselation";
    if (environ_cb(RETRO_ENVIRONMENT_GET_VARIABLE, &var) && var.value)
    {
@@ -841,6 +965,15 @@ static void check_variables(CoreParameter &coreParam)
          g_Config.iTexFiltering = 4;
    }
 
+   var.key = "ppsspp_smart_2d_texture_filtering";
+   if (environ_cb(RETRO_ENVIRONMENT_GET_VARIABLE, &var) && var.value)
+   {
+      if (!strcmp(var.value, "disabled"))
+         g_Config.bSmart2DTexFiltering = false;
+      else
+         g_Config.bSmart2DTexFiltering = true;
+   }
+
    var.key = "ppsspp_texture_replacement";
    if (environ_cb(RETRO_ENVIRONMENT_GET_VARIABLE, &var) && var.value)
    {
@@ -995,16 +1128,18 @@ static void check_variables(CoreParameter &coreParam)
          !g_Config.bRenderDuplicateFrames;
 
    bool updateAvInfo = false;
+   bool updateGeometry = false;
+
    if (!detectVsyncSwapInterval && (vsyncSwapInterval != 1))
    {
       vsyncSwapInterval = 1;
       updateAvInfo = true;
    }
 
-   if (g_Config.iInternalResolution != iInternalResolution_prev && !PSP_IsInited())
+   if (g_Config.iInternalResolution != iInternalResolution_prev && backend != RETRO_HW_CONTEXT_NONE)
    {
-      coreParam.pixelWidth  = coreParam.renderWidth  = g_Config.iInternalResolution * 480;
-      coreParam.pixelHeight = coreParam.renderHeight = g_Config.iInternalResolution * 272;
+      coreParam.pixelWidth  = coreParam.renderWidth  = g_Config.iInternalResolution * NATIVEWIDTH;
+      coreParam.pixelHeight = coreParam.renderHeight = g_Config.iInternalResolution * NATIVEHEIGHT;
 
       if (gpu)
       {
@@ -1016,6 +1151,13 @@ static void check_variables(CoreParameter &coreParam)
       }
    }
 
+   if (g_Config.bDisplayCropTo16x9 != bDisplayCropTo16x9_prev && PSP_IsInited())
+   {
+      updateGeometry = true;
+      if (gpu)
+         gpu->NotifyDisplayResized();
+   }
+
 #if 0 // see issue #16786
    if (g_Config.iMultiSampleLevel != iMultiSampleLevel_prev && PSP_IsInited())
    {
@@ -1032,6 +1174,12 @@ static void check_variables(CoreParameter &coreParam)
       retro_get_system_av_info(&avInfo);
       environ_cb(RETRO_ENVIRONMENT_SET_SYSTEM_AV_INFO, &avInfo);
    }
+   else if (updateGeometry)
+   {
+      retro_system_av_info avInfo;
+      retro_get_system_av_info(&avInfo);
+      environ_cb(RETRO_ENVIRONMENT_SET_GEOMETRY, &avInfo);
+   }
 
    set_variable_visibility();
 }
@@ -1041,11 +1189,36 @@ void retro_set_audio_sample(retro_audio_sample_t cb) { (void)cb; }
 void retro_set_input_poll(retro_input_poll_t cb) { input_poll_cb = cb; }
 void retro_set_input_state(retro_input_state_t cb) { input_state_cb = cb; }
 
+static const struct retro_controller_description psp_controllers[] =
+{
+   { "PSP", RETRO_DEVICE_JOYPAD },
+   { NULL, 0 }
+};
+
+static const struct retro_controller_info ports[] =
+{
+   { psp_controllers, 1 },
+   { NULL, 0 }
+};
+
 void retro_init(void)
 {
-   VsyncSwapIntervalReset();
+   TimeInit();
 
-   g_threadManager.Init(cpu_info.num_cores, cpu_info.logical_cpu_count);
+   struct retro_log_callback log;
+   if (environ_cb(RETRO_ENVIRONMENT_GET_LOG_INTERFACE, &log))
+   {
+      log_cb = log.log;
+      LogManager::Init(&g_Config.bEnableLogging);
+      printfLogger = new PrintfLogger(log);
+      LogManager* logman = LogManager::GetInstance();
+      logman->RemoveListener(logman->GetStdioListener());
+      logman->RemoveListener(logman->GetDebuggerListener());
+      logman->ChangeFileLog(nullptr);
+      logman->AddListener(printfLogger);
+   }
+
+   VsyncSwapIntervalReset();
 
    struct retro_input_descriptor desc[] = {
       { 0, RETRO_DEVICE_JOYPAD, 0, RETRO_DEVICE_ID_JOYPAD_LEFT, "D-Pad Left" },
@@ -1067,26 +1240,17 @@ void retro_init(void)
       { 0 },
    };
    environ_cb(RETRO_ENVIRONMENT_SET_INPUT_DESCRIPTORS, desc);
+   environ_cb(RETRO_ENVIRONMENT_SET_CONTROLLER_INFO, (void*)ports);
 
    if (environ_cb(RETRO_ENVIRONMENT_GET_INPUT_BITMASKS, NULL))
       libretro_supports_bitmasks = true;
 
-   struct retro_log_callback log;
-   if (environ_cb(RETRO_ENVIRONMENT_GET_LOG_INTERFACE, &log))
-   {
-      LogManager::Init(&g_Config.bEnableLogging);
-      printfLogger = new PrintfLogger(log);
-      LogManager* logman = LogManager::GetInstance();
-      logman->RemoveListener(logman->GetConsoleListener());
-      logman->RemoveListener(logman->GetDebuggerListener());
-      logman->ChangeFileLog(nullptr);
-      logman->AddListener(printfLogger);
-      logman->SetAllLogLevels(LogLevel::LINFO);
-   }
-
    g_Config.Load("", "");
    g_Config.iInternalResolution = 0;
 
+   // Log levels must be set after g_Config.Load
+   LogManager::GetInstance()->SetAllLogLevels(LogLevel::LINFO);
+
    const char* nickname = NULL;
    if (environ_cb(RETRO_ENVIRONMENT_GET_USERNAME, &nickname) && nickname)
       g_Config.sNickName = std::string(nickname);
@@ -1108,15 +1272,20 @@ void retro_init(void)
    g_Config.flash0Directory = retro_base_dir / "flash0";
    g_Config.internalDataDirectory = retro_base_dir;
    g_Config.bEnableNetworkChat = false;
-   g_Config.bDiscordPresence = false;
+   g_Config.bDiscordRichPresence = false;
 
    g_VFS.Register("", new DirectoryReader(retro_base_dir));
+
+   g_threadManager.Init(cpu_info.num_cores, cpu_info.logical_cpu_count);
+
+   init_output_audio_buffer(2048);
 }
 
 void retro_deinit(void)
 {
    g_threadManager.Teardown();
    LogManager::Shutdown();
+   log_cb = NULL;
 
    delete printfLogger;
    printfLogger = nullptr;
@@ -1125,6 +1294,8 @@ void retro_deinit(void)
    libretro_supports_option_categories = false;
 
    VsyncSwapIntervalReset();
+
+   free_output_audio_buffer();
 }
 
 void retro_set_controller_port_device(unsigned port, unsigned device)
@@ -1139,7 +1310,7 @@ void retro_get_system_info(struct retro_system_info *info)
    info->library_name     = "PPSSPP";
    info->library_version  = PPSSPP_GIT_VERSION;
    info->need_fullpath    = true;
-   info->valid_extensions = "elf|iso|cso|prx|pbp";
+   info->valid_extensions = "elf|iso|cso|prx|pbp|chd";
 }
 
 void retro_get_system_av_info(struct retro_system_av_info *info)
@@ -1148,11 +1319,23 @@ void retro_get_system_av_info(struct retro_system_av_info *info)
    info->timing.fps            = (60.0 / 1.001) / (double)vsyncSwapInterval;
    info->timing.sample_rate    = SAMPLERATE;
 
-   info->geometry.base_width   = g_Config.iInternalResolution * 480;
-   info->geometry.base_height  = g_Config.iInternalResolution * 272;
-   info->geometry.max_width    = g_Config.iInternalResolution * 480;
-   info->geometry.max_height   = g_Config.iInternalResolution * 272;
-   info->geometry.aspect_ratio = 480.0 / 272.0;  // Not 16:9! But very, very close.
+   info->geometry.base_width   = g_Config.iInternalResolution * NATIVEWIDTH;
+   info->geometry.base_height  = g_Config.iInternalResolution * NATIVEHEIGHT;
+   info->geometry.max_width    = g_Config.iInternalResolution * NATIVEWIDTH;
+   info->geometry.max_height   = g_Config.iInternalResolution * NATIVEHEIGHT;
+
+   if (g_Config.bDisplayCropTo16x9)
+      info->geometry.base_height -= g_Config.iInternalResolution * 2;
+
+   info->geometry.aspect_ratio = (float)info->geometry.base_width / (float)info->geometry.base_height;
+
+   PSP_CoreParameter().pixelWidth  = PSP_CoreParameter().renderWidth  = info->geometry.base_width;
+   PSP_CoreParameter().pixelHeight = PSP_CoreParameter().renderHeight = info->geometry.base_height;
+
+   /* Must reset context to resize render area properly while running,
+    * but not necessary with software, and not working with Vulkan.. (TODO) */
+   if (PSP_IsInited() && ctx && backend != RETRO_HW_CONTEXT_NONE && ctx->GetGPUCore() != GPUCORE_VULKAN)
+      ((LibretroHWRenderContext *)Libretro::ctx)->ContextReset();
 }
 
 unsigned retro_api_version(void) { return RETRO_API_VERSION; }
@@ -1254,18 +1437,40 @@ namespace Libretro
 
 } // namespace Libretro
 
+static void retro_check_backend(void)
+{
+   struct retro_variable var = {0};
+
+   var.key = "ppsspp_backend";
+   if (environ_cb(RETRO_ENVIRONMENT_GET_VARIABLE, &var) && var.value)
+   {
+      if (!strcmp(var.value, "auto"))
+         backend = RETRO_HW_CONTEXT_DUMMY;
+      else if (!strcmp(var.value, "opengl"))
+         backend = RETRO_HW_CONTEXT_OPENGL;
+      else if (!strcmp(var.value, "vulkan"))
+         backend = RETRO_HW_CONTEXT_VULKAN;
+      else if (!strcmp(var.value, "d3d11"))
+         backend = RETRO_HW_CONTEXT_DIRECT3D;
+      else if (!strcmp(var.value, "none"))
+         backend = RETRO_HW_CONTEXT_NONE;
+   }
+}
+
 bool retro_load_game(const struct retro_game_info *game)
 {
    retro_pixel_format fmt = retro_pixel_format::RETRO_PIXEL_FORMAT_XRGB8888;
    if (!environ_cb(RETRO_ENVIRONMENT_SET_PIXEL_FORMAT, &fmt))
    {
-      ERROR_LOG(SYSTEM, "XRGB8888 is not supported.\n");
+      ERROR_LOG(Log::System, "XRGB8888 is not supported.\n");
       return false;
    }
 
+   retro_check_backend();
+
    coreState = CORE_POWERUP;
    ctx       = LibretroGraphicsContext::CreateGraphicsContext();
-   INFO_LOG(SYSTEM, "Using %s backend", ctx->Ident());
+   INFO_LOG(Log::System, "Using %s backend", ctx->Ident());
 
    Core_SetGraphicsContext(ctx);
    SetGPUBackend((GPUBackend)g_Config.iGPUBackend);
@@ -1285,13 +1490,19 @@ bool retro_load_game(const struct retro_game_info *game)
    coreParam.gpuCore         = ctx->GetGPUCore();
    check_variables(coreParam);
 
+   // TODO: OpenGL goes black when inited with software rendering,
+   // therefore start without, set back after init, and reset.
+   softwareRenderInitHack    = ctx->GetGPUCore() == GPUCORE_GLES && g_Config.bSoftwareRendering;
+   if (softwareRenderInitHack)
+      g_Config.bSoftwareRendering = false;
+
    // set cpuCore from libretro setting variable
    coreParam.cpuCore         =  (CPUCore)g_Config.iCpuCore;
 
    std::string error_string;
    if (!PSP_InitStart(coreParam, &error_string))
    {
-      ERROR_LOG(BOOT, "%s", error_string.c_str());
+      ERROR_LOG(Log::Boot, "%s", error_string.c_str());
       return false;
    }
 
@@ -1338,7 +1549,7 @@ void retro_reset(void)
 
    if (!PSP_Init(PSP_CoreParameter(), &error_string))
    {
-      ERROR_LOG(BOOT, "%s", error_string.c_str());
+      ERROR_LOG(Log::Boot, "%s", error_string.c_str());
       environ_cb(RETRO_ENVIRONMENT_SHUTDOWN, nullptr);
    }
 }
@@ -1420,8 +1631,8 @@ static void retro_input(void)
    }
 
    float mappedNorm = norm;
-   x_left = std::clamp(x_left / norm * mappedNorm, -1.0f, 1.0f);
-   y_left = std::clamp(y_left / norm * mappedNorm, -1.0f, 1.0f);
+   x_left = Libretro::clamp(x_left / norm * mappedNorm, -1.0f, 1.0f);
+   y_left = Libretro::clamp(y_left / norm * mappedNorm, -1.0f, 1.0f);
 
    __CtrlSetAnalogXY(CTRL_STICK_LEFT, x_left, y_left);
    __CtrlSetAnalogXY(CTRL_STICK_RIGHT, x_right, y_right);
@@ -1437,10 +1648,18 @@ void retro_run(void)
 
       if (!PSP_IsInited())
       {
-         ERROR_LOG(BOOT, "%s", error_string.c_str());
+         ERROR_LOG(Log::Boot, "%s", error_string.c_str());
          environ_cb(RETRO_ENVIRONMENT_SHUTDOWN, nullptr);
          return;
       }
+
+      if (softwareRenderInitHack)
+      {
+         log_cb(RETRO_LOG_DEBUG, "Software rendering init hack for opengl triggered.\n");
+         softwareRenderInitHack = false;
+         g_Config.bSoftwareRendering = true;
+         retro_reset();
+      }
    }
 
    check_variables(PSP_CoreParameter());
@@ -1449,7 +1668,7 @@ void retro_run(void)
 
    if (useEmuThread)
    {
-      if(   emuThreadState == EmuThreadState::PAUSED ||
+      if (  emuThreadState == EmuThreadState::PAUSED ||
             emuThreadState == EmuThreadState::PAUSE_REQUESTED)
       {
          VsyncSwapIntervalDetect();
@@ -1471,6 +1690,7 @@ void retro_run(void)
 
    VsyncSwapIntervalDetect();
    ctx->SwapBuffers();
+   upload_output_audio_buffer();
 }
 
 unsigned retro_get_region(void) { return RETRO_REGION_NTSC; }
@@ -1487,9 +1707,8 @@ namespace SaveState
 
 size_t retro_serialize_size(void)
 {
-   if(!gpu) { // The HW renderer isn't ready on first pass.
+   if (!gpu) // The HW renderer isn't ready on first pass.
       return 134217728; // 128MB ought to be enough for anybody.
-   }
 
    SaveState::SaveStart state;
    // TODO: Libretro API extension to use the savestate queue
@@ -1502,9 +1721,8 @@ size_t retro_serialize_size(void)
 
 bool retro_serialize(void *data, size_t size)
 {
-   if(!gpu) { // The HW renderer isn't ready on first pass.
+   if (!gpu) // The HW renderer isn't ready on first pass.
       return false;
-   }
 
    // TODO: Libretro API extension to use the savestate queue
    if (useEmuThread)
@@ -1526,6 +1744,9 @@ bool retro_serialize(void *data, size_t size)
 
 bool retro_unserialize(const void *data, size_t size)
 {
+   if (!gpu) // The HW renderer isn't ready on first pass.
+      return false;
+
    // TODO: Libretro API extension to use the savestate queue
    if (useEmuThread)
       EmuThreadPause(); // Does nothing if already paused
@@ -1639,7 +1860,7 @@ void retro_cheat_set(unsigned index, bool enabled, const char *code) {
    }
 }
 
-int System_GetPropertyInt(SystemProperty prop)
+int64_t System_GetPropertyInt(SystemProperty prop)
 {
    switch (prop)
    {
@@ -1666,11 +1887,7 @@ float System_GetPropertyFloat(SystemProperty prop)
    switch (prop)
    {
       case SYSPROP_DISPLAY_REFRESH_RATE:
-         // Have to lie here and report 60 Hz instead
-         // of (60.0 / 1.001), otherwise the internal
-         // stereo resampler will output at the wrong
-         // frequency...
-         return 60.0f;
+         return 60.0f / 1.001f;
       case SYSPROP_DISPLAY_SAFE_INSET_LEFT:
       case SYSPROP_DISPLAY_SAFE_INSET_RIGHT:
       case SYSPROP_DISPLAY_SAFE_INSET_TOP:
@@ -1708,12 +1925,12 @@ void System_Notify(SystemNotification notification) {
       break;
    }
 }
-bool System_MakeRequest(SystemRequestType type, int requestId, const std::string &param1, const std::string &param2, int param3) { return false; }
-void System_PostUIMessage(const std::string &message, const std::string &param) {}
+bool System_MakeRequest(SystemRequestType type, int requestId, const std::string &param1, const std::string &param2, int64_t param3, int64_t param4) { return false; }
+void System_PostUIMessage(UIMessage message, const std::string &param) {}
 void NativeFrame(GraphicsContext *graphicsContext) {}
 void NativeResized() {}
 
-void System_Toast(const char *str) {}
+void System_Toast(std::string_view str) {}
 
 inline int16_t Clamp16(int32_t sample) {
    if (sample < -32767) return -32767;
@@ -1724,6 +1941,8 @@ inline int16_t Clamp16(int32_t sample) {
 void System_AudioPushSamples(const int32_t *audio, int numSamples) {
    // Convert to 16-bit audio for further processing.
    int16_t buffer[1024 * 2];
+   int origSamples = numSamples * 2;
+
    while (numSamples > 0) {
       int blockSize = std::min(1024, numSamples);
       for (int i = 0; i < blockSize; i++) {
@@ -1731,14 +1950,13 @@ void System_AudioPushSamples(const int32_t *audio, int numSamples) {
          buffer[i * 2 + 1] = Clamp16(audio[i * 2 + 1]);
       }
 
-      int framesToWrite = blockSize;
-      uint32_t framesWritten = audio_batch_cb(buffer, framesToWrite);
-      if (framesWritten != framesToWrite) {
-         // Let's just stop, and eat any left-over samples.
-         break;
-      }
       numSamples -= blockSize;
    }
+
+   if (output_audio_buffer.capacity - output_audio_buffer.size < origSamples)
+      ensure_output_audio_buffer_capacity((output_audio_buffer.capacity + origSamples) * 1.5);
+   memcpy(output_audio_buffer.data + output_audio_buffer.size, buffer, origSamples * sizeof(*output_audio_buffer.data));
+   output_audio_buffer.size += origSamples;
 }
 
 void System_AudioGetDebugStats(char *buf, size_t bufSize) { if (buf) buf[0] = '\0'; }
@@ -1749,11 +1967,10 @@ std::vector<std::string> System_GetCameraDeviceList() { return std::vector<std::
 bool System_AudioRecordingIsAvailable() { return false; }
 bool System_AudioRecordingState() { return false; }
 
-void System_InputBoxGetString(const std::string &title, const std::string &defaultValue, std::function<void(bool, const std::string &)> cb) { cb(false, ""); }
 #endif
 
 // TODO: To avoid having to define these here, these should probably be turned into system "requests".
-bool NativeSaveSecret(const char *nameOfSecret, const std::string &data) { return false; }
-std::string NativeLoadSecret(const char *nameOfSecret) {
+bool NativeSaveSecret(std::string_view nameOfSecret, std::string_view data) { return false; }
+std::string NativeLoadSecret(std::string_view nameOfSecret) {
    return "";
 }
